@@ -1,45 +1,20 @@
 ;;;; optargs.scm -- support for optional arguments
 ;;;;
-;;;; 	Copyright (C) 1997, 1998, 1999, 2001, 2002, 2004 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1997, 1998, 1999, 2001, 2002 Free Software Foundation, Inc.
 ;;;;
-;;;; This program is free software; you can redistribute it and/or modify
-;;;; it under the terms of the GNU General Public License as published by
-;;;; the Free Software Foundation; either version 2, or (at your option)
-;;;; any later version.
-;;;;
-;;;; This program is distributed in the hope that it will be useful,
+;;;; This library is free software; you can redistribute it and/or
+;;;; modify it under the terms of the GNU Lesser General Public
+;;;; License as published by the Free Software Foundation; either
+;;;; version 2.1 of the License, or (at your option) any later version.
+;;;; 
+;;;; This library is distributed in the hope that it will be useful,
 ;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;;; GNU General Public License for more details.
-;;;;
-;;;; You should have received a copy of the GNU General Public License
-;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;;;; Boston, MA 02111-1307 USA
-;;;;
-;;;; As a special exception, the Free Software Foundation gives permission
-;;;; for additional uses of the text contained in its release of GUILE.
-;;;;
-;;;; The exception is that, if you link the GUILE library with other files
-;;;; to produce an executable, this does not by itself cause the
-;;;; resulting executable to be covered by the GNU General Public License.
-;;;; Your use of that executable is in no way restricted on account of
-;;;; linking the GUILE library code into it.
-;;;;
-;;;; This exception does not however invalidate any other reasons why
-;;;; the executable file might be covered by the GNU General Public License.
-;;;;
-;;;; This exception applies only to the code released by the
-;;;; Free Software Foundation under the name GUILE.  If you copy
-;;;; code from other Free Software Foundation releases into a copy of
-;;;; GUILE, as the General Public License permits, the exception does
-;;;; not apply to the code that you add in this way.  To avoid misleading
-;;;; anyone as to the status of such modified files, you must delete
-;;;; this exception notice from them.
-;;;;
-;;;; If you write modifications of your own for GUILE, it is your choice
-;;;; whether to permit this exception to apply to your modifications.
-;;;; If you do not wish that, delete this exception notice.
+;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;;; Lesser General Public License for more details.
+;;;; 
+;;;; You should have received a copy of the GNU Lesser General Public
+;;;; License along with this library; if not, write to the Free Software
+;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ;;;;
 ;;;; Contributed by Maciej Stachowiak <mstachow@alum.mit.edu>
 
@@ -150,7 +125,7 @@
 
 (define (let-optional-template REST-ARG BINDINGS BODY let-type)
     (if (null? BINDINGS)
-	`(let () ,@BODY)
+	`(begin ,@BODY)
 	(let-o-k-template REST-ARG BINDINGS BODY let-type
 			  (lambda (optional)
 			    `(,(car optional)
@@ -165,7 +140,7 @@
 
 (define (let-keywords-template REST-ARG ALLOW-OTHER-KEYS? BINDINGS BODY let-type)
     (if (null? BINDINGS)
-	`(let () ,@BODY)
+	`(begin ,@BODY)
 	(let* ((kb-list-gensym (gensym "kb:G"))
 	       (bindfilter (lambda (key)
 			     `(,(car key)
@@ -208,19 +183,6 @@
 		  accum
 		  (loop (car rest) (cdr rest) accum)))))))
 
-;; This is a reader extension to support the (deprecated) use of
-;; "#&optional" instead of "#:optional"
-
-(read-hash-extend #\& (lambda (c port)
-			(issue-deprecation-warning
-			 "`#&' is deprecated, use `#:' instead.")
-			(case (read port)
-			  ((optional) #:optional)
-			  ((key) #:key)
-			  ((rest) #:rest)
-			  ((allow-other-keys) #:allow-other-keys)
-			  (else (error "Bad #& value.")))))
-
 
 ;; lambda* args . body
 ;;   lambda extended for optional and keyword arguments
@@ -273,7 +235,7 @@
   (parse-arglist
    ARGLIST
    (lambda (non-optional-args optionals keys aok? rest-arg)
-     ; Check for syntax errors.
+     ;; Check for syntax errors.
      (if (not (every? symbol? non-optional-args))
 	 (error "Syntax error in fixed argument declaration."))
      (if (not (every? ext-decl? optionals))
@@ -374,13 +336,13 @@
      ((null? arglist) (cont '() '() '() #f #f))
      ((not (pair? arglist)) (cont '() '() '() #f arglist))
      ((not (list? arglist))
-      (let* ((copy (improper-list-copy arglist))
-	     (lp (last-pair copy))
-	     (ra (cdr lp)))
-	(set-cdr! lp '())
-	(if (memq #:rest copy)
-	    (error "Cannot specify both #:rest and dotted rest argument.")
-	    (parse-keys copy ra cont))))
+	  (let* ((copy (improper-list-copy arglist))
+		 (lp (last-pair copy))
+		 (ra (cdr lp)))
+	    (set-cdr! lp '())
+	    (if (memq #:rest copy)
+		(error "Cannot specify both #:rest and dotted rest argument.")
+		(parse-keys copy ra cont))))
      (else (split-list-at
 	    #:rest arglist
 	    (lambda (before after split?)
