@@ -1,43 +1,18 @@
-;;;; 	Copyright (C) 1997, 1999, 2001, 2004 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1997, 1999, 2001 Free Software Foundation, Inc.
 ;;;;
-;;;; This program is free software; you can redistribute it and/or modify
-;;;; it under the terms of the GNU General Public License as published by
-;;;; the Free Software Foundation; either version 2, or (at your option)
-;;;; any later version.
-;;;;
-;;;; This program is distributed in the hope that it will be useful,
+;;;; This library is free software; you can redistribute it and/or
+;;;; modify it under the terms of the GNU Lesser General Public
+;;;; License as published by the Free Software Foundation; either
+;;;; version 2.1 of the License, or (at your option) any later version.
+;;;; 
+;;;; This library is distributed in the hope that it will be useful,
 ;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;;; GNU General Public License for more details.
-;;;;
-;;;; You should have received a copy of the GNU General Public License
-;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;;;; Boston, MA 02111-1307 USA
-;;;;
-;;;; As a special exception, the Free Software Foundation gives permission
-;;;; for additional uses of the text contained in its release of GUILE.
-;;;;
-;;;; The exception is that, if you link the GUILE library with other files
-;;;; to produce an executable, this does not by itself cause the
-;;;; resulting executable to be covered by the GNU General Public License.
-;;;; Your use of that executable is in no way restricted on account of
-;;;; linking the GUILE library code into it.
-;;;;
-;;;; This exception does not however invalidate any other reasons why
-;;;; the executable file might be covered by the GNU General Public License.
-;;;;
-;;;; This exception applies only to the code released by the
-;;;; Free Software Foundation under the name GUILE.  If you copy
-;;;; code from other Free Software Foundation releases into a copy of
-;;;; GUILE, as the General Public License permits, the exception does
-;;;; not apply to the code that you add in this way.  To avoid misleading
-;;;; anyone as to the status of such modified files, you must delete
-;;;; this exception notice from them.
-;;;;
-;;;; If you write modifications of your own for GUILE, it is your choice
-;;;; whether to permit this exception to apply to your modifications.
-;;;; If you do not wish that, delete this exception notice.
+;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;;; Lesser General Public License for more details.
+;;;; 
+;;;; You should have received a copy of the GNU Lesser General Public
+;;;; License along with this library; if not, write to the Free Software
+;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ;;;;
 
 ;;; Commentary:
@@ -68,11 +43,6 @@
 	   string-match regexp-substitute fold-matches list-matches
 	   regexp-substitute/global))
 
-;; References:
-;;
-;; POSIX spec:
-;; http://www.opengroup.org/onlinepubs/009695399/basedefs/xbd_chap09.html
-
 ;;; FIXME:
 ;;;   It is not clear what should happen if a `match' function
 ;;;   is passed a `match number' which is out of bounds for the
@@ -108,21 +78,6 @@
 		(loop (+ 1 i)))
 	       (else #f)))))
 
-;; * . \ ^ $ and [ are special in both regexp/basic and regexp/extended and
-;; can be backslash escaped.
-;;
-;; ( ) + ? { } and | are special in regexp/extended so must be quoted.  But
-;; that can't be done with a backslash since in regexp/basic where they're
-;; not special, adding a backslash makes them become special.  Character
-;; class forms [(] etc are used instead.
-;;
-;; ) is not special when not preceded by a (, and * and ? are not special at
-;; the start of a string, but we quote all of these always, so the result
-;; can be concatenated or merged into some larger regexp.
-;;
-;; ] is not special outside a [ ] character class, so doesn't need to be
-;; quoted.
-;;
 (define (regexp-quote string)
   (call-with-output-string
    (lambda (p)
@@ -130,15 +85,9 @@
        (and (< i (string-length string))
 	    (begin
 	      (case (string-ref string i)
-		((#\* #\. #\\ #\^ #\$ #\[)
-		 (write-char #\\ p)
-		 (write-char (string-ref string i) p))
-		((#\( #\) #\+ #\? #\{ #\} #\|)
-		 (write-char #\[ p)
-		 (write-char (string-ref string i) p)
-		 (write-char #\] p))
-		(else
-		 (write-char (string-ref string i) p)))
+		((#\* #\. #\( #\) #\+ #\? #\\ #\^ #\$ #\{ #\})
+		 (write-char #\\ p)))
+	      (write-char (string-ref string i) p)
 	      (loop (1+ i))))))))
 
 (define (match:start match . args)
