@@ -1,52 +1,28 @@
 /* Printing of backtraces and error messages
- * Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation
+ * Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004 Free Software Foundation
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
- *
- * The author can be reached at djurfeldt@nada.kth.se
- * Mikael Djurfeldt, SANS/NADA KTH, 10044 STOCKHOLM, SWEDEN */
-
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <stdio.h>
 #include <ctype.h>
+#include <assert.h>
 
 #include "libguile/_scm.h"
 
@@ -65,23 +41,34 @@
 #include "libguile/fluids.h"
 #include "libguile/ports.h"
 #include "libguile/strings.h"
+#include "libguile/dynwind.h"
 
 #include "libguile/validate.h"
+#include "libguile/lang.h"
 #include "libguile/backtrace.h"
 #include "libguile/filesys.h"
 
 /* {Error reporting and backtraces}
- * (A first approximation.)
  *
  * Note that these functions shouldn't generate errors themselves.
  */
 
-#ifndef SCM_RECKLESS
+/* Print parameters for error messages. */
+
+#define DISPLAY_ERROR_MESSAGE_MAX_LEVEL   7
+#define DISPLAY_ERROR_MESSAGE_MAX_LENGTH 10
+
+/* Print parameters for failing expressions in error messages.
+ * (See also `print_params' below for backtrace print parameters.)
+ */
+
+#define DISPLAY_EXPRESSION_MAX_LEVEL      2
+#define DISPLAY_EXPRESSION_MAX_LENGTH     3
+
 #undef SCM_ASSERT
 #define SCM_ASSERT(_cond, _arg, _pos, _subr) \
 	if (!(_cond)) \
           return SCM_BOOL_F;
-#endif
 
 SCM scm_the_last_stack_fluid_var;
 
@@ -98,17 +85,17 @@ display_header (SCM source, SCM port)
        * filename with the source properties?  Then we could in case of
        * non-file ports give at least some more details than just
        * "<unnamed port>". */
-      if (SCM_STRINGP (fname))
+      if (scm_is_true (fname))
 	scm_prin1 (fname, port, 0);
       else
 	scm_puts ("<unnamed port>", port);
 
-      if (!SCM_FALSEP (line) && !SCM_FALSEP (col))
+      if (scm_is_true (line) && scm_is_true (col))
 	{
 	  scm_putc (':', port);
-	  scm_intprint (SCM_INUM (line) + 1, 10, port);
+	  scm_intprint (scm_to_long (line) + 1, 10, port);
 	  scm_putc (':', port);
-	  scm_intprint (SCM_INUM (col) + 1, 10, port);
+	  scm_intprint (scm_to_long (col) + 1, 10, port);
 	}
     }
   else
@@ -117,31 +104,80 @@ display_header (SCM source, SCM port)
 }
 
 
-void
-scm_display_error_message (SCM message, SCM args, SCM port)
+struct display_error_message_data {
+  SCM message;
+  SCM args;
+  SCM port;
+  scm_print_state *pstate;
+  int old_fancyp;
+  int old_level;
+  int old_length;
+};
+
+static SCM
+display_error_message (struct display_error_message_data *d)
 {
-  if (SCM_STRINGP (message) && !SCM_FALSEP (scm_list_p (args)))
-    {
-      scm_simple_format (port, message, args);
-      scm_newline (port);
-    }
+  if (scm_is_string (d->message) && scm_is_true (scm_list_p (d->args)))
+    scm_simple_format (d->port, d->message, d->args);
   else
-    {
-      scm_display (message, port);
-      scm_newline (port);
-    }
+    scm_display (d->message, d->port);
+  scm_newline (d->port);
+  return SCM_UNSPECIFIED;
 }
 
 static void
-display_expression (SCM frame,SCM pname,SCM source,SCM port)
+before_display_error_message (struct display_error_message_data *d)
+{
+  scm_print_state *pstate = d->pstate;
+  d->old_fancyp = pstate->fancyp;
+  d->old_level  = pstate->level;
+  d->old_length = pstate->length;
+  pstate->fancyp = 1;
+  pstate->level  = DISPLAY_ERROR_MESSAGE_MAX_LEVEL;
+  pstate->length = DISPLAY_ERROR_MESSAGE_MAX_LENGTH;
+}
+
+static void
+after_display_error_message (struct display_error_message_data *d)
+{
+  scm_print_state *pstate = d->pstate;
+  pstate->fancyp = d->old_fancyp;
+  pstate->level  = d->old_level;
+  pstate->length = d->old_length;
+}
+
+void
+scm_display_error_message (SCM message, SCM args, SCM port)
+{
+  struct display_error_message_data d;
+  SCM print_state;
+  scm_print_state *pstate;
+
+  port = scm_i_port_with_print_state (port, SCM_UNDEFINED);
+  print_state = SCM_PORT_WITH_PS_PS (port);
+  pstate = SCM_PRINT_STATE (print_state);
+  
+  d.message = message;
+  d.args = args;
+  d.port = port;
+  d.pstate = pstate;
+  scm_internal_dynamic_wind ((scm_t_guard) before_display_error_message,
+			     (scm_t_inner) display_error_message,
+			     (scm_t_guard) after_display_error_message,
+			     &d,
+			     &d);
+}
+
+static void
+display_expression (SCM frame, SCM pname, SCM source, SCM port)
 {
   SCM print_state = scm_make_print_state ();
   scm_print_state *pstate = SCM_PRINT_STATE (print_state);
   pstate->writingp = 0;
   pstate->fancyp = 1;
-  pstate->level = 2;
-  pstate->length = 3;
-  if (SCM_SYMBOLP (pname) || SCM_STRINGP (pname))
+  pstate->level  = DISPLAY_EXPRESSION_MAX_LEVEL;
+  pstate->length = DISPLAY_EXPRESSION_MAX_LENGTH;
+  if (scm_is_symbol (pname) || scm_is_string (pname))
     {
       if (SCM_FRAMEP (frame)
 	  && SCM_FRAME_EVAL_ARGS_P (frame))
@@ -153,14 +189,14 @@ display_expression (SCM frame,SCM pname,SCM source,SCM port)
 	{
 	  scm_puts (" in expression ", port);
 	  pstate->writingp = 1;
-	  scm_iprin1 (scm_unmemoize (source), port, pstate);
+	  scm_iprin1 (scm_i_unmemoize_expr (source), port, pstate);
 	}
     }
   else if (SCM_MEMOIZEDP (source))
     {
       scm_puts ("In expression ", port);
       pstate->writingp = 1;
-      scm_iprin1 (scm_unmemoize (source), port, pstate);
+      scm_iprin1 (scm_i_unmemoize_expr (source), port, pstate);
     }
   scm_puts (":\n", port);
   scm_free_print_state (print_state);
@@ -183,20 +219,22 @@ display_error_body (struct display_error_args *a)
   SCM prev_frame = SCM_BOOL_F;
   SCM pname = a->subr;
 
-  if (SCM_DEBUGGINGP
+  if (scm_debug_mode_p
       && SCM_STACKP (a->stack)
       && SCM_STACK_LENGTH (a->stack) > 0)
     {
       current_frame = scm_stack_ref (a->stack, SCM_INUM0);
       source = SCM_FRAME_SOURCE (current_frame);
       prev_frame = SCM_FRAME_PREV (current_frame);
-      if (!SCM_MEMOIZEDP (source) && !SCM_FALSEP (prev_frame))
+      if (!SCM_MEMOIZEDP (source) && scm_is_true (prev_frame))
 	source = SCM_FRAME_SOURCE (prev_frame);
-      if (!SCM_SYMBOLP (pname) && !SCM_STRINGP (pname) && SCM_FRAME_PROC_P (current_frame)
-	  && SCM_EQ_P (scm_procedure_p (SCM_FRAME_PROC (current_frame)), SCM_BOOL_T))
+      if (!scm_is_symbol (pname)
+	  && !scm_is_string (pname)
+	  && SCM_FRAME_PROC_P (current_frame)
+	  && scm_is_true (scm_procedure_p (SCM_FRAME_PROC (current_frame))))
 	pname = scm_procedure_name (SCM_FRAME_PROC (current_frame));
     }
-  if (SCM_SYMBOLP (pname) || SCM_STRINGP (pname) || SCM_MEMOIZEDP (source))
+  if (scm_is_symbol (pname) || scm_is_string (pname) || SCM_MEMOIZEDP (source))
     {
       display_header (source, a->port);
       display_expression (current_frame, pname, source, a->port);
@@ -302,24 +340,21 @@ SCM_DEFINE (scm_set_print_params_x, "set-print-params!", 1, 0, 0,
   print_params_t *new_params;
 
   SCM_VALIDATE_NONEMPTYLIST_COPYLEN (2, params, n);
-  for (ls = params; !SCM_NULLP (ls); ls = SCM_CDR (ls))
+  for (ls = params; !SCM_NULL_OR_NIL_P (ls); ls = SCM_CDR (ls))
     SCM_ASSERT (scm_ilength (SCM_CAR (params)) == 2
-		&& SCM_INUMP (SCM_CAAR (ls))
-		&& SCM_INUM (SCM_CAAR (ls)) >= 0
-		&& SCM_INUMP (SCM_CADAR (ls))
-		&& SCM_INUM (SCM_CADAR (ls)) >= 0,
+		&& scm_is_unsigned_integer (SCM_CAAR (ls), 0, INT_MAX)
+		&& scm_is_unsigned_integer (SCM_CADAR (ls), 0, INT_MAX),
 		params,
 		SCM_ARG2,
 		s_scm_set_print_params_x);
-  new_params = scm_must_malloc (n * sizeof (print_params_t),
-				FUNC_NAME);
+  new_params = scm_malloc (n * sizeof (print_params_t));
   if (print_params != default_print_params)
-    scm_must_free (print_params);
+    free (print_params);
   print_params = new_params;
   for (i = 0; i < n; ++i)
     {
-      print_params[i].level = SCM_INUM (SCM_CAAR (params));
-      print_params[i].length = SCM_INUM (SCM_CADAR (params));
+      print_params[i].level = scm_to_int (SCM_CAAR (params));
+      print_params[i].length = scm_to_int (SCM_CADAR (params));
       params = SCM_CDR (params);
     }
   n_print_params = n;
@@ -337,7 +372,7 @@ indent (int n, SCM port)
 }
 
 static void
-display_frame_expr (char *hdr,SCM exp,char *tlr,int indentation,SCM sport,SCM port,scm_print_state *pstate)
+display_frame_expr (char *hdr, SCM exp, char *tlr, int indentation, SCM sport, SCM port, scm_print_state *pstate)
 {
   SCM string;
   int i = 0, n;
@@ -364,29 +399,38 @@ display_frame_expr (char *hdr,SCM exp,char *tlr,int indentation,SCM sport,SCM po
   while (indentation + n > SCM_BACKTRACE_WIDTH && i < n_print_params);
   ptob->truncate (sport, n);
   string = scm_strport_to_string (sport);
-  /* Remove control characters */
-  for (i = 0; i < n; ++i)
-    if (iscntrl (SCM_STRING_CHARS (string)[i]))
-      SCM_STRING_CHARS (string)[i] = ' ';
-  /* Truncate */
-  if (indentation + n > SCM_BACKTRACE_WIDTH)
-    {
-      n = SCM_BACKTRACE_WIDTH - indentation;
-      SCM_STRING_CHARS (string)[n - 1] = '$';
-    }
+  assert (scm_is_string (string));
+
+  {
+    char *data = scm_i_string_writable_chars (string);
+
+    /* Remove control characters */
+    for (i = 0; i < n; ++i)
+      if (iscntrl (data[i]))
+	data[i] = ' ';
+    /* Truncate */
+    if (indentation + n > SCM_BACKTRACE_WIDTH)
+      {
+	n = SCM_BACKTRACE_WIDTH - indentation;
+	data[n-1] = '$';
+      }
+
+    scm_i_string_stop_writing ();
+  }
       
-  scm_lfwrite (SCM_STRING_CHARS (string), n, port);
+  scm_lfwrite (scm_i_string_chars (string), n, port);
+  scm_remember_upto_here_1 (string);
 }
 
 static void
-display_application (SCM frame,int indentation,SCM sport,SCM port,scm_print_state *pstate)
+display_application (SCM frame, int indentation, SCM sport, SCM port, scm_print_state *pstate)
 {
   SCM proc = SCM_FRAME_PROC (frame);
-  SCM name = (!SCM_FALSEP (scm_procedure_p (proc))
+  SCM name = (scm_is_true (scm_procedure_p (proc))
 	      ? scm_procedure_name (proc)
 	      : SCM_BOOL_F);
   display_frame_expr ("[",
-		      scm_cons (!SCM_FALSEP (name) ? name : proc,
+		      scm_cons (scm_is_true (name) ? name : proc,
 				SCM_FRAME_ARGS (frame)),
 		      SCM_FRAME_EVAL_ARGS_P (frame) ? " ..." : "]",
 		      indentation,
@@ -402,15 +446,13 @@ SCM_DEFINE (scm_display_application, "display-application", 1, 2, 0,
 	    "output.")
 #define FUNC_NAME s_scm_display_application
 {
-  SCM_VALIDATE_FRAME (1,frame);
+  SCM_VALIDATE_FRAME (1, frame);
   if (SCM_UNBNDP (port))
     port = scm_cur_outp;
   else
-    SCM_VALIDATE_OPOUTPORT (2,port);
+    SCM_VALIDATE_OPOUTPORT (2, port);
   if (SCM_UNBNDP (indent))
     indent = SCM_INUM0;
-  else
-    SCM_VALIDATE_INUM (3,indent);
   
   if (SCM_FRAME_PROC_P (frame))
     /* Display an application. */
@@ -420,7 +462,7 @@ SCM_DEFINE (scm_display_application, "display-application", 1, 2, 0,
       
       /* Create a string port used for adaptation of printing parameters. */
       sport = scm_mkstrport (SCM_INUM0,
-			     scm_make_string (SCM_MAKINUM (240),
+			     scm_make_string (scm_from_int (240),
 					      SCM_UNDEFINED),
 			     SCM_OPN | SCM_WRTNG,
 			     FUNC_NAME);
@@ -431,7 +473,7 @@ SCM_DEFINE (scm_display_application, "display-application", 1, 2, 0,
       pstate->writingp = 1;
       pstate->fancyp = 1;
       
-      display_application (frame, SCM_INUM (indent), sport, port, pstate);
+      display_application (frame, scm_to_int (indent), sport, port, pstate);
       return SCM_BOOL_T;
     }
   else
@@ -460,14 +502,14 @@ display_backtrace_file (frame, last_file, port, pstate)
 
   display_backtrace_get_file_line (frame, &file, &line);
 
-  if (file == *last_file)
+  if (scm_is_eq (file, *last_file))
     return;
 
   *last_file = file;
 
   scm_puts ("In ", port);
-  if (SCM_FALSEP (file))
-    if (SCM_FALSEP (line))
+  if (scm_is_false (file))
+    if (scm_is_false (line))
       scm_puts ("unknown file", port);
     else
       scm_puts ("current input", port);
@@ -487,11 +529,11 @@ display_backtrace_file_and_line (SCM frame, SCM port, scm_print_state *pstate)
 
   display_backtrace_get_file_line (frame, &file, &line);
 
-  if (SCM_EQ_P (SCM_SHOW_FILE_NAME, sym_base))
+  if (scm_is_eq (SCM_PACK (SCM_SHOW_FILE_NAME), sym_base))
     {
-      if (file == SCM_BOOL_F)
+      if (scm_is_false (file))
 	{
-	  if (line == SCM_BOOL_F)
+	  if (scm_is_false (line))
 	    scm_putc ('?', port);
 	  else
 	    scm_puts ("<stdin>", port);
@@ -500,7 +542,8 @@ display_backtrace_file_and_line (SCM frame, SCM port, scm_print_state *pstate)
 	{
 	  pstate -> writingp = 0;
 #ifdef HAVE_POSIX
-	  scm_iprin1 (SCM_STRINGP (file) ? scm_basename (file, SCM_UNDEFINED) : file,
+	  scm_iprin1 ((scm_is_string (file)?
+		       scm_basename (file, SCM_UNDEFINED) : file),
 		      port, pstate);
 #else
 	  scm_iprin1 (file, port, pstate);
@@ -510,23 +553,23 @@ display_backtrace_file_and_line (SCM frame, SCM port, scm_print_state *pstate)
 
       scm_putc (':', port);
     }
-  else if (line != SCM_BOOL_F)
+  else if (scm_is_true (line))
     {
       int i, j=0;
-      for (i = SCM_INUM (line)+1; i > 0; i = i/10, j++)
+      for (i = scm_to_int (line)+1; i > 0; i = i/10, j++)
 	;
       indent (4-j, port);
     }
 
-  if (line == SCM_BOOL_F)
+  if (scm_is_false (line))
     scm_puts ("   ?", port);
   else
-    scm_intprint (SCM_INUM (line) + 1, 10, port);
+    scm_intprint (scm_to_int (line) + 1, 10, port);
   scm_puts (": ", port);
 }
 
 static void
-display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print_state *pstate)
+display_frame (SCM frame, int nfield, int indentation, SCM sport, SCM port, scm_print_state *pstate)
 {
   int n, i, j;
 
@@ -538,7 +581,7 @@ display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print
     }
 
   /* display file name and line number */
-  if (!SCM_FALSEP (SCM_SHOW_FILE_NAME))
+  if (scm_is_true (SCM_PACK (SCM_SHOW_FILE_NAME)))
     display_backtrace_file_and_line (frame, port, pstate);
 
   /* Check size of frame number. */
@@ -549,7 +592,7 @@ display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print
   indent (nfield - (i ? i : 1), port);
 
   /* Frame number. */
-  scm_iprin1 (SCM_MAKINUM (n), port, pstate);
+  scm_iprin1 (scm_from_int (n), port, pstate);
 
   /* Real frame marker */
   scm_putc (SCM_FRAME_REAL_P (frame) ? '*' : ' ', port);
@@ -568,7 +611,7 @@ display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print
 		  ? scm_source_property (source, scm_sym_copy)
 		  : SCM_BOOL_F);
       SCM umcopy = (SCM_MEMOIZEDP (source)
-		    ? scm_unmemoize (source)
+		    ? scm_i_unmemoize_expr (source)
 		    : SCM_BOOL_F);
       display_frame_expr ("(",
 			  SCM_CONSP (copy) ? copy : umcopy,
@@ -610,11 +653,11 @@ display_backtrace_body (struct display_backtrace_args *a)
   /* Argument checking and extraction. */
   SCM_VALIDATE_STACK (1, a->stack);
   SCM_VALIDATE_OPOUTPORT (2, a->port);
-  n_frames = SCM_INUM (scm_stack_length (a->stack));
-  n = SCM_INUMP (a->depth) ? SCM_INUM (a->depth) : SCM_BACKTRACE_DEPTH;
+  n_frames = scm_to_int (scm_stack_length (a->stack));
+  n = scm_is_integer (a->depth) ? scm_to_int (a->depth) : SCM_BACKTRACE_DEPTH;
   if (SCM_BACKWARDS_P)
     {
-      beg = SCM_INUMP (a->first) ? SCM_INUM (a->first) : 0;
+      beg = scm_is_integer (a->first) ? scm_to_int (a->first) : 0;
       end = beg + n - 1;
       if (end >= n_frames)
 	end = n_frames - 1;
@@ -622,9 +665,9 @@ display_backtrace_body (struct display_backtrace_args *a)
     }
   else
     {
-      if (SCM_INUMP (a->first))
+      if (scm_is_integer (a->first))
 	{
-	  beg = SCM_INUM (a->first);
+	  beg = scm_to_int (a->first);
 	  end = beg - n + 1;
 	  if (end < 0)
 	    end = 0;
@@ -643,7 +686,7 @@ display_backtrace_body (struct display_backtrace_args *a)
 
   /* Create a string port used for adaptation of printing parameters. */
   sport = scm_mkstrport (SCM_INUM0,
-			 scm_make_string (SCM_MAKINUM (240), SCM_UNDEFINED),
+			 scm_make_string (scm_from_int (240), SCM_UNDEFINED),
 			 SCM_OPN | SCM_WRTNG,
 			 FUNC_NAME);
 
@@ -661,7 +704,7 @@ display_backtrace_body (struct display_backtrace_args *a)
       unsigned int j;
 
       indent_p = 1;
-      frame = scm_stack_ref (a->stack, SCM_MAKINUM (beg));
+      frame = scm_stack_ref (a->stack, scm_from_int (beg));
       for (i = 0, j = 0; i < n; ++i)
 	{
 	  if (SCM_FRAME_REAL_P (frame))
@@ -678,17 +721,17 @@ display_backtrace_body (struct display_backtrace_args *a)
     }
   
   /* Determine size of frame number field. */
-  j = SCM_FRAME_NUMBER (scm_stack_ref (a->stack, SCM_MAKINUM (end)));
+  j = SCM_FRAME_NUMBER (scm_stack_ref (a->stack, scm_from_int (end)));
   for (i = 0; j > 0; ++i) j /= 10;
   nfield = i ? i : 1;
   
   /* Print frames. */
-  frame = scm_stack_ref (a->stack, SCM_MAKINUM (beg));
+  frame = scm_stack_ref (a->stack, scm_from_int (beg));
   indentation = 1;
   last_file = SCM_UNDEFINED;
   for (i = 0; i < n; ++i)
     {
-      if (!SCM_EQ_P (SCM_SHOW_FILE_NAME, sym_base))
+      if (!scm_is_eq (SCM_PACK (SCM_SHOW_FILE_NAME), sym_base))
 	display_backtrace_file (frame, &last_file, a->port, pstate);
 
       display_frame (frame, nfield, indentation, sport, a->port, pstate);
@@ -738,7 +781,7 @@ SCM_DEFINE (scm_backtrace, "backtrace", 0, 0, 0,
 {
   SCM the_last_stack =
     scm_fluid_ref (SCM_VARIABLE_REF (scm_the_last_stack_fluid_var));
-  if (!SCM_FALSEP (the_last_stack))
+  if (scm_is_true (the_last_stack))
     {
       scm_newline (scm_cur_outp);
       scm_puts ("Backtrace:\n", scm_cur_outp);
@@ -747,7 +790,7 @@ SCM_DEFINE (scm_backtrace, "backtrace", 0, 0, 0,
 			     SCM_UNDEFINED,
 			     SCM_UNDEFINED);
       scm_newline (scm_cur_outp);
-      if (SCM_FALSEP (SCM_VARIABLE_REF (scm_has_shown_backtrace_hint_p_var))
+      if (scm_is_false (SCM_VARIABLE_REF (scm_has_shown_backtrace_hint_p_var))
 	  && !SCM_BACKTRACE_P)
 	{
 	  scm_puts ("Type \"(debug-enable 'backtrace)\" if you would like "
