@@ -1,52 +1,26 @@
 /* Debugging extensions for Guile
- * Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2003 Free Software Foundation
+ * Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003 Free Software Foundation
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
- *
- * The author can be reached at djurfeldt@nada.kth.se
- * Mikael Djurfeldt, SANS/NADA KTH, 10044 STOCKHOLM, SWEDEN */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 #include "libguile/_scm.h"
 #include "libguile/eval.h"
+#include "libguile/list.h"
 #include "libguile/stackchk.h"
 #include "libguile/throw.h"
 #include "libguile/macros.h"
@@ -81,17 +55,12 @@ SCM_DEFINE (scm_debug_options, "debug-options-interface", 0, 1, 0,
 {
   SCM ans;
   SCM_DEFER_INTS;
-  ans = scm_options (setting,
-		     scm_debug_opts,
-		     SCM_N_DEBUG_OPTIONS,
-		     FUNC_NAME);
-#ifndef SCM_RECKLESS
+  ans = scm_options (setting, scm_debug_opts, SCM_N_DEBUG_OPTIONS, FUNC_NAME);
   if (!(1 <= SCM_N_FRAMES && SCM_N_FRAMES <= SCM_MAX_FRAME_SIZE))
     {
       scm_options (ans, scm_debug_opts, SCM_N_DEBUG_OPTIONS, FUNC_NAME);
       SCM_OUT_OF_RANGE (1, setting);
     }
-#endif
   SCM_RESET_DEBUG_MODE;
   scm_stack_checking_enabled_p = SCM_STACK_CHECKING_P;
   scm_debug_eframe_size = 2 * SCM_N_FRAMES;
@@ -118,7 +87,7 @@ with_traps_after (void *data)
 static SCM
 with_traps_inner (void *data)
 {
-  SCM thunk = SCM_PACK (data);
+  SCM thunk = SCM_PACK ((scm_t_bits) data);
   return scm_call_0 (thunk);
 }
 
@@ -128,7 +97,7 @@ SCM_DEFINE (scm_with_traps, "with-traps", 1, 0, 0,
 #define FUNC_NAME s_scm_with_traps
 {
   int trap_flag;
-  SCM_VALIDATE_THUNK (1,thunk);
+  SCM_VALIDATE_THUNK (1, thunk);
   return scm_internal_dynamic_wind (with_traps_before,
 				    with_traps_inner,
 				    with_traps_after,
@@ -154,11 +123,7 @@ memoized_print (SCM obj, SCM port, scm_print_state *pstate)
   int writingp = SCM_WRITINGP (pstate);
   scm_puts ("#<memoized ", port);
   SCM_SET_WRITINGP (pstate, 1);
-#ifdef GUILE_DEBUG
   scm_iprin1 (SCM_MEMOIZED_EXP (obj), port, pstate);
-#else
-  scm_iprin1 (scm_unmemoize (obj), port, pstate);
-#endif
   SCM_SET_WRITINGP (pstate, writingp);
   scm_putc ('>', port);
   return 1;
@@ -169,7 +134,7 @@ SCM_DEFINE (scm_memoized_p, "memoized?", 1, 0, 0,
 	    "Return @code{#t} if @var{obj} is memoized.")
 #define FUNC_NAME s_scm_memoized_p
 {
-  return SCM_BOOL(SCM_MEMOIZEDP (obj));
+  return scm_from_bool(SCM_MEMOIZEDP (obj));
 }
 #undef FUNC_NAME
 
@@ -197,17 +162,6 @@ scm_make_memoized (SCM exp, SCM env)
  *     specified, the top-level environment of the current module will
  *     be assumed.  All environments must match.
  *
- * - procedure: make-gloc VARIABLE [ENV]
- *
- *     Return a gloc, encapsulated in a memoized object.
- *
- *     (Glocs can't exist in normal list structures, since they will
- *     be mistaken for structs.)
- *
- * - procedure: gloc? OBJECT
- *
- *     Return #t if OBJECT is a memoized gloc.
- *
  * - procedure: make-iloc FRAME BINDING CDRP
  *
  *     Return an iloc referring to frame no. FRAME, binding
@@ -253,56 +207,6 @@ scm_make_memoized (SCM exp, SCM env)
 #include "libguile/variable.h"
 #include "libguile/procs.h"
 
-SCM_DEFINE (scm_make_gloc, "make-gloc", 1, 1, 0, 
-            (SCM var, SCM env),
-	    "Create a gloc for variable @var{var} in the environment\n"
-	    "@var{env}.")
-#define FUNC_NAME s_scm_make_gloc
-{
-  SCM_VALIDATE_VARIABLE (1,var);
-  if (SCM_UNBNDP (env))
-    env = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
-  else
-    SCM_VALIDATE_NULLORCONS (2,env);
-  return scm_make_memoized (SCM_PACK (SCM_UNPACK (var) + scm_tc3_cons_gloc), env);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_gloc_p, "gloc?", 1, 0, 0, 
-            (SCM obj),
-	    "Return @code{#t} if @var{obj} is a gloc.")
-#define FUNC_NAME s_scm_gloc_p
-{
-  return
-    SCM_BOOL (SCM_MEMOIZEDP (obj) 
-	      && ((SCM_UNPACK(SCM_MEMOIZED_EXP(obj))&7) == scm_tc3_cons_gloc));
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_make_iloc, "make-iloc", 3, 0, 0,
-            (SCM frame, SCM binding, SCM cdrp),
-	    "Return a new iloc with frame offset @var{frame}, binding\n"
-	    "offset @var{binding} and the cdr flag @var{cdrp}.")
-#define FUNC_NAME s_scm_make_iloc
-{
-  SCM_VALIDATE_INUM (1,frame);
-  SCM_VALIDATE_INUM (2,binding);
-  return SCM_PACK (SCM_UNPACK (SCM_ILOC00)
-                   + SCM_IFRINC * SCM_INUM (frame)
-                   + (SCM_NFALSEP (cdrp) ? SCM_ICDR : 0)
-                   + SCM_IDINC * SCM_INUM (binding));
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_iloc_p, "iloc?", 1, 0, 0, 
-          (SCM obj),
-	    "Return @code{#t} if @var{obj} is an iloc.")
-#define FUNC_NAME s_scm_iloc_p
-{
-  return SCM_BOOL(SCM_ILOCP (obj));
-}
-#undef FUNC_NAME
-
 SCM_DEFINE (scm_memcons, "memcons", 2, 1, 0,
             (SCM car, SCM cdr, SCM env),
 	    "Return a new memoized cons cell with @var{car} and @var{cdr}\n"
@@ -314,7 +218,7 @@ SCM_DEFINE (scm_memcons, "memcons", 2, 1, 0,
       /*fixme* environments may be two different but equal top-level envs */
       if (!SCM_UNBNDP (env) && SCM_MEMOIZED_ENV (car) != env)
 	SCM_MISC_ERROR ("environment mismatch arg1 <-> arg3",
-			scm_cons2 (car, env, SCM_EOL));
+			scm_list_2 (car, env));
       else
 	env = SCM_MEMOIZED_ENV (car);
       car = SCM_MEMOIZED_EXP (car);
@@ -323,7 +227,7 @@ SCM_DEFINE (scm_memcons, "memcons", 2, 1, 0,
     {
       if (!SCM_UNBNDP (env) && SCM_MEMOIZED_ENV (cdr) != env)
 	SCM_MISC_ERROR ("environment mismatch arg2 <-> arg3",
-			scm_cons2 (cdr, env, SCM_EOL));
+			scm_list_2 (cdr, env));
       else
 	env = SCM_MEMOIZED_ENV (cdr);
       cdr = SCM_MEMOIZED_EXP (cdr);
@@ -331,25 +235,22 @@ SCM_DEFINE (scm_memcons, "memcons", 2, 1, 0,
   if (SCM_UNBNDP (env))
     env = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
   else
-    SCM_VALIDATE_NULLORCONS (3,env);
+    SCM_VALIDATE_NULLORCONS (3, env);
   return scm_make_memoized (scm_cons (car, cdr), env);
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_mem_to_proc, "mem->proc", 1, 0, 0, 
             (SCM obj),
-	    "Convert a memoized object (which must be a lambda expression)\n"
+	    "Convert a memoized object (which must represent a body)\n"
 	    "to a procedure.")
 #define FUNC_NAME s_scm_mem_to_proc
 {
   SCM env;
-  SCM_VALIDATE_MEMOIZED (1,obj);
+  SCM_VALIDATE_MEMOIZED (1, obj);
   env = SCM_MEMOIZED_ENV (obj);
   obj = SCM_MEMOIZED_EXP (obj);
-  if (!(SCM_NIMP (obj) && SCM_CAR (obj) == SCM_IM_LAMBDA))
-    SCM_MISC_ERROR ("expected lambda expression",
-		    scm_cons (obj, SCM_EOL));
-  return scm_closure (SCM_CDR (obj), env);
+  return scm_closure (obj, env);
 }
 #undef FUNC_NAME
 
@@ -359,20 +260,19 @@ SCM_DEFINE (scm_proc_to_mem, "proc->mem", 1, 0, 0,
 #define FUNC_NAME s_scm_proc_to_mem
 {
   SCM_VALIDATE_CLOSURE (1, obj);
-  return scm_make_memoized (scm_cons (SCM_IM_LAMBDA, SCM_CODE (obj)),
-			    SCM_ENV (obj));
+  return scm_make_memoized (SCM_CODE (obj), SCM_ENV (obj));
 }
 #undef FUNC_NAME
 
 #endif /* GUILE_DEBUG */
 
-SCM_DEFINE (scm_unmemoize, "unmemoize", 1, 0, 0, 
+SCM_DEFINE (scm_i_unmemoize_expr, "unmemoize-expr", 1, 0, 0, 
             (SCM m),
 	    "Unmemoize the memoized expression @var{m},")
-#define FUNC_NAME s_scm_unmemoize
+#define FUNC_NAME s_scm_i_unmemoize_expr
 {
-  SCM_VALIDATE_MEMOIZED (1,m);
-  return scm_unmemocopy (SCM_MEMOIZED_EXP (m), SCM_MEMOIZED_ENV (m));
+  SCM_VALIDATE_MEMOIZED (1, m);
+  return scm_i_unmemocopy_expr (SCM_MEMOIZED_EXP (m), SCM_MEMOIZED_ENV (m));
 }
 #undef FUNC_NAME
 
@@ -381,7 +281,7 @@ SCM_DEFINE (scm_memoized_environment, "memoized-environment", 1, 0, 0,
 	    "Return the environment of the memoized expression @var{m}.")
 #define FUNC_NAME s_scm_memoized_environment
 {
-  SCM_VALIDATE_MEMOIZED (1,m);
+  SCM_VALIDATE_MEMOIZED (1, m);
   return SCM_MEMOIZED_ENV (m);
 }
 #undef FUNC_NAME
@@ -391,7 +291,7 @@ SCM_DEFINE (scm_procedure_name, "procedure-name", 1, 0, 0,
 	    "Return the name of the procedure @var{proc}")
 #define FUNC_NAME s_scm_procedure_name
 {
-  SCM_VALIDATE_PROC (1,proc);
+  SCM_VALIDATE_PROC (1, proc);
   switch (SCM_TYP7 (proc)) {
   case scm_tcs_subrs:
     return SCM_SNAME (proc);
@@ -400,11 +300,11 @@ SCM_DEFINE (scm_procedure_name, "procedure-name", 1, 0, 0,
       SCM name = scm_procedure_property (proc, scm_sym_name);
 #if 0
       /* Source property scm_sym_procname not implemented yet... */
-      SCM name = scm_source_property (SCM_CAR (SCM_CDR (SCM_CODE (proc))), scm_sym_procname);
-      if (SCM_FALSEP (name))
+      SCM name = scm_source_property (SCM_CAR (SCM_CLOSURE_BODY (proc)), scm_sym_procname);
+      if (scm_is_false (name))
 	name = scm_procedure_property (proc, scm_sym_name);
 #endif
-      if (SCM_FALSEP (name) && SCM_CLOSUREP (proc))
+      if (scm_is_false (name) && SCM_CLOSUREP (proc))
 	name = scm_reverse_lookup (SCM_ENV (proc), proc);
       return name;
     }
@@ -412,27 +312,31 @@ SCM_DEFINE (scm_procedure_name, "procedure-name", 1, 0, 0,
 }
 #undef FUNC_NAME
 
-#define scm_tcs_struct scm_tcs_cons_gloc
-
 SCM_DEFINE (scm_procedure_source, "procedure-source", 1, 0, 0, 
             (SCM proc),
 	    "Return the source of the procedure @var{proc}.")
 #define FUNC_NAME s_scm_procedure_source
 {
-  SCM_VALIDATE_NIM (1,proc);
+  SCM_VALIDATE_NIM (1, proc);
  again:
   switch (SCM_TYP7 (proc)) {
   case scm_tcs_closures:
     {
-      SCM formals = SCM_CLOSURE_FORMALS (proc);
-      SCM src = scm_source_property (SCM_CDR (SCM_CODE (proc)), scm_sym_copy);
-      if (!SCM_FALSEP (src))
-	return scm_cons2 (scm_sym_lambda, formals, src);
-      return scm_cons (scm_sym_lambda,
-		       scm_unmemocopy (SCM_CODE (proc),
-				       SCM_EXTEND_ENV (formals,
-						       SCM_EOL,
-						       SCM_ENV (proc))));
+      const SCM formals = SCM_CLOSURE_FORMALS (proc);
+      const SCM body = SCM_CLOSURE_BODY (proc);
+      const SCM src = scm_source_property (body, scm_sym_copy);
+
+      if (scm_is_true (src))
+        {
+          return scm_cons2 (scm_sym_lambda, formals, src);
+        }
+      else
+        {
+          const SCM env = SCM_EXTEND_ENV (formals, SCM_EOL, SCM_ENV (proc));
+          return scm_cons2 (scm_sym_lambda,
+                            scm_i_finite_list_copy (formals),
+                            scm_i_unmemocopy_body (body, env));
+        }
     }
   case scm_tcs_struct:
     if (!SCM_I_OPERATORP (proc))
@@ -452,7 +356,7 @@ SCM_DEFINE (scm_procedure_source, "procedure-source", 1, 0, 0,
   case scm_tc7_pws:
     {
       SCM src = scm_procedure_property (proc, scm_sym_source);
-      if (!SCM_FALSEP (src))
+      if (scm_is_true (src))
 	return src;
       proc = SCM_PROCEDURE (proc);
       goto again;
@@ -470,7 +374,7 @@ SCM_DEFINE (scm_procedure_environment, "procedure-environment", 1, 0, 0,
 	    "Return the environment of the procedure @var{proc}.")
 #define FUNC_NAME s_scm_procedure_environment
 {
-  SCM_VALIDATE_NIM (1,proc);
+  SCM_VALIDATE_NIM (1, proc);
   switch (SCM_TYP7 (proc)) {
   case scm_tcs_closures:
     return SCM_ENV (proc);
@@ -524,12 +428,12 @@ scm_reverse_lookup (SCM env, SCM data)
       SCM values = SCM_CDAR (env);
       while (SCM_CONSP (names))
 	{
-	  if (SCM_EQ_P (SCM_CAR (values), data))
+	  if (scm_is_eq (SCM_CAR (values), data))
 	    return SCM_CAR (names);
 	  names = SCM_CDR (names);
 	  values = SCM_CDR (values);
 	}
-      if (!SCM_NULLP (names) && SCM_EQ_P (values, data))
+      if (!SCM_NULLP (names) && scm_is_eq (values, data))
 	return names;
       env = SCM_CDR (env);
     }
@@ -559,8 +463,8 @@ scm_m_start_stack (SCM exp, SCM env)
 #define FUNC_NAME s_start_stack
 {
   exp = SCM_CDR (exp);
-  if (!SCM_ECONSP (exp) 
-      || !SCM_ECONSP (SCM_CDR (exp))
+  if (!SCM_CONSP (exp) 
+      || !SCM_CONSP (SCM_CDR (exp))
       || !SCM_NULLP (SCM_CDDR (exp)))
     SCM_WRONG_NUM_ARGS ();
   return scm_start_stack (scm_eval_car (exp, env), SCM_CADR (exp), env);
@@ -589,7 +493,7 @@ SCM_DEFINE (scm_debug_object_p, "debug-object?", 1, 0, 0,
 	    "Return @code{#t} if @var{obj} is a debug object.")
 #define FUNC_NAME s_scm_debug_object_p
 {
-  return SCM_BOOL(SCM_DEBUGOBJP (obj));
+  return scm_from_bool(SCM_DEBUGOBJP (obj));
 }
 #undef FUNC_NAME
 
@@ -597,13 +501,7 @@ SCM_DEFINE (scm_debug_object_p, "debug-object?", 1, 0, 0,
 SCM
 scm_make_debugobj (scm_t_debug_frame *frame)
 {
-  register SCM z;
-  SCM_NEWCELL (z);
-  SCM_ENTER_A_SECTION;
-  SCM_SET_DEBUGOBJ_FRAME (z, frame);
-  SCM_SET_CELL_TYPE (z, scm_tc16_debugobj);
-  SCM_EXIT_A_SECTION;
-  return z;
+  return scm_cell (scm_tc16_debugobj, (scm_t_bits) frame);
 }
 
 
