@@ -1,43 +1,19 @@
-/* Copyright (C) 1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1999,2000,2001, 2003 Free Software Foundation, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -46,6 +22,7 @@
 #include "libguile/eval.h"
 #include "libguile/gh.h"
 #include "libguile/hash.h"
+#include "libguile/list.h"
 #include "libguile/ports.h"
 #include "libguile/smob.h"
 #include "libguile/symbols.h"
@@ -58,7 +35,7 @@
 
 scm_t_bits scm_tc16_environment;
 scm_t_bits scm_tc16_observer;
-#define DEFAULT_OBARRAY_SIZE 137
+#define DEFAULT_OBARRAY_SIZE 31
 
 SCM scm_system_environment;
 
@@ -119,13 +96,7 @@ scm_error_environment_immutable_location (const char *func, SCM env, SCM symbol)
 SCM
 scm_make_environment (void *type)
 {
-  SCM env;
-
-  SCM_NEWCELL (env);
-  SCM_SET_CELL_WORD_1 (env, type);
-  SCM_SET_CELL_TYPE (env, scm_tc16_environment);
-
-  return env;
+  return scm_cell (scm_tc16_environment, (scm_t_bits) type);
 }
 
 
@@ -135,7 +106,7 @@ SCM_DEFINE (scm_environment_p, "environment?", 1, 0, 0,
 	    "otherwise.")
 #define FUNC_NAME s_scm_environment_p
 {
-  return SCM_BOOL (SCM_ENVIRONMENT_P (obj));
+  return scm_from_bool (SCM_ENVIRONMENT_P (obj));
 }
 #undef FUNC_NAME
 
@@ -147,9 +118,9 @@ SCM_DEFINE (scm_environment_bound_p, "environment-bound?", 2, 0, 0,
 #define FUNC_NAME s_scm_environment_bound_p
 {
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, FUNC_NAME);
 
-  return SCM_BOOL (SCM_ENVIRONMENT_BOUND_P (env, sym));
+  return scm_from_bool (SCM_ENVIRONMENT_BOUND_P (env, sym));
 }
 #undef FUNC_NAME
 
@@ -164,7 +135,7 @@ SCM_DEFINE (scm_environment_ref, "environment-ref", 2, 0, 0,
   SCM val;
 
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, FUNC_NAME);
 
   val = SCM_ENVIRONMENT_REF (env, sym);
 
@@ -184,7 +155,7 @@ SCM
 scm_c_environment_ref (SCM env, SCM sym)
 {
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, "scm_c_environment_ref");
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, "scm_c_environment_ref");
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, "scm_c_environment_ref");
   return SCM_ENVIRONMENT_REF (env, sym);
 }
 
@@ -232,7 +203,7 @@ SCM_DEFINE (scm_environment_fold, "environment-fold", 3, 0, 0,
 #define FUNC_NAME s_scm_environment_fold
 {
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_EQ_P (scm_procedure_p (proc), SCM_BOOL_T), 
+  SCM_ASSERT (scm_is_true (scm_procedure_p (proc)), 
 	      proc, SCM_ARG2, FUNC_NAME);
 
   return SCM_ENVIRONMENT_FOLD (env, environment_default_folder, proc, init);
@@ -269,13 +240,13 @@ SCM_DEFINE (scm_environment_define, "environment-define", 3, 0, 0,
   SCM status;
 
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, FUNC_NAME);
 
   status = SCM_ENVIRONMENT_DEFINE (env, sym, val);
 
-  if (SCM_EQ_P (status, SCM_ENVIRONMENT_SUCCESS))
+  if (scm_is_eq (status, SCM_ENVIRONMENT_SUCCESS))
     return SCM_UNSPECIFIED;
-  else if (SCM_EQ_P (status, SCM_ENVIRONMENT_BINDING_IMMUTABLE))
+  else if (scm_is_eq (status, SCM_ENVIRONMENT_BINDING_IMMUTABLE))
     scm_error_environment_immutable_binding (FUNC_NAME, env, sym);
   else
     abort();
@@ -295,13 +266,13 @@ SCM_DEFINE (scm_environment_undefine, "environment-undefine", 2, 0, 0,
   SCM status;
 
   SCM_ASSERT(SCM_ENVIRONMENT_P(env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT(SCM_SYMBOLP(sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT(scm_is_symbol(sym), sym, SCM_ARG2, FUNC_NAME);
 
   status = SCM_ENVIRONMENT_UNDEFINE (env, sym);
 
-  if (SCM_EQ_P (status, SCM_ENVIRONMENT_SUCCESS))
+  if (scm_is_eq (status, SCM_ENVIRONMENT_SUCCESS))
     return SCM_UNSPECIFIED;
-  else if (SCM_EQ_P (status, SCM_ENVIRONMENT_BINDING_IMMUTABLE))
+  else if (scm_is_eq (status, SCM_ENVIRONMENT_BINDING_IMMUTABLE))
     scm_error_environment_immutable_binding (FUNC_NAME, env, sym);
   else
     abort();
@@ -323,15 +294,15 @@ SCM_DEFINE (scm_environment_set_x, "environment-set!", 3, 0, 0,
   SCM status;
 
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, FUNC_NAME);
 
   status = SCM_ENVIRONMENT_SET (env, sym, val);
 
-  if (SCM_EQ_P (status, SCM_ENVIRONMENT_SUCCESS))
+  if (scm_is_eq (status, SCM_ENVIRONMENT_SUCCESS))
     return SCM_UNSPECIFIED;
   else if (SCM_UNBNDP (status))
     scm_error_environment_unbound (FUNC_NAME, env, sym);
-  else if (SCM_EQ_P (status, SCM_ENVIRONMENT_LOCATION_IMMUTABLE))
+  else if (scm_is_eq (status, SCM_ENVIRONMENT_LOCATION_IMMUTABLE))
     scm_error_environment_immutable_binding (FUNC_NAME, env, sym);
   else
     abort();
@@ -358,15 +329,15 @@ SCM_DEFINE (scm_environment_cell, "environment-cell", 3, 0, 0,
   SCM location;
 
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, FUNC_NAME);
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, FUNC_NAME);
-  SCM_ASSERT (SCM_BOOLP (for_write), for_write, SCM_ARG3, FUNC_NAME);
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT (scm_is_bool (for_write), for_write, SCM_ARG3, FUNC_NAME);
 
-  location = SCM_ENVIRONMENT_CELL (env, sym, !SCM_FALSEP (for_write));
+  location = SCM_ENVIRONMENT_CELL (env, sym, scm_is_true (for_write));
   if (!SCM_IMP (location))
     return location;
   else if (SCM_UNBNDP (location))
     scm_error_environment_unbound (FUNC_NAME, env, sym);
-  else if (SCM_EQ_P (location, SCM_ENVIRONMENT_LOCATION_IMMUTABLE))
+  else if (scm_is_eq (location, SCM_ENVIRONMENT_LOCATION_IMMUTABLE))
     scm_error_environment_immutable_location (FUNC_NAME, env, sym);
   else /* no cell */
     return location;
@@ -384,7 +355,7 @@ SCM
 scm_c_environment_cell(SCM env, SCM sym, int for_write)
 {
   SCM_ASSERT (SCM_ENVIRONMENT_P (env), env, SCM_ARG1, "scm_c_environment_cell");
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG2, "scm_c_environment_cell");
+  SCM_ASSERT (scm_is_symbol (sym), sym, SCM_ARG2, "scm_c_environment_cell");
 
   return SCM_ENVIRONMENT_CELL (env, sym, for_write);
 }
@@ -482,7 +453,8 @@ environment_mark (SCM env)
 static size_t
 environment_free (SCM env)
 {
-  return (*(SCM_ENVIRONMENT_FUNCS (env)->free)) (env);
+  (*(SCM_ENVIRONMENT_FUNCS (env)->free)) (env);
+  return 0;
 }
 
 
@@ -508,11 +480,11 @@ observer_mark (SCM observer)
 static int
 observer_print (SCM type, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
-  SCM address = scm_ulong2num (SCM_UNPACK (type));
-  SCM base16 = scm_number_to_string (address, SCM_MAKINUM (16));
+  SCM address = scm_from_size_t (SCM_UNPACK (type));
+  SCM base16 = scm_number_to_string (address, scm_from_int (16));
 
   scm_puts ("#<observer ", port);
-  scm_puts (SCM_STRING_CHARS (base16), port);
+  scm_display (base16, port);
   scm_puts (">", port);
 
   return 1;
@@ -535,10 +507,12 @@ observer_print (SCM type, SCM port, scm_print_state *pstate SCM_UNUSED)
 static SCM
 obarray_enter (SCM obarray, SCM symbol, SCM data)
 {
-  size_t hash = SCM_SYMBOL_HASH (symbol) % SCM_VECTOR_LENGTH (obarray);
+  size_t hash = scm_i_symbol_hash (symbol) % SCM_HASHTABLE_N_BUCKETS (obarray);
   SCM entry = scm_cons (symbol, data);
-  SCM slot = scm_cons (entry, SCM_VELTS (obarray)[hash]);
-  SCM_VELTS (obarray)[hash] = slot;
+  SCM slot = scm_cons (entry, SCM_HASHTABLE_BUCKETS (obarray)[hash]);
+  SCM_SET_HASHTABLE_BUCKET  (obarray, hash, slot);
+  if (SCM_HASHTABLE_N_ITEMS (obarray) > SCM_HASHTABLE_UPPER (obarray))
+    scm_i_rehash (obarray, scm_i_hash_symbol, 0, "obarray_enter");
 
   return entry;
 }
@@ -551,23 +525,27 @@ obarray_enter (SCM obarray, SCM symbol, SCM data)
 static SCM
 obarray_replace (SCM obarray, SCM symbol, SCM data)
 {
-  size_t hash = SCM_SYMBOL_HASH (symbol) % SCM_VECTOR_LENGTH (obarray);
+  size_t hash = scm_i_symbol_hash (symbol) % SCM_HASHTABLE_N_BUCKETS (obarray);
   SCM new_entry = scm_cons (symbol, data);
   SCM lsym;
   SCM slot;
 
-  for (lsym = SCM_VELTS (obarray)[hash]; !SCM_NULLP (lsym); lsym = SCM_CDR (lsym))
+  for (lsym = SCM_HASHTABLE_BUCKETS (obarray)[hash];
+       !SCM_NULLP (lsym);
+       lsym = SCM_CDR (lsym))
     {
       SCM old_entry = SCM_CAR (lsym);
-      if (SCM_EQ_P (SCM_CAR (old_entry), symbol))
+      if (scm_is_eq (SCM_CAR (old_entry), symbol))
 	{
 	  SCM_SETCAR (lsym, new_entry);
 	  return old_entry;
 	}
     }
 
-  slot = scm_cons (new_entry, SCM_VELTS (obarray)[hash]);
-  SCM_VELTS (obarray)[hash] = slot;
+  slot = scm_cons (new_entry, SCM_HASHTABLE_BUCKETS (obarray)[hash]);
+  SCM_SET_HASHTABLE_BUCKET (obarray, hash, slot);
+  if (SCM_HASHTABLE_N_ITEMS (obarray) > SCM_HASHTABLE_UPPER (obarray))
+    scm_i_rehash (obarray, scm_i_hash_symbol, 0, "obarray_replace");
 
   return SCM_BOOL_F;
 }
@@ -579,13 +557,15 @@ obarray_replace (SCM obarray, SCM symbol, SCM data)
 static SCM
 obarray_retrieve (SCM obarray, SCM sym)
 {
-  size_t hash = SCM_SYMBOL_HASH (sym) % SCM_VECTOR_LENGTH (obarray);
+  size_t hash = scm_i_symbol_hash (sym) % SCM_HASHTABLE_N_BUCKETS (obarray);
   SCM lsym;
 
-  for (lsym = SCM_VELTS (obarray)[hash]; !SCM_NULLP (lsym); lsym = SCM_CDR (lsym))
+  for (lsym = SCM_HASHTABLE_BUCKETS (obarray)[hash];
+       !SCM_NULLP (lsym);
+       lsym = SCM_CDR (lsym))
     {
       SCM entry = SCM_CAR (lsym);
-      if (SCM_EQ_P (SCM_CAR (entry), sym))
+      if (scm_is_eq (SCM_CAR (entry), sym))
 	return entry;
     }
 
@@ -600,36 +580,32 @@ obarray_retrieve (SCM obarray, SCM sym)
 static SCM
 obarray_remove (SCM obarray, SCM sym)
 {
-  size_t hash = SCM_SYMBOL_HASH (sym) % SCM_VECTOR_LENGTH (obarray);
-  SCM lsym;
-  SCM *lsymp;
+  size_t hash = scm_i_symbol_hash (sym) % SCM_HASHTABLE_N_BUCKETS (obarray);
+  SCM table_entry = SCM_HASHTABLE_BUCKETS (obarray)[hash];
+  SCM handle = scm_sloppy_assq (sym, table_entry);
 
-  /* Dirk:FIXME:: gc problem due to use of &SCM_VELTS[hash] */
-  for (lsym = *(lsymp = &SCM_VELTS (obarray)[hash]);
-       !SCM_NULLP (lsym);
-       lsym = *(lsymp = SCM_CDRLOC (lsym)))
+  if (SCM_CONSP (handle))
     {
-      SCM entry = SCM_CAR (lsym);
-      if (SCM_EQ_P (SCM_CAR (entry), sym))
-	{
-	  *lsymp = SCM_CDR (lsym);
-	  return entry;
-	}
+      SCM new_table_entry = scm_delq1_x (handle, table_entry);
+      SCM_SET_HASHTABLE_BUCKET (obarray, hash, new_table_entry);
+      SCM_HASHTABLE_DECREMENT (obarray);
     }
-  return SCM_BOOL_F;
+
+  return handle;
 }
 
 
 static void
 obarray_remove_all (SCM obarray)
 {
-  size_t size = SCM_VECTOR_LENGTH (obarray);
+  size_t size = SCM_HASHTABLE_N_BUCKETS (obarray);
   size_t i;
 
   for (i = 0; i < size; i++)
     {
-      SCM_VELTS (obarray)[i] = SCM_EOL;
+      SCM_SET_HASHTABLE_BUCKET (obarray, i, SCM_EOL);
     }
+  SCM_SET_HASHTABLE_N_ITEMS (obarray, 0);
 }
 
 
@@ -660,20 +636,17 @@ struct core_environments_base {
 #define CORE_ENVIRONMENT_WEAK_OBSERVERS(env) \
   (SCM_VELTS (CORE_ENVIRONMENT_WEAK_OBSERVER_VECTOR (env)) [0])
 #define SCM_SET_CORE_ENVIRONMENT_WEAK_OBSERVERS(env, v) \
-  (SCM_VELTS (CORE_ENVIRONMENT_WEAK_OBSERVER_VECTOR (env)) [0] = (v))
+  (SCM_VECTOR_SET (CORE_ENVIRONMENT_WEAK_OBSERVER_VECTOR (env), 0, (v)))
 
 
 
 static SCM
 core_environments_observe (SCM env, scm_environment_observer proc, SCM data, int weak_p)
 {
-  SCM observer;
-
-  SCM_NEWCELL2 (observer);
-  SCM_SET_CELL_OBJECT_1 (observer, env);
-  SCM_SET_CELL_OBJECT_2 (observer, data);
-  SCM_SET_CELL_WORD_3 (observer, proc);
-  SCM_SET_CELL_TYPE (observer, scm_tc16_observer);
+  SCM observer = scm_double_cell (scm_tc16_observer,
+				  SCM_UNPACK (env),
+				  SCM_UNPACK (data),
+				  (scm_t_bits) proc);
 
   if (!weak_p)
     {
@@ -709,7 +682,7 @@ core_environments_unobserve (SCM env, SCM observer)
 	    ? SCM_CDAR (l)
 	    : SCM_CAR (l);
 
-	  if (SCM_EQ_P (first, observer))
+	  if (scm_is_eq (first, observer))
 	    {
 	      /* Remove the first observer */
 	      handling_weaks
@@ -727,7 +700,7 @@ core_environments_unobserve (SCM env, SCM observer)
 		  ? SCM_CDAR (l)
 		  : SCM_CAR (l);
 
-		if (SCM_EQ_P (next, observer))
+		if (scm_is_eq (next, observer))
 		  {
 		    SCM_SETCDR (l, SCM_CDR (rest));
 		    return;
@@ -772,7 +745,7 @@ core_environments_init (struct core_environments_base *body,
 {
   body->funcs = funcs;
   body->observers = SCM_EOL;
-  body->weak_observers = scm_make_weak_value_hash_table (SCM_MAKINUM (1));
+  body->weak_observers = scm_make_weak_value_alist_vector (scm_from_int (1));
 }
 
 
@@ -814,7 +787,8 @@ update_catch_handler (void *ptr, SCM tag, SCM args)
 {
   struct update_data *data = (struct update_data *) ptr;
   SCM observer = data->observer;
-  SCM message = scm_makfrom0str ("Observer `~A' signals `~A' error: ~S");
+  SCM message =
+    scm_from_locale_string ("Observer `~A' signals `~A' error: ~S");
 
   return scm_cons (message, scm_list_3 (observer, tag, args));
 }
@@ -910,10 +884,12 @@ leaf_environment_fold (SCM env, scm_environment_folder proc, SCM data, SCM init)
   SCM result = init;
   SCM obarray = LEAF_ENVIRONMENT (env)->obarray;
 
-  for (i = 0; i < SCM_VECTOR_LENGTH (obarray); i++)
+  for (i = 0; i < SCM_HASHTABLE_N_BUCKETS (obarray); i++)
     {
       SCM l;
-      for (l = SCM_VELTS (obarray)[i]; !SCM_NULLP (l); l = SCM_CDR (l))
+      for (l = SCM_HASHTABLE_BUCKETS (obarray)[i];
+	   !SCM_NULLP (l);
+	   l = SCM_CDR (l))
 	{
 	  SCM binding = SCM_CAR (l);
 	  SCM symbol = SCM_CAR (binding);
@@ -946,7 +922,7 @@ leaf_environment_undefine (SCM env, SCM sym)
   SCM obarray = LEAF_ENVIRONMENT (env)->obarray;
   SCM removed = obarray_remove (obarray, sym);
   
-  if (!SCM_FALSEP (removed))
+  if (scm_is_true (removed))
     core_environments_broadcast (env);
 
   return SCM_ENVIRONMENT_SUCCESS;
@@ -991,24 +967,23 @@ leaf_environment_mark (SCM env)
 }
 
 
-static size_t
+static void
 leaf_environment_free (SCM env)
 {
   core_environments_finalize (env);
-
-  free (LEAF_ENVIRONMENT (env));
-  return sizeof (struct leaf_environment);
+  scm_gc_free (LEAF_ENVIRONMENT (env), sizeof (struct leaf_environment),
+	       "leaf environment");
 }
 
 
 static int
 leaf_environment_print (SCM type, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
-  SCM address = scm_ulong2num (SCM_UNPACK (type));
-  SCM base16 = scm_number_to_string (address, SCM_MAKINUM (16));
+  SCM address = scm_from_size_t (SCM_UNPACK (type));
+  SCM base16 = scm_number_to_string (address, scm_from_int (16));
 
   scm_puts ("#<leaf environment ", port);
-  scm_puts (SCM_STRING_CHARS (base16), port);
+  scm_display (base16, port);
   scm_puts (">", port);
 
   return 1;
@@ -1041,7 +1016,7 @@ SCM_DEFINE (scm_make_leaf_environment, "make-leaf-environment", 0, 0, 0,
 #define FUNC_NAME s_scm_make_leaf_environment
 {
   size_t size = sizeof (struct leaf_environment);
-  struct leaf_environment *body = scm_must_malloc (size, FUNC_NAME);
+  struct leaf_environment *body = scm_gc_malloc (size, "leaf environment");
   SCM env;
 
   core_environments_preinit (&body->base);
@@ -1063,7 +1038,7 @@ SCM_DEFINE (scm_leaf_environment_p, "leaf-environment?", 1, 0, 0,
 	    "otherwise.")
 #define FUNC_NAME s_scm_leaf_environment_p
 {
-  return SCM_BOOL (SCM_LEAF_ENVIRONMENT_P (object));
+  return scm_from_bool (SCM_LEAF_ENVIRONMENT_P (object));
 }
 #undef FUNC_NAME
 
@@ -1110,9 +1085,9 @@ struct eval_environment {
 #define EVAL_ENVIRONMENT(env) \
   ((struct eval_environment *) SCM_CELL_WORD_1 (env))
 
-#define IMMUTABLE SCM_MAKINUM (0)
-#define MUTABLE   SCM_MAKINUM (1)
-#define UNKNOWN   SCM_MAKINUM (2)
+#define IMMUTABLE SCM_I_MAKINUM (0)
+#define MUTABLE   SCM_I_MAKINUM (1)
+#define UNKNOWN   SCM_I_MAKINUM (2)
 
 #define CACHED_LOCATION(x) SCM_CAR (x)
 #define CACHED_MUTABILITY(x) SCM_CADR (x)
@@ -1150,10 +1125,10 @@ eval_environment_lookup (SCM env, SCM sym, int for_write)
 	    return location;
 
 	  mutability = CACHED_MUTABILITY (entry);
-	  if (SCM_EQ_P (mutability, MUTABLE))
+	  if (scm_is_eq (mutability, MUTABLE))
 	    return location;
 
-	  if (SCM_EQ_P (mutability, UNKNOWN))
+	  if (scm_is_eq (mutability, UNKNOWN))
 	    {
 	      SCM source_env = CACHED_SOURCE_ENVIRONMENT (entry);
 	      SCM location = SCM_ENVIRONMENT_CELL (source_env, sym, 1);
@@ -1205,7 +1180,7 @@ eval_environment_lookup (SCM env, SCM sym, int for_write)
 		  obarray_enter (obarray, sym, entry);
 		  return location;
 		}
-	      else if (SCM_EQ_P (location, SCM_ENVIRONMENT_LOCATION_NO_CELL))
+	      else if (scm_is_eq (location, SCM_ENVIRONMENT_LOCATION_NO_CELL))
 		{
 		  obarray_enter (obarray, sym, source_env);
 		  return source_env;
@@ -1246,7 +1221,7 @@ eval_environment_folder (SCM extended_data, SCM symbol, SCM value, SCM tail)
   if (!SCM_ENVIRONMENT_BOUND_P (local, symbol))
     {
       SCM proc_as_nr = SCM_CADR (extended_data);
-      unsigned long int proc_as_ul = scm_num2ulong (proc_as_nr, 0, NULL);
+      unsigned long int proc_as_ul = scm_to_ulong (proc_as_nr);
       scm_environment_folder proc = (scm_environment_folder) proc_as_ul;
       SCM data = SCM_CDDR (extended_data);
 
@@ -1264,7 +1239,7 @@ eval_environment_fold (SCM env, scm_environment_folder proc, SCM data, SCM init)
 {
   SCM local = EVAL_ENVIRONMENT (env)->local;
   SCM imported = EVAL_ENVIRONMENT (env)->imported;
-  SCM proc_as_nr = scm_ulong2num ((unsigned long int) proc);
+  SCM proc_as_nr = scm_from_ulong ((unsigned long) proc);
   SCM extended_data = scm_cons2 (local, proc_as_nr, data);
   SCM tmp_result = scm_c_environment_fold (imported, eval_environment_folder, extended_data, init);
 
@@ -1307,7 +1282,7 @@ eval_environment_set_x (SCM env, SCM sym, SCM val)
     {
       return SCM_ENVIRONMENT_SET (location, sym, val);
     }
-  else if (SCM_EQ_P (location, IMMUTABLE))
+  else if (scm_is_eq (location, IMMUTABLE))
     {
       return SCM_ENVIRONMENT_LOCATION_IMMUTABLE;
     }
@@ -1329,7 +1304,7 @@ eval_environment_cell (SCM env, SCM sym, int for_write)
     return location;
   else if (SCM_ENVIRONMENT_P (location))
     return SCM_ENVIRONMENT_LOCATION_NO_CELL;
-  else if (SCM_EQ_P (location, IMMUTABLE))
+  else if (scm_is_eq (location, IMMUTABLE))
     return SCM_ENVIRONMENT_LOCATION_IMMUTABLE;
   else
     return SCM_UNDEFINED;
@@ -1352,24 +1327,23 @@ eval_environment_mark (SCM env)
 }
 
 
-static size_t
+static void
 eval_environment_free (SCM env)
 {
   core_environments_finalize (env);
-
-  free (EVAL_ENVIRONMENT (env));
-  return sizeof (struct eval_environment);
+  scm_gc_free (EVAL_ENVIRONMENT (env), sizeof (struct eval_environment),
+	       "eval environment");
 }
 
 
 static int
 eval_environment_print (SCM type, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
-  SCM address = scm_ulong2num (SCM_UNPACK (type));
-  SCM base16 = scm_number_to_string (address, SCM_MAKINUM (16));
+  SCM address = scm_from_size_t (SCM_UNPACK (type));
+  SCM base16 = scm_number_to_string (address, scm_from_int (16));
 
   scm_puts ("#<eval environment ", port);
-  scm_puts (SCM_STRING_CHARS (base16), port);
+  scm_display (base16, port);
   scm_puts (">", port);
 
   return 1;
@@ -1435,7 +1409,7 @@ SCM_DEFINE (scm_make_eval_environment, "make-eval-environment", 2, 0, 0,
   SCM_ASSERT (SCM_ENVIRONMENT_P (local), local, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (SCM_ENVIRONMENT_P (imported), imported, SCM_ARG2, FUNC_NAME);
 
-  body = scm_must_malloc (sizeof (struct eval_environment), FUNC_NAME);
+  body = scm_gc_malloc (sizeof (struct eval_environment), "eval environment");
 
   core_environments_preinit (&body->base);
   body->obarray = SCM_BOOL_F;
@@ -1466,7 +1440,7 @@ SCM_DEFINE (scm_eval_environment_p, "eval-environment?", 1, 0, 0,
 	    "otherwise.")
 #define FUNC_NAME s_scm_eval_environment_p
 {
-  return SCM_BOOL (SCM_EVAL_ENVIRONMENT_P (object));
+  return scm_from_bool (SCM_EVAL_ENVIRONMENT_P (object));
 }
 #undef FUNC_NAME
 
@@ -1652,11 +1626,11 @@ import_environment_folder (SCM extended_data, SCM symbol, SCM value, SCM tail)
   SCM imported_env = SCM_CADR (extended_data);
   SCM owner = import_environment_lookup (import_env, symbol);
   SCM proc_as_nr = SCM_CADDR (extended_data);
-  unsigned long int proc_as_ul = scm_num2ulong (proc_as_nr, 0, NULL);
+  unsigned long int proc_as_ul = scm_to_ulong (proc_as_nr);
   scm_environment_folder proc = (scm_environment_folder) proc_as_ul;
   SCM data = SCM_CDDDR (extended_data);
 
-  if (SCM_CONSP (owner) && SCM_EQ_P (SCM_CAR (owner), imported_env))
+  if (SCM_CONSP (owner) && scm_is_eq (SCM_CAR (owner), imported_env))
     owner = import_environment_conflict (import_env, symbol, owner);
 
   if (SCM_ENVIRONMENT_P (owner))
@@ -1670,7 +1644,7 @@ import_environment_folder (SCM extended_data, SCM symbol, SCM value, SCM tail)
 static SCM
 import_environment_fold (SCM env, scm_environment_folder proc, SCM data, SCM init)
 {
-  SCM proc_as_nr = scm_ulong2num ((unsigned long int) proc);
+  SCM proc_as_nr = scm_from_ulong ((unsigned long) proc);
   SCM result = init;
   SCM l;
 
@@ -1771,13 +1745,12 @@ import_environment_mark (SCM env)
 }
 
 
-static size_t
+static void
 import_environment_free (SCM env)
 {
   core_environments_finalize (env);
-
-  free (IMPORT_ENVIRONMENT (env));
-  return sizeof (struct import_environment);
+  scm_gc_free (IMPORT_ENVIRONMENT (env), sizeof (struct import_environment),
+	       "import environment");
 }
 
 
@@ -1785,11 +1758,11 @@ static int
 import_environment_print (SCM type, SCM port, 
 			  scm_print_state *pstate SCM_UNUSED)
 {
-  SCM address = scm_ulong2num (SCM_UNPACK (type));
-  SCM base16 = scm_number_to_string (address, SCM_MAKINUM (16));
+  SCM address = scm_from_size_t (SCM_UNPACK (type));
+  SCM base16 = scm_number_to_string (address, scm_from_int (16));
 
   scm_puts ("#<import environment ", port);
-  scm_puts (SCM_STRING_CHARS (base16), port);
+  scm_display (base16, port);
   scm_puts (">", port);
 
   return 1;
@@ -1851,7 +1824,7 @@ SCM_DEFINE (scm_make_import_environment, "make-import-environment", 2, 0, 0,
 #define FUNC_NAME s_scm_make_import_environment
 {
   size_t size = sizeof (struct import_environment);
-  struct import_environment *body = scm_must_malloc (size, FUNC_NAME);
+  struct import_environment *body = scm_gc_malloc (size, "import environment");
   SCM env;
 
   core_environments_preinit (&body->base);
@@ -1879,7 +1852,7 @@ SCM_DEFINE (scm_import_environment_p, "import-environment?", 1, 0, 0,
 	    "@code{#f} otherwise.")
 #define FUNC_NAME s_scm_import_environment_p
 {
-  return SCM_BOOL (SCM_IMPORT_ENVIRONMENT_P (object));
+  return scm_from_bool (SCM_IMPORT_ENVIRONMENT_P (object));
 }
 #undef FUNC_NAME
 
@@ -1974,7 +1947,7 @@ export_environment_ref (SCM env, SCM sym)
   struct export_environment *body = EXPORT_ENVIRONMENT (env);
   SCM entry = scm_assq (sym, body->signature);
 
-  if (SCM_FALSEP (entry))
+  if (scm_is_false (entry))
     return SCM_UNDEFINED;
   else
     return SCM_ENVIRONMENT_REF (body->private, sym);
@@ -2027,13 +2000,13 @@ export_environment_set_x (SCM env, SCM sym, SCM val)
   struct export_environment *body = EXPORT_ENVIRONMENT (env);
   SCM entry = scm_assq (sym, body->signature);
 
-  if (SCM_FALSEP (entry))
+  if (scm_is_false (entry))
     {
       return SCM_UNDEFINED;
     }
   else
     {
-      if (SCM_EQ_P (SCM_CADR (entry), symbol_mutable_location))
+      if (scm_is_eq (SCM_CADR (entry), symbol_mutable_location))
 	return SCM_ENVIRONMENT_SET (body->private, sym, val);
       else
 	return SCM_ENVIRONMENT_LOCATION_IMMUTABLE;
@@ -2049,13 +2022,13 @@ export_environment_cell (SCM env, SCM sym, int for_write)
   struct export_environment *body = EXPORT_ENVIRONMENT (env);
   SCM entry = scm_assq (sym, body->signature);
 
-  if (SCM_FALSEP (entry))
+  if (scm_is_false (entry))
     {
       return SCM_UNDEFINED;
     }
   else
     {
-      if (!for_write || SCM_EQ_P (SCM_CADR (entry), symbol_mutable_location))
+      if (!for_write || scm_is_eq (SCM_CADR (entry), symbol_mutable_location))
 	return SCM_ENVIRONMENT_CELL (body->private, sym, for_write);
       else
 	return SCM_ENVIRONMENT_LOCATION_IMMUTABLE;
@@ -2077,13 +2050,12 @@ export_environment_mark (SCM env)
 }
 
 
-static size_t
+static void
 export_environment_free (SCM env)
 {
   core_environments_finalize (env);
-
-  free (EXPORT_ENVIRONMENT (env));
-  return sizeof (struct export_environment);
+  scm_gc_free (EXPORT_ENVIRONMENT (env), sizeof (struct export_environment),
+	       "export environment");
 }
 
 
@@ -2091,11 +2063,11 @@ static int
 export_environment_print (SCM type, SCM port,
 			  scm_print_state *pstate SCM_UNUSED)
 {
-  SCM address = scm_ulong2num (SCM_UNPACK (type));
-  SCM base16 = scm_number_to_string (address, SCM_MAKINUM (16));
+  SCM address = scm_from_size_t (SCM_UNPACK (type));
+  SCM base16 = scm_number_to_string (address, scm_from_int (16));
 
   scm_puts ("#<export environment ", port);
-  scm_puts (SCM_STRING_CHARS (base16), port);
+  scm_display (base16, port);
   scm_puts (">", port);
 
   return 1;
@@ -2178,7 +2150,7 @@ SCM_DEFINE (scm_make_export_environment, "make-export-environment", 2, 0, 0,
   SCM_ASSERT (SCM_ENVIRONMENT_P (private), private, SCM_ARG1, FUNC_NAME);
 
   size = sizeof (struct export_environment);
-  body = scm_must_malloc (size, FUNC_NAME);
+  body = scm_gc_malloc (size, "export environment");
 
   core_environments_preinit (&body->base);
   body->private = SCM_BOOL_F;
@@ -2206,7 +2178,7 @@ SCM_DEFINE (scm_export_environment_p, "export-environment?", 1, 0, 0,
 	    "@code{#f} otherwise.")
 #define FUNC_NAME s_scm_export_environment_p
 {
-  return SCM_BOOL (SCM_EXPORT_ENVIRONMENT_P (object));
+  return scm_from_bool (SCM_EXPORT_ENVIRONMENT_P (object));
 }
 #undef FUNC_NAME
 
@@ -2267,7 +2239,7 @@ export_environment_parse_signature (SCM signature, const char* caller)
     {
       SCM entry = SCM_CAR (l);
 
-      if (SCM_SYMBOLP (entry))
+      if (scm_is_symbol (entry))
 	{
 	  SCM new_entry = scm_cons2 (entry, symbol_immutable_location, SCM_EOL);
 	  result = scm_cons (new_entry, result);
@@ -2282,16 +2254,16 @@ export_environment_parse_signature (SCM signature, const char* caller)
 	  SCM l2;
 
 	  SCM_ASSERT (SCM_CONSP (entry), entry, SCM_ARGn, caller);
-	  SCM_ASSERT (SCM_SYMBOLP (SCM_CAR (entry)), entry, SCM_ARGn, caller);
+	  SCM_ASSERT (scm_is_symbol (SCM_CAR (entry)), entry, SCM_ARGn, caller);
 
 	  sym = SCM_CAR (entry);
 
 	  for (l2 = SCM_CDR (entry); SCM_CONSP (l2); l2 = SCM_CDR (l2))
 	    {
 	      SCM attribute = SCM_CAR (l2);
-	      if (SCM_EQ_P (attribute, symbol_immutable_location))
+	      if (scm_is_eq (attribute, symbol_immutable_location))
 		immutable = 1;
-	      else if (SCM_EQ_P (attribute, symbol_mutable_location))
+	      else if (scm_is_eq (attribute, symbol_mutable_location))
 		mutable = 1;
 	      else
 		SCM_ASSERT (0, entry, SCM_ARGn, caller);
