@@ -1,46 +1,25 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001,2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
-
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "libguile/_scm.h"
 #include "libguile/ramap.h"
 #include "libguile/stackchk.h"
@@ -54,6 +33,12 @@
 #include "libguile/validate.h"
 #include "libguile/eq.h"
 
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
+
 SCM_DEFINE1 (scm_eq_p, "eq?", scm_tc7_rpsubr,
              (SCM x, SCM y),
 	     "Return @code{#t} iff @var{x} references the same object as @var{y}.\n"
@@ -62,11 +47,21 @@ SCM_DEFINE1 (scm_eq_p, "eq?", scm_tc7_rpsubr,
 	     "@code{eqv?}.")
 #define FUNC_NAME s_scm_eq_p
 {
-  return SCM_BOOL (SCM_EQ_P (x, y));
+  return scm_from_bool (scm_is_eq (x, y));
 }
 #undef FUNC_NAME
 
+/* We compare doubles in a special way for 'eqv?' to be able to
+   distinguish plus and minus zero and to identify NaNs.
+*/
 
+static int
+real_eqv (double x, double y)
+{
+  return !memcmp (&x, &y, sizeof(double));
+}
+
+#include <stdio.h>
 SCM_PRIMITIVE_GENERIC_1 (scm_eqv_p, "eqv?", scm_tc7_rpsubr,
              (SCM x, SCM y),
 	     "The @code{eqv?} procedure defines a useful equivalence relation on objects.\n"
@@ -76,38 +71,53 @@ SCM_PRIMITIVE_GENERIC_1 (scm_eqv_p, "eqv?", scm_tc7_rpsubr,
 	     "and inexact numbers.")
 #define FUNC_NAME s_scm_eqv_p
 {
-  if (SCM_EQ_P (x, y))
+  if (scm_is_eq (x, y))
     return SCM_BOOL_T;
   if (SCM_IMP (x))
     return SCM_BOOL_F;
   if (SCM_IMP (y))
     return SCM_BOOL_F;
   /* this ensures that types and scm_length are the same. */
+
   if (SCM_CELL_TYPE (x) != SCM_CELL_TYPE (y))
     {
+      /* fractions use 0x10000 as a flag (at the suggestion of Marius Vollmer),
+	 but this checks the entire type word, so fractions may be accidentally
+	 flagged here as unequal.  Perhaps I should use the 4th double_cell word?
+      */
+
       /* treat mixes of real and complex types specially */
-      if (SCM_SLOPPY_INEXACTP (x))
+      if (SCM_INEXACTP (x))
 	{
-	  if (SCM_SLOPPY_REALP (x))
-	    return SCM_BOOL (SCM_SLOPPY_COMPLEXP (y)
-			     && SCM_REAL_VALUE (x) == SCM_COMPLEX_REAL (y)
-			     && 0.0 == SCM_COMPLEX_IMAG (y));
+	  if (SCM_REALP (x))
+	    return scm_from_bool (SCM_COMPLEXP (y)
+			     && real_eqv (SCM_REAL_VALUE (x),
+					  SCM_COMPLEX_REAL (y))
+			     && SCM_COMPLEX_IMAG (y) == 0.0);
 	  else
-	    return SCM_BOOL (SCM_SLOPPY_REALP (y)
-			     && SCM_COMPLEX_REAL (x) == SCM_REAL_VALUE (y)
+	    return scm_from_bool (SCM_REALP (y)
+			     && real_eqv (SCM_COMPLEX_REAL (x),
+					  SCM_REAL_VALUE (y))
 			     && SCM_COMPLEX_IMAG (x) == 0.0);
 	}
+
+      if (SCM_FRACTIONP (x) && SCM_FRACTIONP (y))
+	return scm_i_fraction_equalp (x, y);
       return SCM_BOOL_F;
     }
   if (SCM_NUMP (x))
     {
       if (SCM_BIGP (x)) {
-	return SCM_BOOL (0 == scm_bigcomp (x, y));
-      } else if (SCM_SLOPPY_REALP (x)) {
-	return SCM_BOOL (SCM_REAL_VALUE (x) == SCM_REAL_VALUE (y));
+	return scm_from_bool (scm_i_bigcmp (x, y) == 0);
+      } else if (SCM_REALP (x)) {
+	return scm_from_bool (real_eqv (SCM_REAL_VALUE (x), SCM_REAL_VALUE (y)));
+      } else if (SCM_FRACTIONP (x)) {
+	return scm_i_fraction_equalp (x, y);
       } else { /* complex */
-	return SCM_BOOL (SCM_COMPLEX_REAL (x) == SCM_COMPLEX_REAL (y) 
-			 && SCM_COMPLEX_IMAG (x) == SCM_COMPLEX_IMAG (y));
+	return scm_from_bool (real_eqv (SCM_COMPLEX_REAL (x),
+				   SCM_COMPLEX_REAL (y)) 
+			 && real_eqv (SCM_COMPLEX_IMAG (x),
+				      SCM_COMPLEX_IMAG (y)));
       }
     }
   if (SCM_UNPACK (g_scm_eqv_p))
@@ -131,7 +141,7 @@ SCM_PRIMITIVE_GENERIC_1 (scm_equal_p, "equal?", scm_tc7_rpsubr,
   SCM_CHECK_STACK;
  tailrecurse:
   SCM_TICK;
-  if (SCM_EQ_P (x, y))
+  if (scm_is_eq (x, y))
     return SCM_BOOL_T;
   if (SCM_IMP (x))
     return SCM_BOOL_F;
@@ -139,35 +149,48 @@ SCM_PRIMITIVE_GENERIC_1 (scm_equal_p, "equal?", scm_tc7_rpsubr,
     return SCM_BOOL_F;
   if (SCM_CONSP (x) && SCM_CONSP (y))
     {
-      if (SCM_FALSEP (scm_equal_p (SCM_CAR (x), SCM_CAR (y))))
+      if (scm_is_false (scm_equal_p (SCM_CAR (x), SCM_CAR (y))))
 	return SCM_BOOL_F;
       x = SCM_CDR(x);
       y = SCM_CDR(y);
       goto tailrecurse;
     }
-  if (SCM_TYP7S (x) == scm_tc7_string && SCM_TYP7S (y) == scm_tc7_string)
+  if (SCM_TYP7 (x) == scm_tc7_string && SCM_TYP7 (y) == scm_tc7_string)
     return scm_string_equal_p (x, y);
   /* This ensures that types and scm_length are the same.  */
   if (SCM_CELL_TYPE (x) != SCM_CELL_TYPE (y))
     {
       /* treat mixes of real and complex types specially */
-      if (SCM_SLOPPY_INEXACTP (x))
+      if (SCM_INEXACTP (x) && SCM_INEXACTP (y))
 	{
-	  if (SCM_SLOPPY_REALP (x))
-	    return SCM_BOOL (SCM_SLOPPY_COMPLEXP (y)
+	  if (SCM_REALP (x))
+	    return scm_from_bool (SCM_COMPLEXP (y)
 			     && SCM_REAL_VALUE (x) == SCM_COMPLEX_REAL (y)
-			     && 0.0 == SCM_COMPLEX_IMAG (y));
+			     && SCM_COMPLEX_IMAG (y) == 0.0);
 	  else
-	    return SCM_BOOL (SCM_SLOPPY_REALP (y)
+	    return scm_from_bool (SCM_REALP (y)
 			     && SCM_COMPLEX_REAL (x) == SCM_REAL_VALUE (y)
 			     && SCM_COMPLEX_IMAG (x) == 0.0);
 	}
+
       return SCM_BOOL_F;
     }
   switch (SCM_TYP7 (x))
     {
     default:
       break;
+    case scm_tc7_number:
+      switch SCM_TYP16 (x)
+        {
+        case scm_tc16_big:
+          return scm_bigequal (x, y);
+        case scm_tc16_real:
+          return scm_real_equalp (x, y);
+        case scm_tc16_complex:
+          return scm_complex_equalp (x, y);
+	case scm_tc16_fraction:
+          return scm_i_fraction_equalp (x, y);
+        }
     case scm_tc7_vector:
     case scm_tc7_wvect:
       return scm_vector_equal_p (x, y);
@@ -181,11 +204,11 @@ SCM_PRIMITIVE_GENERIC_1 (scm_equal_p, "equal?", scm_tc7_rpsubr,
 	else
 	  break;
       }
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_bvect: case scm_tc7_uvect: case scm_tc7_ivect:
     case scm_tc7_fvect:	case scm_tc7_cvect: case scm_tc7_dvect:
     case scm_tc7_svect:
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
     case scm_tc7_llvect:
 #endif
     case scm_tc7_byvect:
