@@ -1,51 +1,32 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2004 Free Software Foundation, Inc.
+ * 
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 
 #include "libguile/_scm.h"
+#include "libguile/dynwind.h"
 #include "libguile/pairs.h"
 #include "libguile/strings.h"
 #include "libguile/throw.h"
@@ -71,36 +52,19 @@
  */
 
 
-/* All errors should pass through here.  */
+/* Scheme interface to scm_error_scm.  */
 void
 scm_error (SCM key, const char *subr, const char *message, SCM args, SCM rest)
 {
-  SCM arg_list;
-  if (scm_gc_running_p)
-    {
-      /* The error occured during GC --- abort */
-      fprintf (stderr, "Error in %s during GC: %s\n",
-	       subr ? subr : "unknown function",
-	       message ? message : "<empty message>");
-      abort ();
-    }
-  arg_list = scm_list_4 (subr ? scm_makfrom0str (subr) : SCM_BOOL_F,
-			 message ? scm_makfrom0str (message) : SCM_BOOL_F,
-			 args,
-			 rest);
-  scm_ithrow (key, arg_list, 1);
-
-  /* No return, but just in case: */
-  {
-    const char msg[] = "guile:scm_error:scm_ithrow returned!\n";
-
-    write (2, msg, (sizeof msg) - 1);
-  }
-  exit (1);
+  scm_error_scm 
+    (key,
+     (subr == NULL) ? SCM_BOOL_F : scm_from_locale_string (subr),
+     (message == NULL) ? SCM_BOOL_F : scm_from_locale_string (message),
+     args, rest);
 }
 
-/* Scheme interface to scm_error.  */
-SCM_DEFINE (scm_error_scm, "scm-error", 5, 0, 0,
+/* All errors should pass through here.  */
+SCM_DEFINE (scm_error_scm, "scm-error", 5, 0, 0, 
            (SCM key, SCM subr, SCM message, SCM args, SCM data),
 	    "Raise an error with key @var{key}.  @var{subr} can be a string\n"
 	    "naming the procedure associated with the error, or @code{#f}.\n"
@@ -117,50 +81,64 @@ SCM_DEFINE (scm_error_scm, "scm-error", 5, 0, 0,
 	    "it will usually be @code{#f}.")
 #define FUNC_NAME s_scm_error_scm
 {
-  char *szSubr;
-  char *szMessage;
-
-  SCM_VALIDATE_SYMBOL (1, key);
-
-  if (SCM_FALSEP (subr))
+  if (scm_gc_running_p)
     {
-      szSubr = NULL;
-    }
-  else if (SCM_SYMBOLP (subr))
-    {
-      szSubr = SCM_SYMBOL_CHARS (subr);
-    }
-  else
-    {
-      SCM_VALIDATE_STRING (2, subr);
-      SCM_STRING_COERCE_0TERMINATION_X (subr);
-      szSubr = SCM_STRING_CHARS (subr);
+      /* The error occured during GC --- abort */
+      fprintf (stderr, "Guile: error during GC.\n"),
+      abort ();
     }
 
-  if (SCM_FALSEP (message))
-    {
-      szMessage = NULL;
-    }
-  else
-    {
-      SCM_VALIDATE_STRING (2, message);
-      SCM_STRING_COERCE_0TERMINATION_X (message);
-      szMessage = SCM_STRING_CHARS (message);
-    }
-
-  scm_error (key, szSubr, szMessage, args, data);
-  /* not reached.  */
+  scm_ithrow (key, scm_list_4 (subr, message, args, data), 1);
+  
+  /* No return, but just in case: */
+  fprintf (stderr, "Guile scm_ithrow returned!\n");
+  exit (1);
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_strerror, "strerror", 1, 0, 0,
+#ifdef __MINGW32__
+# include "win32-socket.h"
+# define SCM_I_STRERROR(err) \
+    ((err >= WSABASEERR) ? scm_i_socket_strerror (err) : strerror (err))
+# define SCM_I_ERRNO() \
+    (errno ? errno : scm_i_socket_errno ())
+#else
+# define SCM_I_STRERROR(err) strerror (err)
+# define SCM_I_ERRNO() errno
+#endif /* __MINGW32__ */
+
+/* strerror may not be thread safe, for instance in glibc (version 2.3.2) an
+   error number not among the known values results in a string like "Unknown
+   error 9999" formed in a static buffer, which will be overwritten by a
+   similar call in another thread.  A test program running two threads with
+   different unknown error numbers can trip this fairly quickly.
+
+   Some systems don't do what glibc does, instead just giving a single
+   "Unknown error" for unrecognised numbers.  It doesn't seem worth trying
+   to tell if that's the case, a mutex is reasonably fast, and strerror
+   isn't needed very often.
+
+   strerror_r (when available) could be used, it might be a touch faster
+   than a frame and a mutex, though there's probably not much
+   difference.  */
+
+SCM_DEFINE (scm_strerror, "strerror", 1, 0, 0, 
             (SCM err),
 	    "Return the Unix error message corresponding to @var{err}, which\n"
 	    "must be an integer value.")
 #define FUNC_NAME s_scm_strerror
 {
-  SCM_VALIDATE_INUM (1,err);
-  return scm_makfrom0str (strerror (SCM_INUM (err)));
+  SCM ret;
+  scm_frame_begin (0);
+  scm_frame_unwind_handler ((void(*)(void*)) scm_mutex_unlock,
+                            &scm_i_misc_mutex,
+                            SCM_F_WIND_EXPLICITLY);
+  scm_mutex_lock (&scm_i_misc_mutex);
+
+  ret = scm_from_locale_string (SCM_I_STRERROR (scm_to_int (err)));
+
+  scm_frame_end ();
+  return ret;
 }
 #undef FUNC_NAME
 
@@ -168,13 +146,12 @@ SCM_GLOBAL_SYMBOL (scm_system_error_key, "system-error");
 void
 scm_syserror (const char *subr)
 {
-  int save_errno = errno;
-
+  SCM err = scm_from_int (SCM_I_ERRNO ());
   scm_error (scm_system_error_key,
 	     subr,
 	     "~A",
-	     scm_cons (scm_makfrom0str (strerror (save_errno)), SCM_EOL),
-	     scm_cons (SCM_MAKINUM (save_errno), SCM_EOL));
+	     scm_cons (scm_strerror (err), SCM_EOL),
+	     scm_cons (err, SCM_EOL));
 }
 
 void
@@ -184,7 +161,7 @@ scm_syserror_msg (const char *subr, const char *message, SCM args, int eno)
 	     subr,
 	     message,
 	     args,
-	     scm_cons (SCM_MAKINUM (eno), SCM_EOL));
+	     scm_cons (scm_from_int (eno), SCM_EOL));
 }
 
 SCM_GLOBAL_SYMBOL (scm_num_overflow_key, "numerical-overflow");
@@ -215,7 +192,7 @@ scm_out_of_range_pos (const char *subr, SCM bad_value, SCM pos)
   scm_error (scm_out_of_range_key,
 	     subr,
 	     "Argument ~S out of range: ~S",
-             scm_list_2 (pos,bad_value),
+             scm_list_2 (pos, bad_value),
 	     SCM_BOOL_F);
 }
 
@@ -238,7 +215,7 @@ scm_error_num_args_subr (const char *subr)
   scm_error (scm_args_number_key,
 	     NULL,
 	     "Wrong number of arguments to ~A",
-	     scm_list_1 (scm_makfrom0str (subr)),
+	     scm_list_1 (scm_from_locale_string (subr)),
 	     SCM_BOOL_F);
 }
 
@@ -252,14 +229,14 @@ scm_wrong_type_arg (const char *subr, int pos, SCM bad_value)
 	     (pos == 0) ? "Wrong type argument: ~S"
 	     : "Wrong type argument in position ~A: ~S",
 	     (pos == 0) ? scm_list_1 (bad_value)
-	     : scm_list_2 (SCM_MAKINUM (pos), bad_value),
+	     : scm_list_2 (scm_from_int (pos), bad_value),
 	     SCM_BOOL_F);
 }
 
 void
 scm_wrong_type_arg_msg (const char *subr, int pos, SCM bad_value, const char *szMessage)
 {
-  SCM msg = scm_makfrom0str(szMessage);
+  SCM msg = scm_from_locale_string (szMessage);
   if (pos == 0) {
     scm_error (scm_arg_type_key,
                subr, "Wrong type argument (expecting ~A): ~S",
@@ -269,7 +246,7 @@ scm_wrong_type_arg_msg (const char *subr, int pos, SCM bad_value, const char *sz
     scm_error (scm_arg_type_key,
                subr,
                "Wrong type argument in position ~A (expecting ~A): ~S",
-               scm_list_3 (SCM_MAKINUM (pos), msg, bad_value),
+               scm_list_3 (scm_from_int (pos), msg, bad_value),
                SCM_BOOL_F);
   }
 }
@@ -279,11 +256,8 @@ SCM_GLOBAL_SYMBOL (scm_memory_alloc_key, "memory-allocation-error");
 void
 scm_memory_error (const char *subr)
 {
-  scm_error (scm_memory_alloc_key,
-	     subr,
-	     "Memory allocation error",
-	     SCM_BOOL_F,
-	     SCM_BOOL_F);
+  fprintf (stderr, "FATAL: memory error in %s\n", subr);
+  abort ();
 }
 
 SCM_GLOBAL_SYMBOL (scm_misc_error_key, "misc-error");
@@ -293,57 +267,6 @@ scm_misc_error (const char *subr, const char *message, SCM args)
   scm_error (scm_misc_error_key, subr, message, args, SCM_BOOL_F);
 }
 
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-SCM
-scm_wta (SCM arg, const char *pos, const char *s_subr)
-{
-  if (!s_subr || !*s_subr)
-    s_subr = NULL;
-  if ((~0x1fL) & (long) pos)
-    {
-      /* error string supplied.  */
-      scm_misc_error (s_subr, pos, scm_list_1 (arg));
-    }
-  else
-    {
-      /* numerical error code.  */
-      scm_t_bits error = (scm_t_bits) pos;
-
-      switch (error)
-	{
-	case SCM_ARGn:
-	  scm_wrong_type_arg (s_subr, 0, arg);
-	case SCM_ARG1:
-	  scm_wrong_type_arg (s_subr, 1, arg);
-	case SCM_ARG2:
-	  scm_wrong_type_arg (s_subr, 2, arg);
-	case SCM_ARG3:
-	  scm_wrong_type_arg (s_subr, 3, arg);
-	case SCM_ARG4:
-	  scm_wrong_type_arg (s_subr, 4, arg);
-	case SCM_ARG5:
-	  scm_wrong_type_arg (s_subr, 5, arg);
-	case SCM_ARG6:
-	  scm_wrong_type_arg (s_subr, 6, arg);
-	case SCM_ARG7:
-	  scm_wrong_type_arg (s_subr, 7, arg);
-	case SCM_WNA:
-	  scm_wrong_num_args (arg);
-	case SCM_OUTOFRANGE:
-	  scm_out_of_range (s_subr, arg);
-	case SCM_NALLOC:
-	  scm_memory_error (s_subr);
-	default:
-	  /* this shouldn't happen.  */
-	  scm_misc_error (s_subr, "Unknown error", SCM_EOL);
-	}
-    }
-  return SCM_UNSPECIFIED;
-}
-
-#endif  /* SCM_DEBUG_DEPRECATED == 0 */
-
 void
 scm_init_error ()
 {
