@@ -1,46 +1,29 @@
-/* Copyright (C) 1996,1997,1998,1999,2000,2001,2004 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,1999,2000,2001, 2002, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
+#define _GNU_SOURCE              /* ask glibc for everything */
+#define _POSIX_C_SOURCE 199506L  /* for readdir_r elsewhere */
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 
@@ -51,6 +34,8 @@
 #include "libguile/iselect.h"
 #include "libguile/strings.h"
 #include "libguile/vectors.h"
+#include "libguile/lang.h"
+#include "libguile/dynwind.h"
 
 #include "libguile/validate.h"
 #include "libguile/filesys.h"
@@ -60,6 +45,10 @@
 #include <io.h>
 #endif
 
+#ifdef HAVE_DIRECT_H
+#include <direct.h>
+#endif
+
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -96,7 +85,10 @@
 #endif
 
 
-#if HAVE_DIRENT_H
+#if defined (__MINGW32__) || defined (_MSC_VER) || defined (__BORLANDC__)
+# include "win32-dirent.h"
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#elif HAVE_DIRENT_H
 # include <dirent.h>
 # define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
@@ -121,15 +113,60 @@
 /* The MinGW gcc does not define the S_ISSOCK macro. Any other native Windows
    compiler like BorlandC or MSVC has none of these macros defined. */
 #ifdef __MINGW32__
-# define S_ISSOCK(mode) (0)
-#endif
-#if defined (__BORLANDC__) || defined (_MSC_VER)
-# define S_ISBLK(mode) (0)
+
+# ifdef _S_IFIFO
+#  undef _S_IFIFO
+# endif
+# ifdef _S_IFCHR
+#  undef _S_IFCHR
+# endif
+# ifdef _S_IFBLK
+#  undef _S_IFBLK
+# endif
+# ifdef _S_IFDIR
+#  undef _S_IFDIR
+# endif
+# ifdef _S_IFREG
+#  undef _S_IFREG
+# endif
+# ifdef _S_IFSOCK
+#  undef _S_IFSOCK
+# endif
+
+# define _S_IFIFO        0x1000  /* FIFO */
+# define _S_IFCHR        0x2000  /* Character */
+# define _S_IFBLK        0x3000  /* Block */
+# define _S_IFDIR        0x4000  /* Directory */
+# define _S_IFREG        0x8000  /* Regular */
+# define _S_IFSOCK       0xC000  /* Socket */
+
+# ifdef S_ISBLK
+#  undef S_ISBLK
+# endif
+# ifdef S_ISFIFO
+#  undef S_ISFIFO
+# endif
+# ifdef S_ISCHR
+#  undef S_ISCHR
+# endif
+# ifdef S_ISDIR
+#  undef S_ISDIR
+# endif
+# ifdef S_ISREG
+#  undef S_ISREG
+# endif
+# ifdef S_ISSOCK
+#  undef S_ISSOCK
+# endif
+
+# define S_ISBLK(mode)  (((mode) & _S_IFMT) == _S_IFBLK)
 # define S_ISFIFO(mode) (((mode) & _S_IFMT) == _S_IFIFO)
-# define S_ISCHR(mode) (((mode) & _S_IFMT) == _S_IFCHR)
-# define S_ISDIR(mode) (((mode) & _S_IFMT) == _S_IFDIR)
-# define S_ISREG(mode) (((mode) & _S_IFMT) == _S_IFREG)
-#endif
+# define S_ISCHR(mode)  (((mode) & _S_IFMT) == _S_IFCHR)
+# define S_ISDIR(mode)  (((mode) & _S_IFMT) == _S_IFDIR)
+# define S_ISREG(mode)  (((mode) & _S_IFMT) == _S_IFREG)
+# define S_ISSOCK(mode) (((mode) & _S_IFMT) == _S_IFSOCK)
+
+#endif /* __MINGW32__ */
 
 /* Some more definitions for the native Windows port. */
 #ifdef __MINGW32__
@@ -139,6 +176,28 @@
 #endif /* __MINGW32__ */
 
 
+/* Two helper macros for an often used pattern */
+
+#define STRING_SYSCALL(str,cstr,code)        \
+  do {                                       \
+    int eno;                                 \
+    char *cstr = scm_to_locale_string (str); \
+    SCM_SYSCALL (code);                      \
+    eno = errno; free (cstr); errno = eno;   \
+  } while (0)
+
+#define STRING2_SYSCALL(str1,cstr1,str2,cstr2,code)  \
+  do {                                               \
+    int eno;                                         \
+    char *cstr1, *cstr2;                             \
+    scm_frame_begin (0);                             \
+    cstr1 = scm_to_locale_string (str1);             \
+    scm_frame_free (cstr1);                          \
+    cstr2 = scm_to_locale_string (str2);             \
+    scm_frame_free (cstr2);                          \
+    SCM_SYSCALL (code);                              \
+    eno = errno; scm_frame_end (); errno = eno;      \
+  } while (0)
 
 
 
@@ -165,23 +224,20 @@ SCM_DEFINE (scm_chown, "chown", 3, 0, 0,
 
   object = SCM_COERCE_OUTPORT (object);
 
-  SCM_VALIDATE_INUM (2,owner);
-  SCM_VALIDATE_INUM (3,group);
 #ifdef HAVE_FCHOWN
-  if (SCM_INUMP (object) || (SCM_OPFPORTP (object)))
+  if (scm_is_integer (object) || (SCM_OPFPORTP (object)))
     {
-      int fdes = SCM_INUMP (object) ? SCM_INUM (object)
-	: SCM_FPORT_FDES (object);
+      int fdes = (SCM_OPFPORTP (object)?
+		  SCM_FPORT_FDES (object) : scm_to_int (object));
 
-      SCM_SYSCALL (rv = fchown (fdes, SCM_INUM (owner), SCM_INUM (group)));
+      SCM_SYSCALL (rv = fchown (fdes, scm_to_int (owner), scm_to_int (group)));
     }
   else
 #endif
     {
-      SCM_VALIDATE_STRING (1, object);
-      SCM_STRING_COERCE_0TERMINATION_X (object);
-      SCM_SYSCALL (rv = chown (SCM_STRING_CHARS (object),
-			       SCM_INUM (owner), SCM_INUM (group)));
+      STRING_SYSCALL (object, c_object,
+		      rv = chown (c_object,
+				  scm_to_int (owner), scm_to_int (group)));
     }
   if (rv == -1)
     SCM_SYSERROR;
@@ -207,20 +263,18 @@ SCM_DEFINE (scm_chmod, "chmod", 2, 0, 0,
 
   object = SCM_COERCE_OUTPORT (object);
 
-  SCM_VALIDATE_INUM (2,mode);
-  if (SCM_INUMP (object) || SCM_OPFPORTP (object))
+  if (scm_is_integer (object) || SCM_OPFPORTP (object))
     {
-      if (SCM_INUMP (object))
-	fdes = SCM_INUM (object);
+      if (scm_is_integer (object))
+	fdes = scm_to_int (object);
       else
 	fdes = SCM_FPORT_FDES (object);
-      SCM_SYSCALL (rv = fchmod (fdes, SCM_INUM (mode)));
+      SCM_SYSCALL (rv = fchmod (fdes, scm_to_int (mode)));
     }
   else
     {
-      SCM_VALIDATE_STRING (1, object);
-      SCM_STRING_COERCE_0TERMINATION_X (object);
-      SCM_SYSCALL (rv = chmod (SCM_STRING_CHARS (object), SCM_INUM (mode)));
+      STRING_SYSCALL (object, c_object,
+		      rv = chmod (c_object, scm_to_int (mode)));
     }
   if (rv == -1)
     SCM_SYSERROR;
@@ -244,10 +298,9 @@ SCM_DEFINE (scm_umask, "umask", 0, 1, 0,
     }
   else
     {
-      SCM_VALIDATE_INUM (1,mode);
-      mask = umask (SCM_INUM (mode));
+      mask = umask (scm_to_uint (mode));
     }
-  return SCM_MAKINUM (mask);
+  return scm_from_uint (mask);
 }
 #undef FUNC_NAME
 
@@ -263,14 +316,12 @@ SCM_DEFINE (scm_open_fdes, "open-fdes", 2, 1, 0,
   int iflags;
   int imode;
 
-  SCM_VALIDATE_STRING (1, path);
-  SCM_STRING_COERCE_0TERMINATION_X (path);
   iflags = SCM_NUM2INT (2, flags);
   imode = SCM_NUM2INT_DEF (3, mode, 0666);
-  SCM_SYSCALL (fd = open (SCM_STRING_CHARS (path), iflags, imode));
+  STRING_SYSCALL (path, c_path, fd = open (c_path, iflags, imode));
   if (fd == -1)
     SCM_SYSERROR;
-  return SCM_MAKINUM (fd);
+  return scm_from_int (fd);
 }
 #undef FUNC_NAME
 
@@ -307,7 +358,7 @@ SCM_DEFINE (scm_open, "open", 2, 1, 0,
   int fd;
   int iflags;
 
-  fd = SCM_INUM (scm_open_fdes (path, flags, mode));
+  fd = scm_to_int (scm_open_fdes (path, flags, mode));
   iflags = SCM_NUM2INT (2, flags);
   if (iflags & O_RDWR)
     {
@@ -347,15 +398,14 @@ SCM_DEFINE (scm_close, "close", 1, 0, 0,
 
   if (SCM_PORTP (fd_or_port))
     return scm_close_port (fd_or_port);
-  SCM_VALIDATE_INUM (1,fd_or_port);
-  fd = SCM_INUM (fd_or_port);
+  fd = scm_to_int (fd_or_port);
   scm_evict_ports (fd);		/* see scsh manual.  */
   SCM_SYSCALL (rv = close (fd));
   /* following scsh, closing an already closed file descriptor is
      not an error.  */
   if (rv < 0 && errno != EBADF)
     SCM_SYSERROR;
-  return SCM_BOOL (rv >= 0);
+  return scm_from_bool (rv >= 0);
 }
 #undef FUNC_NAME
 
@@ -371,7 +421,7 @@ SCM_DEFINE (scm_close_fdes, "close-fdes", 1, 0, 0,
   int c_fd;
   int rv;
 
-  SCM_VALIDATE_INUM_COPY (1, fd, c_fd);
+  c_fd = scm_to_int (fd);
   SCM_SYSCALL (rv = close (c_fd));
   if (rv < 0)
     SCM_SYSERROR;
@@ -398,58 +448,57 @@ static SCM
 scm_stat2scm (struct stat *stat_temp)
 {
   SCM ans = scm_c_make_vector (15, SCM_UNSPECIFIED);
-  SCM *ve = SCM_VELTS (ans);
   
-  ve[0] = scm_ulong2num ((unsigned long) stat_temp->st_dev);
-  ve[1] = scm_ulong2num ((unsigned long) stat_temp->st_ino);
-  ve[2] = scm_ulong2num ((unsigned long) stat_temp->st_mode);
-  ve[3] = scm_ulong2num ((unsigned long) stat_temp->st_nlink);
-  ve[4] = scm_ulong2num ((unsigned long) stat_temp->st_uid);
-  ve[5] = scm_ulong2num ((unsigned long) stat_temp->st_gid);
+  SCM_VECTOR_SET(ans, 0, scm_from_ulong (stat_temp->st_dev));
+  SCM_VECTOR_SET(ans, 1, scm_from_ulong (stat_temp->st_ino));
+  SCM_VECTOR_SET(ans, 2, scm_from_ulong (stat_temp->st_mode));
+  SCM_VECTOR_SET(ans, 3, scm_from_ulong (stat_temp->st_nlink));
+  SCM_VECTOR_SET(ans, 4, scm_from_ulong (stat_temp->st_uid));
+  SCM_VECTOR_SET(ans, 5, scm_from_ulong (stat_temp->st_gid));
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
-  ve[6] = scm_ulong2num ((unsigned long) stat_temp->st_rdev);
+  SCM_VECTOR_SET(ans, 6, scm_from_ulong (stat_temp->st_rdev));
 #else
-  ve[6] = SCM_BOOL_F;
+  SCM_VECTOR_SET(ans, 6, SCM_BOOL_F);
 #endif
-  ve[7] = scm_ulong2num ((unsigned long) stat_temp->st_size);
-  ve[8] = scm_ulong2num ((unsigned long) stat_temp->st_atime);
-  ve[9] = scm_ulong2num ((unsigned long) stat_temp->st_mtime);
-  ve[10] = scm_ulong2num ((unsigned long) stat_temp->st_ctime);
+  SCM_VECTOR_SET(ans, 7, scm_from_ulong (stat_temp->st_size));
+  SCM_VECTOR_SET(ans, 8, scm_from_ulong (stat_temp->st_atime));
+  SCM_VECTOR_SET(ans, 9, scm_from_ulong (stat_temp->st_mtime));
+  SCM_VECTOR_SET(ans, 10, scm_from_ulong (stat_temp->st_ctime));
 #ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-  ve[11] = scm_ulong2num ((unsigned long) stat_temp->st_blksize);
+  SCM_VECTOR_SET(ans, 11, scm_from_ulong (stat_temp->st_blksize));
 #else
-  ve[11] = scm_ulong2num (4096L);
+  SCM_VECTOR_SET(ans, 11, scm_from_ulong (4096L));
 #endif
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-  ve[12] = scm_ulong2num ((unsigned long) stat_temp->st_blocks);
+  SCM_VECTOR_SET(ans, 12, scm_from_ulong (stat_temp->st_blocks));
 #else
-  ve[12] = SCM_BOOL_F;
+  SCM_VECTOR_SET(ans, 12, SCM_BOOL_F);
 #endif
   {
     int mode = stat_temp->st_mode;
     
     if (S_ISREG (mode))
-      ve[13] = scm_sym_regular;
+      SCM_VECTOR_SET(ans, 13, scm_sym_regular);
     else if (S_ISDIR (mode))
-      ve[13] = scm_sym_directory;
+      SCM_VECTOR_SET(ans, 13, scm_sym_directory);
 #ifdef HAVE_S_ISLNK
     else if (S_ISLNK (mode))
-      ve[13] = scm_sym_symlink;
+      SCM_VECTOR_SET(ans, 13, scm_sym_symlink);
 #endif
     else if (S_ISBLK (mode))
-      ve[13] = scm_sym_block_special;
+      SCM_VECTOR_SET(ans, 13, scm_sym_block_special);
     else if (S_ISCHR (mode))
-      ve[13] = scm_sym_char_special;
+      SCM_VECTOR_SET(ans, 13, scm_sym_char_special);
     else if (S_ISFIFO (mode))
-      ve[13] = scm_sym_fifo;
+      SCM_VECTOR_SET(ans, 13, scm_sym_fifo);
 #ifdef S_ISSOCK
     else if (S_ISSOCK (mode))
-      ve[13] = scm_sym_sock;
+      SCM_VECTOR_SET(ans, 13, scm_sym_sock);
 #endif
     else
-      ve[13] = scm_sym_unknown;
+      SCM_VECTOR_SET(ans, 13, scm_sym_unknown);
 
-    ve[14] = SCM_MAKINUM ((~S_IFMT) & mode);
+    SCM_VECTOR_SET(ans, 14, scm_from_int ((~S_IFMT) & mode));
 
     /* the layout of the bits in ve[14] is intended to be portable.
        If there are systems that don't follow the usual convention,
@@ -478,7 +527,7 @@ scm_stat2scm (struct stat *stat_temp)
        tmp <<= 1;
        if (S_IXOTH & mode) tmp += 1; 
 
-       ve[14] = SCM_MAKINUM (tmp);
+       SCM_VECTOR_SET(ans, 14, scm_from_int (tmp));
        
        */
   }  
@@ -486,6 +535,31 @@ scm_stat2scm (struct stat *stat_temp)
   return ans;
 }
 
+#ifdef __MINGW32__
+/*
+ * Try getting the appropiate stat buffer for a given file descriptor
+ * under Windows. It differentiates between file, pipe and socket 
+ * descriptors.
+ */
+static int fstat_Win32 (int fdes, struct stat *buf)
+{
+  int error, optlen = sizeof (int);
+
+  memset (buf, 0, sizeof (struct stat));
+
+  /* Is this a socket ? */
+  if (getsockopt (fdes, SOL_SOCKET, SO_ERROR, (void *) &error, &optlen) >= 0)
+    {
+      buf->st_mode = _S_IFSOCK | _S_IREAD | _S_IWRITE | _S_IEXEC;
+      buf->st_nlink = 1;
+      buf->st_atime = buf->st_ctime = buf->st_mtime = time (NULL);
+      return 0;
+    }
+  /* Maybe a regular file or pipe ? */
+  return fstat (fdes, buf);
+}
+#endif /* __MINGW32__ */
+
 SCM_DEFINE (scm_stat, "stat", 1, 0, 0, 
             (SCM object),
 	    "Return an object containing various information about the file\n"
@@ -550,21 +624,36 @@ SCM_DEFINE (scm_stat, "stat", 1, 0, 0,
   int fdes;
   struct stat stat_temp;
 
-  if (SCM_INUMP (object))
+  if (scm_is_integer (object))
     {
-      SCM_SYSCALL (rv = fstat (SCM_INUM (object), &stat_temp));
+#ifdef __MINGW32__
+      SCM_SYSCALL (rv = fstat_Win32 (scm_to_int (object), &stat_temp));
+#else
+      SCM_SYSCALL (rv = fstat (scm_to_int (object), &stat_temp));
+#endif
     }
-  else if (SCM_STRINGP (object))
+  else if (scm_is_string (object))
     {
-      SCM_STRING_COERCE_0TERMINATION_X (object);
-      SCM_SYSCALL (rv = stat (SCM_STRING_CHARS (object), &stat_temp));
+      char *file = scm_to_locale_string (object);
+#ifdef __MINGW32__
+      char *p;
+      p = file + strlen (file) - 1;
+      while (p > file && (*p == '/' || *p == '\\'))
+	*p-- = '\0';
+#endif
+      SCM_SYSCALL (rv = stat (file, &stat_temp));
+      free (file);
     }
   else
     {
       object = SCM_COERCE_OUTPORT (object);
       SCM_VALIDATE_OPFPORT (1, object);
       fdes = SCM_FPORT_FDES (object);
+#ifdef __MINGW32__
+      SCM_SYSCALL (rv = fstat_Win32 (fdes, &stat_temp));
+#else
       SCM_SYSCALL (rv = fstat (fdes, &stat_temp));
+#endif
     }
 
   if (rv == -1)
@@ -572,7 +661,7 @@ SCM_DEFINE (scm_stat, "stat", 1, 0, 0,
       int en = errno;
 
       SCM_SYSERROR_MSG ("~A: ~S",
-			scm_list_2 (scm_makfrom0str (strerror (errno)),
+			scm_list_2 (scm_strerror (scm_from_int (en)),
 				    object),
 			en);
     }
@@ -595,12 +684,9 @@ SCM_DEFINE (scm_link, "link", 2, 0, 0,
 {
   int val;
 
-  SCM_VALIDATE_STRING (1, oldpath);
-  SCM_STRING_COERCE_0TERMINATION_X (oldpath);
-  SCM_VALIDATE_STRING (2, newpath);
-  SCM_STRING_COERCE_0TERMINATION_X (newpath);
-  SCM_SYSCALL (val = link (SCM_STRING_CHARS (oldpath),
-			   SCM_STRING_CHARS (newpath)));
+  STRING2_SYSCALL (oldpath, c_oldpath,
+		   newpath, c_newpath,
+		   val = link (c_oldpath, c_newpath));
   if (val != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -608,7 +694,25 @@ SCM_DEFINE (scm_link, "link", 2, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_LINK */
 
+#ifdef HAVE_RENAME
+#define my_rename rename
+#else
+static int
+my_rename (const char *oldname, const char *newname)
+{
+  int rv;
 
+  SCM_SYSCALL (rv = link (oldname, newname));
+  if (rv == 0)
+    {
+      SCM_SYSCALL (rv = unlink (oldname));
+      if (rv != 0)
+	/* unlink failed.  remove new name */
+	SCM_SYSCALL (unlink (newname)); 
+    }
+  return rv;
+}
+#endif
 
 SCM_DEFINE (scm_rename, "rename-file", 2, 0, 0,
             (SCM oldname, SCM newname),
@@ -617,22 +721,10 @@ SCM_DEFINE (scm_rename, "rename-file", 2, 0, 0,
 #define FUNC_NAME s_scm_rename
 {
   int rv;
-  SCM_VALIDATE_STRING (1, oldname);
-  SCM_VALIDATE_STRING (2, newname);
-  SCM_STRING_COERCE_0TERMINATION_X (oldname);
-  SCM_STRING_COERCE_0TERMINATION_X (newname);
-#ifdef HAVE_RENAME
-  SCM_SYSCALL (rv = rename (SCM_STRING_CHARS (oldname), SCM_STRING_CHARS (newname)));
-#else
-  SCM_SYSCALL (rv = link (SCM_STRING_CHARS (oldname), SCM_STRING_CHARS (newname)));
-  if (rv == 0)
-    {
-      SCM_SYSCALL (rv = unlink (SCM_STRING_CHARS (oldname)));;
-      if (rv != 0)
-	/* unlink failed.  remove new name */
-	SCM_SYSCALL (unlink (SCM_STRING_CHARS (newname))); 
-    }
-#endif
+
+  STRING2_SYSCALL (oldname, c_oldname,
+		   newname, c_newname,
+		   rv = my_rename (c_oldname, c_newname));
   if (rv != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -646,9 +738,7 @@ SCM_DEFINE (scm_delete_file, "delete-file", 1, 0, 0,
 #define FUNC_NAME s_scm_delete_file
 {
   int ans;
-  SCM_VALIDATE_STRING (1, str);
-  SCM_STRING_COERCE_0TERMINATION_X (str);
-  SCM_SYSCALL (ans = unlink (SCM_STRING_CHARS (str)));
+  STRING_SYSCALL (str, c_str, ans = unlink (c_str));
   if (ans != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -666,18 +756,16 @@ SCM_DEFINE (scm_mkdir, "mkdir", 1, 1, 0,
 {
   int rv;
   mode_t mask;
-  SCM_VALIDATE_STRING (1, path);
-  SCM_STRING_COERCE_0TERMINATION_X (path);
+
   if (SCM_UNBNDP (mode))
     {
       mask = umask (0);
       umask (mask);
-      SCM_SYSCALL (rv = mkdir (SCM_STRING_CHARS (path), 0777 ^ mask));
+      STRING_SYSCALL (path, c_path, rv = mkdir (c_path, 0777 ^ mask));
     }
   else
     {
-      SCM_VALIDATE_INUM (2,mode);
-      SCM_SYSCALL (rv = mkdir (SCM_STRING_CHARS (path), SCM_INUM (mode)));
+      STRING_SYSCALL (path, c_path, rv = mkdir (c_path, scm_to_uint (mode)));
     }
   if (rv != 0)
     SCM_SYSERROR;
@@ -695,9 +783,7 @@ SCM_DEFINE (scm_rmdir, "rmdir", 1, 0, 0,
 {
   int val;
 
-  SCM_VALIDATE_STRING (1, path);
-  SCM_STRING_COERCE_0TERMINATION_X (path);
-  SCM_SYSCALL (val = rmdir (SCM_STRING_CHARS (path)));
+  STRING_SYSCALL (path, c_path, val = rmdir (c_path));
   if (val != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -719,7 +805,7 @@ SCM_DEFINE (scm_directory_stream_p, "directory-stream?", 1, 0, 0,
 	    "stream as returned by @code{opendir}.")
 #define FUNC_NAME s_scm_directory_stream_p
 {
-  return SCM_BOOL (SCM_DIRP (obj));
+  return scm_from_bool (SCM_DIRP (obj));
 }
 #undef FUNC_NAME
 
@@ -731,9 +817,7 @@ SCM_DEFINE (scm_opendir, "opendir", 1, 0, 0,
 #define FUNC_NAME s_scm_opendir
 {
   DIR *ds;
-  SCM_VALIDATE_STRING (1, dirname);
-  SCM_STRING_COERCE_0TERMINATION_X (dirname);
-  SCM_SYSCALL (ds = opendir (SCM_STRING_CHARS (dirname)));
+  STRING_SYSCALL (dirname, c_dirname, ds = opendir (c_dirname));
   if (ds == NULL)
     SCM_SYSERROR;
   SCM_RETURN_NEWSMOB (scm_tc16_dir | SCM_DIR_FLAG_OPEN, ds);
@@ -741,6 +825,11 @@ SCM_DEFINE (scm_opendir, "opendir", 1, 0, 0,
 #undef FUNC_NAME
 
 
+/* FIXME: The glibc manual has a portability note that readdir_r may not
+   null-terminate its return string.  The circumstances outlined for this
+   are not clear, nor is it clear what should be done about it.  Lets worry
+   about this if/when someone can figure it out.  */
+
 SCM_DEFINE (scm_readdir, "readdir", 1, 0, 0, 
             (SCM port),
 	    "Return (as a string) the next directory entry from the directory stream\n"
@@ -755,12 +844,28 @@ SCM_DEFINE (scm_readdir, "readdir", 1, 0, 0,
     SCM_MISC_ERROR ("Directory ~S is not open.", scm_list_1 (port));
 
   errno = 0;
-  SCM_SYSCALL (rdent = readdir ((DIR *) SCM_CELL_WORD_1 (port)));
-  if (errno != 0)
-    SCM_SYSERROR;
+  {
+#if HAVE_READDIR_R
+    /* On Solaris 2.7, struct dirent only contains "char d_name[1]" and one is
+       expected to provide a buffer of "sizeof(struct dirent) + NAME_MAX"
+       bytes.  The glibc 2.3.2 manual notes this sort of thing too, and
+       advises "offsetof(struct dirent,d_name) + NAME_MAX + 1".  Either should
+       suffice, we give both to be certain.  */
+    union {
+      struct dirent ent;
+      char pad1 [sizeof(struct dirent) + NAME_MAX];
+      char pad2 [offsetof (struct dirent, d_name) + NAME_MAX + 1];
+    } u;
+    SCM_SYSCALL (readdir_r ((DIR *) SCM_CELL_WORD_1 (port), &u.ent, &rdent));
+#else
+    SCM_SYSCALL (rdent = readdir ((DIR *) SCM_CELL_WORD_1 (port)));
+#endif
+    if (errno != 0)
+      SCM_SYSERROR;
 
-  return (rdent ? scm_mem2string (rdent->d_name, NAMLEN (rdent))
-	  : SCM_EOF_VAL);
+    return (rdent ? scm_from_locale_stringn (rdent->d_name, NAMLEN (rdent))
+            : SCM_EOF_VAL);
+  }
 }
 #undef FUNC_NAME
 
@@ -840,9 +945,7 @@ SCM_DEFINE (scm_chdir, "chdir", 1, 0, 0,
 {
   int ans;
 
-  SCM_VALIDATE_STRING (1, str);
-  SCM_STRING_COERCE_0TERMINATION_X (str);
-  SCM_SYSCALL (ans = chdir (SCM_STRING_CHARS (str)));
+  STRING_SYSCALL (str, c_str, ans = chdir (c_str));
   if (ans != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -860,12 +963,12 @@ SCM_DEFINE (scm_getcwd, "getcwd", 0, 0, 0,
   char *wd;
   SCM result;
 
-  wd = scm_must_malloc (size, FUNC_NAME);
+  wd = scm_malloc (size);
   while ((rv = getcwd (wd, size)) == 0 && errno == ERANGE)
     {
-      scm_must_free (wd);
+      free (wd);
       size *= 2;
-      wd = scm_must_malloc (size, FUNC_NAME);
+      wd = scm_malloc (size);
     }
   if (rv == 0)
     {
@@ -874,8 +977,8 @@ SCM_DEFINE (scm_getcwd, "getcwd", 0, 0, 0,
       errno = save_errno;
       SCM_SYSERROR;
     }
-  result = scm_mem2string (wd, strlen (wd));
-  scm_must_free (wd);
+  result = scm_from_locale_stringn (wd, strlen (wd));
+  free (wd);
   return result;
 }
 #undef FUNC_NAME
@@ -895,9 +998,9 @@ set_element (SELECT_TYPE *set, SCM *ports_ready, SCM element, int pos)
 {
   int fd;
 
-  if (SCM_INUMP (element))
+  if (scm_is_integer (element))
     {
-      fd = SCM_INUM (element);
+      fd = scm_to_int (element);
     }
   else
     {
@@ -944,7 +1047,7 @@ fill_select_type (SELECT_TYPE *set, SCM *ports_ready, SCM list_or_vec, int pos)
   if (SCM_VECTORP (list_or_vec))
     {
       int i = SCM_VECTOR_LENGTH (list_or_vec);
-      SCM *ve = SCM_VELTS (list_or_vec);
+      SCM const *ve = SCM_VELTS (list_or_vec);
       
       while (--i >= 0)
 	{
@@ -956,7 +1059,7 @@ fill_select_type (SELECT_TYPE *set, SCM *ports_ready, SCM list_or_vec, int pos)
     }
   else
     {
-      while (!SCM_NULLP (list_or_vec))
+      while (!SCM_NULL_OR_NIL_P (list_or_vec))
 	{
 	  int fd = set_element (set, ports_ready, SCM_CAR (list_or_vec), pos);
 
@@ -976,9 +1079,9 @@ get_element (SELECT_TYPE *set, SCM element, SCM list)
 {
   int fd;
 
-  if (SCM_INUMP (element))
+  if (scm_is_integer (element))
     {
-      fd = SCM_INUM (element);
+      fd = scm_to_int (element);
     }
   else
     {
@@ -1005,7 +1108,7 @@ retrieve_select_type (SELECT_TYPE *set, SCM ports_ready, SCM list_or_vec)
   if (SCM_VECTORP (list_or_vec))
     {
       int i = SCM_VECTOR_LENGTH (list_or_vec);
-      SCM *ve = SCM_VELTS (list_or_vec);
+      SCM const  *ve = SCM_VELTS (list_or_vec);
 
       while (--i >= 0)
 	{
@@ -1016,7 +1119,7 @@ retrieve_select_type (SELECT_TYPE *set, SCM ports_ready, SCM list_or_vec)
   else
     {
       /* list_or_vec must be a list.  */
-      while (!SCM_NULLP (list_or_vec))
+      while (!SCM_NULL_OR_NIL_P (list_or_vec))
 	{
 	  answer_list = get_element (set, SCM_CAR (list_or_vec), answer_list);
 	  list_or_vec = SCM_CDR (list_or_vec);
@@ -1124,24 +1227,21 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
       timeout.tv_usec = 0;
       time_ptr = &timeout;
     }
-  else if (SCM_UNBNDP (secs) || SCM_FALSEP (secs))
+  else if (SCM_UNBNDP (secs) || scm_is_false (secs))
     time_ptr = 0;
   else
     {
-      if (SCM_INUMP (secs))
+      if (scm_is_unsigned_integer (secs, 0, ULONG_MAX))
 	{
-	  timeout.tv_sec = SCM_INUM (secs);
+	  timeout.tv_sec = scm_to_ulong (secs);
 	  if (SCM_UNBNDP (usecs))
 	    timeout.tv_usec = 0;
 	  else
-	    {
-              SCM_VALIDATE_INUM (5,usecs);
-	      timeout.tv_usec = SCM_INUM (usecs);
-	    }
+	    timeout.tv_usec = scm_to_long (usecs);
 	}
       else
 	{
-	  double fl = scm_num2dbl (secs, FUNC_NAME);
+	  double fl = scm_to_double (secs);
 
 	  if (!SCM_UNBNDP (usecs))
 	    SCM_WRONG_TYPE_ARG (4, secs);
@@ -1154,14 +1254,9 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
     }
 
   {
-#ifdef GUILE_ISELECT
     int rv = scm_internal_select (max_fd + 1,
 				  &read_set, &write_set, &except_set,
 				  time_ptr);
-#else
-    int rv = select (max_fd + 1,
-		     &read_set, &write_set, &except_set, time_ptr);
-#endif
     if (rv < 0)
       SCM_SYSERROR;
   }
@@ -1208,25 +1303,20 @@ SCM_DEFINE (scm_fcntl, "fcntl", 2, 1, 0,
 
   object = SCM_COERCE_OUTPORT (object);
 
-  SCM_VALIDATE_INUM (2,cmd);
   if (SCM_OPFPORTP (object))
     fdes = SCM_FPORT_FDES (object);
   else
-    {
-      SCM_VALIDATE_INUM (1,object);
-      fdes = SCM_INUM (object);
-    }
+    fdes = scm_to_int (object);
 
-  if (SCM_UNBNDP (value)) {
+  if (SCM_UNBNDP (value))
     ivalue = 0;
-  } else {
-    SCM_VALIDATE_INUM_COPY (SCM_ARG3, value, ivalue);
-  }
+  else
+    ivalue = scm_to_int (value);
 
-  SCM_SYSCALL (rv = fcntl (fdes, SCM_INUM (cmd), ivalue));
+  SCM_SYSCALL (rv = fcntl (fdes, scm_to_int (cmd), ivalue));
   if (rv == -1)
     SCM_SYSERROR;
-  return SCM_MAKINUM (rv);
+  return scm_from_int (rv);
 }
 #undef FUNC_NAME
 #endif /* HAVE_FCNTL */
@@ -1249,10 +1339,8 @@ SCM_DEFINE (scm_fsync, "fsync", 1, 0, 0,
       fdes = SCM_FPORT_FDES (object);
     }
   else
-    {
-      SCM_VALIDATE_INUM (1,object);
-      fdes = SCM_INUM (object);
-    }
+    fdes = scm_to_int (object);
+
   if (fsync (fdes) == -1)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -1268,11 +1356,9 @@ SCM_DEFINE (scm_symlink, "symlink", 2, 0, 0,
 {
   int val;
 
-  SCM_VALIDATE_STRING (1, oldpath);
-  SCM_VALIDATE_STRING (2, newpath);
-  SCM_STRING_COERCE_0TERMINATION_X (oldpath);
-  SCM_STRING_COERCE_0TERMINATION_X (newpath);
-  SCM_SYSCALL (val = symlink (SCM_STRING_CHARS (oldpath), SCM_STRING_CHARS (newpath)));
+  STRING2_SYSCALL (oldpath, c_oldpath,
+		   newpath, c_newpath,
+		   val = symlink (c_oldpath, c_newpath));
   if (val != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -1291,14 +1377,20 @@ SCM_DEFINE (scm_readlink, "readlink", 1, 0, 0,
   int size = 100;
   char *buf;
   SCM result;
-  SCM_VALIDATE_STRING (1, path);
-  SCM_STRING_COERCE_0TERMINATION_X (path);
-  buf = scm_must_malloc (size, FUNC_NAME);
-  while ((rv = readlink (SCM_STRING_CHARS (path), buf, size)) == size)
+  char *c_path;
+  
+  scm_frame_begin (0);
+
+  c_path = scm_to_locale_string (path);
+  scm_frame_free (c_path);
+
+  buf = scm_malloc (size);
+
+  while ((rv = readlink (c_path, buf, size)) == size)
     {
-      scm_must_free (buf);
+      free (buf);
       size *= 2;
-      buf = scm_must_malloc (size, FUNC_NAME);
+      buf = scm_malloc (size);
     }
   if (rv == -1)
     {
@@ -1307,8 +1399,9 @@ SCM_DEFINE (scm_readlink, "readlink", 1, 0, 0,
       errno = save_errno;
       SCM_SYSERROR;
     }
-  result = scm_mem2string (buf, rv);
-  scm_must_free (buf);
+  result = scm_take_locale_stringn (buf, rv);
+
+  scm_frame_end ();
   return result;
 }
 #undef FUNC_NAME
@@ -1325,18 +1418,16 @@ SCM_DEFINE (scm_lstat, "lstat", 1, 0, 0,
   int rv;
   struct stat stat_temp;
 
-  SCM_VALIDATE_STRING (1, str);
-  SCM_STRING_COERCE_0TERMINATION_X (str);
-  SCM_SYSCALL (rv = lstat (SCM_STRING_CHARS (str), &stat_temp));
+  STRING_SYSCALL (str, c_str, rv = lstat (c_str, &stat_temp));
   if (rv != 0)
     {
       int en = errno;
 
       SCM_SYSERROR_MSG ("~A: ~S",
-			scm_list_2 (scm_makfrom0str (strerror (errno)), str),
+			scm_list_2 (scm_strerror (scm_from_int (en)), str),
 			en);
     }
-  return scm_stat2scm(&stat_temp);
+  return scm_stat2scm (&stat_temp);
 }
 #undef FUNC_NAME
 #endif /* HAVE_LSTAT */
@@ -1347,26 +1438,33 @@ SCM_DEFINE (scm_copy_file, "copy-file", 2, 0, 0,
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_copy_file
 {
+  char *c_oldfile, *c_newfile;
   int oldfd, newfd;
   int n, rv;
   char buf[BUFSIZ];
   struct stat oldstat;
 
-  SCM_VALIDATE_STRING (1, oldfile);
-  SCM_STRING_COERCE_0TERMINATION_X (oldfile);
-  SCM_VALIDATE_STRING (2, newfile);
-  SCM_STRING_COERCE_0TERMINATION_X (newfile);
+  scm_frame_begin (0);
+  
+  c_oldfile = scm_to_locale_string (oldfile);
+  scm_frame_free (c_oldfile);
+  c_newfile = scm_to_locale_string (newfile);
+  scm_frame_free (c_newfile);
 
-  oldfd = open (SCM_STRING_CHARS (oldfile), O_RDONLY);
+  oldfd = open (c_oldfile, O_RDONLY);
   if (oldfd == -1)
     SCM_SYSERROR;
 
+#ifdef __MINGW32__
+  SCM_SYSCALL (rv = fstat_Win32 (oldfd, &oldstat));
+#else
   SCM_SYSCALL (rv = fstat (oldfd, &oldstat));
+#endif
   if (rv == -1)
     goto err_close_oldfd;
 
   /* use POSIX flags instead of 07777?.  */
-  newfd = open (SCM_STRING_CHARS (newfile), O_WRONLY | O_CREAT | O_TRUNC,
+  newfd = open (c_newfile, O_WRONLY | O_CREAT | O_TRUNC,
 		oldstat.st_mode & 07777);
   if (newfd == -1)
     {
@@ -1385,6 +1483,8 @@ SCM_DEFINE (scm_copy_file, "copy-file", 2, 0, 0,
   close (oldfd);
   if (close (newfd) == -1)
     SCM_SYSERROR;
+
+  scm_frame_end ();
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1401,19 +1501,19 @@ SCM_DEFINE (scm_dirname, "dirname", 1, 0, 0,
 	    "component, @code{.} is returned.")
 #define FUNC_NAME s_scm_dirname
 {
-  char *s;
+  const char *s;
   long int i;
   unsigned long int len;
 
-  SCM_VALIDATE_STRING (1,filename);
+  SCM_VALIDATE_STRING (1, filename);
 
-  s = SCM_STRING_CHARS (filename);
-  len = SCM_STRING_LENGTH (filename);
+  s = scm_i_string_chars (filename);
+  len = scm_i_string_length (filename);
 
   i = len - 1;
 #ifdef __MINGW32__
   while (i >= 0 && (s[i] == '/' || s[i] == '\\')) --i;
-  while (i >= 0 && (s[i] != '/' || s[i] != '\\')) --i;
+  while (i >= 0 && (s[i] != '/' && s[i] != '\\')) --i;
   while (i >= 0 && (s[i] == '/' || s[i] == '\\')) --i;
 #else
   while (i >= 0 && s[i] == '/') --i;
@@ -1427,12 +1527,12 @@ SCM_DEFINE (scm_dirname, "dirname", 1, 0, 0,
 #else
       if (len > 0 && s[0] == '/')
 #endif /* ndef __MINGW32__ */
-	return scm_substring (filename, SCM_INUM0, SCM_MAKINUM (1));
+	return scm_c_substring (filename, 0, 1);
       else
 	return scm_dot_string;
     }
   else
-    return scm_substring (filename, SCM_INUM0, SCM_MAKINUM (i + 1));
+    return scm_c_substring (filename, 0, i + 1);
 }
 #undef FUNC_NAME
 
@@ -1444,20 +1544,20 @@ SCM_DEFINE (scm_basename, "basename", 1, 1, 0,
 	    "@var{basename}, it is removed also.")
 #define FUNC_NAME s_scm_basename
 {
-  char *f, *s = 0;
+  const char *f, *s = 0;
   int i, j, len, end;
 
-  SCM_VALIDATE_STRING (1,filename);
-  f = SCM_STRING_CHARS (filename);
-  len = SCM_STRING_LENGTH (filename);
+  SCM_VALIDATE_STRING (1, filename);
+  f = scm_i_string_chars (filename);
+  len = scm_i_string_length (filename);
 
   if (SCM_UNBNDP (suffix))
     j = -1;
   else
     {
       SCM_VALIDATE_STRING (2, suffix);
-      s = SCM_STRING_CHARS (suffix);
-      j = SCM_STRING_LENGTH (suffix) - 1;
+      s = scm_i_string_chars (suffix);
+      j = scm_i_string_length (suffix) - 1;
     }
   i = len - 1;
 #ifdef __MINGW32__
@@ -1470,23 +1570,23 @@ SCM_DEFINE (scm_basename, "basename", 1, 1, 0,
   if (j == -1)
     end = i;
 #ifdef __MINGW32__
-  while (i >= 0 && (f[i] != '/' || f[i] != '\\')) --i;
+  while (i >= 0 && f[i] != '/' && f[i] != '\\') --i;
 #else
   while (i >= 0 && f[i] != '/') --i;
 #endif /* ndef __MINGW32__ */
   if (i == end)
     {
 #ifdef __MINGW32__
-      if (len > 0 && (f[0] == '/' || f[i] == '\\'))
+      if (len > 0 && (f[0] == '/' || f[0] == '\\'))
 #else
       if (len > 0 && f[0] == '/')
 #endif /* ndef __MINGW32__ */
-	return scm_substring (filename, SCM_INUM0, SCM_MAKINUM (1));
+	return scm_c_substring (filename, 0, 1);
       else
 	return scm_dot_string;
     }
   else
-    return scm_substring (filename, SCM_MAKINUM (i + 1), SCM_MAKINUM (end + 1));
+    return scm_c_substring (filename, i+1, end+1);
 }
 #undef FUNC_NAME
 
@@ -1501,65 +1601,65 @@ scm_init_filesys ()
   scm_set_smob_free (scm_tc16_dir, scm_dir_free);
   scm_set_smob_print (scm_tc16_dir, scm_dir_print);
 
-  scm_dot_string = scm_permanent_object (scm_makfrom0str ("."));
+  scm_dot_string = scm_permanent_object (scm_from_locale_string ("."));
   
 #ifdef O_RDONLY
-  scm_c_define ("O_RDONLY", scm_long2num (O_RDONLY));
+  scm_c_define ("O_RDONLY", scm_from_long (O_RDONLY));
 #endif 	       
 #ifdef O_WRONLY
-  scm_c_define ("O_WRONLY", scm_long2num (O_WRONLY));
+  scm_c_define ("O_WRONLY", scm_from_long (O_WRONLY));
 #endif 	       
 #ifdef O_RDWR
-  scm_c_define ("O_RDWR", scm_long2num (O_RDWR));
+  scm_c_define ("O_RDWR", scm_from_long (O_RDWR));
 #endif 	       
 #ifdef O_CREAT
-  scm_c_define ("O_CREAT", scm_long2num (O_CREAT));
+  scm_c_define ("O_CREAT", scm_from_long (O_CREAT));
 #endif 	       
 #ifdef O_EXCL  
-  scm_c_define ("O_EXCL", scm_long2num (O_EXCL));
+  scm_c_define ("O_EXCL", scm_from_long (O_EXCL));
 #endif 	       
 #ifdef O_NOCTTY
-  scm_c_define ("O_NOCTTY", scm_long2num (O_NOCTTY));
+  scm_c_define ("O_NOCTTY", scm_from_long (O_NOCTTY));
 #endif 	       
 #ifdef O_TRUNC 
-  scm_c_define ("O_TRUNC", scm_long2num (O_TRUNC));
+  scm_c_define ("O_TRUNC", scm_from_long (O_TRUNC));
 #endif 	       
 #ifdef O_APPEND
-  scm_c_define ("O_APPEND", scm_long2num (O_APPEND));
+  scm_c_define ("O_APPEND", scm_from_long (O_APPEND));
 #endif 	       
 #ifdef O_NONBLOCK
-  scm_c_define ("O_NONBLOCK", scm_long2num (O_NONBLOCK));
+  scm_c_define ("O_NONBLOCK", scm_from_long (O_NONBLOCK));
 #endif 	       
 #ifdef O_NDELAY
-  scm_c_define ("O_NDELAY", scm_long2num (O_NDELAY));
+  scm_c_define ("O_NDELAY", scm_from_long (O_NDELAY));
 #endif 	       
 #ifdef O_SYNC  
-  scm_c_define ("O_SYNC", scm_long2num (O_SYNC));
+  scm_c_define ("O_SYNC", scm_from_long (O_SYNC));
 #endif 
 
 #ifdef F_DUPFD  
-  scm_c_define ("F_DUPFD", scm_long2num (F_DUPFD));
+  scm_c_define ("F_DUPFD", scm_from_long (F_DUPFD));
 #endif 
 #ifdef F_GETFD  
-  scm_c_define ("F_GETFD", scm_long2num (F_GETFD));
+  scm_c_define ("F_GETFD", scm_from_long (F_GETFD));
 #endif 
 #ifdef F_SETFD  
-  scm_c_define ("F_SETFD", scm_long2num (F_SETFD));
+  scm_c_define ("F_SETFD", scm_from_long (F_SETFD));
 #endif 
 #ifdef F_GETFL  
-  scm_c_define ("F_GETFL", scm_long2num (F_GETFL));
+  scm_c_define ("F_GETFL", scm_from_long (F_GETFL));
 #endif 
 #ifdef F_SETFL  
-  scm_c_define ("F_SETFL", scm_long2num (F_SETFL));
+  scm_c_define ("F_SETFL", scm_from_long (F_SETFL));
 #endif 
 #ifdef F_GETOWN  
-  scm_c_define ("F_GETOWN", scm_long2num (F_GETOWN));
+  scm_c_define ("F_GETOWN", scm_from_long (F_GETOWN));
 #endif 
 #ifdef F_SETOWN  
-  scm_c_define ("F_SETOWN", scm_long2num (F_SETOWN));
+  scm_c_define ("F_SETOWN", scm_from_long (F_SETOWN));
 #endif 
 #ifdef FD_CLOEXEC  
-  scm_c_define ("FD_CLOEXEC", scm_long2num (FD_CLOEXEC));
+  scm_c_define ("FD_CLOEXEC", scm_from_long (FD_CLOEXEC));
 #endif
 
 #include "libguile/filesys.x"
