@@ -1,49 +1,25 @@
 /* GDB interface for Guile
- * Copyright (C) 1996,1997,1999,2000,2001 Free Software Foundation, Inc.
+ * Copyright (C) 1996,1997,1999,2000,2001,2002,2004
+ * Free Software Foundation, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
- *
- * The author can be reached at djurfeldt@nada.kth.se
- * Mikael Djurfeldt, SANS/NADA KTH, 10044 STOCKHOLM, SWEDEN */
-
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 
@@ -90,7 +66,6 @@
  */
 #define SCM_BEGIN_FOREIGN_BLOCK \
 do { \
-  old_ints = scm_ints_disabled; scm_ints_disabled = 1; \
   old_gc = scm_block_gc; scm_block_gc = 1; \
   scm_print_carefully_p = 1; \
 } while (0)
@@ -100,7 +75,6 @@ do { \
 do { \
   scm_print_carefully_p = 0; \
   scm_block_gc = old_gc; \
-  scm_ints_disabled = old_ints; \
 } while (0)
 
 
@@ -135,21 +109,21 @@ static SCM tok_buf;
 static int tok_buf_mark_p;
 
 static SCM gdb_output_port;
-static int old_ints, old_gc;
+static int old_gc;
 
 
 static void
 unmark_port (SCM port)
 {
   SCM stream, string;
-  port_mark_p = SCM_GCMARKP (port);
-  SCM_CLRGCMARK (port);
+  port_mark_p = SCM_GC_MARK_P (port);
+  SCM_CLEAR_GC_MARK (port);
   stream = SCM_PACK (SCM_STREAM (port));
-  stream_mark_p = SCM_GCMARKP (stream);
-  SCM_CLRGCMARK (stream);
+  stream_mark_p = SCM_GC_MARK_P (stream);
+  SCM_CLEAR_GC_MARK (stream);
   string = SCM_CDR (stream);
-  string_mark_p = SCM_GCMARKP (string);
-  SCM_CLRGCMARK (string);
+  string_mark_p = SCM_GC_MARK_P (string);
+  SCM_CLEAR_GC_MARK (string);
 }
 
 
@@ -158,16 +132,19 @@ remark_port (SCM port)
 {
   SCM stream = SCM_PACK (SCM_STREAM (port));
   SCM string = SCM_CDR (stream);
-  if (string_mark_p) SCM_SETGCMARK (string);
-  if (stream_mark_p) SCM_SETGCMARK (stream);
-  if (port_mark_p) SCM_SETGCMARK (port);
+  if (string_mark_p)
+    SCM_SET_GC_MARK (string);
+  if (stream_mark_p)
+    SCM_SET_GC_MARK (stream);
+  if (port_mark_p)
+    SCM_SET_GC_MARK (port);
 }
 
 
 int
 gdb_maybe_valid_type_p (SCM value)
 {
-  return SCM_IMP (value) || scm_cellp (value);
+  return SCM_IMP (value) || scm_in_heap_p (value);
 }
 
 
@@ -206,13 +183,13 @@ gdb_read (char *str)
     }
   SCM_BEGIN_FOREIGN_BLOCK;
   unmark_port (gdb_input_port);
-  scm_seek (gdb_input_port, SCM_INUM0, SCM_MAKINUM (SEEK_SET));
+  scm_seek (gdb_input_port, SCM_INUM0, scm_from_int (SEEK_SET));
   scm_puts (str, gdb_input_port);
   scm_truncate_file (gdb_input_port, SCM_UNDEFINED);
-  scm_seek (gdb_input_port, SCM_INUM0, SCM_MAKINUM (SEEK_SET));
+  scm_seek (gdb_input_port, SCM_INUM0, scm_from_int (SEEK_SET));
   /* Read one object */
-  tok_buf_mark_p = SCM_GCMARKP (tok_buf);
-  SCM_CLRGCMARK (tok_buf);
+  tok_buf_mark_p = SCM_GC_MARK_P (tok_buf);
+  SCM_CLEAR_GC_MARK (tok_buf);
   ans = scm_lreadr (&tok_buf, gdb_input_port, &ans);
   if (SCM_GC_P)
     {
@@ -229,7 +206,7 @@ gdb_read (char *str)
     scm_permanent_object (ans);
 exit:
   if (tok_buf_mark_p)
-    SCM_SETGCMARK (tok_buf);
+    SCM_SET_GC_MARK (tok_buf);
   remark_port (gdb_input_port);
   SCM_END_FOREIGN_BLOCK;
   return status;
@@ -240,11 +217,6 @@ int
 gdb_eval (SCM exp)
 {
   RESET_STRING;
-  if (SCM_IMP (exp))
-    {
-      gdb_result = exp;
-      return 0;
-    }
   if (SCM_GC_P)
     {
       SEND_STRING ("Can't evaluate lisp expressions during gc");
@@ -253,7 +225,7 @@ gdb_eval (SCM exp)
   SCM_BEGIN_FOREIGN_BLOCK;
   {
     SCM env = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
-    gdb_result = scm_permanent_object (scm_ceval (exp, env));
+    gdb_result = scm_permanent_object (scm_i_eval_x (exp, env));
   }
   SCM_END_FOREIGN_BLOCK;
   return 0;
@@ -270,7 +242,7 @@ gdb_print (SCM obj)
       RESET_STRING;
       SCM_BEGIN_FOREIGN_BLOCK;
       /* Reset stream */
-      scm_seek (gdb_output_port, SCM_INUM0, SCM_MAKINUM (SEEK_SET));
+      scm_seek (gdb_output_port, SCM_INUM0, scm_from_int (SEEK_SET));
       scm_write (obj, gdb_output_port);
       scm_truncate_file (gdb_output_port, SCM_UNDEFINED);
       {
@@ -313,18 +285,18 @@ scm_init_gdbint ()
   scm_print_carefully_p = 0;
   
   port = scm_mkstrport (SCM_INUM0,
-			scm_make_string (SCM_MAKINUM (0), SCM_UNDEFINED),
+			scm_c_make_string (0, SCM_UNDEFINED),
 			SCM_OPN | SCM_WRTNG,
 			s);
   gdb_output_port = scm_permanent_object (port);
   
   port = scm_mkstrport (SCM_INUM0,
-			scm_make_string (SCM_MAKINUM (0), SCM_UNDEFINED),
+			scm_c_make_string (0, SCM_UNDEFINED),
 			SCM_OPN | SCM_RDNG | SCM_WRTNG,
 			s);
   gdb_input_port = scm_permanent_object (port);
 
-  tok_buf = scm_permanent_object (scm_allocate_string (30));
+  tok_buf = scm_permanent_object (scm_c_make_string (30, SCM_UNDEFINED));
 }
 
 /*
