@@ -1,46 +1,26 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2004 Free Software Foundation, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 /* data initialization and C<->Scheme data conversion */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "libguile/gh.h"
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -51,27 +31,27 @@
 SCM 
 gh_bool2scm (int x)
 {
-  return SCM_BOOL(x);
+  return scm_from_bool(x);
 }
 SCM 
 gh_int2scm (int x)
 {
-  return scm_long2num ((long) x);
+  return scm_from_long ((long) x);
 }
 SCM 
 gh_ulong2scm (unsigned long x)
 {
-  return scm_ulong2num (x);
+  return scm_from_ulong (x);
 }
 SCM 
 gh_long2scm (long x)
 {
-  return scm_long2num (x);
+  return scm_from_long (x);
 }
 SCM 
 gh_double2scm (double x)
 {
-  return scm_make_real (x);
+  return scm_from_double (x);
 }
 SCM 
 gh_char2scm (char c)
@@ -81,12 +61,12 @@ gh_char2scm (char c)
 SCM 
 gh_str2scm (const char *s, size_t len)
 {
-  return scm_mem2string (s, len);
+  return scm_from_locale_stringn (s, len);
 }
 SCM 
 gh_str02scm (const char *s)
 {
-  return scm_makfrom0str (s);
+  return scm_from_locale_string (s);
 }
 /* Copy LEN characters at SRC into the *existing* Scheme string DST,
    starting at START.  START is an index into DST; zero means the
@@ -95,18 +75,19 @@ gh_str02scm (const char *s)
    If START + LEN is off the end of DST, signal an out-of-range
    error.  */
 void 
-gh_set_substr (char *src, SCM dst, long start, size_t len)
+gh_set_substr (const char *src, SCM dst, long start, size_t len)
 {
   char *dst_ptr;
   size_t dst_len;
 
-  SCM_ASSERT (SCM_STRINGP (dst), dst, SCM_ARG3, "gh_set_substr");
+  SCM_ASSERT (scm_is_string (dst), dst, SCM_ARG3, "gh_set_substr");
 
-  dst_ptr = SCM_STRING_CHARS (dst);
-  dst_len = SCM_STRING_LENGTH (dst);
+  dst_len = scm_i_string_length (dst);
   SCM_ASSERT (start + len <= dst_len, dst, SCM_ARG4, "gh_set_substr");
-  
+
+  dst_ptr = scm_i_string_writable_chars (dst);
   memmove (dst_ptr + start, src, len);
+  scm_i_string_stop_writing ();
   scm_remember_upto_here_1 (dst);
 }
 
@@ -114,7 +95,7 @@ gh_set_substr (char *src, SCM dst, long start, size_t len)
 SCM 
 gh_symbol2scm (const char *symbol_str)
 {
-  return scm_str2symbol(symbol_str);
+  return scm_from_locale_symbol(symbol_str);
 }
 
 SCM
@@ -122,10 +103,8 @@ gh_ints2scm (const int *d, long n)
 {
   long i;
   SCM v = scm_c_make_vector (n, SCM_UNSPECIFIED);
-  SCM *velts = SCM_VELTS(v);
-
   for (i = 0; i < n; ++i)
-    velts[i] = (SCM_FIXABLE (d[i]) ? SCM_MAKINUM (d[i]) : scm_i_long2big (d[i]));
+    SCM_VECTOR_SET (v, i, scm_from_int (d[i]));
 
   return v;
 }
@@ -135,33 +114,26 @@ gh_doubles2scm (const double *d, long n)
 {
   long i;
   SCM v = scm_c_make_vector (n, SCM_UNSPECIFIED);
-  SCM *velts = SCM_VELTS(v);
 
   for(i = 0; i < n; i++) 
-    velts[i] = scm_make_real (d[i]);
+    SCM_VECTOR_SET (v, i, scm_from_double (d[i]));
   return v;
 }
 
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
 /* Do not use this function for building normal Scheme vectors, unless
    you arrange for the elements to be protected from GC while you
    initialize the vector.  */
 static SCM
 makvect (char *m, size_t len, int type)
 {
-  SCM ans;
-  SCM_NEWCELL (ans);
-  SCM_DEFER_INTS;
-  SCM_SET_UVECTOR_BASE (ans, m);
-  SCM_SET_UVECTOR_LENGTH (ans, len, type);
-  SCM_ALLOW_INTS;
-  return ans;
+  return scm_cell (SCM_MAKE_UVECTOR_TAG (len, type), (scm_t_bits) m);
 }
 
 SCM
 gh_chars2byvect (const char *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (char), "vector");
+  char *m = scm_gc_malloc (n * sizeof (char), "vector");
   memcpy (m, d, n * sizeof (char));
   return makvect (m, n, scm_tc7_byvect);
 }
@@ -169,7 +141,7 @@ gh_chars2byvect (const char *d, long n)
 SCM
 gh_shorts2svect (const short *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (short), "vector");
+  char *m = scm_gc_malloc (n * sizeof (short), "vector");
   memcpy (m, d, n * sizeof (short));
   return makvect (m, n, scm_tc7_svect);
 }
@@ -177,7 +149,7 @@ gh_shorts2svect (const short *d, long n)
 SCM
 gh_longs2ivect (const long *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (long), "vector");
+  char *m = scm_gc_malloc (n * sizeof (long), "vector");
   memcpy (m, d, n * sizeof (long));
   return makvect (m, n, scm_tc7_ivect);
 }
@@ -185,7 +157,7 @@ gh_longs2ivect (const long *d, long n)
 SCM
 gh_ulongs2uvect (const unsigned long *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (unsigned long), "vector");
+  char *m = scm_gc_malloc (n * sizeof (unsigned long), "vector");
   memcpy (m, d, n * sizeof (unsigned long));
   return makvect (m, n, scm_tc7_uvect);
 }
@@ -193,7 +165,7 @@ gh_ulongs2uvect (const unsigned long *d, long n)
 SCM
 gh_floats2fvect (const float *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (float), "vector");
+  char *m = scm_gc_malloc (n * sizeof (float), "vector");
   memcpy (m, d, n * sizeof (float));
   return makvect (m, n, scm_tc7_fvect);
 }
@@ -201,7 +173,7 @@ gh_floats2fvect (const float *d, long n)
 SCM
 gh_doubles2dvect (const double *d, long n)
 {
-  char *m = scm_must_malloc (n * sizeof (double), "vector");
+  char *m = scm_gc_malloc (n * sizeof (double), "vector");
   memcpy (m, d, n * sizeof (double));
   return makvect (m, n, scm_tc7_dvect);
 }
@@ -211,27 +183,27 @@ gh_doubles2dvect (const double *d, long n)
 int 
 gh_scm2bool (SCM obj)
 {
-  return (SCM_FALSEP (obj)) ? 0 : 1;
+  return (scm_is_false (obj)) ? 0 : 1;
 }
 unsigned long 
 gh_scm2ulong (SCM obj)
 {
-  return scm_num2ulong (obj, SCM_ARG1, "gh_scm2ulong");
+  return scm_to_ulong (obj);
 }
 long 
 gh_scm2long (SCM obj)
 {
-  return scm_num2long (obj, SCM_ARG1, "gh_scm2long");
+  return scm_to_long (obj);
 }
 int 
 gh_scm2int (SCM obj)
 {
-  return (int) scm_num2int (obj, SCM_ARG1, "gh_scm2int");
+  return scm_to_int (obj);
 }
 double 
 gh_scm2double (SCM obj)
 {
-  return scm_num2dbl (obj, "gh_scm2double");
+  return scm_to_double (obj);
 }
 char 
 gh_scm2char (SCM obj)
@@ -261,9 +233,9 @@ gh_scm2chars (SCM obj, char *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (SCM_INUMP (val))
+	  if (SCM_I_INUMP (val))
 	    {
-	      v = SCM_INUM (val);
+	      v = SCM_I_INUM (val);
 	      if (v < -128 || v > 255)
 		scm_out_of_range (0, obj);
 	    }
@@ -275,9 +247,9 @@ gh_scm2chars (SCM obj, char *m)
       if (m == NULL)
 	return NULL;
       for (i = 0; i < n; ++i)
-	m[i] = SCM_INUM (SCM_VELTS (obj)[i]);
+	m[i] = SCM_I_INUM (SCM_VELTS (obj)[i]);
       break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_byvect:
       n = SCM_UVECTOR_LENGTH (obj);
       if (m == 0)
@@ -288,13 +260,12 @@ gh_scm2chars (SCM obj, char *m)
       break;
 #endif
     case scm_tc7_string:
-    case scm_tc7_substring:
-      n = SCM_STRING_LENGTH (obj);
+      n = scm_i_string_length (obj);
       if (m == 0)
 	m = (char *) malloc (n * sizeof (char));
       if (m == NULL)
 	return NULL;
-      memcpy (m, SCM_VELTS (obj), n * sizeof (char));
+      memcpy (m, scm_i_string_chars (obj), n * sizeof (char));
       break;
     default:
       scm_wrong_type_arg (0, 0, obj);
@@ -321,9 +292,9 @@ gh_scm2shorts (SCM obj, short *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (SCM_INUMP (val))
+	  if (SCM_I_INUMP (val))
 	    {
-	      v = SCM_INUM (val);
+	      v = SCM_I_INUM (val);
 	      if (v < -32768 || v > 65535)
 		scm_out_of_range (0, obj);
 	    }
@@ -335,9 +306,9 @@ gh_scm2shorts (SCM obj, short *m)
       if (m == NULL)
 	return NULL;
       for (i = 0; i < n; ++i)
-	m[i] = SCM_INUM (SCM_VELTS (obj)[i]);
+	m[i] = SCM_I_INUM (SCM_VELTS (obj)[i]);
       break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_svect:
       n = SCM_UVECTOR_LENGTH (obj);
       if (m == 0)
@@ -371,7 +342,7 @@ gh_scm2longs (SCM obj, long *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (!SCM_INUMP (val) && !SCM_BIGP (val))
+	  if (!SCM_I_INUMP (val) && !SCM_BIGP (val))
 	    scm_wrong_type_arg (0, 0, obj);
 	}
       if (m == 0)
@@ -381,12 +352,12 @@ gh_scm2longs (SCM obj, long *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  m[i] = SCM_INUMP (val) 
-	    ? SCM_INUM (val) 
-	    : scm_num2long (val, 0, NULL);
+	  m[i] = SCM_I_INUMP (val) 
+	    ? SCM_I_INUM (val) 
+	    : scm_to_long (val);
 	}
       break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_ivect:
     case scm_tc7_uvect:
       n = SCM_UVECTOR_LENGTH (obj);
@@ -421,7 +392,7 @@ gh_scm2floats (SCM obj, float *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (!SCM_INUMP (val)
+	  if (!SCM_I_INUMP (val)
 	      && !(SCM_BIGP (val) || SCM_REALP (val)))
 	    scm_wrong_type_arg (0, 0, val);
 	}
@@ -432,15 +403,15 @@ gh_scm2floats (SCM obj, float *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (SCM_INUMP (val))
-	    m[i] = SCM_INUM (val);
+	  if (SCM_I_INUMP (val))
+	    m[i] = SCM_I_INUM (val);
 	  else if (SCM_BIGP (val))
-	    m[i] = scm_num2long (val, 0, NULL);
+	    m[i] = scm_to_long (val);
 	  else
 	    m[i] = SCM_REAL_VALUE (val);
 	}
       break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_fvect:
       n = SCM_UVECTOR_LENGTH (obj);
       if (m == 0)
@@ -484,7 +455,7 @@ gh_scm2doubles (SCM obj, double *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (!SCM_INUMP (val)
+	  if (!SCM_I_INUMP (val)
 	      && !(SCM_BIGP (val) || SCM_REALP (val)))
 	    scm_wrong_type_arg (0, 0, val);
 	}
@@ -495,15 +466,15 @@ gh_scm2doubles (SCM obj, double *m)
       for (i = 0; i < n; ++i)
 	{
 	  val = SCM_VELTS (obj)[i];
-	  if (SCM_INUMP (val))
-	    m[i] = SCM_INUM (val);
+	  if (SCM_I_INUMP (val))
+	    m[i] = SCM_I_INUM (val);
 	  else if (SCM_BIGP (val))
-	    m[i] = scm_num2long (val, 0, NULL);
+	    m[i] = scm_to_long (val);
 	  else
 	    m[i] = SCM_REAL_VALUE (val);
 	}
       break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
     case scm_tc7_fvect:
       n = SCM_UVECTOR_LENGTH (obj);
       if (m == 0)
@@ -548,30 +519,17 @@ char *
 gh_scm2newstr (SCM str, size_t *lenp)
 {
   char *ret_str;
-  size_t len;
 
-  SCM_ASSERT (SCM_STRINGP (str), str, SCM_ARG3, "gh_scm2newstr");
-
-  len = SCM_STRING_LENGTH (str);
-
-  ret_str = (char *) malloc ((len + 1) * sizeof (char));
-  if (ret_str == NULL)
-    return NULL;
-  /* so we copy tmp_str to ret_str, which is what we will allocate */
-  memcpy (ret_str, SCM_STRING_CHARS (str), len);
-  scm_remember_upto_here_1 (str);
-  /* now make sure we null-terminate it */
-  ret_str[len] = '\0';
-
-  if (lenp != NULL)
-    {
-      *lenp = len;
-    }
+  /* We can't use scm_to_locale_stringn directly since it does not
+     guarantee null-termination when lenp is non-NULL.
+   */
 
+  ret_str = scm_to_locale_string (str);
+  if (lenp)
+    *lenp = scm_i_string_length (str);
   return ret_str;
 }
 
-
 /* Copy LEN characters at START from the Scheme string SRC to memory
    at DST.  START is an index into SRC; zero means the beginning of
    the string.  DST has already been allocated by the caller.
@@ -583,11 +541,11 @@ void
 gh_get_substr (SCM src, char *dst, long start, size_t len)
 {
   size_t src_len, effective_length;
-  SCM_ASSERT (SCM_STRINGP (src), src, SCM_ARG3, "gh_get_substr");
+  SCM_ASSERT (scm_is_string (src), src, SCM_ARG3, "gh_get_substr");
 
-  src_len = SCM_STRING_LENGTH (src);
+  src_len = scm_i_string_length (src);
   effective_length = (len < src_len) ? len : src_len;
-  memcpy (dst + start, SCM_STRING_CHARS (src), effective_length * sizeof (char));
+  memcpy (dst + start, scm_i_string_chars (src), effective_length * sizeof (char));
   /* FIXME: must signal an error if len > src_len */
   scm_remember_upto_here_1 (src);
 }
@@ -604,28 +562,7 @@ gh_get_substr (SCM src, char *dst, long start, size_t len)
 char *
 gh_symbol2newstr (SCM sym, size_t *lenp)
 {
-  char *ret_str;
-  size_t len;
-
-  SCM_ASSERT (SCM_SYMBOLP (sym), sym, SCM_ARG3, "gh_scm2newsymbol");
-
-  len = SCM_SYMBOL_LENGTH (sym);
-
-  ret_str = (char *) malloc ((len + 1) * sizeof (char));
-  if (ret_str == NULL)
-    return NULL;
-  /* so we copy sym to ret_str, which is what we will allocate */
-  memcpy (ret_str, SCM_SYMBOL_CHARS (sym), len);
-  scm_remember_upto_here_1 (sym);
-  /* now make sure we null-terminate it */
-  ret_str[len] = '\0';
-
-  if (lenp != NULL)
-    {
-      *lenp = len;
-    }
-
-  return ret_str;
+  return gh_scm2newstr (scm_symbol_to_string (sym), lenp);
 }
 
 
@@ -658,7 +595,7 @@ gh_vector_length (SCM v)
   return (unsigned long) SCM_VECTOR_LENGTH (v);
 }
 
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
 /* uniform vector support */
 
 /* returns the length as a C unsigned long integer */
@@ -708,7 +645,7 @@ gh_module_lookup (SCM module, const char *sname)
 
   SCM_VALIDATE_MODULE (SCM_ARG1, module);
 
-  sym = gh_symbol2scm (sname);
+  sym = scm_from_locale_symbol (sname);
   var = scm_sym2var (sym, scm_module_lookup_closure (module), SCM_BOOL_F);
   if (var != SCM_BOOL_F)
     return SCM_VARIABLE_REF (var);
