@@ -1,44 +1,19 @@
-/*	Copyright (C) 1995,1996,1997,1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
-
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -111,11 +86,9 @@ create_gsubr (int define, const char *name,
 	  }
 	SCM_SET_GSUBR_PROC (cclo, subr);
 	SCM_SET_GSUBR_TYPE (cclo,
-			    SCM_MAKINUM (SCM_GSUBR_MAKTYPE (req, opt, rst)));
-#ifdef DEBUG_EXTENSIONS
+			    scm_from_int (SCM_GSUBR_MAKTYPE (req, opt, rst)));
 	if (SCM_REC_PROCNAMES_P)
 	  scm_set_procedure_property_x (cclo, scm_sym_name, sym);
-#endif
 	if (define)
 	  scm_define (sym, cclo);
       return cclo;
@@ -214,20 +187,18 @@ scm_gsubr_apply (SCM args)
   SCM self = SCM_CAR (args);
   SCM (*fcn)() = SCM_SUBRF (SCM_GSUBR_PROC (self));
   SCM v[SCM_GSUBR_MAX];
-  long typ = SCM_INUM (SCM_GSUBR_TYPE (self));
+  int typ = scm_to_int (SCM_GSUBR_TYPE (self));
   long i, n = SCM_GSUBR_REQ (typ) + SCM_GSUBR_OPT (typ) + SCM_GSUBR_REST (typ);
 #if 0
   if (n > SCM_GSUBR_MAX)
     scm_misc_error (FUNC_NAME,
 		    "Function ~S has illegal arity ~S.",
-		    scm_list_2 (self, SCM_MAKINUM (n)));
+		    scm_list_2 (self, scm_from_int (n)));
 #endif
   args = SCM_CDR (args);
   for (i = 0; i < SCM_GSUBR_REQ (typ); i++) {
-#ifndef SCM_RECKLESS
     if (SCM_NULLP (args))
       scm_wrong_num_args (SCM_SNAME (SCM_GSUBR_PROC (self)));
-#endif
     v[i] = SCM_CAR(args);
     args = SCM_CDR(args);
   }
@@ -280,7 +251,6 @@ gsubr_21l(SCM req1, SCM req2, SCM opt, SCM rst)
 #endif
 
 
-
 void
 scm_init_gsubr()
 {
@@ -293,32 +263,6 @@ scm_init_gsubr()
 #include "libguile/gsubr.x"
 }
 
-#if SCM_DEBUG_DEPRECATED == 0
-
-SCM
-scm_make_gsubr (const char *name, int req, int opt, int rst, SCM (*fcn)())
-{
-  scm_c_issue_deprecation_warning
-    ("`scm_make_gsubr' is deprecated.  Use `scm_c_define_gsubr' instead.");
-
-  return scm_c_define_gsubr (name, req, opt, rst, fcn);
-}
-
-SCM
-scm_make_gsubr_with_generic (const char *name,
-			     int req, int opt, int rst,
-			     SCM (*fcn)(), SCM *gf)
-{
-  scm_c_issue_deprecation_warning
-    ("`scm_make_gsubr_with_generic' is deprecated.  "
-     "Use `scm_c_define_gsubr_with_generic' instead.");
-
-  return scm_c_define_gsubr_with_generic (name, req, opt, rst, fcn, gf);
-}
-
-#endif /* !SCM_DEBUG_DEPRECATED */
-
-
 /*
   Local Variables:
   c-file-style: "gnu"
