@@ -1,48 +1,29 @@
-/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <string.h>
+#include <stdio.h>
 
 #include "libguile/_scm.h"
 #include "libguile/libpath.h"
@@ -55,6 +36,9 @@
 #include "libguile/root.h"
 #include "libguile/strings.h"
 #include "libguile/modules.h"
+#include "libguile/lang.h"
+#include "libguile/chars.h"
+#include "libguile/srfi-13.h"
 
 #include "libguile/validate.h"
 #include "libguile/load.h"
@@ -112,16 +96,16 @@ SCM_DEFINE (scm_primitive_load, "primitive-load", 1, 0, 0,
 {
   SCM hook = *scm_loc_load_hook;
   SCM_VALIDATE_STRING (1, filename);
-  if (!SCM_FALSEP (hook) && !SCM_EQ_P (scm_procedure_p (hook), SCM_BOOL_T))
+  if (scm_is_true (hook) && scm_is_false (scm_procedure_p (hook)))
     SCM_MISC_ERROR ("value of %load-hook is neither a procedure nor #f",
 		    SCM_EOL);
 
-  if (! SCM_FALSEP (hook))
+  if (!scm_is_false (hook))
     scm_call_1 (hook, filename);
 
   { /* scope */
     SCM port, save_port;
-    port = scm_open_file (filename, scm_mem2string ("r", sizeof (char)));
+    port = scm_open_file (filename, scm_from_locale_string ("r"));
     save_port = port;
     scm_internal_dynamic_wind (swap_port,
 			       load,
@@ -137,7 +121,7 @@ SCM_DEFINE (scm_primitive_load, "primitive-load", 1, 0, 0,
 SCM
 scm_c_primitive_load (const char *filename)
 {
-  return scm_primitive_load (scm_makfrom0str (filename));
+  return scm_primitive_load (scm_from_locale_string (filename));
 }
 
 
@@ -150,7 +134,7 @@ SCM_DEFINE (scm_sys_package_data_dir, "%package-data-dir", 0, 0, 0,
 	    "@samp{/usr/local/share/guile}.")
 #define FUNC_NAME s_scm_sys_package_data_dir
 {
-  return scm_makfrom0str (SCM_PKGDATA_DIR);
+  return scm_from_locale_string (SCM_PKGDATA_DIR);
 }
 #undef FUNC_NAME
 #endif /* SCM_PKGDATA_DIR */
@@ -162,7 +146,7 @@ SCM_DEFINE (scm_sys_library_dir, "%library-dir", 0,0,0,
 	    "E.g., may return \"/usr/share/guile/1.3.5\".")
 #define FUNC_NAME s_scm_sys_library_dir
 {
-  return scm_makfrom0str(SCM_LIBRARY_DIR);
+  return scm_from_locale_string (SCM_LIBRARY_DIR);
 }
 #undef FUNC_NAME
 #endif /* SCM_LIBRARY_DIR */
@@ -174,7 +158,7 @@ SCM_DEFINE (scm_sys_site_dir, "%site-dir", 0,0,0,
 	    "E.g., may return \"/usr/share/guile/site\".")
 #define FUNC_NAME s_scm_sys_site_dir
 {
-  return scm_makfrom0str(SCM_SITE_DIR);
+  return scm_from_locale_string (SCM_SITE_DIR);
 }
 #undef FUNC_NAME
 #endif /* SCM_SITE_DIR */
@@ -191,33 +175,6 @@ static SCM *scm_loc_load_path;
 static SCM *scm_loc_load_extensions;
 
 
-/* Parse the null-terminated string PATH as if it were a standard path
-   environment variable (i.e. a colon-separated list of strings), and
-   prepend the elements to TAIL.  */
-SCM
-scm_internal_parse_path (char *path, SCM tail)
-{
-  if (path && path[0] != '\0')
-    {
-      char *scan, *elt_end;
-
-      /* Scan backwards from the end of the string, to help
-	 construct the list in the right order.  */
-      scan = elt_end = path + strlen (path);
-      do {
-	/* Scan back to the beginning of the current element.  */
-	do scan--;
-	while (scan >= path && *scan != ':');
-	tail = scm_cons (scm_mem2string (scan + 1, elt_end - (scan + 1)),
-			 tail);
-	elt_end = scan;
-      } while (scan >= path);
-    }
-
-  return tail;
-}
-
-
 SCM_DEFINE (scm_parse_path, "parse-path", 1, 1, 0, 
             (SCM path, SCM tail),
 	    "Parse @var{path}, which is expected to be a colon-separated\n"
@@ -226,14 +183,17 @@ SCM_DEFINE (scm_parse_path, "parse-path", 1, 1, 0,
 	    "is returned.")
 #define FUNC_NAME s_scm_parse_path
 {
-  SCM_ASSERT (SCM_FALSEP (path) || (SCM_STRINGP (path)),
-	      path,
-	      SCM_ARG1, FUNC_NAME);
+#ifdef __MINGW32__
+  SCM sep = SCM_MAKE_CHAR (';');
+#else
+  SCM sep = SCM_MAKE_CHAR (':');
+#endif
+  
   if (SCM_UNBNDP (tail))
     tail = SCM_EOL;
-  return (SCM_FALSEP (path)
+  return (scm_is_false (path)
 	  ? tail
-	  : scm_internal_parse_path (SCM_STRING_CHARS (path), tail));
+	  : scm_append_x (scm_list_2 (scm_string_split (path, sep), tail)));
 }
 #undef FUNC_NAME
 
@@ -244,21 +204,95 @@ SCM_DEFINE (scm_parse_path, "parse-path", 1, 1, 0,
 void
 scm_init_load_path ()
 {
+  char *env;
   SCM path = SCM_EOL;
 
 #ifdef SCM_LIBRARY_DIR
-  path = scm_list_3 (scm_makfrom0str (SCM_SITE_DIR),
-		     scm_makfrom0str (SCM_LIBRARY_DIR),
-		     scm_makfrom0str (SCM_PKGDATA_DIR));
+  path = scm_list_3 (scm_from_locale_string (SCM_SITE_DIR),
+		     scm_from_locale_string (SCM_LIBRARY_DIR),
+		     scm_from_locale_string (SCM_PKGDATA_DIR));
 #endif /* SCM_LIBRARY_DIR */
 
-  path = scm_internal_parse_path (getenv ("GUILE_LOAD_PATH"), path);
+  env = getenv ("GUILE_LOAD_PATH");
+  if (env)
+    path = scm_parse_path (scm_from_locale_string (env), path);
 
   *scm_loc_load_path = path;
 }
 
 SCM scm_listofnullstr;
 
+/* Utility functions for assembling C strings in a buffer.
+ */
+
+struct stringbuf {
+  char *buf, *ptr;
+  size_t buf_len;
+};
+
+static void
+stringbuf_free (void *data)
+{
+  struct stringbuf *buf = (struct stringbuf *)data;
+  free (buf->buf);
+}
+
+static void
+stringbuf_grow (struct stringbuf *buf)
+{
+  size_t ptroff = buf->ptr - buf->buf;
+  buf->buf_len *= 2; 
+  buf->buf = scm_realloc (buf->buf, buf->buf_len);
+  buf->ptr = buf->buf + ptroff;
+}
+
+static void
+stringbuf_cat_locale_string (struct stringbuf *buf, SCM str)
+{
+  size_t max_len = buf->buf_len - (buf->ptr - buf->buf) - 1;
+  size_t len = scm_to_locale_stringbuf (str, buf->ptr, max_len);
+  if (len > max_len)
+    {
+      /* buffer is too small, double its size and try again. 
+       */
+      stringbuf_grow (buf);
+      stringbuf_cat_locale_string (buf, str);
+    }
+  else
+    {
+      /* string fits, terminate it and check for embedded '\0'.
+       */
+      buf->ptr[len] = '\0';
+      if (strlen (buf->ptr) != len)
+	scm_misc_error (NULL,
+			"string contains #\\nul character: ~S",
+			scm_list_1 (str));
+      buf->ptr += len;
+    }
+}
+
+static void
+stringbuf_cat (struct stringbuf *buf, char *str)
+{
+  size_t max_len = buf->buf_len - (buf->ptr - buf->buf) - 1;
+  size_t len = strlen (str);
+  if (len > max_len)
+    {
+      /* buffer is too small, double its size and try again. 
+       */
+      stringbuf_grow (buf);
+      stringbuf_cat (buf, str);
+    }
+  else
+    {
+      /* string fits, copy it into buffer.
+       */
+      strcpy (buf->ptr, str);
+      buf->ptr += len;
+    }
+}
+
+  
 /* Search PATH for a directory containing a file named FILENAME.
    The file must be readable, and not a directory.
    If we find one, return its full filename; otherwise, return #f.
@@ -276,39 +310,35 @@ SCM_DEFINE (scm_search_path, "search-path", 2, 1, 0,
 	    "concatenated with each @var{extension}.")
 #define FUNC_NAME s_scm_search_path
 {
+  struct stringbuf buf;
   char *filename_chars;
-  int filename_len;
-  size_t max_path_len;		/* maximum length of any PATH element */
-  size_t max_ext_len;		/* maximum length of any EXTENSIONS element */
+  size_t filename_len;
+  SCM result = SCM_BOOL_F;
 
-  SCM_VALIDATE_LIST (1,path);
-  SCM_VALIDATE_STRING (2, filename);
   if (SCM_UNBNDP (extensions))
     extensions = SCM_EOL;
-  else
-    SCM_VALIDATE_LIST (3,extensions);
 
-  filename_chars = SCM_STRING_CHARS (filename);
-  filename_len = SCM_STRING_LENGTH (filename);
+  scm_frame_begin (0);
+
+  filename_chars = scm_to_locale_string (filename);
+  filename_len = strlen (filename_chars);
+  scm_frame_free (filename_chars);
 
   /* If FILENAME is absolute, return it unchanged.  */
+#ifdef __MINGW32__
+  if (((filename_len >= 1) && 
+       (filename_chars[0] == '/' || filename_chars[0] == '\\')) ||
+      ((filename_len >= 3) && filename_chars[1] == ':' &&
+       ((filename_chars[0] >= 'a' && filename_chars[0] <= 'z') ||
+	(filename_chars[0] >= 'A' && filename_chars[0] <= 'Z')) &&
+       (filename_chars[2] == '/' || filename_chars[2] == '\\')))
+#else
   if (filename_len >= 1 && filename_chars[0] == '/')
-    return filename;
-
-  /* Find the length of the longest element of path.  */
-  {
-    SCM walk;
-
-    max_path_len = 0;
-    for (walk = path; !SCM_NULLP (walk); walk = SCM_CDR (walk))
-      {
-	SCM elt = SCM_CAR (walk);
-	SCM_ASSERT_TYPE (SCM_STRINGP (elt), path, 1, FUNC_NAME,
-			 "list of strings");
-	if (SCM_STRING_LENGTH (elt) > max_path_len)
-	  max_path_len = SCM_STRING_LENGTH (elt);
-      }
-  }
+#endif
+    {
+      scm_frame_end ();
+      return filename;
+    }
 
   /* If FILENAME has an extension, don't try to add EXTENSIONS to it.  */
   {
@@ -325,85 +355,79 @@ SCM_DEFINE (scm_search_path, "search-path", 2, 1, 0,
 	    extensions = SCM_EOL;
 	    break;
 	  }
+#ifdef __MINGW32__
+	else if (*endp == '/' || *endp == '\\')
+#else
 	else if (*endp == '/')
+#endif
 	  /* This filename has no extension, so keep the current list
              of extensions.  */
 	  break;
       }
   }
 
-  /* Find the length of the longest element of the load extensions
-     list.  */
-  { /* scope */
-    SCM walk;
+  /* This simplifies the loop below a bit.
+   */
+  if (SCM_NULLP (extensions))
+    extensions = scm_listofnullstr;
 
-    max_ext_len = 0;
-    for (walk = extensions; !SCM_NULLP (walk); walk = SCM_CDR (walk))
-      {
-	SCM elt = SCM_CAR (walk);
-	SCM_ASSERT_TYPE (SCM_STRINGP (elt), elt, 3, FUNC_NAME,
-			 "list of strings");
-	if (SCM_STRING_LENGTH (elt) > max_ext_len)
-	  max_ext_len = SCM_STRING_LENGTH (elt);
-      }
-  }
+  buf.buf_len = 512;
+  buf.buf = scm_malloc (buf.buf_len);
+  scm_frame_unwind_handler (stringbuf_free, &buf, SCM_F_WIND_EXPLICITLY);
 
-  SCM_DEFER_INTS;
+  /* Try every path element.
+   */
+  for (; SCM_CONSP (path); path = SCM_CDR (path))
+    {
+      SCM dir = SCM_CAR (path);
+      SCM exts;
+      size_t sans_ext_len;
 
-  { /* scope */
-    SCM result = SCM_BOOL_F;
-    size_t buf_size = max_path_len + 1 + filename_len + max_ext_len + 1;
-    char *buf = SCM_MUST_MALLOC (buf_size);
+      buf.ptr = buf.buf;
+      stringbuf_cat_locale_string (&buf, dir);
+	
+      /* Concatenate the path name and the filename. */
+      
+#ifdef __MINGW32__
+      if (buf.ptr > buf.buf && buf.ptr[-1] != '/' && buf.ptr[-1] != '\\')
+#else
+      if (buf.ptr > buf.buf >= 1 && buf.ptr[-1] != '/')
+#endif
+	stringbuf_cat (&buf, "/");
 
-    /* This simplifies the loop below a bit.  */
-    if (SCM_NULLP (extensions))
-      extensions = scm_listofnullstr;
+      stringbuf_cat (&buf, filename_chars);
+      sans_ext_len = buf.ptr - buf.buf;
 
-    /* Try every path element.  At this point, we know the path is a
-       proper list of strings.  */
-    for (; !SCM_NULLP (path); path = SCM_CDR (path))
-      {
-	size_t len;
-	SCM dir = SCM_CAR (path);
-	SCM exts;
+      /* Try every extension. */
+      for (exts = extensions; SCM_CONSP (exts); exts = SCM_CDR (exts))
+	{
+	  SCM ext = SCM_CAR (exts);
+	  struct stat mode;
+	  
+	  buf.ptr = buf.buf + sans_ext_len;
+	  stringbuf_cat_locale_string (&buf, ext);
+	  
+	  /* If the file exists at all, we should return it.  If the
+	     file is inaccessible, then that's an error.  */
 
-	/* Concatenate the path name and the filename. */
-	len = SCM_STRING_LENGTH (dir);
-	memcpy (buf, SCM_STRING_CHARS (dir), len);
-	if (len >= 1 && buf[len - 1] != '/')
-	  buf[len++] = '/';
-	memcpy (buf + len, filename_chars, filename_len);
-	len += filename_len;
+	  if (stat (buf.buf, &mode) == 0
+	      && ! (mode.st_mode & S_IFDIR))
+	    {
+	      result = scm_from_locale_string (buf.buf);
+	      goto end;
+	    }
+	}
+      
+      if (!SCM_NULL_OR_NIL_P (exts))
+	scm_wrong_type_arg_msg (NULL, 0, extensions, "proper list");
+    }
 
-	/* Try every extension.  At this point, we know the extension
-	   list is a proper, nonempty list of strings.  */
-	for (exts = extensions; !SCM_NULLP (exts); exts = SCM_CDR (exts))
-	  {
-	    SCM ext = SCM_CAR (exts);
-	    size_t ext_len = SCM_STRING_LENGTH (ext);
-	    struct stat mode;
+  if (!SCM_NULL_OR_NIL_P (path))
+    scm_wrong_type_arg_msg (NULL, 0, path, "proper list");
 
-	    /* Concatenate the extension. */
-	    memcpy (buf + len, SCM_STRING_CHARS (ext), ext_len);
-	    buf[len + ext_len] = '\0';
-	    
-	    /* If the file exists at all, we should return it.  If the
-	       file is inaccessible, then that's an error.  */
-	    if (stat (buf, &mode) == 0
-		&& ! (mode.st_mode & S_IFDIR))
-	      {
-		result = scm_mem2string (buf, len + ext_len);
-		goto end;
-	      }
-	  }
-      }
-
-  end:
-    scm_must_free (buf);
-    scm_done_free (buf_size);
-    SCM_ALLOW_INTS;
-    return result;
-  }
+ end:
+  scm_frame_end ();
+  return result;
 }
 #undef FUNC_NAME
 
@@ -446,19 +470,11 @@ SCM_DEFINE (scm_primitive_load_path, "primitive-load-path", 1, 0, 0,
 {
   SCM full_filename;
 
-  SCM_VALIDATE_STRING (1, filename);
-
   full_filename = scm_sys_search_load_path (filename);
 
-  if (SCM_FALSEP (full_filename))
-    {
-      int absolute = (SCM_STRING_LENGTH (filename) >= 1
-		      && SCM_STRING_CHARS (filename)[0] == '/');
-      SCM_MISC_ERROR ((absolute
-		       ? "Unable to load file ~S"
-		       : "Unable to find file ~S in load path"),
-		      scm_list_1 (filename));
-    }
+  if (scm_is_false (full_filename))
+    SCM_MISC_ERROR ("Unable to find file ~S in load path",
+		    scm_list_1 (filename));
 
   return scm_primitive_load (full_filename);
 }
@@ -467,33 +483,9 @@ SCM_DEFINE (scm_primitive_load_path, "primitive-load-path", 1, 0, 0,
 SCM
 scm_c_primitive_load_path (const char *filename)
 {
-  return scm_primitive_load_path (scm_makfrom0str (filename));
+  return scm_primitive_load_path (scm_from_locale_string (filename));
 }
 
-#if SCM_DEBUG_DEPRECATED == 0
-
-/* Eval now copies source properties, so this function is no longer required.
- */
-
-SCM_SYMBOL (scm_end_of_file_key, "end-of-file");
-
-SCM_DEFINE (scm_read_and_eval_x, "read-and-eval!", 0, 1, 0, 
-            (SCM port),
-	    "Read a form from @var{port} (standard input by default), and evaluate it\n"
-	    "(memoizing it in the process) in the top-level environment.  If no data\n"
-	    "is left to be read from @var{port}, an @code{end-of-file} error is\n"
-	    "signalled.")
-#define FUNC_NAME s_scm_read_and_eval_x
-{
-  SCM form = scm_read (port);
-  if (SCM_EOF_OBJECT_P (form))
-    scm_ithrow (scm_end_of_file_key, SCM_EOL, 1);
-  return scm_eval_x (form, scm_current_module ());
-}
-#undef FUNC_NAME
-
-#endif
-
 
 /* Information about the build environment.  */
 
@@ -507,12 +499,13 @@ init_build_info ()
   unsigned long i;
 
   for (i = 0; i < (sizeof (info) / sizeof (info[0])); i++)
-    *loc = scm_acons (scm_str2symbol (info[i].name),
-		      scm_makfrom0str (info[i].value),
-		      *loc);
+    {
+      SCM key = scm_from_locale_symbol (info[i].name);
+      SCM val = scm_from_locale_string (info[i].value);
+      *loc = scm_acons (key, val, *loc);
+    }
 }
 
-
 
 void
 scm_init_load ()
@@ -521,8 +514,8 @@ scm_init_load ()
   scm_loc_load_path = SCM_VARIABLE_LOC (scm_c_define ("%load-path", SCM_EOL));
   scm_loc_load_extensions
     = SCM_VARIABLE_LOC (scm_c_define ("%load-extensions",
-				    scm_list_2 (scm_makfrom0str (".scm"),
-						scm_nullstr)));
+				      scm_list_2 (scm_from_locale_string (".scm"),
+						  scm_nullstr)));
   scm_loc_load_hook = SCM_VARIABLE_LOC (scm_c_define ("%load-hook", SCM_BOOL_F));
 
   init_build_info ();
