@@ -1,43 +1,19 @@
-/* Copyright (C) 1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1998,2000,2001,2002, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -91,13 +67,6 @@ SCM_DEFINE (scm_set_current_module, "set-current-module", 1, 0, 0,
   old = scm_current_module ();
   scm_fluid_set_x (the_module, module);
 
-#if SCM_DEBUG_DEPRECATED == 0
-  scm_fluid_set_x (SCM_VARIABLE_REF (scm_top_level_lookup_closure_var),
-		   scm_current_module_lookup_closure ());
-  scm_fluid_set_x (SCM_VARIABLE_REF (scm_system_transformer),
-		   scm_current_module_transformer ());
-#endif
-
   return old;
 }
 #undef FUNC_NAME
@@ -122,6 +91,10 @@ scm_c_call_with_current_module (SCM module,
   return scm_c_with_fluid (the_module, module, func, data);
 }
 
+
+/*
+  convert "A B C" to scheme list (A B C)
+ */
 static SCM
 convert_module_name (const char *name)
 {
@@ -138,7 +111,8 @@ convert_module_name (const char *name)
 	ptr++;
       if (ptr > name)
 	{
-	  *tail = scm_cons (scm_mem2symbol (name, ptr-name), SCM_EOL);
+	  SCM sym = scm_from_locale_symboln (name, ptr-name);
+	  *tail = scm_cons (sym, SCM_EOL);
 	  tail = SCM_CDRLOC (*tail);
 	}
       name = ptr;
@@ -183,13 +157,36 @@ scm_c_use_module (const char *name)
 
 static SCM module_export_x_var;
 
+
+/*
+  TODO: should export this function? --hwn.
+ */
+static SCM
+scm_export (SCM module, SCM namelist)
+{
+  return scm_call_2 (SCM_VARIABLE_REF (module_export_x_var),
+		     module, namelist);
+}
+
+
+/*
+  @code{scm_c_export}(@var{name-list})
+
+  @code{scm_c_export} exports the named bindings from the current
+  module, making them visible to users of the module. This function
+  takes a list of string arguments, terminated by NULL, e.g.
+
+  @example
+    scm_c_export ("add-double-record", "bamboozle-money", NULL);
+  @end example
+*/
 void
 scm_c_export (const char *name, ...)
 {
   if (name)
     {
       va_list ap;
-      SCM names = scm_cons (scm_str2symbol (name), SCM_EOL);
+      SCM names = scm_cons (scm_from_locale_symbol (name), SCM_EOL);
       SCM *tail = SCM_CDRLOC (names);
       va_start (ap, name);
       while (1)
@@ -197,15 +194,15 @@ scm_c_export (const char *name, ...)
 	  const char *n = va_arg (ap, const char *);
 	  if (n == NULL)
 	    break;
-	  *tail = scm_cons (scm_str2symbol (n), SCM_EOL);
+	  *tail = scm_cons (scm_from_locale_symbol (n), SCM_EOL);
 	  tail = SCM_CDRLOC (*tail);
 	}
       va_end (ap);
-      scm_call_2 (SCM_VARIABLE_REF (module_export_x_var),
-		  scm_current_module (), names);
+      scm_export (scm_current_module(), names);
     }
 }
 
+
 /* Environments */
 
 SCM
@@ -220,11 +217,11 @@ scm_top_level_env (SCM thunk)
 SCM
 scm_env_top_level (SCM env)
 {
-  while (SCM_NIMP (env))
+  while (SCM_CONSP (env))
     {
-      if (!SCM_CONSP (SCM_CAR (env))
-	  && SCM_NFALSEP (scm_procedure_p (SCM_CAR (env))))
-	return SCM_CAR (env);
+      SCM car_env = SCM_CAR (env);
+      if (!SCM_CONSP (car_env) && scm_is_true (scm_procedure_p (car_env)))
+	return car_env;
       env = SCM_CDR (env);
     }
   return SCM_BOOL_F;
@@ -246,14 +243,14 @@ the_root_module ()
 SCM
 scm_lookup_closure_module (SCM proc)
 {
-  if (SCM_FALSEP (proc))
+  if (scm_is_false (proc))
     return the_root_module ();
   else if (SCM_EVAL_CLOSURE_P (proc))
     return SCM_PACK (SCM_SMOB_DATA (proc));
   else
     {
       SCM mod = scm_procedure_property (proc, sym_module);
-      if (mod == SCM_BOOL_F)
+      if (scm_is_false (mod))
 	mod = the_root_module ();
       return mod;
     }
@@ -280,17 +277,20 @@ static SCM module_make_local_var_x_var;
 static SCM
 module_variable (SCM module, SCM sym)
 {
+#define SCM_BOUND_THING_P(b) \
+  (scm_is_true (b))
+
   /* 1. Check module obarray */
   SCM b = scm_hashq_ref (SCM_MODULE_OBARRAY (module), sym, SCM_UNDEFINED);
-  if (SCM_VARIABLEP (b))
+  if (SCM_BOUND_THING_P (b))
     return b;
   {
     SCM binder = SCM_MODULE_BINDER (module);
-    if (SCM_NFALSEP (binder))
+    if (scm_is_true (binder))
       /* 2. Custom binder */
       {
 	b = scm_call_3 (binder, module, sym, SCM_BOOL_F);
-	if (SCM_NFALSEP (b))
+	if (SCM_BOUND_THING_P (b))
 	  return b;
       }
   }
@@ -300,12 +300,13 @@ module_variable (SCM module, SCM sym)
     while (SCM_CONSP (uses))
       {
 	b = module_variable (SCM_CAR (uses), sym);
-	if (SCM_NFALSEP (b))
+	if (SCM_BOUND_THING_P (b))
 	  return b;
 	uses = SCM_CDR (uses);
       }
     return SCM_BOOL_F;
   }
+#undef SCM_BOUND_THING_P
 }
 
 scm_t_bits scm_tc16_eval_closure;
@@ -320,7 +321,7 @@ SCM
 scm_eval_closure_lookup (SCM eclo, SCM sym, SCM definep)
 {
   SCM module = SCM_PACK (SCM_SMOB_DATA (eclo));
-  if (SCM_NFALSEP (definep))
+  if (scm_is_true (definep))
     {
       if (SCM_EVAL_CLOSURE_INTERFACE_P (eclo))
 	return SCM_BOOL_F;
@@ -355,7 +356,7 @@ SCM_DEFINE (scm_standard_interface_eval_closure,
 SCM
 scm_module_lookup_closure (SCM module)
 {
-  if (module == SCM_BOOL_F)
+  if (scm_is_false (module))
     return SCM_BOOL_F;
   else
     return SCM_MODULE_EVAL_CLOSURE (module);
@@ -373,7 +374,7 @@ scm_current_module_lookup_closure ()
 SCM
 scm_module_transformer (SCM module)
 {
-  if (module == SCM_BOOL_F)
+  if (scm_is_false (module))
     return SCM_BOOL_F;
   else
     return SCM_MODULE_TRANSFORMER (module);
@@ -388,6 +389,43 @@ scm_current_module_transformer ()
     return SCM_BOOL_F;
 }
 
+SCM_DEFINE (scm_module_import_interface, "module-import-interface", 2, 0, 0,
+	    (SCM module, SCM sym),
+	    "")
+#define FUNC_NAME s_scm_module_import_interface
+{
+#define SCM_BOUND_THING_P(b) (scm_is_true (b))
+  SCM uses;
+  SCM_VALIDATE_MODULE (SCM_ARG1, module);
+  /* Search the use list */
+  uses = SCM_MODULE_USES (module);
+  while (SCM_CONSP (uses))
+    {
+      SCM _interface = SCM_CAR (uses);
+      /* 1. Check module obarray */
+      SCM b = scm_hashq_ref (SCM_MODULE_OBARRAY (_interface), sym, SCM_BOOL_F);
+      if (SCM_BOUND_THING_P (b))
+	return _interface;
+      {
+	SCM binder = SCM_MODULE_BINDER (_interface);
+	if (scm_is_true (binder))
+	  /* 2. Custom binder */
+	  {
+	    b = scm_call_3 (binder, _interface, sym, SCM_BOOL_F);
+	    if (SCM_BOUND_THING_P (b))
+	      return _interface;
+	  }
+      }
+      /* 3. Search use list recursively. */
+      _interface = scm_module_import_interface (_interface, sym);
+      if (scm_is_true (_interface))
+	return _interface;
+      uses = SCM_CDR (uses);
+    }
+  return SCM_BOOL_F;
+}
+#undef FUNC_NAME
+
 /* scm_sym2var
  *
  * looks up the variable bound to SYM according to PROC.  PROC should be
@@ -423,25 +461,22 @@ scm_sym2var (SCM sym, SCM proc, SCM definep)
     {
       SCM handle;
 
-      if (definep == SCM_BOOL_F)
+      if (scm_is_false (definep))
 	var = scm_hashq_ref (scm_pre_modules_obarray, sym, SCM_BOOL_F);
       else
 	{
 	  handle = scm_hashq_create_handle_x (scm_pre_modules_obarray,
 					      sym, SCM_BOOL_F);
 	  var = SCM_CDR (handle);
-	  if (var == SCM_BOOL_F)
+	  if (scm_is_false (var))
 	    {
 	      var = scm_make_variable (SCM_UNDEFINED);
-#if SCM_ENABLE_VCELLS
-	      scm_variable_set_name_hint (var, sym);
-#endif
 	      SCM_SETCDR (handle, var);
 	    }
 	}
     }
 
-  if (var != SCM_BOOL_F && !SCM_VARIABLEP (var))
+  if (scm_is_true (var) && !SCM_VARIABLEP (var))
     SCM_MISC_ERROR ("~S is not bound to a variable", scm_list_1 (sym));
 
   return var;
@@ -451,7 +486,7 @@ scm_sym2var (SCM sym, SCM proc, SCM definep)
 SCM
 scm_c_module_lookup (SCM module, const char *name)
 {
-  return scm_module_lookup (module, scm_str2symbol (name));
+  return scm_module_lookup (module, scm_from_locale_symbol (name));
 }
 
 SCM
@@ -462,7 +497,7 @@ scm_module_lookup (SCM module, SCM sym)
   SCM_VALIDATE_MODULE (1, module);
 
   var = scm_sym2var (sym, scm_module_lookup_closure (module), SCM_BOOL_F);
-  if (SCM_FALSEP (var))
+  if (scm_is_false (var))
     SCM_MISC_ERROR ("unbound variable: ~S", scm_list_1 (sym));
   return var;
 }
@@ -471,7 +506,7 @@ scm_module_lookup (SCM module, SCM sym)
 SCM
 scm_c_lookup (const char *name)
 {
-  return scm_lookup (scm_str2symbol (name));
+  return scm_lookup (scm_from_locale_symbol (name));
 }
 
 SCM
@@ -479,7 +514,7 @@ scm_lookup (SCM sym)
 {
   SCM var = 
     scm_sym2var (sym, scm_current_module_lookup_closure (), SCM_BOOL_F);
-  if (SCM_FALSEP (var))
+  if (scm_is_false (var))
     scm_misc_error ("scm_lookup", "unbound variable: ~S", scm_list_1 (sym));
   return var;
 }
@@ -487,7 +522,7 @@ scm_lookup (SCM sym)
 SCM
 scm_c_module_define (SCM module, const char *name, SCM value)
 {
-  return scm_module_define (module, scm_str2symbol (name), value);
+  return scm_module_define (module, scm_from_locale_symbol (name), value);
 }
 
 SCM
@@ -506,7 +541,7 @@ scm_module_define (SCM module, SCM sym, SCM value)
 SCM
 scm_c_define (const char *name, SCM value)
 {
-  return scm_define (scm_str2symbol (name), value);
+  return scm_define (scm_from_locale_symbol (name), value);
 }
 
 SCM
@@ -525,7 +560,7 @@ scm_module_reverse_lookup (SCM module, SCM variable)
   SCM obarray;
   long i, n;
 
-  if (module == SCM_BOOL_F)
+  if (scm_is_false (module))
     obarray = scm_pre_modules_obarray;
   else
     {
@@ -533,13 +568,16 @@ scm_module_reverse_lookup (SCM module, SCM variable)
       obarray = SCM_MODULE_OBARRAY (module);
     }
 
+  if (!SCM_HASHTABLE_P (obarray))
+      return SCM_BOOL_F;
+
   /* XXX - We do not use scm_hash_fold here to avoid searching the
      whole obarray.  We should have a scm_hash_find procedure. */
 
-  n = SCM_VECTOR_LENGTH (obarray);
+  n = SCM_HASHTABLE_N_BUCKETS (obarray);
   for (i = 0; i < n; ++i)
     {
-      SCM ls = SCM_VELTS (obarray)[i], handle;
+      SCM ls = SCM_HASHTABLE_BUCKETS (obarray)[i], handle;
       while (!SCM_NULLP (ls))
 	{
 	  handle = SCM_CAR (ls);
@@ -556,7 +594,7 @@ scm_module_reverse_lookup (SCM module, SCM variable)
     while (SCM_CONSP (uses))
       {
 	SCM sym = scm_module_reverse_lookup (SCM_CAR (uses), variable);
-	if (sym != SCM_BOOL_F)
+	if (scm_is_true (sym))
 	  return sym;
 	uses = SCM_CDR (uses);
       }
@@ -577,27 +615,15 @@ SCM_DEFINE (scm_get_pre_modules_obarray, "%get-pre-modules-obarray", 0, 0, 0,
 }
 #undef FUNC_NAME
 
-#if SCM_DEBUG_DEPRECATED == 0
-
-static SCM root_module_lookup_closure;
-SCM_SYMBOL (scm_sym_app, "app");
-SCM_SYMBOL (scm_sym_modules, "modules");
-static SCM module_prefix;
-static SCM make_modules_in_var;
-static SCM beautify_user_module_x_var;
-static SCM try_module_autoload_var;
-
-#endif
-
 SCM_SYMBOL (scm_sym_system_module, "system-module");
 
 SCM
 scm_system_module_env_p (SCM env)
 {
   SCM proc = scm_env_top_level (env);
-  if (SCM_FALSEP (proc))
+  if (scm_is_false (proc))
     return SCM_BOOL_T;
-  return ((SCM_NFALSEP (scm_procedure_property (proc,
+  return ((scm_is_true (scm_procedure_property (proc,
 						scm_sym_system_module)))
 	  ? SCM_BOOL_T
 	  : SCM_BOOL_F);
@@ -607,7 +633,7 @@ void
 scm_modules_prehistory ()
 {
   scm_pre_modules_obarray 
-    = scm_permanent_object (scm_c_make_hash_table (2001));
+    = scm_permanent_object (scm_c_make_hash_table (1533));
 }
 
 void
@@ -629,7 +655,7 @@ scm_post_boot_init_modules ()
 #define PERM(x) scm_permanent_object(x)
 
   SCM module_type = SCM_VARIABLE_REF (scm_c_lookup ("module-type"));
-  scm_module_tag = (SCM_CELL_WORD_1 (module_type) + scm_tc3_cons_gloc);
+  scm_module_tag = (SCM_CELL_WORD_1 (module_type) + scm_tc3_struct);
 
   resolve_module_var = PERM (scm_c_lookup ("resolve-module"));
   process_define_module_var = PERM (scm_c_lookup ("process-define-module"));
@@ -637,73 +663,9 @@ scm_post_boot_init_modules ()
   module_export_x_var = PERM (scm_c_lookup ("module-export!"));
   the_root_module_var = PERM (scm_c_lookup ("the-root-module"));
 
-#if SCM_DEBUG_DEPRECATED == 0
-
-  module_prefix = PERM (scm_list_2 (scm_sym_app, scm_sym_modules));
-  make_modules_in_var = PERM (scm_c_lookup ("make-modules-in"));
-  root_module_lookup_closure =
-    PERM (scm_module_lookup_closure (SCM_VARIABLE_REF (the_root_module_var)));
-  beautify_user_module_x_var = PERM (scm_c_lookup ("beautify-user-module!"));
-  try_module_autoload_var = PERM (scm_c_lookup ("try-module-autoload"));
-
-#endif
-
   scm_module_system_booted_p = 1;
 }
 
-#if SCM_DEBUG_DEPRECATED == 0
-
-SCM
-scm_the_root_module ()
-{
-  scm_c_issue_deprecation_warning ("`scm_the_root_module' is deprecated. "
-				   "Use `scm_c_resolve_module (\"guile\")' "
-				   "instead.");
-
-  return the_root_module ();
-}
-
-static SCM
-scm_module_full_name (SCM name)
-{
-  if (SCM_EQ_P (SCM_CAR (name), scm_sym_app))
-    return name;
-  else
-    return scm_append (scm_list_2 (module_prefix, name));
-}
-
-SCM
-scm_make_module (SCM name)
-{
-  scm_c_issue_deprecation_warning ("`scm_make_module' is deprecated. "
-				   "Use `scm_c_define_module instead.");
-
-  return scm_call_2 (SCM_VARIABLE_REF (make_modules_in_var),
-		     scm_the_root_module (),
-		     scm_module_full_name (name));
-}
-
-SCM
-scm_ensure_user_module (SCM module)
-{
-  scm_c_issue_deprecation_warning ("`scm_ensure_user_module' is deprecated. "
-				   "Use `scm_c_define_module instead.");
-
-  scm_call_1 (SCM_VARIABLE_REF (beautify_user_module_x_var), module);
-  return SCM_UNSPECIFIED;
-}
-
-SCM
-scm_load_scheme_module (SCM name)
-{
-  scm_c_issue_deprecation_warning ("`scm_load_scheme_module' is deprecated. "
-				   "Use `scm_c_resolve_module instead.");
-
-  return scm_call_1 (SCM_VARIABLE_REF (try_module_autoload_var), name);
-}
-
-#endif
-
 /*
   Local Variables:
   c-file-style: "gnu"
