@@ -1,43 +1,19 @@
-/* Copyright (C) 1995, 1996, 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1999,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -72,14 +48,12 @@ SCM scm_class_boolean, scm_class_char, scm_class_pair;
 SCM scm_class_procedure, scm_class_string, scm_class_symbol;
 SCM scm_class_procedure_with_setter, scm_class_primitive_generic;
 SCM scm_class_vector, scm_class_null;
-SCM scm_class_integer, scm_class_real, scm_class_complex;
+SCM scm_class_integer, scm_class_real, scm_class_complex, scm_class_fraction;
 SCM scm_class_unknown;
 
 SCM *scm_port_class = 0;
 SCM *scm_smob_class = 0;
 
-int scm_classes_initialized = 0;
-
 SCM scm_no_applicable_method;
 
 /* This function is used for efficient type dispatch.  */
@@ -88,9 +62,6 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 	    "Return the class of @var{x}.")
 #define FUNC_NAME s_scm_class_of
 {
-  if (!scm_classes_initialized)
-    scm_misc_error (FUNC_NAME, "GOOPS not loaded yet.", SCM_EOL);
-
   switch (SCM_ITAG3 (x))
     {
     case scm_tc3_int_1:
@@ -100,19 +71,12 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
     case scm_tc3_imm24:
       if (SCM_CHARP (x))
 	return scm_class_char;
+      else if (scm_is_bool (x))
+        return scm_class_boolean;
+      else if (SCM_NULLP (x))
+        return scm_class_null;
       else
-	{
-	  switch (SCM_ISYMNUM (x))
-	    {
-	    case SCM_ISYMNUM (SCM_BOOL_F):
-	    case SCM_ISYMNUM (SCM_BOOL_T):
-	      return scm_class_boolean;
-	    case SCM_ISYMNUM (SCM_EOL):
-	      return scm_class_null;
-	    default:
-	      return scm_class_unknown;
-	    }
-	}
+        return scm_class_unknown;
 
     case scm_tc3_cons:
       switch (SCM_TYP7 (x))
@@ -125,7 +89,7 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 	  return scm_class_symbol;
 	case scm_tc7_vector:
 	case scm_tc7_wvect:
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
 	case scm_tc7_bvect:
 	case scm_tc7_byvect:
 	case scm_tc7_svect:
@@ -137,11 +101,22 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 #endif
 	  return scm_class_vector;
 	case scm_tc7_string:
-	case scm_tc7_substring:
 	  return scm_class_string;
+        case scm_tc7_number:
+          switch SCM_TYP16 (x) {
+          case scm_tc16_big:
+            return scm_class_integer;
+          case scm_tc16_real:
+            return scm_class_real;
+          case scm_tc16_complex:
+            return scm_class_complex;
+	  case scm_tc16_fraction:
+	    return scm_class_fraction;
+          }
 	case scm_tc7_asubr:
 	case scm_tc7_subr_0:
 	case scm_tc7_subr_1:
+	case scm_tc7_dsubr:
 	case scm_tc7_cxr:
 	case scm_tc7_subr_3:
 	case scm_tc7_subr_2:
@@ -173,14 +148,13 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 				    ? SCM_INOUT_PCLASS_INDEX | SCM_PTOBNUM (x)
 				    : SCM_OUT_PCLASS_INDEX | SCM_PTOBNUM (x))
 				 : SCM_IN_PCLASS_INDEX | SCM_PTOBNUM (x))];
-	case scm_tcs_cons_gloc:
-	  /* must be a struct */
+	case scm_tcs_struct:
 	  if (SCM_OBJ_CLASS_FLAGS (x) & SCM_CLASSF_GOOPS_VALID)
 	    return SCM_CLASS_OF (x);
 	  else if (SCM_OBJ_CLASS_FLAGS (x) & SCM_CLASSF_GOOPS)
 	    {
 	      /* Goops object */
-	      if (! SCM_FALSEP (SCM_OBJ_CLASS_REDEF (x)))
+	      if (! scm_is_false (SCM_OBJ_CLASS_REDEF (x)))
 		scm_change_object_class (x,
 					 SCM_CLASS_OF (x),         /* old */
 					 SCM_OBJ_CLASS_REDEF (x)); /* new */
@@ -190,14 +164,15 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 	    {
 	      /* ordinary struct */
 	      SCM handle = scm_struct_create_handle (SCM_STRUCT_VTABLE (x));
-	      if (!SCM_FALSEP (SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle))))
+	      if (scm_is_true (SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle))))
 		return SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle));
 	      else
 		{
 		  SCM name = SCM_STRUCT_TABLE_NAME (SCM_CDR (handle));
-		  SCM class = scm_make_extended_class (!SCM_FALSEP (name)
-						       ? SCM_SYMBOL_CHARS (name)
-						       : 0);
+		  SCM class = scm_make_extended_class (scm_is_true (name)
+						       ? scm_i_symbol_chars (name)
+						       : 0,
+						       SCM_I_OPERATORP (x));
 		  SCM_SET_STRUCT_TABLE_CLASS (SCM_CDR (handle), class);
 		  return class;
 		}
@@ -209,7 +184,7 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 	    return scm_class_unknown;
 	}
 
-    case scm_tc3_cons_gloc:
+    case scm_tc3_struct:
     case scm_tc3_tc7_1:
     case scm_tc3_tc7_2:
     case scm_tc3_closure:
@@ -259,18 +234,25 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 SCM
 scm_mcache_lookup_cmethod (SCM cache, SCM args)
 {
-  long i, n, end, mask;
+  unsigned long i, mask, n, end;
   SCM ls, methods, z = SCM_CDDR (cache);
-  n = SCM_INUM (SCM_CAR (z)); /* maximum number of specializers */
+  n = scm_to_ulong (SCM_CAR (z)); /* maximum number of specializers */
   methods = SCM_CADR (z);
 
-  if (SCM_INUMP (methods))
+  if (SCM_VECTORP (methods))
+    {
+      /* cache format #1: prepare for linear search */
+      mask = -1;
+      i = 0;
+      end = SCM_VECTOR_LENGTH (methods);
+    }
+  else
     {
       /* cache format #2: compute a hash value */
-      long hashset = SCM_INUM (methods);
+      unsigned long hashset = scm_to_ulong (methods);
       long j = n;
       z = SCM_CDDR (z);
-      mask = SCM_INUM (SCM_CAR (z));
+      mask = scm_to_ulong (SCM_CAR (z));
       methods = SCM_CADR (z);
       i = 0;
       ls = args;
@@ -285,13 +267,6 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
       i &= mask;
       end = i;
     }
-  else /* SCM_VECTORP (methods) */
-    {
-      /* cache format #1: prepare for linear search */
-      mask = -1;
-      i = 0;
-      end = SCM_VECTOR_LENGTH (methods);
-    }
 
   /* Search for match  */
   do
@@ -303,7 +278,7 @@ scm_mcache_lookup_cmethod (SCM cache, SCM args)
 	do
 	  {
 	    /* More arguments than specifiers => CLASS != ENV */
-	    if (! SCM_EQ_P (scm_class_of (SCM_CAR (ls)), SCM_CAR (z)))
+	    if (! scm_is_eq (scm_class_of (SCM_CAR (ls)), SCM_CAR (z)))
 	      goto next_method;
 	    ls = SCM_CDR (ls);
 	    z = SCM_CDR (z);
@@ -322,7 +297,7 @@ SCM
 scm_mcache_compute_cmethod (SCM cache, SCM args)
 {
   SCM cmethod = scm_mcache_lookup_cmethod (cache, args);
-  if (SCM_FALSEP (cmethod))
+  if (scm_is_false (cmethod))
     /* No match - memoize */
     return scm_memoize_method (cache, args);
   return cmethod;
@@ -367,7 +342,7 @@ SCM_DEFINE (scm_entity_p, "entity?", 1, 0, 0,
 	    "Return @code{#t} if @var{obj} is an entity.")
 #define FUNC_NAME s_scm_entity_p
 {
-  return SCM_BOOL(SCM_STRUCTP (obj) && SCM_I_ENTITYP (obj));
+  return scm_from_bool(SCM_STRUCTP (obj) && SCM_I_ENTITYP (obj));
 }
 #undef FUNC_NAME
 
@@ -376,7 +351,7 @@ SCM_DEFINE (scm_operator_p, "operator?", 1, 0, 0,
 	    "Return @code{#t} if @var{obj} is an operator.")
 #define FUNC_NAME s_scm_operator_p
 {
-  return SCM_BOOL(SCM_STRUCTP (obj)
+  return scm_from_bool(SCM_STRUCTP (obj)
                   && SCM_I_OPERATORP (obj)
                   && !SCM_I_ENTITYP (obj));
 }
@@ -475,9 +450,9 @@ SCM_DEFINE (scm_make_class_object, "make-class-object", 2, 0, 0,
 #define FUNC_NAME s_scm_make_class_object
 {
   unsigned long flags = 0;
-  SCM_VALIDATE_STRUCT (1,metaclass);
-  SCM_VALIDATE_STRING (2,layout);
-  if (SCM_EQ_P (metaclass, scm_metaclass_operator))
+  SCM_VALIDATE_STRUCT (1, metaclass);
+  SCM_VALIDATE_STRING (2, layout);
+  if (scm_is_eq (metaclass, scm_metaclass_operator))
     flags = SCM_CLASSF_OPERATOR;
   return scm_i_make_class_object (metaclass, layout, flags);
 }
@@ -490,11 +465,10 @@ SCM_DEFINE (scm_make_subclass_object, "make-subclass-object", 2, 0, 0,
 #define FUNC_NAME s_scm_make_subclass_object
 {
   SCM pl;
-  SCM_VALIDATE_STRUCT (1,class);
-  SCM_VALIDATE_STRING (2,layout);
+  SCM_VALIDATE_STRUCT (1, class);
+  SCM_VALIDATE_STRING (2, layout);
   pl = SCM_PACK (SCM_STRUCT_DATA (class) [scm_vtable_index_layout]);
-  /* Convert symbol->string */
-  pl = scm_mem2string (SCM_SYMBOL_CHARS (pl), SCM_SYMBOL_LENGTH (pl));
+  pl = scm_symbol_to_string (pl);
   return scm_i_make_class_object (SCM_STRUCT_VTABLE (class),
 				  scm_string_append (scm_list_2 (pl, layout)),
 				  SCM_CLASS_FLAGS (class));
@@ -504,15 +478,15 @@ SCM_DEFINE (scm_make_subclass_object, "make-subclass-object", 2, 0, 0,
 void
 scm_init_objects ()
 {
-  SCM ms = scm_makfrom0str (SCM_METACLASS_STANDARD_LAYOUT);
+  SCM ms = scm_from_locale_string (SCM_METACLASS_STANDARD_LAYOUT);
   SCM mt = scm_make_vtable_vtable (ms, SCM_INUM0,
 				   scm_list_3 (SCM_BOOL_F, SCM_EOL, SCM_EOL));
   
-  SCM os = scm_makfrom0str (SCM_METACLASS_OPERATOR_LAYOUT);
+  SCM os = scm_from_locale_string (SCM_METACLASS_OPERATOR_LAYOUT);
   SCM ot = scm_make_vtable_vtable (os, SCM_INUM0,
 				   scm_list_3 (SCM_BOOL_F, SCM_EOL, SCM_EOL));
   
-  SCM es = scm_makfrom0str (SCM_ENTITY_LAYOUT);
+  SCM es = scm_from_locale_string (SCM_ENTITY_LAYOUT);
   SCM el = scm_make_struct_layout (es);
   SCM et = scm_make_struct (mt, SCM_INUM0,
 			    scm_list_4 (el, SCM_BOOL_F, SCM_EOL, SCM_EOL));
