@@ -1,48 +1,28 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
 /* Headers.  */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 
@@ -51,13 +31,15 @@
 #include "libguile/objects.h"
 #include "libguile/smob.h"
 #include "libguile/chars.h"
+#include "libguile/dynwind.h"
 
 #include "libguile/keywords.h"
 #include "libguile/root.h"
 #include "libguile/strings.h"
-
+#include "libguile/mallocs.h"
 #include "libguile/validate.h"
 #include "libguile/ports.h"
+#include "libguile/vectors.h"
 
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -67,6 +49,10 @@
 #include <malloc.h>
 #endif
 
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -120,6 +106,12 @@ end_input_default (SCM port SCM_UNUSED, int offset SCM_UNUSED)
 {
 }
 
+static size_t
+scm_port_free0 (SCM port)
+{
+  return 0;
+}
+
 scm_t_bits
 scm_make_port_type (char *name,
 		    int (*fill_input) (SCM port),
@@ -138,7 +130,7 @@ scm_make_port_type (char *name,
 
       scm_ptobs[scm_numptob].name = name;
       scm_ptobs[scm_numptob].mark = 0;
-      scm_ptobs[scm_numptob].free = scm_free0;
+      scm_ptobs[scm_numptob].free = scm_port_free0;
       scm_ptobs[scm_numptob].print = scm_port_print;
       scm_ptobs[scm_numptob].equalp = 0;
       scm_ptobs[scm_numptob].close = 0;
@@ -239,14 +231,15 @@ SCM_DEFINE (scm_char_ready_p, "char-ready?", 0, 1, 0,
 	    "@code{#t} then the next @code{read-char} operation on\n"
 	    "@var{port} is guaranteed not to hang.  If @var{port} is a file\n"
 	    "port at end of file then @code{char-ready?} returns @code{#t}.\n"
-	    "@footnote{@code{char-ready?} exists to make it possible for a\n"
+	    "\n"
+	    "@code{char-ready?} exists to make it possible for a\n"
 	    "program to accept characters from interactive ports without\n"
 	    "getting stuck waiting for input.  Any input editors associated\n"
 	    "with such ports must make sure that characters whose existence\n"
 	    "has been asserted by @code{char-ready?} cannot be rubbed out.\n"
 	    "If @code{char-ready?} were to return @code{#f} at end of file,\n"
 	    "a port at end of file would be indistinguishable from an\n"
-	    "interactive port that has no ready characters.}")
+	    "interactive port that has no ready characters.")
 #define FUNC_NAME s_scm_char_ready_p
 {
   scm_t_port *pt;
@@ -254,7 +247,7 @@ SCM_DEFINE (scm_char_ready_p, "char-ready?", 0, 1, 0,
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
   else
-    SCM_VALIDATE_OPINPORT (1,port);
+    SCM_VALIDATE_OPINPORT (1, port);
 
   pt = SCM_PTAB_ENTRY (port);
 
@@ -270,7 +263,7 @@ SCM_DEFINE (scm_char_ready_p, "char-ready?", 0, 1, 0,
       scm_t_ptob_descriptor *ptob = &scm_ptobs[SCM_PTOBNUM (port)];
       
       if (ptob->input_waiting)
-	return SCM_BOOL(ptob->input_waiting (port));
+	return scm_from_bool(ptob->input_waiting (port));
       else
 	return SCM_BOOL_T;
     }
@@ -327,19 +320,19 @@ SCM_DEFINE (scm_drain_input, "drain-input", 1, 0, 0,
 #define FUNC_NAME s_scm_drain_input
 {
   SCM result;
+  char *data;
   scm_t_port *pt;
   long count;
 
-  SCM_VALIDATE_OPINPORT (1,port);
+  SCM_VALIDATE_OPINPORT (1, port);
   pt = SCM_PTAB_ENTRY (port);
 
   count = pt->read_end - pt->read_pos;
   if (pt->read_buf == pt->putback_buf)
     count += pt->saved_read_end - pt->saved_read_pos;
 
-  result = scm_allocate_string (count);
-  scm_take_from_input_buffers (port, SCM_STRING_CHARS (result), count);
-
+  result = scm_i_make_string (count, &data);
+  scm_take_from_input_buffers (port, data, count);
   return result;
 }
 #undef FUNC_NAME
@@ -400,7 +393,7 @@ SCM_DEFINE (scm_set_current_input_port, "set-current-input-port", 1, 0, 0,
 #define FUNC_NAME s_scm_set_current_input_port
 {
   SCM oinp = scm_cur_inp;
-  SCM_VALIDATE_OPINPORT (1,port);
+  SCM_VALIDATE_OPINPORT (1, port);
   scm_cur_inp = port;
   return oinp;
 }
@@ -414,7 +407,7 @@ SCM_DEFINE (scm_set_current_output_port, "set-current-output-port", 1, 0, 0,
 {
   SCM ooutp = scm_cur_outp;
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPOUTPORT (1,port);
+  SCM_VALIDATE_OPOUTPORT (1, port);
   scm_cur_outp = port;
   return ooutp;
 }
@@ -428,63 +421,139 @@ SCM_DEFINE (scm_set_current_error_port, "set-current-error-port", 1, 0, 0,
 {
   SCM oerrp = scm_cur_errp;
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPOUTPORT (1,port);
+  SCM_VALIDATE_OPOUTPORT (1, port);
   scm_cur_errp = port;
   return oerrp;
 }
 #undef FUNC_NAME
 
+typedef struct {
+  SCM value;
+  SCM (*getter) (void);
+  SCM (*setter) (SCM);
+} swap_data;
+
+static void
+swap_port (SCM scm_data)
+{
+  swap_data *d = (swap_data *)SCM_MALLOCDATA (scm_data);
+  SCM t;
+
+  t = d->getter ();
+  d->setter (d->value);
+  d->value = t;
+}
+
+static void
+scm_frame_current_foo_port (SCM port,
+			   SCM (*getter) (void), SCM (*setter) (SCM))
+{
+  SCM scm_data = scm_malloc_obj (sizeof (swap_data));
+  swap_data *data = (swap_data *)SCM_MALLOCDATA (scm_data);
+  data->value = port;
+  data->getter = getter;
+  data->setter = setter;
+  
+  scm_frame_rewind_handler_with_scm (swap_port, scm_data,
+				     SCM_F_WIND_EXPLICITLY);
+  scm_frame_unwind_handler_with_scm (swap_port, scm_data,
+				     SCM_F_WIND_EXPLICITLY);
+}
+
+void
+scm_frame_current_input_port (SCM port)
+{
+  scm_frame_current_foo_port (port,
+			      scm_current_input_port,
+			      scm_set_current_input_port);
+}
+
+void
+scm_frame_current_output_port (SCM port)
+{
+  scm_frame_current_foo_port (port,
+			      scm_current_output_port,
+			      scm_set_current_output_port);
+}
+
+void
+scm_frame_current_error_port (SCM port)
+{
+  scm_frame_current_foo_port (port,
+			      scm_current_error_port,
+			      scm_set_current_error_port);
+}
+
 
 /* The port table --- an array of pointers to ports.  */
 
-scm_t_port **scm_port_table;
+scm_t_port **scm_i_port_table;
 
-long scm_port_table_size = 0;	/* Number of ports in scm_port_table.  */
-long scm_port_table_room = 20;	/* Size of the array.  */
+long scm_i_port_table_size = 0;	/* Number of ports in scm_i_port_table.  */
+long scm_i_port_table_room = 20;	/* Size of the array.  */
 
-/* Add a port to the table.  */
+SCM_GLOBAL_MUTEX (scm_i_port_table_mutex);
 
-scm_t_port *
-scm_add_to_port_table (SCM port)
-#define FUNC_NAME "scm_add_to_port_table"
+/* This function is not and should not be thread safe. */
+
+SCM
+scm_new_port_table_entry (scm_t_bits tag)
+#define FUNC_NAME "scm_new_port_table_entry"
 {
-  scm_t_port *entry;
-
-  if (scm_port_table_size == scm_port_table_room)
+  /*
+    We initialize the cell to empty, this is in case scm_gc_calloc
+    triggers GC ; we don't want the GC to scan a half-finished Z.
+   */
+  
+  SCM z = scm_cons (SCM_EOL, SCM_EOL);
+  scm_t_port *entry = (scm_t_port *) scm_gc_calloc (sizeof (scm_t_port), "port");
+  if (scm_i_port_table_size == scm_i_port_table_room)
     {
-      /* initial malloc is in gc.c.  this doesn't use scm_must_malloc etc.,
+      /* initial malloc is in gc.c.  this doesn't use scm_gc_malloc etc.,
 	 since it can never be freed during gc.  */
-      void *newt = realloc ((char *) scm_port_table,
-			    (size_t) (sizeof (scm_t_port *)
-					 * scm_port_table_room * 2));
-      if (newt == NULL)
-	scm_memory_error ("scm_add_to_port_table");
-      scm_port_table = (scm_t_port **) newt;
-      scm_port_table_room *= 2;
+      void *newt = scm_realloc ((char *) scm_i_port_table,
+				(size_t) (sizeof (scm_t_port *)
+					  * scm_i_port_table_room * 2));
+      scm_i_port_table = (scm_t_port **) newt;
+      scm_i_port_table_room *= 2;
     }
-  entry = (scm_t_port *) scm_must_malloc (sizeof (scm_t_port), FUNC_NAME);
 
-  entry->port = port;
-  entry->entry = scm_port_table_size;
-  entry->revealed = 0;
-  entry->stream = 0;
+  entry->entry = scm_i_port_table_size;
+
   entry->file_name = SCM_BOOL_F;
-  entry->line_number = 0;
-  entry->column_number = 0;
-  entry->putback_buf = 0;
-  entry->putback_buf_size = 0;
   entry->rw_active = SCM_PORT_NEITHER;
-  entry->rw_random = 0;
 
-  scm_port_table[scm_port_table_size] = entry;
-  scm_port_table_size++;
+  scm_i_port_table[scm_i_port_table_size] = entry;
+  scm_i_port_table_size++;
 
-  return entry;
+  entry->port = z;
+  SCM_SET_CELL_TYPE(z, tag);
+  SCM_SETPTAB_ENTRY(z, entry);
+  
+  return z;
 }
 #undef FUNC_NAME
 
+#if SCM_ENABLE_DEPRECATED==1
+SCM_API scm_t_port *
+scm_add_to_port_table (SCM port)
+{
+  SCM z = scm_new_port_table_entry (scm_tc7_port);
+  scm_t_port * pt = SCM_PTAB_ENTRY(z);
+
+  pt->port = port;
+  SCM_SETCAR(z, SCM_EOL);
+  SCM_SETCDR(z, SCM_EOL);
+  SCM_SETPTAB_ENTRY (port, pt);
+  return pt;
+}
+#endif
+
+
 /* Remove a port from the table and destroy it.  */
 
+/* This function is not and should not be thread safe. */
+
 void
 scm_remove_from_port_table (SCM port)
 #define FUNC_NAME "scm_remove_from_port_table"
@@ -492,20 +561,20 @@ scm_remove_from_port_table (SCM port)
   scm_t_port *p = SCM_PTAB_ENTRY (port);
   long i = p->entry;
 
-  if (i >= scm_port_table_size)
+  if (i >= scm_i_port_table_size)
     SCM_MISC_ERROR ("Port not in table: ~S", scm_list_1 (port));
   if (p->putback_buf)
-    scm_must_free (p->putback_buf);
-  scm_must_free (p);
+    scm_gc_free (p->putback_buf, p->putback_buf_size, "putback buffer");
+  scm_gc_free (p, sizeof (scm_t_port), "port");
   /* Since we have just freed slot i we can shrink the table by moving
      the last entry to that slot... */
-  if (i < scm_port_table_size - 1)
+  if (i < scm_i_port_table_size - 1)
     {
-      scm_port_table[i] = scm_port_table[scm_port_table_size - 1];
-      scm_port_table[i]->entry = i;
+      scm_i_port_table[i] = scm_i_port_table[scm_i_port_table_size - 1];
+      scm_i_port_table[i]->entry = i;
     }
   SCM_SETPTAB_ENTRY (port, 0);
-  scm_port_table_size--;
+  scm_i_port_table_size--;
 }
 #undef FUNC_NAME
 
@@ -519,7 +588,7 @@ SCM_DEFINE (scm_pt_size, "pt-size", 0, 0, 0,
 	    "is only included in @code{--enable-guile-debug} builds.")
 #define FUNC_NAME s_scm_pt_size
 {
-  return SCM_MAKINUM (scm_port_table_size);
+  return scm_from_int (scm_i_port_table_size);
 }
 #undef FUNC_NAME
 
@@ -530,12 +599,11 @@ SCM_DEFINE (scm_pt_member, "pt-member", 1, 0, 0,
 	    "@code{--enable-guile-debug} builds.")
 #define FUNC_NAME s_scm_pt_member
 {
-  long i;
-  SCM_VALIDATE_INUM_COPY (1,index,i);
-  if (i < 0 || i >= scm_port_table_size)
+  size_t i = scm_to_size_t (index);
+  if (i >= scm_i_port_table_size)
     return SCM_BOOL_F;
   else
-    return scm_port_table[i]->port;
+    return scm_i_port_table[i]->port;
 }
 #undef FUNC_NAME
 #endif
@@ -572,8 +640,8 @@ SCM_DEFINE (scm_port_revealed, "port-revealed", 1, 0, 0,
 #define FUNC_NAME s_scm_port_revealed
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  return SCM_MAKINUM (scm_revealed_count (port));
+  SCM_VALIDATE_OPENPORT (1, port);
+  return scm_from_int (scm_revealed_count (port));
 }
 #undef FUNC_NAME
 
@@ -585,9 +653,8 @@ SCM_DEFINE (scm_set_port_revealed_x, "set-port-revealed!", 2, 0, 0,
 #define FUNC_NAME s_scm_set_port_revealed_x
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  SCM_VALIDATE_INUM (2,rcount);
-  SCM_REVEALED (port) = SCM_INUM (rcount);
+  SCM_VALIDATE_OPENPORT (1, port);
+  SCM_REVEALED (port) = scm_to_int (rcount);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -602,18 +669,37 @@ SCM_DEFINE (scm_set_port_revealed_x, "set-port-revealed!", 2, 0, 0,
  * See PORT FLAGS in scm.h
  */
 
+static long
+scm_i_mode_bits_n (const char *modes, size_t n)
+{
+  return (SCM_OPN
+	  | (memchr (modes, 'r', n) || memchr (modes, '+', n) ? SCM_RDNG : 0)
+	  | (   memchr (modes, 'w', n)
+	     || memchr (modes, 'a', n)
+	     || memchr (modes, '+', n) ? SCM_WRTNG : 0)
+	  | (memchr (modes, '0', n) ? SCM_BUF0 : 0)
+	  | (memchr (modes, 'l', n) ? SCM_BUFLINE : 0));
+}
+
 long
 scm_mode_bits (char *modes)
 {
-  return (SCM_OPN
-	  | (strchr (modes, 'r') || strchr (modes, '+') ? SCM_RDNG : 0)
-	  | (   strchr (modes, 'w')
-	     || strchr (modes, 'a')
-	     || strchr (modes, '+') ? SCM_WRTNG : 0)
-	  | (strchr (modes, '0') ? SCM_BUF0 : 0)
-	  | (strchr (modes, 'l') ? SCM_BUFLINE : 0));
+  return scm_i_mode_bits_n (modes, strlen (modes));
 }
 
+long
+scm_i_mode_bits (SCM modes)
+{
+  long bits;
+
+  if (!scm_is_string (modes))
+    scm_wrong_type_arg_msg (NULL, 0, modes, "string");
+
+  bits = scm_i_mode_bits_n (scm_i_string_chars (modes),
+			    scm_i_string_length (modes));
+  scm_remember_upto_here_1 (modes);
+  return bits;
+}
 
 /* Return the mode flags from an open port.
  * Some modes such as "append" are only used when opening
@@ -631,7 +717,7 @@ SCM_DEFINE (scm_port_mode, "port-mode", 1, 0, 0,
   modes[0] = '\0';
 
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPPORT (1,port);
+  SCM_VALIDATE_OPPORT (1, port);
   if (SCM_CELL_WORD_0 (port) & SCM_RDNG) {
     if (SCM_CELL_WORD_0 (port) & SCM_WRTNG)
       strcpy (modes, "r+");
@@ -642,7 +728,7 @@ SCM_DEFINE (scm_port_mode, "port-mode", 1, 0, 0,
     strcpy (modes, "w");
   if (SCM_CELL_WORD_0 (port) & SCM_BUF0)
     strcat (modes, "0");
-  return scm_mem2string (modes, strlen (modes));
+  return scm_from_locale_string (modes);
 }
 #undef FUNC_NAME
 
@@ -677,9 +763,11 @@ SCM_DEFINE (scm_close_port, "close-port", 1, 0, 0,
     rv = (scm_ptobs[i].close) (port);
   else
     rv = 0;
+  scm_mutex_lock (&scm_i_port_table_mutex);
   scm_remove_from_port_table (port);
+  scm_mutex_unlock (&scm_i_port_table_mutex);
   SCM_CLR_PORT_OPEN_FLAG (port);
-  return SCM_BOOL (rv >= 0);
+  return scm_from_bool (rv >= 0);
 }
 #undef FUNC_NAME
 
@@ -714,6 +802,35 @@ SCM_DEFINE (scm_close_output_port, "close-output-port", 1, 0, 0,
 }
 #undef FUNC_NAME
 
+void
+scm_c_port_for_each (void (*proc)(void *data, SCM p), void *data)
+{
+  long i;
+  size_t n;
+  SCM ports;
+
+  /* Even without pre-emptive multithreading, running arbitrary code
+     while scanning the port table is unsafe because the port table
+     can change arbitrarily (from a GC, for example).  So we first
+     collect the ports into a vector. -mvo */
+
+  scm_mutex_lock (&scm_i_port_table_mutex);
+  n = scm_i_port_table_size;
+  scm_mutex_unlock (&scm_i_port_table_mutex);
+
+  ports = scm_make_vector (scm_from_size_t (n), SCM_BOOL_F);
+
+  scm_mutex_lock (&scm_i_port_table_mutex);
+  if (n > scm_i_port_table_size)
+    n = scm_i_port_table_size;
+  for (i = 0; i < n; i++)
+    SCM_VECTOR_SET (ports, i, scm_i_port_table[i]->port);
+  scm_mutex_unlock (&scm_i_port_table_mutex);
+
+  for (i = 0; i < n; i++)
+    proc (data, SCM_VECTOR_REF (ports, i));
+}
+
 SCM_DEFINE (scm_port_for_each, "port-for-each", 1, 0, 0,
 	    (SCM proc),
 	    "Apply @var{proc} to each port in the Guile port table\n"
@@ -724,77 +841,13 @@ SCM_DEFINE (scm_port_for_each, "port-for-each", 1, 0, 0,
 	    "have no effect as far as @var{port-for-each} is concerned.") 
 #define FUNC_NAME s_scm_port_for_each
 {
-  long i;
-  SCM ports;
-
   SCM_VALIDATE_PROC (1, proc);
 
-  /* when pre-emptive multithreading is supported, access to the port
-     table will need to be controlled by a mutex.  */
-
-  /* Even without pre-emptive multithreading, running arbitrary code
-     while scanning the port table is unsafe because the port table
-     can change arbitrarily (from a GC, for example).  So we build a
-     list in advance while blocking the GC. -mvo */
-
-  SCM_DEFER_INTS;
-  scm_block_gc++;
-  ports = SCM_EOL;
-  for (i = 0; i < scm_port_table_size; i++)
-    ports = scm_cons (scm_port_table[i]->port, ports);
-  scm_block_gc--;
-  SCM_ALLOW_INTS;
-
-  while (ports != SCM_EOL)
-    {
-      scm_call_1 (proc, SCM_CAR (ports));
-      ports = SCM_CDR (ports);
-    }
-
+  scm_c_port_for_each ((void (*)(void*,SCM))scm_call_1, proc);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-SCM_DEFINE (scm_close_all_ports_except, "close-all-ports-except", 0, 0, 1,
-           (SCM ports),
-	    "[DEPRECATED] Close all open file ports used by the interpreter\n"
-	    "except for those supplied as arguments.  This procedure\n"
-	    "was intended to be used before an exec call to close file descriptors\n"
-	    "which are not needed in the new process.  However it has the\n"
-	    "undesirable side effect of flushing buffers, so it's deprecated.\n"
-	    "Use port-for-each instead.")
-#define FUNC_NAME s_scm_close_all_ports_except
-{
-  long i = 0;
-  SCM_VALIDATE_REST_ARGUMENT (ports);
-  while (i < scm_port_table_size)
-    {
-      SCM thisport = scm_port_table[i]->port;
-      int found = 0;
-      SCM ports_ptr = ports;
-
-      while (!SCM_NULLP (ports_ptr))
-	{
-	  SCM port = SCM_COERCE_OUTPORT (SCM_CAR (ports_ptr));
-	  if (i == 0)
-            SCM_VALIDATE_OPPORT (SCM_ARG1,port);
-	  if (SCM_EQ_P (port, thisport))
-	    found = 1;
-	  ports_ptr = SCM_CDR (ports_ptr);
-	}
-      if (found)
-	i++;
-      else
-	/* i is not to be incremented here.  */
-	scm_close_port (thisport);
-    }
-  return SCM_UNSPECIFIED;
-}
-#undef FUNC_NAME
-
-#endif
 
 
 /* Utter miscellany.  Gosh, we should clean this up some time.  */
@@ -806,7 +859,7 @@ SCM_DEFINE (scm_input_port_p, "input-port?", 1, 0, 0,
 	    "@code{port?}.")
 #define FUNC_NAME s_scm_input_port_p
 {
-  return SCM_BOOL (SCM_INPUT_PORT_P (x));
+  return scm_from_bool (SCM_INPUT_PORT_P (x));
 }
 #undef FUNC_NAME
 
@@ -818,7 +871,7 @@ SCM_DEFINE (scm_output_port_p, "output-port?", 1, 0, 0,
 #define FUNC_NAME s_scm_output_port_p
 {
   x = SCM_COERCE_OUTPORT (x);
-  return SCM_BOOL (SCM_OUTPUT_PORT_P (x));
+  return scm_from_bool (SCM_OUTPUT_PORT_P (x));
 }
 #undef FUNC_NAME
 
@@ -829,7 +882,7 @@ SCM_DEFINE (scm_port_p, "port?", 1, 0, 0,
 	    "@var{x}))}.")
 #define FUNC_NAME s_scm_port_p
 {
-  return SCM_BOOL (SCM_PORTP (x));
+  return scm_from_bool (SCM_PORTP (x));
 }
 #undef FUNC_NAME
 
@@ -839,8 +892,8 @@ SCM_DEFINE (scm_port_closed_p, "port-closed?", 1, 0, 0,
 	    "open.")
 #define FUNC_NAME s_scm_port_closed_p
 {
-  SCM_VALIDATE_PORT (1,port);
-  return SCM_BOOL (!SCM_OPPORTP (port));
+  SCM_VALIDATE_PORT (1, port);
+  return scm_from_bool (!SCM_OPPORTP (port));
 }
 #undef FUNC_NAME
 
@@ -850,7 +903,7 @@ SCM_DEFINE (scm_eof_object_p, "eof-object?", 1, 0, 0,
 	    "return @code{#f}.")
 #define FUNC_NAME s_scm_eof_object_p
 {
-  return SCM_BOOL(SCM_EOF_OBJECT_P (x));
+  return scm_from_bool(SCM_EOF_OBJECT_P (x));
 }
 #undef FUNC_NAME
 
@@ -869,7 +922,7 @@ SCM_DEFINE (scm_force_output, "force-output", 0, 1, 0,
   else
     {
       port = SCM_COERCE_OUTPORT (port);
-      SCM_VALIDATE_OPOUTPORT (1,port);
+      SCM_VALIDATE_OPOUTPORT (1, port);
     }
   scm_flush (port);
   return SCM_UNSPECIFIED;
@@ -884,11 +937,13 @@ SCM_DEFINE (scm_flush_all_ports, "flush-all-ports", 0, 0, 0,
 {
   size_t i;
 
-  for (i = 0; i < scm_port_table_size; i++)
+  scm_mutex_lock (&scm_i_port_table_mutex);
+  for (i = 0; i < scm_i_port_table_size; i++)
     {
-      if (SCM_OPOUTPORTP (scm_port_table[i]->port))
-	scm_flush (scm_port_table[i]->port);
+      if (SCM_OPOUTPORTP (scm_i_port_table[i]->port))
+	scm_flush (scm_i_port_table[i]->port);
     }
+  scm_mutex_unlock (&scm_i_port_table_mutex);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -903,7 +958,7 @@ SCM_DEFINE (scm_read_char, "read-char", 0, 1, 0,
   int c;
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
-  SCM_VALIDATE_OPINPORT (1,port);
+  SCM_VALIDATE_OPINPORT (1, port);
   c = scm_getc (port);
   if (EOF == c)
     return SCM_EOF_VAL;
@@ -939,10 +994,8 @@ scm_getc (SCM port)
   scm_t_port *pt = SCM_PTAB_ENTRY (port);
 
   if (pt->rw_active == SCM_PORT_WRITE)
-    {
-      /* may be marginally faster than calling scm_flush.  */
-      scm_ptobs[SCM_PTOBNUM (port)].flush (port);
-    }
+    /* may be marginally faster than calling scm_flush.  */
+    scm_ptobs[SCM_PTOBNUM (port)].flush (port);
   
   if (pt->rw_random)
     pt->rw_active = SCM_PORT_READ;
@@ -957,17 +1010,9 @@ scm_getc (SCM port)
 
   switch (c)
     {
-      case '\a':
-        break;
-      case '\b':
-        SCM_DECCOL (port);
-        break;
       case '\n':
         SCM_INCLINE (port);
         break;
-      case '\r':
-        SCM_ZEROCOL (port);
-        break;
       case '\t':
         SCM_TABCOL (port);
         break;
@@ -975,7 +1020,7 @@ scm_getc (SCM port)
         SCM_INCCOL (port);
         break;
     }
-
+ 
   return c;
 }
 
@@ -1008,17 +1053,9 @@ scm_lfwrite (const char *ptr, size_t size, SCM port)
   ptob->write (port, ptr, size);
 
   for (; size; ptr++, size--) {
-    if (*ptr == '\a') {
-    }
-    else if (*ptr == '\b') {
-      SCM_DECCOL(port);
-    }
-    else if (*ptr == '\n') {
+    if (*ptr == '\n') {
       SCM_INCLINE(port);
     }
-    else if (*ptr == '\r') {
-      SCM_ZEROCOL(port);
-    }
     else if (*ptr == '\t') {
       SCM_TABCOL(port);
     }
@@ -1152,8 +1189,8 @@ scm_ungetc (int c, SCM port)
 	{
 	  size_t new_size = pt->read_buf_size * 2;
 	  unsigned char *tmp = (unsigned char *)
-	    scm_must_realloc (pt->putback_buf, pt->read_buf_size, new_size,
-			      FUNC_NAME);
+	    scm_gc_realloc (pt->putback_buf, pt->read_buf_size, new_size,
+			    "putback buffer");
 
 	  pt->read_pos = pt->read_buf = pt->putback_buf = tmp;
 	  pt->read_end = pt->read_buf + pt->read_buf_size;
@@ -1179,8 +1216,8 @@ scm_ungetc (int c, SCM port)
       if (pt->putback_buf == NULL)
 	{
 	  pt->putback_buf
-	    = (unsigned char *) scm_must_malloc (SCM_INITIAL_PUTBACK_BUF_SIZE,
-						 FUNC_NAME);
+	    = (unsigned char *) scm_gc_malloc (SCM_INITIAL_PUTBACK_BUF_SIZE,
+					       "putback buffer");
 	  pt->putback_buf_size = SCM_INITIAL_PUTBACK_BUF_SIZE;
 	}
 
@@ -1231,7 +1268,9 @@ SCM_DEFINE (scm_peek_char, "peek-char", 0, 1, 0,
 	    "Return the next character available from @var{port},\n"
 	    "@emph{without} updating @var{port} to point to the following\n"
 	    "character.  If no more characters are available, the\n"
-	    "end-of-file object is returned.@footnote{The value returned by\n"
+	    "end-of-file object is returned.\n"
+	    "\n"
+	    "The value returned by\n"
 	    "a call to @code{peek-char} is the same as the value that would\n"
 	    "have been returned by a call to @code{read-char} on the same\n"
 	    "port.  The only difference is that the very next call to\n"
@@ -1239,18 +1278,20 @@ SCM_DEFINE (scm_peek_char, "peek-char", 0, 1, 0,
 	    "return the value returned by the preceding call to\n"
 	    "@code{peek-char}.  In particular, a call to @code{peek-char} on\n"
 	    "an interactive port will hang waiting for input whenever a call\n"
-	    "to @code{read-char} would have hung.}")
+	    "to @code{read-char} would have hung.")
 #define FUNC_NAME s_scm_peek_char
 {
-  int c;
+  int c, column;
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
   else
-    SCM_VALIDATE_OPINPORT (1,port);
+    SCM_VALIDATE_OPINPORT (1, port);
+  column = SCM_COL(port);
   c = scm_getc (port);
   if (EOF == c)
     return SCM_EOF_VAL;
   scm_ungetc (c, port);
+  SCM_COL(port) = column;
   return SCM_MAKE_CHAR (c);
 }
 #undef FUNC_NAME
@@ -1265,11 +1306,11 @@ SCM_DEFINE (scm_unread_char, "unread-char", 1, 1, 0,
 {
   int c;
 
-  SCM_VALIDATE_CHAR (1,cobj);
+  SCM_VALIDATE_CHAR (1, cobj);
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
   else
-    SCM_VALIDATE_OPINPORT (2,port);
+    SCM_VALIDATE_OPINPORT (2, port);
 
   c = SCM_CHAR (cobj);
 
@@ -1286,13 +1327,13 @@ SCM_DEFINE (scm_unread_string, "unread-string", 2, 0, 0,
 	    "@var{port} is not supplied, the current-input-port is used.")
 #define FUNC_NAME s_scm_unread_string
 {
-  SCM_VALIDATE_STRING (1,str);
+  SCM_VALIDATE_STRING (1, str);
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
   else
-    SCM_VALIDATE_OPINPORT (2,port);
+    SCM_VALIDATE_OPINPORT (2, port);
 
-  scm_ungets (SCM_STRING_CHARS (str), SCM_STRING_LENGTH (str), port);
+  scm_ungets (scm_i_string_chars (str), scm_i_string_length (str), port);
   
   return str;
 }
@@ -1331,8 +1372,12 @@ SCM_DEFINE (scm_seek, "seek", 3, 0, 0,
 
   fd_port = SCM_COERCE_OUTPORT (fd_port);
 
-  off = SCM_NUM2LONG (2, offset);
-  SCM_VALIDATE_INUM_COPY (3, whence, how);
+  if (sizeof (off_t) == sizeof (scm_t_intmax))
+    off = scm_to_intmax (offset);
+  else
+    off = scm_to_long (offset);
+  how = scm_to_int (whence);
+
   if (how != SEEK_SET && how != SEEK_CUR && how != SEEK_END)
     SCM_OUT_OF_RANGE (3, whence);
   if (SCM_OPPORTP (fd_port))
@@ -1347,12 +1392,11 @@ SCM_DEFINE (scm_seek, "seek", 3, 0, 0,
     }
   else /* file descriptor?.  */
     {
-      SCM_VALIDATE_INUM (1,fd_port);
-      rv = lseek (SCM_INUM (fd_port), off, how);
+      rv = lseek (scm_to_int (fd_port), off, how);
       if (rv == -1)
 	SCM_SYSERROR;
     }
-  return scm_long2num (rv);
+  return scm_from_intmax (rv);
 }
 #undef FUNC_NAME
 
@@ -1376,7 +1420,7 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
 	    "@var{length} bytes.  @var{object} can be a string containing a\n"
 	    "file name or an integer file descriptor or a port.\n"
 	    "@var{length} may be omitted if @var{object} is not a file name,\n"
-	    "in which case the truncation occurs at the current port.\n"
+	    "in which case the truncation occurs at the current port\n"
 	    "position.  The return value is unspecified.")
 #define FUNC_NAME s_scm_truncate_file
 {
@@ -1388,19 +1432,17 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
   if (SCM_UNBNDP (length))
     {
       /* must supply length if object is a filename.  */
-      if (SCM_STRINGP (object))
-        SCM_MISC_ERROR("must supply length if OBJECT is a filename",SCM_EOL);
+      if (scm_is_string (object))
+        SCM_MISC_ERROR("must supply length if OBJECT is a filename", SCM_EOL);
       
-      length = scm_seek (object, SCM_INUM0, SCM_MAKINUM (SEEK_CUR));
+      length = scm_seek (object, SCM_INUM0, scm_from_int (SEEK_CUR));
     }
-  c_length = SCM_NUM2LONG (2,length);
-  if (c_length < 0)
-    SCM_MISC_ERROR ("negative offset", SCM_EOL);
+  c_length = scm_to_size_t (length);
 
   object = SCM_COERCE_OUTPORT (object);
-  if (SCM_INUMP (object))
+  if (scm_is_integer (object))
     {
-      SCM_SYSCALL (rv = ftruncate (SCM_INUM (object), c_length));
+      SCM_SYSCALL (rv = ftruncate (scm_to_int (object), c_length));
     }
   else if (SCM_OPOUTPORTP (object))
     {
@@ -1419,9 +1461,12 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
     }
   else
     {
-      SCM_VALIDATE_STRING (1, object);
-      SCM_STRING_COERCE_0TERMINATION_X (object);
-      SCM_SYSCALL (rv = truncate (SCM_STRING_CHARS (object), c_length));
+      char *str = scm_to_locale_string (object);
+      int eno;
+      SCM_SYSCALL (rv = truncate (str, c_length));
+      eno = errno;
+      free (str);
+      errno = eno;
     }
   if (rv == -1)
     SCM_SYSERROR;
@@ -1431,33 +1476,37 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
 
 SCM_DEFINE (scm_port_line, "port-line", 1, 0, 0,
             (SCM port),
-	    "Return the current line number for @var{port}.")
+	    "Return the current line number for @var{port}.\n"
+	    "\n"
+	    "The first line of a file is 0.  But you might want to add 1\n"
+	    "when printing line numbers, since starting from 1 is\n"
+	    "traditional in error messages, and likely to be more natural to\n"
+	    "non-programmers.")
 #define FUNC_NAME s_scm_port_line
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  return SCM_MAKINUM (SCM_LINUM (port));
+  SCM_VALIDATE_OPENPORT (1, port);
+  return scm_from_int (SCM_LINUM (port));
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_set_port_line_x, "set-port-line!", 2, 0, 0,
             (SCM port, SCM line),
-	    "Set the current line number for @var{port} to @var{line}.")
+	    "Set the current line number for @var{port} to @var{line}.  The\n"
+	    "first line of a file is 0.")
 #define FUNC_NAME s_scm_set_port_line_x
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  SCM_VALIDATE_INUM (2,line);
-  SCM_PTAB_ENTRY (port)->line_number = SCM_INUM (line);
+  SCM_VALIDATE_OPENPORT (1, port);
+  SCM_PTAB_ENTRY (port)->line_number = scm_to_int (line);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_port_column, "port-column", 1, 0, 0,
             (SCM port),
-	    "@deffnx {Scheme Procedure} port-line port\n"
-	    "Return the current column number or line number of @var{port},\n"
-	    "using the current input port if none is specified.  If the number is\n"
+	    "Return the current column number of @var{port}.\n"
+	    "If the number is\n"
 	    "unknown, the result is #f.  Otherwise, the result is a 0-origin integer\n"
 	    "- i.e. the first character of the first line is line 0, column 0.\n"
 	    "(However, when you display a file position, for example in an error\n"
@@ -1467,22 +1516,20 @@ SCM_DEFINE (scm_port_column, "port-column", 1, 0, 0,
 #define FUNC_NAME s_scm_port_column
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  return SCM_MAKINUM (SCM_COL (port));
+  SCM_VALIDATE_OPENPORT (1, port);
+  return scm_from_int (SCM_COL (port));
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_set_port_column_x, "set-port-column!", 2, 0, 0,
             (SCM port, SCM column),
-	    "@deffnx {Scheme Procedure} set-port-line! port line\n"
-	    "Set the current column or line number of @var{port}, using the\n"
-	    "current input port if none is specified.")
+	    "Set the current column of @var{port}.  Before reading the first\n"
+	    "character on a line the column should be 0.")
 #define FUNC_NAME s_scm_set_port_column_x
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
-  SCM_VALIDATE_INUM (2,column);
-  SCM_PTAB_ENTRY (port)->column_number = SCM_INUM (column);
+  SCM_VALIDATE_OPENPORT (1, port);
+  SCM_PTAB_ENTRY (port)->column_number = scm_to_int (column);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1495,7 +1542,7 @@ SCM_DEFINE (scm_port_filename, "port-filename", 1, 0, 0,
 #define FUNC_NAME s_scm_port_filename
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
+  SCM_VALIDATE_OPENPORT (1, port);
   return SCM_FILENAME (port);
 }
 #undef FUNC_NAME
@@ -1509,17 +1556,13 @@ SCM_DEFINE (scm_set_port_filename_x, "set-port-filename!", 2, 0, 0,
 #define FUNC_NAME s_scm_set_port_filename_x
 {
   port = SCM_COERCE_OUTPORT (port);
-  SCM_VALIDATE_OPENPORT (1,port);
+  SCM_VALIDATE_OPENPORT (1, port);
   /* We allow the user to set the filename to whatever he likes.  */
   SCM_SET_FILENAME (port, filename);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-#ifndef ttyname
-extern char * ttyname();
-#endif
-
 void
 scm_print_port_mode (SCM exp, SCM port)
 {
@@ -1554,7 +1597,7 @@ void
 scm_ports_prehistory ()
 {
   scm_numptob = 0;
-  scm_ptobs = (scm_t_ptob_descriptor *) malloc (sizeof (scm_t_ptob_descriptor));
+  scm_ptobs = (scm_t_ptob_descriptor *) scm_malloc (sizeof (scm_t_ptob_descriptor));
 }
 
 
@@ -1575,23 +1618,27 @@ write_void_port (SCM port SCM_UNUSED,
 {
 }
 
+static SCM
+scm_i_void_port (long mode_bits)
+{
+  scm_mutex_lock (&scm_i_port_table_mutex);
+  {
+    SCM answer = scm_new_port_table_entry (scm_tc16_void_port);
+    scm_t_port * pt = SCM_PTAB_ENTRY(answer);
+
+    scm_port_non_buffer (pt);
+  
+    SCM_SETSTREAM (answer, 0);
+    SCM_SET_CELL_TYPE (answer, scm_tc16_void_port | mode_bits);
+    scm_mutex_unlock (&scm_i_port_table_mutex);
+    return answer;
+  }
+}
+
 SCM
 scm_void_port (char *mode_str)
 {
-  int mode_bits;
-  SCM answer;
-  scm_t_port * pt;
-
-  SCM_NEWCELL (answer);
-  SCM_DEFER_INTS;
-  mode_bits = scm_mode_bits (mode_str);
-  pt = scm_add_to_port_table (answer);
-  scm_port_non_buffer (pt);
-  SCM_SETPTAB_ENTRY (answer, pt);
-  SCM_SETSTREAM (answer, 0);
-  SCM_SET_CELL_TYPE (answer, scm_tc16_void_port | mode_bits);
-  SCM_ALLOW_INTS;
-  return answer;
+  return scm_i_void_port (scm_mode_bits (mode_str));
 }
 
 SCM_DEFINE (scm_sys_make_void_port, "%make-void-port", 1, 0, 0,
@@ -1602,9 +1649,7 @@ SCM_DEFINE (scm_sys_make_void_port, "%make-void-port", 1, 0, 0,
 	    "documentation for @code{open-file} in @ref{File Ports}.")
 #define FUNC_NAME s_scm_sys_make_void_port
 {
-  SCM_VALIDATE_STRING (1, mode);
-  SCM_STRING_COERCE_0TERMINATION_X (mode);
-  return scm_void_port (SCM_STRING_CHARS (mode));
+  return scm_i_void_port (scm_i_mode_bits (mode));
 }
 #undef FUNC_NAME
 
@@ -1615,9 +1660,9 @@ void
 scm_init_ports ()
 {
   /* lseek() symbols.  */
-  scm_c_define ("SEEK_SET", SCM_MAKINUM (SEEK_SET));
-  scm_c_define ("SEEK_CUR", SCM_MAKINUM (SEEK_CUR));
-  scm_c_define ("SEEK_END", SCM_MAKINUM (SEEK_END));
+  scm_c_define ("SEEK_SET", scm_from_int (SEEK_SET));
+  scm_c_define ("SEEK_CUR", scm_from_int (SEEK_CUR));
+  scm_c_define ("SEEK_END", scm_from_int (SEEK_END));
 
   scm_tc16_void_port = scm_make_port_type ("void", fill_input_void_port, 
 					   write_void_port);
