@@ -1,43 +1,19 @@
-/* Copyright (C) 1995-1999,2000,2001, 2003 Free Software Foundation, Inc.
+/* Copyright (C) 1995-1999,2000,2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -62,6 +38,8 @@
 #include "libguile/strings.h"
 #include "libguile/strports.h"
 #include "libguile/vectors.h"
+#include "libguile/lang.h"
+#include "libguile/numbers.h"
 
 #include "libguile/validate.h"
 #include "libguile/print.h"
@@ -72,58 +50,22 @@
  * This table must agree with the declarations in scm.h: {Immediate Symbols}.
  */
 
-char *scm_isymnames[] =
+/* This table must agree with the list of flags in tags.h.  */
+static const char *iflagnames[] =
 {
-  /* This table must agree with the declarations */
-  "#@and",
-  "#@begin",
-  "#@case",
-  "#@cond",
-  "#@do",
-  "#@if",
-  "#@lambda",
-  "#@let",
-  "#@let*",
-  "#@letrec",
-  "#@or",
-  "#@quote",
-  "#@set!",
-  "#@define",
-#if 0
-  "#@literal-variable-ref",
-  "#@literal-variable-set!",
-#endif
-  "#@apply",
-  "#@call-with-current-continuation",
-
- /* user visible ISYMS */
- /* other keywords */
- /* Flags */
-
   "#f",
   "#t",
   "#<undefined>",
   "#<eof>",
   "()",
   "#<unspecified>",
-  "#@dispatch",
-  "#@slot-ref",
-  "#@slot-set!",
 
-  /* Multi-language support */
-  
-  "#@nil-cond",
-  "#@nil-ify",
-  "#@t-ify",
-  "#@0-cond",
-  "#@0-ify",
-  "#@1-ify",
-  "#@bind",
-  
-  "#@delay",
-  "#@call-with-values",
+  /* Unbound slot marker for GOOPS.  For internal use in GOOPS only.  */
+  "#<unbound>",
 
-  "#<unbound>"
+  /* Elisp nil value.  This is its Scheme name; whenever it's printed in
+   * Elisp, it should appear as the symbol `nil'.  */
+  "#nil"
 };
 
 scm_t_option scm_print_opts[] = {
@@ -170,7 +112,7 @@ do { \
 do { \
   register unsigned long i; \
   for (i = 0; i < pstate->top; ++i) \
-    if (SCM_EQ_P (pstate->ref_stack[i], (obj))) \
+    if (scm_is_eq (pstate->ref_stack[i], (obj))) \
       goto label; \
   if (pstate->fancyp) \
     { \
@@ -185,21 +127,21 @@ do { \
 
 #define EXIT_NESTED_DATA(pstate) { --pstate->top; }
 
-SCM scm_print_state_vtable;
-
-static SCM print_state_pool;
+SCM scm_print_state_vtable = SCM_BOOL_F;
+static SCM print_state_pool = SCM_EOL;
+SCM_MUTEX (print_state_mutex);
 
 #ifdef GUILE_DEBUG /* Used for debugging purposes */
 
 SCM_DEFINE (scm_current_pstate, "current-pstate", 0, 0, 0, 
            (),
-	    "Return the current-pstate -- the cadr of the\n"
+	    "Return the current-pstate -- the car of the\n"
 	    "@code{print_state_pool}.  @code{current-pstate} is only\n"
 	    "included in @code{--enable-guile-debug} builds.")
 #define FUNC_NAME s_scm_current_pstate
 {
-  if (SCM_NNULLP (SCM_CDR (print_state_pool)))
-    return SCM_CADR (print_state_pool);
+  if (!SCM_NULLP (print_state_pool))
+    return SCM_CAR (print_state_pool);
   else
     return SCM_BOOL_F;
 }
@@ -212,12 +154,11 @@ SCM_DEFINE (scm_current_pstate, "current-pstate", 0, 0, 0,
 static SCM
 make_print_state (void)
 {
-  SCM print_state = scm_make_struct (SCM_CAR (print_state_pool), /* pstate type */
-				     SCM_INUM0,
-				     SCM_EOL);
+  SCM print_state
+    = scm_make_struct (scm_print_state_vtable, SCM_INUM0, SCM_EOL);
   scm_print_state *pstate = SCM_PRINT_STATE (print_state);
   pstate->ref_vect = scm_c_make_vector (PSTATE_SIZE, SCM_UNDEFINED);
-  pstate->ref_stack = SCM_VELTS (pstate->ref_vect);
+  pstate->ref_stack = SCM_WRITABLE_VELTS (pstate->ref_vect);
   pstate->ceiling = SCM_VECTOR_LENGTH (pstate->ref_vect);
   return print_state;
 }
@@ -228,15 +169,15 @@ scm_make_print_state ()
   SCM answer = SCM_BOOL_F;
 
   /* First try to allocate a print state from the pool */
-  SCM_DEFER_INTS;
-  if (SCM_NNULLP (SCM_CDR (print_state_pool)))
+  scm_i_plugin_mutex_lock (&print_state_mutex);
+  if (!SCM_NULLP (print_state_pool))
     {
-      answer = SCM_CADR (print_state_pool);
-      SCM_SETCDR (print_state_pool, SCM_CDDR (print_state_pool));
+      answer = SCM_CAR (print_state_pool);
+      print_state_pool = SCM_CDR (print_state_pool);
     }
-  SCM_ALLOW_INTS;
+  scm_i_plugin_mutex_unlock (&print_state_mutex);
   
-  return SCM_FALSEP (answer) ? make_print_state () : answer;
+  return scm_is_false (answer) ? make_print_state () : answer;
 }
 
 void
@@ -251,35 +192,49 @@ scm_free_print_state (SCM print_state)
    */
   pstate->fancyp = 0;
   pstate->revealed = 0;
-  SCM_NEWCELL (handle);
-  SCM_DEFER_INTS;
-  SCM_SET_CELL_WORD_0 (handle, print_state);
-  SCM_SET_CELL_WORD_1 (handle, SCM_CDR (print_state_pool));
-  SCM_SETCDR (print_state_pool, handle);
-  SCM_ALLOW_INTS;
+  scm_i_plugin_mutex_lock (&print_state_mutex);
+  handle = scm_cons (print_state, print_state_pool);
+  print_state_pool = handle;
+  scm_i_plugin_mutex_unlock (&print_state_mutex);
+}
+
+SCM
+scm_i_port_with_print_state (SCM port, SCM print_state)
+{
+  if (SCM_UNBNDP (print_state))
+    {
+      if (SCM_PORT_WITH_PS_P (port))
+	return port;
+      else
+	print_state = scm_make_print_state ();
+      /* port does not need to be coerced since it doesn't have ps */
+    }
+  else
+    port = SCM_COERCE_OUTPORT (port);
+  SCM_RETURN_NEWSMOB (scm_tc16_port_with_ps,
+		      SCM_UNPACK (scm_cons (port, print_state)));
 }
 
 static void
 grow_ref_stack (scm_print_state *pstate)
 {
   unsigned long int old_size = SCM_VECTOR_LENGTH (pstate->ref_vect);
-  SCM *old_elts = SCM_VELTS (pstate->ref_vect);
+  SCM const *old_elts = SCM_VELTS (pstate->ref_vect);
   unsigned long int new_size = 2 * pstate->ceiling;
   SCM new_vect = scm_c_make_vector (new_size, SCM_UNDEFINED);
-  SCM *new_elts = SCM_VELTS (new_vect);
   unsigned long int i;
 
   for (i = 0; i != old_size; ++i)
-    new_elts [i] = old_elts [i];
+    SCM_VECTOR_SET (new_vect, i, old_elts [i]);
 
   pstate->ref_vect = new_vect;
-  pstate->ref_stack = new_elts;
+  pstate->ref_stack = SCM_WRITABLE_VELTS(new_vect);
   pstate->ceiling = new_size;
 }
 
 
 static void
-print_circref (SCM port,scm_print_state *pstate,SCM ref)
+print_circref (SCM port, scm_print_state *pstate, SCM ref)
 {
   register long i;
   long self = pstate->top - 1;
@@ -288,16 +243,16 @@ print_circref (SCM port,scm_print_state *pstate,SCM ref)
     {
       while (i > 0)
 	{
-	  if (SCM_NCONSP (pstate->ref_stack[i - 1])
-	      || !SCM_EQ_P (SCM_CDR (pstate->ref_stack[i - 1]), 
-			    pstate->ref_stack[i]))
+	  if (!SCM_CONSP (pstate->ref_stack[i - 1])
+	      || !scm_is_eq (SCM_CDR (pstate->ref_stack[i - 1]), 
+			     pstate->ref_stack[i]))
 	    break;
 	  --i;
 	}
       self = i;
     }
   for (i = pstate->top - 1; 1; --i)
-    if (SCM_EQ_P (pstate->ref_stack[i], ref))
+    if (scm_is_eq (pstate->ref_stack[i], ref))
       break;
   scm_putc ('#', port);
   scm_intprint (i - self, 10, port);
@@ -309,27 +264,34 @@ print_circref (SCM port,scm_print_state *pstate,SCM ref)
 void
 scm_print_symbol_name (const char *str, size_t len, SCM port)
 {
-  size_t pos;
+  /* This points to the first character that has not yet been written to the
+   * port. */
+  size_t pos = 0;
+  /* This points to the character we're currently looking at. */
   size_t end;
-  int weird;
-  int maybe_weird;
+  /* If the name contains weird characters, we'll escape them with
+   * backslashes and set this flag; it indicates that we should surround the
+   * name with "#{" and "}#". */
+  int weird = 0;
+  /* Backslashes are not sufficient to make a name weird, but if a name is
+   * weird because of other characters, backslahes need to be escaped too.
+   * The first time we see a backslash, we set maybe_weird, and mw_pos points
+   * to the backslash.  Then if the name turns out to be weird, we re-process
+   * everything starting from mw_pos.
+   * We could instead make backslashes always weird.  This is not necessary
+   * to ensure that the output is (read)-able, but it would make this code
+   * simpler and faster. */
+  int maybe_weird = 0;
   size_t mw_pos = 0;
-  
-  pos = 0;
-  weird = 0;
-  maybe_weird = 0;
-  
-  /* XXX - Lots of weird symbol names are missed, such as "12" or
-     "'a". */
 
-  if (len == 0)
-    scm_lfwrite ("#{}#", 4, port);
-  else if (str[0] == '#' || str[0] == ':' || str[len-1] == ':')
+  if (len == 0 || str[0] == '\'' || str[0] == '`' || str[0] == ',' ||
+      str[0] == ':' || str[len-1] == ':' || (str[0] == '.' && len == 1) ||
+      scm_is_true (scm_i_mem2number(str, len, 10)))
     {
       scm_lfwrite ("#{", 2, port);
       weird = 1;
     }
-  
+
   for (end = pos; end < len; ++end)
     switch (str[end])
       {
@@ -341,6 +303,7 @@ scm_print_symbol_name (const char *str, size_t len, SCM port)
       case ')':
       case '"':
       case ';':
+      case '#':
       case SCM_WHITE_SPACES:
       case SCM_LINE_INCREMENTORS:
       weird_handler:
@@ -355,9 +318,7 @@ scm_print_symbol_name (const char *str, size_t len, SCM port)
 	    weird = 1;
 	  }
 	if (pos < end)
-	  {
-	    scm_lfwrite (str + pos, end - pos, port);
-	  }
+	  scm_lfwrite (str + pos, end - pos, port);
 	{
 	  char buf[2];
 	  buf[0] = '\\';
@@ -375,11 +336,6 @@ scm_print_symbol_name (const char *str, size_t len, SCM port)
 	    mw_pos = pos;
 	  }
 	break;
-      case '}':
-      case '#':
-	if (weird)
-	  goto weird_handler;
-	break;
       default:
 	break;
       }
@@ -397,7 +353,6 @@ SCM_GPROC(s_display, "display", 1, 1, 0, scm_display, g_display);
 void 
 scm_iprin1 (SCM exp, SCM port, scm_print_state *pstate)
 {
-taloop:
   switch (SCM_ITAG3 (exp))
     {
     case scm_tc3_closure:
@@ -411,7 +366,7 @@ taloop:
       break;
     case scm_tc3_int_1:
     case scm_tc3_int_2:
-      scm_intprint (SCM_INUM (exp), 10, port);
+      scm_intprint (SCM_I_INUM (exp), 10, port);
       break;
     case scm_tc3_imm24:
       if (SCM_CHARP (exp))
@@ -436,14 +391,17 @@ taloop:
 	    scm_putc (i, port);
 	}
       else if (SCM_IFLAGP (exp)
-	       && ((size_t) SCM_ISYMNUM (exp) < (sizeof scm_isymnames / sizeof (char *))))
-	  scm_puts (SCM_ISYMCHARS (exp), port);
+	       && ((size_t) SCM_IFLAGNUM (exp) < (sizeof iflagnames / sizeof (char *))))
+        {
+          scm_puts (iflagnames [SCM_IFLAGNUM (exp)], port);
+        }
+      else if (SCM_ISYMP (exp))
+        {
+          scm_i_print_isym (exp, port);
+        }
       else if (SCM_ILOCP (exp))
 	{
-	  scm_puts ("#@", port);
-	  scm_intprint ((long) SCM_IFRAME (exp), 10, port);
-	  scm_putc (SCM_ICDRP (exp) ? '-' : '+', port);
-	  scm_intprint ((long) SCM_IDIST (exp), 10, port);
+          scm_i_print_iloc (exp, port);
 	}
       else
 	{
@@ -451,40 +409,29 @@ taloop:
 	  scm_ipruk ("immediate", exp, port);
 	}
       break;
-    case scm_tc3_cons_gloc:
-      /* gloc */
-      scm_puts ("#@", port);
-      exp = scm_module_reverse_lookup (scm_current_module (),
-				       SCM_GLOC_VAR (exp));
-      goto taloop;
     case scm_tc3_cons:
       switch (SCM_TYP7 (exp))
 	{
-	case scm_tcs_cons_gloc:
-
-	  if (SCM_STRUCT_VTABLE_DATA (exp) [scm_vtable_index_vcell] == 0)
-	    {
-	      ENTER_NESTED_DATA (pstate, exp, circref);
-	      if (SCM_OBJ_CLASS_FLAGS (exp) & SCM_CLASSF_GOOPS)
-		{
-		  SCM pwps, print = pstate->writingp ? g_write : g_display;
-		  if (!print)
-		    goto print_struct;
-		  SCM_NEWSMOB (pwps,
-			       scm_tc16_port_with_ps,
-			       SCM_UNPACK (scm_cons (port, pstate->handle)));
-		  pstate->revealed = 1;
-		  scm_call_generic_2 (print, exp, pwps);
-		}
-	      else
-		{
-		print_struct:
-		  scm_print_struct (exp, port, pstate);
-		}
-	      EXIT_NESTED_DATA (pstate);
-	      break;
-	    }
-
+	case scm_tcs_struct:
+	  {
+	    ENTER_NESTED_DATA (pstate, exp, circref);
+	    if (SCM_OBJ_CLASS_FLAGS (exp) & SCM_CLASSF_GOOPS)
+	      {
+		SCM pwps, print = pstate->writingp ? g_write : g_display;
+		if (!print)
+		  goto print_struct;
+		pwps = scm_i_port_with_print_state (port, pstate->handle);
+		pstate->revealed = 1;
+		scm_call_generic_2 (print, exp, pwps);
+	      }
+	    else
+	      {
+	      print_struct:
+		scm_print_struct (exp, port, pstate);
+	      }
+	    EXIT_NESTED_DATA (pstate);
+	  }
+	  break;
 	case scm_tcs_cons_imcar:
 	case scm_tcs_cons_nimcar:
 	  ENTER_NESTED_DATA (pstate, exp, circref);
@@ -495,8 +442,8 @@ taloop:
 	  print_circref (port, pstate, exp);
 	  break;
 	case scm_tcs_closures:
-	  if (SCM_FALSEP (scm_procedure_p (SCM_PRINT_CLOSURE))
-	      || SCM_FALSEP (scm_printer_apply (SCM_PRINT_CLOSURE,
+	  if (scm_is_false (scm_procedure_p (SCM_PRINT_CLOSURE))
+	      || scm_is_false (scm_printer_apply (SCM_PRINT_CLOSURE,
 						exp, port, pstate)))
 	    {
 	      SCM formals = SCM_CLOSURE_FORMALS (exp);
@@ -508,7 +455,7 @@ taloop:
 		{
 		  SCM env = SCM_ENV (exp);
 		  SCM xenv = SCM_EXTEND_ENV (formals, SCM_EOL, env);
-		  SCM src = scm_unmemocopy (SCM_CODE (exp), xenv);
+		  SCM src = scm_i_unmemocopy_body (SCM_CODE (exp), xenv);
 		  ENTER_NESTED_DATA (pstate, exp, circref);
 		  scm_iprin1 (src, port, pstate);
 		  EXIT_NESTED_DATA (pstate);
@@ -518,33 +465,78 @@ taloop:
 	      scm_putc ('>', port);
 	    }
 	  break;
-	case scm_tc7_substring:
+	case scm_tc7_number:
+          switch SCM_TYP16 (exp) {
+          case scm_tc16_big:
+            scm_bigprint (exp, port, pstate);
+            break;
+          case scm_tc16_real:
+            scm_print_real (exp, port, pstate);
+            break;
+          case scm_tc16_complex:
+            scm_print_complex (exp, port, pstate);
+            break;
+          case scm_tc16_fraction:
+            scm_i_print_fraction (exp, port, pstate);
+            break;
+          }
+	  break;
 	case scm_tc7_string:
 	  if (SCM_WRITINGP (pstate))
 	    {
-	      size_t i;
+	      size_t i, len;
+	      const char *data;
 
 	      scm_putc ('"', port);
-	      for (i = 0; i < SCM_STRING_LENGTH (exp); ++i)
-		switch (SCM_STRING_CHARS (exp)[i])
-		  {
-		  case '"':
-		  case '\\':
-		    scm_putc ('\\', port);
-		  default:
-		    scm_putc (SCM_STRING_CHARS (exp)[i], port);
-		  }
+	      len = scm_i_string_length (exp);
+	      data = scm_i_string_chars (exp);
+	      for (i = 0; i < len; ++i)
+		{
+		  unsigned char ch = data[i];
+		  if ((ch < 32 && ch != '\n') || (127 <= ch && ch < 148))
+		    {
+		      static char const hex[]="0123456789abcdef";
+		      scm_putc ('\\', port);
+		      scm_putc ('x', port);
+		      scm_putc (hex [ch / 16], port);
+		      scm_putc (hex [ch % 16], port);
+		    }
+		  else
+		    {
+		      if (ch == '"' || ch == '\\')
+			scm_putc ('\\', port);
+		      scm_putc (ch, port);
+		    }
+		}
 	      scm_putc ('"', port);
-	      break;
+	      scm_remember_upto_here_1 (exp);
 	    }
 	  else
-	    scm_lfwrite (SCM_STRING_CHARS (exp), SCM_STRING_LENGTH (exp), port);
+	    scm_lfwrite (scm_i_string_chars (exp), scm_i_string_length (exp),
+			 port);
+	  scm_remember_upto_here_1 (exp);
 	  break;
 	case scm_tc7_symbol:
-	  scm_print_symbol_name (SCM_SYMBOL_CHARS (exp),
-				 SCM_SYMBOL_LENGTH (exp),
-				 port);
-	  scm_remember_upto_here_1 (exp);
+	  if (scm_i_symbol_is_interned (exp))
+	    {
+	      scm_print_symbol_name (scm_i_symbol_chars (exp),
+				     scm_i_symbol_length (exp),
+				     port);
+	      scm_remember_upto_here_1 (exp);
+	    }
+	  else
+	    {
+	      scm_puts ("#<uninterned-symbol ", port);
+	      scm_print_symbol_name (scm_i_symbol_chars (exp),
+				     scm_i_symbol_length (exp),
+				     port);
+	      scm_putc (' ', port);
+	      scm_intprint ((long)exp, 16, port);
+	      scm_putc ('>', port);
+	    }
+	  break;
+	case scm_tc7_variable:
+	  scm_i_variable_print (exp, port, pstate);
 	  break;
 	case scm_tc7_wvect:
 	  ENTER_NESTED_DATA (pstate, exp, circref);
@@ -584,7 +576,7 @@ taloop:
 	  }
 	  EXIT_NESTED_DATA (pstate);
 	  break;
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
 	case scm_tc7_bvect:
 	case scm_tc7_byvect:
 	case scm_tc7_svect:
@@ -593,33 +585,33 @@ taloop:
 	case scm_tc7_fvect:
 	case scm_tc7_dvect:
 	case scm_tc7_cvect:
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
 	case scm_tc7_llvect:
 #endif
 	  scm_raprin1 (exp, port, pstate);
 	  break;
 #endif
 	case scm_tcs_subrs:
-	  scm_puts (SCM_SUBR_GENERIC (exp) && *SCM_SUBR_GENERIC (exp)
+	  scm_puts (SCM_SUBR_GENERIC (exp)
 		    ? "#<primitive-generic "
 		    : "#<primitive-procedure ",
 		    port);
-	  scm_puts (SCM_SYMBOL_CHARS (SCM_SNAME (exp)), port);
+	  scm_puts (scm_i_symbol_chars (SCM_SNAME (exp)), port);
 	  scm_putc ('>', port);
 	  break;
 #ifdef CCLO
 	case scm_tc7_cclo:
 	  {
 	    SCM proc = SCM_CCLO_SUBR (exp);
-	    if (SCM_EQ_P (proc, scm_f_gsubr_apply))
+	    if (scm_is_eq (proc, scm_f_gsubr_apply))
 	      {
 		/* Print gsubrs as primitives */
 		SCM name = scm_procedure_name (exp);
 		scm_puts ("#<primitive-procedure", port);
-		if (SCM_NFALSEP (name))
+		if (scm_is_true (name))
 		  {
 		    scm_putc (' ', port);
-		    scm_puts (SCM_SYMBOL_CHARS (name), port);
+		    scm_puts (scm_i_symbol_chars (name), port);
 		  }
 	      }
 	    else
@@ -635,7 +627,7 @@ taloop:
 	  scm_puts ("#<procedure-with-setter", port);
 	  {
 	    SCM name = scm_procedure_name (exp);
-	    if (SCM_NFALSEP (name))
+	    if (scm_is_true (name))
 	      {
 		scm_putc (' ', port);
 		scm_display (name, port);
@@ -692,15 +684,15 @@ scm_prin1 (SCM exp, SCM port, int writingp)
   else
     {
       /* First try to allocate a print state from the pool */
-      SCM_DEFER_INTS;
-      if (SCM_NNULLP (SCM_CDR (print_state_pool)))
+      scm_i_plugin_mutex_lock (&print_state_mutex);
+      if (!SCM_NULLP (print_state_pool))
 	{
-	  handle = SCM_CDR (print_state_pool);
-	  SCM_SETCDR (print_state_pool, SCM_CDDR (print_state_pool));
+	  handle = print_state_pool;
+	  print_state_pool = SCM_CDR (print_state_pool);
 	}
-      SCM_ALLOW_INTS;
-      if (SCM_FALSEP (handle))
-	handle = scm_cons (make_print_state (), SCM_EOL);
+      scm_i_plugin_mutex_unlock (&print_state_mutex);
+      if (scm_is_false (handle))
+	handle = scm_list_1 (make_print_state ());
       pstate_scm = SCM_CAR (handle);
     }
 
@@ -713,12 +705,12 @@ scm_prin1 (SCM exp, SCM port, int writingp)
   /* Return print state to pool if it has been created above and
      hasn't escaped to Scheme. */
 
-  if (!SCM_FALSEP (handle) && !pstate->revealed)
+  if (scm_is_true (handle) && !pstate->revealed)
     {
-      SCM_DEFER_INTS;
-      SCM_SETCDR (handle, SCM_CDR (print_state_pool));
-      SCM_SETCDR (print_state_pool, handle);
-      SCM_ALLOW_INTS;
+      scm_i_plugin_mutex_lock (&print_state_mutex);
+      SCM_SETCDR (handle, print_state_pool);
+      print_state_pool = handle;
+      scm_i_plugin_mutex_unlock (&print_state_mutex);
     }
 }
 
@@ -741,7 +733,7 @@ scm_ipruk (char *hdr, SCM ptr, SCM port)
 {
   scm_puts ("#<unknown-", port);
   scm_puts (hdr, port);
-  if (SCM_CELLP (ptr))
+  if (scm_in_heap_p (ptr))
     {
       scm_puts (" (0x", port);
       scm_intprint (SCM_CELL_WORD_0 (ptr), 16, port);
@@ -755,12 +747,10 @@ scm_ipruk (char *hdr, SCM ptr, SCM port)
 }
 
 
-/* Print a list.  The list may be either a list of ordinary data, or it may be
-   a list that represents code.  Lists that represent code may contain gloc
-   cells.
+/* Print a list.
  */
 void 
-scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
+scm_iprlist (char *hdr, SCM exp, int tlr, SCM port, scm_print_state *pstate)
 {
   register SCM hare, tortoise;
   long floor = pstate->top - 2;
@@ -773,12 +763,12 @@ scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
      O(depth * N) instead of O(N^2). */
   hare = SCM_CDR (exp);
   tortoise = exp;
-  while (SCM_ECONSP (hare))
+  while (SCM_CONSP (hare))
     {
-      if (SCM_EQ_P (hare, tortoise))
+      if (scm_is_eq (hare, tortoise))
 	goto fancy_printing;
       hare = SCM_CDR (hare);
-      if (SCM_IMP (hare) || SCM_NECONSP (hare))
+      if (!SCM_CONSP (hare))
 	break;
       hare = SCM_CDR (hare);
       tortoise = SCM_CDR (tortoise);
@@ -786,19 +776,19 @@ scm_iprlist (char *hdr,SCM exp,int tlr,SCM port,scm_print_state *pstate)
   
   /* No cdr cycles intrinsic to this list */
   scm_iprin1 (SCM_CAR (exp), port, pstate);
-  for (exp = SCM_CDR (exp); SCM_ECONSP (exp); exp = SCM_CDR (exp))
+  for (exp = SCM_CDR (exp); SCM_CONSP (exp); exp = SCM_CDR (exp))
     {
       register long i;
 
       for (i = floor; i >= 0; --i)
-	if (SCM_EQ_P (pstate->ref_stack[i], exp))
+	if (scm_is_eq (pstate->ref_stack[i], exp))
 	  goto circref;
       PUSH_REF (pstate, exp);
       scm_putc (' ', port);
       /* CHECK_INTS; */
       scm_iprin1 (SCM_CAR (exp), port, pstate);
     }
-  if (!SCM_NULLP (exp))
+  if (!SCM_NULL_OR_NIL_P (exp))
     {
       scm_puts (" . ", port);
       scm_iprin1 (exp, port, pstate);
@@ -815,12 +805,12 @@ fancy_printing:
     
     scm_iprin1 (SCM_CAR (exp), port, pstate);
     exp = SCM_CDR (exp); --n;
-    for (; SCM_ECONSP (exp); exp = SCM_CDR (exp))
+    for (; SCM_CONSP (exp); exp = SCM_CDR (exp))
       {
 	register unsigned long i;
 
 	for (i = 0; i < pstate->top; ++i)
-	  if (SCM_EQ_P (pstate->ref_stack[i], exp))
+	  if (scm_is_eq (pstate->ref_stack[i], exp))
 	    goto fancy_circref;
 	if (pstate->fancyp)
 	  {
@@ -839,7 +829,7 @@ fancy_printing:
 	scm_iprin1 (SCM_CAR (exp), port, pstate);
       }
   }
-  if (SCM_NNULLP (exp))
+  if (!SCM_NULL_OR_NIL_P (exp))
     {
       scm_puts (" . ", port);
       scm_iprin1 (exp, port, pstate);
@@ -924,35 +914,36 @@ SCM_DEFINE (scm_simple_format, "simple-format", 2, 0, 1,
 	    "containing the formatted text. Does not add a trailing newline.")
 #define FUNC_NAME s_scm_simple_format
 {
-  SCM answer = SCM_UNSPECIFIED;
+  SCM port, answer = SCM_UNSPECIFIED;
   int fReturnString = 0;
   int writingp;
-  char *start;
-  char *end;
-  char *p;
+  const char *start;
+  const char *end;
+  const char *p;
 
-  if (SCM_EQ_P (destination, SCM_BOOL_T))
+  if (scm_is_eq (destination, SCM_BOOL_T))
     {
-      destination = scm_cur_outp;
+      destination = port = scm_cur_outp;
     }
-  else if (SCM_FALSEP (destination))
+  else if (scm_is_false (destination))
     {
       fReturnString = 1;
-      destination = scm_mkstrport (SCM_INUM0, 
-				   scm_make_string (SCM_INUM0, SCM_UNDEFINED),
-				   SCM_OPN | SCM_WRTNG,
-				   FUNC_NAME);
+      port = scm_mkstrport (SCM_INUM0, 
+			    scm_make_string (SCM_INUM0, SCM_UNDEFINED),
+			    SCM_OPN | SCM_WRTNG,
+			    FUNC_NAME);
+      destination = port;
     }
   else
     {
       SCM_VALIDATE_OPORT_VALUE (1, destination);
-      destination = SCM_COERCE_OUTPORT (destination);
+      port = SCM_COERCE_OUTPORT (destination);
     }
   SCM_VALIDATE_STRING (2, message);
   SCM_VALIDATE_REST_ARGUMENT (args);
 
-  start = SCM_STRING_CHARS (message);
-  end = start + SCM_STRING_LENGTH (message);
+  start = scm_i_string_chars (message);
+  end = start + scm_i_string_length (message);
   for (p = start; p != end; ++p)
     if (*p == '~')
       {
@@ -968,12 +959,12 @@ SCM_DEFINE (scm_simple_format, "simple-format", 2, 0, 1,
 	    writingp = 1;
 	    break;
 	  case '~':
-	    scm_lfwrite (start, p - start, destination);
+	    scm_lfwrite (start, p - start, port);
 	    start = p + 1;
 	    continue;
 	  case '%':
-	    scm_lfwrite (start, p - start - 1, destination);
-	    scm_newline (destination);
+	    scm_lfwrite (start, p - start - 1, port);
+	    scm_newline (port);
 	    start = p + 1;
 	    continue;
 	  default:
@@ -987,14 +978,15 @@ SCM_DEFINE (scm_simple_format, "simple-format", 2, 0, 1,
 	  SCM_MISC_ERROR ("FORMAT: Missing argument for ~~~A",
 			  scm_list_1 (SCM_MAKE_CHAR (*p)));
 			  		
-	scm_lfwrite (start, p - start - 1, destination);
+	scm_lfwrite (start, p - start - 1, port);
+	/* we pass destination here */
 	scm_prin1 (SCM_CAR (args), destination, writingp);
 	args = SCM_CDR (args);
 	start = p + 1;
       }
 
-  scm_lfwrite (start, p - start, destination);
-  if (args != SCM_EOL)
+  scm_lfwrite (start, p - start, port);
+  if (!scm_is_eq (args, SCM_EOL))
     SCM_MISC_ERROR ("FORMAT: ~A superfluous arguments",
 		    scm_list_1 (scm_length (args)));
 
@@ -1015,7 +1007,7 @@ SCM_DEFINE (scm_newline, "newline", 0, 1, 0,
   if (SCM_UNBNDP (port))
     port = scm_cur_outp;
 
-  SCM_VALIDATE_OPORT_VALUE (1,port);
+  SCM_VALIDATE_OPORT_VALUE (1, port);
 
   scm_putc ('\n', SCM_COERCE_OUTPORT (port));
   return SCM_UNSPECIFIED;
@@ -1030,8 +1022,8 @@ SCM_DEFINE (scm_write_char, "write-char", 1, 1, 0,
   if (SCM_UNBNDP (port))
     port = scm_cur_outp;
 
-  SCM_VALIDATE_CHAR (1,chr);
-  SCM_VALIDATE_OPORT_VALUE (2,port);
+  SCM_VALIDATE_CHAR (1, chr);
+  SCM_VALIDATE_OPORT_VALUE (2, port);
 
   scm_putc ((int) SCM_CHAR (chr), SCM_COERCE_OUTPORT (port));
 #ifdef HAVE_PIPE
@@ -1068,25 +1060,23 @@ port_with_ps_print (SCM obj, SCM port, scm_print_state *pstate)
 SCM
 scm_printer_apply (SCM proc, SCM exp, SCM port, scm_print_state *pstate)
 {
-  SCM pwps;
-  SCM pair = scm_cons (port, pstate->handle);
-  SCM_NEWSMOB (pwps, scm_tc16_port_with_ps, SCM_UNPACK (pair));
   pstate->revealed = 1;
-  return scm_call_2 (proc, exp, pwps);
+  return scm_call_2 (proc, exp,
+		     scm_i_port_with_print_state (port, pstate->handle));
 }
 
-SCM_DEFINE (scm_port_with_print_state, "port-with-print-state", 2, 0, 0, 
+SCM_DEFINE (scm_port_with_print_state, "port-with-print-state", 1, 1, 0, 
             (SCM port, SCM pstate),
 	    "Create a new port which behaves like @var{port}, but with an\n"
-	    "included print state @var{pstate}.")
+	    "included print state @var{pstate}.  @var{pstate} is optional.\n"
+	    "If @var{pstate} isn't supplied and @var{port} already has\n"
+	    "a print state, the old print state is reused.")
 #define FUNC_NAME s_scm_port_with_print_state
 {
-  SCM pwps;
-  SCM_VALIDATE_OPORT_VALUE (1,port);
-  SCM_VALIDATE_PRINTSTATE (2,pstate);
-  port = SCM_COERCE_OUTPORT (port);
-  SCM_NEWSMOB (pwps, scm_tc16_port_with_ps, SCM_UNPACK (scm_cons (port, pstate)));
-  return pwps;
+  SCM_VALIDATE_OPORT_VALUE (1, port);
+  if (!SCM_UNBNDP (pstate))
+    SCM_VALIDATE_PRINTSTATE (2, pstate);
+  return scm_i_port_with_print_state (port, pstate);
 }
 #undef FUNC_NAME
 
@@ -1110,14 +1100,16 @@ void
 scm_init_print ()
 {
   SCM vtable, layout, type;
-  
-  scm_init_opts (scm_print_options, scm_print_opts, SCM_N_PRINT_OPTIONS);
-  vtable = scm_make_vtable_vtable (scm_nullstr, SCM_INUM0, SCM_EOL);
-  layout = scm_make_struct_layout (scm_makfrom0str (SCM_PRINT_STATE_LAYOUT));
-  type = scm_make_struct (vtable, SCM_INUM0, scm_list_1 (layout));
-  scm_set_struct_vtable_name_x (type, scm_str2symbol ("print-state"));
-  print_state_pool = scm_permanent_object (scm_cons (type, SCM_EOL));
 
+  scm_init_opts (scm_print_options, scm_print_opts, SCM_N_PRINT_OPTIONS);
+
+  scm_gc_register_root (&print_state_pool);
+  scm_gc_register_root (&scm_print_state_vtable);
+  vtable = scm_make_vtable_vtable (scm_nullstr, SCM_INUM0, SCM_EOL);
+  layout =
+    scm_make_struct_layout (scm_from_locale_string (SCM_PRINT_STATE_LAYOUT));
+  type = scm_make_struct (vtable, SCM_INUM0, scm_list_1 (layout));
+  scm_set_struct_vtable_name_x (type, scm_from_locale_symbol ("print-state"));
   scm_print_state_vtable = type;
 
   /* Don't want to bind a wrapper class in GOOPS, so pass 0 as arg1. */
