@@ -1,43 +1,19 @@
-/* Copyright (C) 1995,1996,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,2000,2001,2003,2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -77,6 +53,7 @@ scm_i_procedure_arity (SCM proc)
     case scm_tc7_subr_2o:
       o = 1;
     case scm_tc7_subr_1:
+    case scm_tc7_dsubr:
     case scm_tc7_cxr:
       a += 1;
       break;
@@ -109,9 +86,9 @@ scm_i_procedure_arity (SCM proc)
 	  return SCM_BOOL_F;
 	}
     case scm_tc7_cclo:
-      if (SCM_EQ_P (SCM_CCLO_SUBR (proc), scm_f_gsubr_apply))
+      if (scm_is_eq (SCM_CCLO_SUBR (proc), scm_f_gsubr_apply))
 	{
-	  int type = SCM_INUM (SCM_GSUBR_TYPE (proc));
+	  int type = scm_to_int (SCM_GSUBR_TYPE (proc));
 	  a += SCM_GSUBR_REQ (type);
 	  o = SCM_GSUBR_OPT (type);
 	  r = SCM_GSUBR_REST (type);
@@ -138,7 +115,7 @@ scm_i_procedure_arity (SCM proc)
       if (!SCM_NULLP (proc))
 	r = 1;
       break;
-    case scm_tcs_cons_gloc:
+    case scm_tcs_struct:
       if (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC)
 	{
 	  r = 1;
@@ -154,22 +131,27 @@ scm_i_procedure_arity (SCM proc)
     default:
       return SCM_BOOL_F;
     }
-  return scm_list_3 (SCM_MAKINUM (a), SCM_MAKINUM (o), SCM_BOOL(r));
+  return scm_list_3 (scm_from_int (a), scm_from_int (o), scm_from_bool(r));
 }
 
+/* XXX - instead of using a stand-in value for everything except
+   closures, we should find other ways to store the procedure
+   properties for those other kinds of procedures.  For example, subrs
+   have their own property slot, which is unused at present.
+*/
+
 static SCM
 scm_stand_in_scm_proc(SCM proc)
 {
   SCM handle, answer;
   handle = scm_hashq_get_handle (scm_stand_in_procs, proc);
-  if (SCM_FALSEP (handle))
+  if (scm_is_false (handle))
     {
       answer = scm_closure (scm_list_2 (SCM_EOL, SCM_BOOL_F), SCM_EOL);
       scm_hashq_set_x (scm_stand_in_procs, proc, answer);
     }
-  else {
-    answer = SCM_CDR(handle);
-  }
+  else
+    answer = SCM_CDR (handle);
   return answer;
 }
 
@@ -178,7 +160,7 @@ SCM_DEFINE (scm_procedure_properties, "procedure-properties", 1, 0, 0,
 	    "Return @var{obj}'s property list.")
 #define FUNC_NAME s_scm_procedure_properties
 {
-  SCM_VALIDATE_PROC (1,proc);
+  SCM_VALIDATE_PROC (1, proc);
   return scm_acons (scm_sym_arity, scm_i_procedure_arity (proc),
 		    SCM_PROCPROPS (SCM_CLOSUREP (proc)
 				   ? proc
@@ -193,7 +175,7 @@ SCM_DEFINE (scm_set_procedure_properties_x, "set-procedure-properties!", 2, 0, 0
 {
   if (!SCM_CLOSUREP (proc))
     proc = scm_stand_in_scm_proc(proc);
-  SCM_VALIDATE_CLOSURE (1,proc);
+  SCM_VALIDATE_CLOSURE (1, proc);
   SCM_SETPROCPROPS (proc, new_val);
   return SCM_UNSPECIFIED;
 }
@@ -205,14 +187,14 @@ SCM_DEFINE (scm_procedure_property, "procedure-property", 2, 0, 0,
 #define FUNC_NAME s_scm_procedure_property
 {
   SCM assoc;
-  if (SCM_EQ_P (k, scm_sym_arity))
+  if (scm_is_eq (k, scm_sym_arity))
     {
       SCM arity;
-      SCM_ASSERT (SCM_NFALSEP (arity = scm_i_procedure_arity (p)),
+      SCM_ASSERT (scm_is_true (arity = scm_i_procedure_arity (p)),
 		  p, SCM_ARG1, FUNC_NAME);
       return arity;
     }
-  SCM_VALIDATE_PROC (1,p);
+  SCM_VALIDATE_PROC (1, p);
   assoc = scm_sloppy_assq (k,
 			   SCM_PROCPROPS (SCM_CLOSUREP (p)
 					  ? p
@@ -230,8 +212,8 @@ SCM_DEFINE (scm_set_procedure_property_x, "set-procedure-property!", 3, 0, 0,
   SCM assoc;
   if (!SCM_CLOSUREP (p))
     p = scm_stand_in_scm_proc(p);
-  SCM_VALIDATE_CLOSURE (1,p);
-  if (SCM_EQ_P (k, scm_sym_arity))
+  SCM_VALIDATE_CLOSURE (1, p);
+  if (scm_is_eq (k, scm_sym_arity))
     SCM_MISC_ERROR ("arity is a read-only property", SCM_EOL);
   assoc = scm_sloppy_assq (k, SCM_PROCPROPS (p));
   if (SCM_NIMP (assoc))
