@@ -1,43 +1,19 @@
-/* Copyright (C) 1996,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1998,2000,2001,2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 /*
@@ -207,7 +183,7 @@ scm_ra_matchp (SCM ra0, SCM ras)
     case scm_tc7_uvect:
     case scm_tc7_ivect:
     case scm_tc7_svect:
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
     case scm_tc7_llvect:
 #endif
     case scm_tc7_fvect:
@@ -215,7 +191,7 @@ scm_ra_matchp (SCM ra0, SCM ras)
     case scm_tc7_cvect:
       s0->lbnd = 0;
       s0->inc = 1;
-      s0->ubnd = SCM_INUM (scm_uniform_vector_length (ra0)) - 1;
+      s0->ubnd = scm_to_long (scm_uniform_vector_length (ra0)) - 1;
       break;
     case scm_tc7_smob:
       if (!SCM_ARRAYP (ra0))
@@ -243,7 +219,7 @@ scm_ra_matchp (SCM ra0, SCM ras)
 	case scm_tc7_uvect:
 	case scm_tc7_ivect:
 	case scm_tc7_svect:
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
 	case scm_tc7_llvect:
 #endif
 	case scm_tc7_fvect:
@@ -255,7 +231,7 @@ scm_ra_matchp (SCM ra0, SCM ras)
 	    if (1 != ndim)
 	      return 0;
 
-	    length = SCM_INUM (scm_uniform_vector_length (ra1));
+	    length = scm_to_ulong (scm_uniform_vector_length (ra1));
 
 	    switch (exact)
 	      {
@@ -323,7 +299,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
     {
     default:
     case 0:
-      scm_misc_error (what, "array shape mismatch: ~S", ra0);
+      scm_misc_error (what, "array shape mismatch: ~S", scm_list_1 (ra0));
     case 2:
     case 3:
     case 4:			/* Try unrolling arrays */
@@ -334,7 +310,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
       if (SCM_IMP (vra0)) goto gencase;
       if (!SCM_ARRAYP (vra0))
 	{
-	  unsigned long int length = SCM_INUM (scm_uniform_vector_length (vra0));
+	  unsigned long int length = scm_to_ulong (scm_uniform_vector_length (vra0));
 	  vra1 = scm_make_ra (1);
 	  SCM_ARRAY_BASE (vra1) = 0;
 	  SCM_ARRAY_DIMS (vra1)->lbnd = 0;
@@ -392,7 +368,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
       }
     else
       {
-	unsigned long int length = SCM_INUM (scm_uniform_vector_length (ra0));
+	unsigned long length = scm_to_ulong (scm_uniform_vector_length (ra0));
 	kmax = 0;
 	SCM_ARRAY_DIMS (vra0)->lbnd = 0;
 	SCM_ARRAY_DIMS (vra0)->ubnd = length - 1;
@@ -423,7 +399,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
 	*plvra = scm_cons (vra1, SCM_EOL);
 	plvra = SCM_CDRLOC (*plvra);
       }
-    inds = scm_make_uve (SCM_ARRAY_NDIM (ra0), SCM_MAKINUM (-1L));
+    inds = scm_make_uve (SCM_ARRAY_NDIM (ra0), scm_from_int (-1));
     vinds = (long *) SCM_VELTS (inds);
     for (k = 0; k <= kmax; k++)
       vinds[k] = SCM_ARRAY_DIMS (ra0)[k].lbnd;
@@ -483,26 +459,30 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
     {
     default:
       for (i = base; n--; i += inc)
-	scm_array_set_x (ra, fill, SCM_MAKINUM (i));
+	scm_array_set_x (ra, fill, scm_from_ulong (i));
       break;
     case scm_tc7_vector:
     case scm_tc7_wvect:
       for (i = base; n--; i += inc)
-	SCM_VELTS (ra)[i] = fill;
+	SCM_VECTOR_SET (ra, i, fill);
       break;
     case scm_tc7_string:
       SCM_ASRTGO (SCM_CHARP (fill), badarg2);
-      for (i = base; n--; i += inc)
-	SCM_STRING_CHARS (ra)[i] = SCM_CHAR (fill);
+      {
+	char *data = scm_i_string_writable_chars (ra);
+	for (i = base; n--; i += inc)
+	  data[i] = SCM_CHAR (fill);
+	scm_i_string_stop_writing ();
+      }
       break;
     case scm_tc7_byvect:
       if (SCM_CHARP (fill))
-	fill = SCM_MAKINUM ((signed char) SCM_CHAR (fill));
-      SCM_ASRTGO (SCM_INUMP (fill), badarg2);
+	fill = SCM_I_MAKINUM ((signed char) SCM_CHAR (fill));
+      SCM_ASRTGO (SCM_I_INUMP (fill), badarg2);
       SCM_ASSERT_RANGE (SCM_ARG2, fill,
-                        -128 <= SCM_INUM (fill) && SCM_INUM (fill) < 128);
+                        -128 <= SCM_I_INUM (fill) && SCM_I_INUM (fill) < 128);
       for (i = base; n--; i += inc)
-	((char *) SCM_UVECTOR_BASE (ra))[i] = SCM_INUM (fill);
+	((char *) SCM_UVECTOR_BASE (ra))[i] = SCM_I_INUM (fill);
       break;
     case scm_tc7_bvect:
       { /* scope */
@@ -510,7 +490,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
 	if (1 == inc && (n >= SCM_LONG_BIT || n == SCM_BITVECTOR_LENGTH (ra)))
 	  {
 	    i = base / SCM_LONG_BIT;
-	    if (SCM_FALSEP (fill))
+	    if (scm_is_false (fill))
 	      {
 		if (base % SCM_LONG_BIT) /* leading partial word */
 		  ve[i++] &= ~(~0L << (base % SCM_LONG_BIT));
@@ -519,7 +499,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
 		if ((base + n) % SCM_LONG_BIT) /* trailing partial word */
 		  ve[i] &= (~0L << ((base + n) % SCM_LONG_BIT));
 	      }
-	    else if (SCM_EQ_P (fill, SCM_BOOL_T))
+	    else if (scm_is_eq (fill, SCM_BOOL_T))
 	      {
 		if (base % SCM_LONG_BIT)
 		  ve[i++] |= ~0L << (base % SCM_LONG_BIT);
@@ -533,10 +513,10 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
 	  }
 	else
 	  {
-	    if (SCM_FALSEP (fill))
+	    if (scm_is_false (fill))
 	      for (i = base; n--; i += inc)
 		ve[i / SCM_LONG_BIT] &= ~(1L << (i % SCM_LONG_BIT));
-	    else if (SCM_EQ_P (fill, SCM_BOOL_T))
+	    else if (scm_is_eq (fill, SCM_BOOL_T))
 	      for (i = base; n--; i += inc)
 		ve[i / SCM_LONG_BIT] |= (1L << (i % SCM_LONG_BIT));
 	    else
@@ -563,18 +543,18 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
 	break;
       }
     case scm_tc7_svect:
-      SCM_ASRTGO (SCM_INUMP (fill), badarg2);
+      SCM_ASRTGO (SCM_I_INUMP (fill), badarg2);
       { /* scope */
-	short f = SCM_INUM (fill);
+	short f = SCM_I_INUM (fill);
 	short *ve = (short *) SCM_VELTS (ra);
 
-	if (f != SCM_INUM (fill))
+	if (f != SCM_I_INUM (fill))
 	  SCM_OUT_OF_RANGE (2, fill);
 	for (i = base; n--; i += inc)
 	  ve[i] = f;
 	break;
       }
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
     case scm_tc7_llvect:
       { /* scope */
 	long long f = SCM_NUM2LONG_LONG (2, fill);
@@ -588,8 +568,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
     case scm_tc7_fvect:
       { /* scope */
 	float f, *ve = (float *) SCM_VELTS (ra);
-	SCM_ASRTGO (SCM_REALP (fill), badarg2);
-	f = SCM_REAL_VALUE (fill);
+        f = (float) scm_to_double (fill);
 	for (i = base; n--; i += inc)
 	  ve[i] = f;
 	break;
@@ -597,8 +576,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
     case scm_tc7_dvect:
       { /* scope */
 	double f, *ve = (double *) SCM_VELTS (ra);
-	SCM_ASRTGO (SCM_REALP (fill), badarg2);
-	f = SCM_REAL_VALUE (fill);
+        f = scm_to_double (fill);
 	for (i = base; n--; i += inc)
 	  ve[i] = f;
 	break;
@@ -651,13 +629,18 @@ racp (SCM src, SCM dst)
       for (; n-- > 0; i_s += inc_s, i_d += inc_d)
 	scm_array_set_x (dst,
 			 scm_cvref (src, i_s, SCM_UNDEFINED),
-			 SCM_MAKINUM (i_d));
+			 scm_from_ulong (i_d));
       break;
     case scm_tc7_string:
       if (SCM_TYP7 (src) != scm_tc7_string)
 	goto gencase;
-      for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	SCM_STRING_CHARS (dst)[i_d] = SCM_STRING_CHARS (src)[i_s];
+      {
+	char *dst_data = scm_i_string_writable_chars (dst);
+	const char *src_data = scm_i_string_chars (src);
+	for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	  dst_data[i_d] = src_data[i_s];
+	scm_i_string_stop_writing ();
+      }
       break;
     case scm_tc7_byvect:
       if (SCM_TYP7 (src) != scm_tc7_byvect)
@@ -863,7 +846,7 @@ scm_ra_eqp (SCM ra0, SCM ras)
 	SCM e1 = SCM_UNDEFINED, e2 = SCM_UNDEFINED;
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  if (SCM_BITVEC_REF (ra0, i0))
-	    if (SCM_FALSEP(scm_eq_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
+	    if (scm_is_false(scm_eq_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
 	      SCM_BITVEC_CLR (ra0, i0);
 	break;
       }
@@ -905,7 +888,7 @@ scm_ra_eqp (SCM ra0, SCM ras)
 /* opt 0 means <, nonzero means >= */
 
 static int
-ra_compare (SCM ra0,SCM ra1,SCM ra2,int opt)
+ra_compare (SCM ra0, SCM ra1, SCM ra2, int opt)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
   unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
@@ -923,8 +906,8 @@ ra_compare (SCM ra0,SCM ra1,SCM ra2,int opt)
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  if (SCM_BITVEC_REF (ra0, i0))
 	    if (opt ?
-		SCM_NFALSEP (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))) :
-		SCM_FALSEP (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
+		scm_is_true (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))) :
+		scm_is_false (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
 	      SCM_BITVEC_CLR (ra0, i0);
 	break;
       }
@@ -1005,7 +988,7 @@ scm_ra_sum (SCM ra0, SCM ras)
   unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if (SCM_NNULLP(ras))
+  if (!SCM_NULLP(ras))
     {
       SCM ra1 = SCM_CAR (ras);
       unsigned long i1 = SCM_ARRAY_BASE (ra1);
@@ -1018,7 +1001,7 @@ scm_ra_sum (SCM ra0, SCM ras)
 	    SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
 	    for (; n-- > 0; i0 += inc0, i1 += inc1)
 	      scm_array_set_x (ra0, scm_sum (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)),
-			       SCM_MAKINUM (i0));
+			       scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_uvect:
@@ -1054,7 +1037,7 @@ scm_ra_difference (SCM ra0, SCM ras)
 	    for (; n-- > 0; i0 += inc0)
 	      scm_array_set_x (ra0, 
                                scm_difference (RVREF (ra0, i0, e0), SCM_UNDEFINED), 
-                               SCM_MAKINUM (i0));
+                               scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_fvect:
@@ -1077,7 +1060,7 @@ scm_ra_difference (SCM ra0, SCM ras)
 	  {
 	    SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
 	    for (; n-- > 0; i0 += inc0, i1 += inc1)
-	      scm_array_set_x (ra0, scm_difference (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)), SCM_MAKINUM (i0));
+	      scm_array_set_x (ra0, scm_difference (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)), scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_fvect:
@@ -1100,7 +1083,7 @@ scm_ra_product (SCM ra0, SCM ras)
   unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if (SCM_NNULLP (ras))
+  if (!SCM_NULLP (ras))
     {
       SCM ra1 = SCM_CAR (ras);
       unsigned long i1 = SCM_ARRAY_BASE (ra1);
@@ -1113,7 +1096,7 @@ scm_ra_product (SCM ra0, SCM ras)
 	    SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
 	    for (; n-- > 0; i0 += inc0, i1 += inc1)
 	      scm_array_set_x (ra0, scm_product (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)),
-			       SCM_MAKINUM (i0));
+			       scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_uvect:
@@ -1159,7 +1142,7 @@ scm_ra_divide (SCM ra0, SCM ras)
 	  {
 	    SCM e0 = SCM_UNDEFINED;
 	    for (; n-- > 0; i0 += inc0)
-	      scm_array_set_x (ra0, scm_divide (RVREF (ra0, i0, e0), SCM_UNDEFINED), SCM_MAKINUM (i0));
+	      scm_array_set_x (ra0, scm_divide (RVREF (ra0, i0, e0), SCM_UNDEFINED), scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_fvect:
@@ -1192,7 +1175,7 @@ scm_ra_divide (SCM ra0, SCM ras)
 	  {
 	    SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
 	    for (; n-- > 0; i0 += inc0, i1 += inc1)
-	      scm_array_set_x (ra0, scm_divide (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)), SCM_MAKINUM (i0));
+	      scm_array_set_x (ra0, scm_divide (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)), scm_from_ulong (i0));
 	    break;
 	  }
 	case scm_tc7_fvect:
@@ -1230,7 +1213,7 @@ scm_array_identity (SCM dst, SCM src)
 
 
 static int 
-ramap (SCM ra0,SCM proc,SCM ras)
+ramap (SCM ra0, SCM proc, SCM ras)
 {
   long i = SCM_ARRAY_DIMS (ra0)->lbnd;
   long inc = SCM_ARRAY_DIMS (ra0)->inc;
@@ -1239,11 +1222,12 @@ ramap (SCM ra0,SCM proc,SCM ras)
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
     for (; i <= n; i++)
-      scm_array_set_x (ra0, scm_call_0 (proc), SCM_MAKINUM (i * inc + base));
+      scm_array_set_x (ra0, scm_call_0 (proc), scm_from_long (i * inc + base));
   else
     {
       SCM ra1 = SCM_CAR (ras);
-      SCM args, *ve = &ras;
+      SCM args;
+      SCM const *ve = &ras;
       unsigned long k, i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
@@ -1255,13 +1239,14 @@ ramap (SCM ra0,SCM proc,SCM ras)
 	  ras = scm_vector (ras);
 	  ve = SCM_VELTS (ras);
 	}
+      
       for (; i <= n; i++, i1 += inc1)
 	{
 	  args = SCM_EOL;
-	  for (k = SCM_INUM (scm_uniform_vector_length (ras)); k--;)
-	    args = scm_cons (scm_uniform_vector_ref (ve[k], SCM_MAKINUM (i)), args);
+	  for (k = scm_to_ulong (scm_uniform_vector_length (ras)); k--;)
+	    args = scm_cons (scm_uniform_vector_ref (ve[k], scm_from_long (i)), args);
 	  args = scm_cons (scm_cvref (ra1, i1, SCM_UNDEFINED), args);
-	  scm_array_set_x (ra0, scm_apply_0 (proc, args), SCM_MAKINUM (i * inc + base));
+	  scm_array_set_x (ra0, scm_apply_0 (proc, args), scm_from_long (i * inc + base));
 	}
     }
   return 1;
@@ -1269,7 +1254,7 @@ ramap (SCM ra0,SCM proc,SCM ras)
 
 
 static int
-ramap_cxr (SCM ra0,SCM proc,SCM ras)
+ramap_dsubr (SCM ra0, SCM proc, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
@@ -1283,7 +1268,7 @@ ramap_cxr (SCM ra0,SCM proc,SCM ras)
     default:
     gencase:
  for (; n-- > 0; i0 += inc0, i1 += inc1)
-   scm_array_set_x (ra0, scm_call_1 (proc, RVREF (ra1, i1, e1)), SCM_MAKINUM (i0));
+   scm_array_set_x (ra0, scm_call_1 (proc, RVREF (ra1, i1, e1)), scm_from_ulong (i0));
  break;
     case scm_tc7_fvect:
       {
@@ -1330,7 +1315,7 @@ ramap_cxr (SCM ra0,SCM proc,SCM ras)
 
 
 static int
-ramap_rp (SCM ra0,SCM proc,SCM ras)
+ramap_rp (SCM ra0, SCM proc, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras), ra2 = SCM_CAR (SCM_CDR (ras));
   SCM e1 = SCM_UNDEFINED, e2 = SCM_UNDEFINED;
@@ -1347,7 +1332,7 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
     default:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	if (SCM_BITVEC_REF  (ra0, i0))
-	  if (SCM_FALSEP (SCM_SUBRF (proc) (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
+	  if (scm_is_false (SCM_SUBRF (proc) (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
 	    SCM_BITVEC_CLR (ra0, i0);
       break;
     case scm_tc7_uvect:
@@ -1356,46 +1341,45 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
 	if (SCM_BITVEC_REF (ra0, i0))
 	  {
 	    /* DIRK:FIXME:: There should be a way to access the elements
-	       of a cell as raw data.  Further:  How can we be sure that
-	       the values fit into an inum?
+	       of a cell as raw data.
 	     */
-	    SCM n1 = SCM_MAKINUM (((long *) SCM_UNPACK (SCM_CDR (ra1)))[i1]);
-	    SCM n2 = SCM_MAKINUM (((long *) SCM_UNPACK (SCM_CDR (ra2)))[i2]);
-	    if (SCM_FALSEP (SCM_SUBRF (proc) (n1, n2)))
+	    SCM n1 = scm_from_long (((long *) SCM_UNPACK (SCM_CDR (ra1)))[i1]);
+	    SCM n2 = scm_from_long (((long *) SCM_UNPACK (SCM_CDR (ra2)))[i2]);
+	    if (scm_is_false (SCM_SUBRF (proc) (n1, n2)))
 	      SCM_BITVEC_CLR (ra0, i0);
 	  }
       break;
     case scm_tc7_fvect:
       {
-	SCM a1 = scm_make_real (1.0), a2 = scm_make_real (1.0);
+	SCM a1 = scm_from_double (1.0), a2 = scm_from_double (1.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  if (SCM_BITVEC_REF (ra0, i0))
 	    {
 	      SCM_REAL_VALUE (a1) = ((float *) SCM_VELTS (ra1))[i1];
 	      SCM_REAL_VALUE (a2) = ((float *) SCM_VELTS (ra2))[i2];
-	      if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
+	      if (scm_is_false (SCM_SUBRF (proc) (a1, a2)))
 		SCM_BITVEC_CLR (ra0, i0);
 	    }
 	break;
       }
     case scm_tc7_dvect:
       {
-	SCM a1 = scm_make_real (1.0 / 3.0);
-	SCM a2 = scm_make_real (1.0 / 3.0);
+	SCM a1 = scm_from_double (1.0 / 3.0);
+	SCM a2 = scm_from_double (1.0 / 3.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  if (SCM_BITVEC_REF (ra0, i0))
 	    {
 	      SCM_REAL_VALUE (a1) = ((double *) SCM_VELTS (ra1))[i1];
 	      SCM_REAL_VALUE (a2) = ((double *) SCM_VELTS (ra2))[i2];
-	      if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
+	      if (scm_is_false (SCM_SUBRF (proc) (a1, a2)))
 		SCM_BITVEC_CLR (ra0, i0);
 	    }
 	break;
       }
     case scm_tc7_cvect:
       {
-	SCM a1 = scm_make_complex (1.0, 1.0);
-	SCM a2 = scm_make_complex (1.0, 1.0);
+	SCM a1 = scm_c_make_rectangular (1.0, 1.0);
+	SCM a2 = scm_c_make_rectangular (1.0, 1.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  if (SCM_BITVEC_REF (ra0, i0))
 	    {
@@ -1403,7 +1387,7 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
 	      SCM_COMPLEX_IMAG (a1) = ((double *) SCM_VELTS (ra1))[2 * i1 + 1];
 	      SCM_COMPLEX_REAL (a2) = ((double *) SCM_VELTS (ra2))[2 * i2];
 	      SCM_COMPLEX_IMAG (a2) = ((double *) SCM_VELTS (ra2))[2 * i2 + 1];
-	      if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
+	      if (scm_is_false (SCM_SUBRF (proc) (a1, a2)))
 		SCM_BITVEC_CLR (ra0, i0);
 	    }
 	break;
@@ -1420,29 +1404,23 @@ ramap_1 (SCM ra0, SCM proc, SCM ras)
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  unsigned long i0 = SCM_ARRAY_BASE (ra0);
-  unsigned long i1 = SCM_ARRAY_BASE (ra1);
-  long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
-  long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
+  long inc0 = SCM_ARRAY_DIMS (ra0)->inc, inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  ra1 = SCM_ARRAY_V (ra1);  
+  ra1 = SCM_ARRAY_V (ra1);
   if (scm_tc7_vector == SCM_TYP7 (ra0) || scm_tc7_wvect == SCM_TYP7 (ra0))
     for (; n-- > 0; i0 += inc0, i1 += inc1)
-      scm_array_set_x (ra0,
-                       SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED)),
-                       SCM_MAKINUM (i0));
+      scm_array_set_x (ra0, SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED)), scm_from_ulong (i0));
   else
     for (; n-- > 0; i0 += inc0, i1 += inc1)
-      scm_array_set_x (ra0,
-                       SCM_SUBRF (proc) (RVREF (ra1, i1, e1)),
-                       SCM_MAKINUM (i0));
+      scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra1, i1, e1)), scm_from_ulong (i0));
   return 1;
 }
 
 
 
 static int
-ramap_2o (SCM ra0,SCM proc,SCM ras)
+ramap_2o (SCM ra0, SCM proc, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
@@ -1459,11 +1437,11 @@ ramap_2o (SCM ra0,SCM proc,SCM ras)
 
 	for (; n-- > 0; i0 += inc0, i1 += inc1)
 	  scm_array_set_x (ra0, SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED), SCM_UNDEFINED),
-			   SCM_MAKINUM (i0));
+			   scm_from_ulong (i0));
       else
 	for (; n-- > 0; i0 += inc0, i1 += inc1)
 	  scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra1, i1, e1), SCM_UNDEFINED),
-			   SCM_MAKINUM (i0));
+			   scm_from_ulong (i0));
     }
   else
     {
@@ -1476,12 +1454,12 @@ ramap_2o (SCM ra0,SCM proc,SCM ras)
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  scm_array_set_x (ra0,
 			   SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED), scm_cvref (ra2, i2, SCM_UNDEFINED)),
-			   SCM_MAKINUM (i0));
+			   scm_from_ulong (i0));
       else
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  scm_array_set_x (ra0,
 			   SCM_SUBRF (proc) (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2)),
-			   SCM_MAKINUM (i0));
+			   scm_from_ulong (i0));
     }
   return 1;
 }
@@ -1489,7 +1467,7 @@ ramap_2o (SCM ra0,SCM proc,SCM ras)
 
 
 static int
-ramap_a (SCM ra0,SCM proc,SCM ras)
+ramap_a (SCM ra0, SCM proc, SCM ras)
 {
   SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
@@ -1498,7 +1476,7 @@ ramap_a (SCM ra0,SCM proc,SCM ras)
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
     for (; n-- > 0; i0 += inc0)
-      scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra0, i0, e0), SCM_UNDEFINED), SCM_MAKINUM (i0));
+      scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra0, i0, e0), SCM_UNDEFINED), scm_from_ulong (i0));
   else
     {
       SCM ra1 = SCM_CAR (ras);
@@ -1507,7 +1485,7 @@ ramap_a (SCM ra0,SCM proc,SCM ras)
       ra1 = SCM_ARRAY_V (ra1);
       for (; n-- > 0; i0 += inc0, i1 += inc1)
 	scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)),
-			 SCM_MAKINUM (i0));
+			 scm_from_ulong (i0));
     }
   return 1;
 }
@@ -1517,114 +1495,109 @@ SCM_REGISTER_PROC(s_array_map_in_order_x, "array-map-in-order!", 2, 0, 1, scm_ar
 
 
 SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
-	    (SCM dest, SCM proc, SCM sources),
-	    "@var{dest} must be an array, and @var{sources} must be a non-empty list\n"
-            "of arrays where each element must have the same number of dimensions as\n"
-            "@var{dest} and must have a range for each index which includes the range\n"
-	    "for the corresponding index in @var{dest}.  @var{proc} is applied to\n"
-	    "each tuple of elements of @var{sources} and the result is stored as the\n"
-            "corresponding element in @var{dest}.  The value returned is unspecified, and\n"
-            "the order of application is unspecified.\n")
+	    (SCM ra0, SCM proc, SCM lra),
+	    "@deffnx {Scheme Procedure} array-map-in-order! ra0 proc . lra\n"
+	    "@var{array1}, @dots{} must have the same number of dimensions as\n"
+	    "@var{array0} and have a range for each index which includes the range\n"
+	    "for the corresponding index in @var{array0}.  @var{proc} is applied to\n"
+	    "each tuple of elements of @var{array1} @dots{} and the result is stored\n"
+	    "as the corresponding element in @var{array0}.  The value returned is\n"
+	    "unspecified.  The order of application is unspecified.")
 #define FUNC_NAME s_scm_array_map_x
 {
-  SCM_VALIDATE_PROC (2,proc);
-
-  if (!(SCM_CONSP (sources)))
-    SCM_WRONG_NUM_ARGS ();
-
+  SCM_VALIDATE_PROC (2, proc);
+  SCM_VALIDATE_REST_ARGUMENT (lra);
   switch (SCM_TYP7 (proc))
     {
     default:
     gencase:
-      scm_ramapc (ramap, proc, dest, sources, FUNC_NAME);
-      return SCM_UNSPECIFIED;
+ scm_ramapc (ramap, proc, ra0, lra, FUNC_NAME);
+ return SCM_UNSPECIFIED;
     case scm_tc7_subr_1:
-      scm_ramapc (ramap_1, proc, dest, sources, FUNC_NAME);
+      scm_ramapc (ramap_1, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_subr_2:
     case scm_tc7_subr_2o:
-      scm_ramapc (ramap_2o, proc, dest, sources, FUNC_NAME);
+      scm_ramapc (ramap_2o, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
-    case scm_tc7_cxr:
-      if (!SCM_SUBRF (proc))
-	goto gencase;
-      scm_ramapc (ramap_cxr, proc, dest, sources, FUNC_NAME);
+    case scm_tc7_dsubr:
+      scm_ramapc (ramap_dsubr, proc, ra0, lra, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_rpsubr:
       {
 	ra_iproc *p;
-	if (SCM_FALSEP (scm_array_p (dest, SCM_BOOL_T)))
+	if (scm_is_false (scm_array_p (ra0, SCM_BOOL_T)))
 	  goto gencase;
-	scm_array_fill_x (dest, SCM_BOOL_T);
+	scm_array_fill_x (ra0, SCM_BOOL_T);
 	for (p = ra_rpsubrs; p->name; p++)
-	  if (SCM_EQ_P (proc, p->sproc))
+	  if (scm_is_eq (proc, p->sproc))
 	    {
-	      while (SCM_NNULLP (sources) && SCM_NNULLP (SCM_CDR (sources)))
+	      while (!SCM_NULLP (lra) && !SCM_NULLP (SCM_CDR (lra)))
 		{
-		  scm_ramapc (p->vproc, SCM_UNDEFINED, dest, sources, FUNC_NAME);
-		  sources = SCM_CDR (sources);
+		  scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, FUNC_NAME);
+		  lra = SCM_CDR (lra);
 		}
 	      return SCM_UNSPECIFIED;
 	    }
-	while (SCM_NNULLP (sources) && SCM_NNULLP (SCM_CDR (sources)))
+	while (!SCM_NULLP (lra) && !SCM_NULLP (SCM_CDR (lra)))
 	  {
-	    scm_ramapc (ramap_rp, proc, dest, sources, FUNC_NAME);
-	    sources = SCM_CDR (sources);
+	    scm_ramapc (ramap_rp, proc, ra0, lra, FUNC_NAME);
+	    lra = SCM_CDR (lra);
 	  }
 	return SCM_UNSPECIFIED;
       }
     case scm_tc7_asubr:
-      if (SCM_NULLP (sources))
+      if (SCM_NULLP (lra))
 	{
 	  SCM prot, fill = SCM_SUBRF (proc) (SCM_UNDEFINED, SCM_UNDEFINED);
-	  if (SCM_INUMP(fill))
+	  if (SCM_I_INUMP(fill))
 	    {
-	      prot = scm_array_prototype (dest);
+	      prot = scm_array_prototype (ra0);
 	      if (SCM_INEXACTP (prot))
-		fill = scm_make_real ((double) SCM_INUM (fill));
+		fill = scm_from_double ((double) SCM_I_INUM (fill));
 	    }
 
-	  scm_array_fill_x (dest, fill);
+	  scm_array_fill_x (ra0, fill);
 	}
       else
 	{
-	  SCM tail, ra1 = SCM_CAR (sources);
-	  SCM v0 = (SCM_ARRAYP (dest) ? SCM_ARRAY_V (dest) : dest);
+	  SCM tail, ra1 = SCM_CAR (lra);
+	  SCM v0 = (SCM_ARRAYP (ra0) ? SCM_ARRAY_V (ra0) : ra0);
 	  ra_iproc *p;
 	  /* Check to see if order might matter.
 	     This might be an argument for a separate
 	     SERIAL-ARRAY-MAP! */
-	  if (SCM_EQ_P (v0, ra1) 
-	      || (SCM_ARRAYP (ra1) && SCM_EQ_P (v0, SCM_ARRAY_V (ra1))))
-	    if (!SCM_EQ_P (dest, ra1) 
-		|| (SCM_ARRAYP(dest) && !SCM_ARRAY_CONTP(dest)))
+	  if (scm_is_eq (v0, ra1) 
+	      || (SCM_ARRAYP (ra1) && scm_is_eq (v0, SCM_ARRAY_V (ra1))))
+	    if (!scm_is_eq (ra0, ra1) 
+		|| (SCM_ARRAYP(ra0) && !SCM_ARRAY_CONTP(ra0)))
 	      goto gencase;
-	  for (tail = SCM_CDR (sources); SCM_NNULLP (tail); tail = SCM_CDR (tail))
+	  for (tail = SCM_CDR (lra); !SCM_NULLP (tail); tail = SCM_CDR (tail))
 	    {
 	      ra1 = SCM_CAR (tail);
-	      if (SCM_EQ_P (v0, ra1) 
-		  || (SCM_ARRAYP (ra1) && SCM_EQ_P (v0, SCM_ARRAY_V (ra1))))
+	      if (scm_is_eq (v0, ra1) 
+		  || (SCM_ARRAYP (ra1) && scm_is_eq (v0, SCM_ARRAY_V (ra1))))
 		goto gencase;
 	    }
 	  for (p = ra_asubrs; p->name; p++)
-	    if (SCM_EQ_P (proc, p->sproc))
+	    if (scm_is_eq (proc, p->sproc))
 	      {
-		if (!SCM_EQ_P (dest, SCM_CAR (sources)))
-		  scm_ramapc (scm_array_identity, SCM_UNDEFINED, dest, scm_cons (SCM_CAR (sources), SCM_EOL), FUNC_NAME);
-		sources = SCM_CDR (sources);
+		if (!scm_is_eq (ra0, SCM_CAR (lra)))
+		  scm_ramapc (scm_array_identity, SCM_UNDEFINED, ra0, scm_cons (SCM_CAR (lra), SCM_EOL), FUNC_NAME);
+		lra = SCM_CDR (lra);
 		while (1)
 		  {
-		    scm_ramapc (p->vproc, SCM_UNDEFINED, dest, sources, FUNC_NAME);
-		    if (SCM_IMP (sources) || SCM_IMP (SCM_CDR (sources)))
+		    scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, FUNC_NAME);
+		    if (SCM_IMP (lra) || SCM_IMP (SCM_CDR (lra)))
 		      return SCM_UNSPECIFIED;
-		    sources = SCM_CDR (sources);
+		    lra = SCM_CDR (lra);
 		  }
 	      }
-	  scm_ramapc (ramap_2o, proc, dest, sources, FUNC_NAME);
-	  sources = SCM_CDR (sources);
-	  if (SCM_NIMP (sources))
-	    for (sources = SCM_CDR (sources); SCM_NIMP (sources); sources = SCM_CDR (sources))
-	      scm_ramapc (ramap_a, proc, dest, sources, FUNC_NAME);
+	  scm_ramapc (ramap_2o, proc, ra0, lra, FUNC_NAME);
+	  lra = SCM_CDR (lra);
+	  if (SCM_NIMP (lra))
+	    for (lra = SCM_CDR (lra); SCM_NIMP (lra); lra = SCM_CDR (lra))
+	      scm_ramapc (ramap_a, proc, ra0, lra, FUNC_NAME);
 	}
       return SCM_UNSPECIFIED;
     }
@@ -1633,7 +1606,7 @@ SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
 
 
 static int
-rafe (SCM ra0,SCM proc,SCM ras)
+rafe (SCM ra0, SCM proc, SCM ras)
 {
   long i = SCM_ARRAY_DIMS (ra0)->lbnd;
   unsigned long i0 = SCM_ARRAY_BASE (ra0);
@@ -1646,7 +1619,8 @@ rafe (SCM ra0,SCM proc,SCM ras)
   else
     {
       SCM ra1 = SCM_CAR (ras);
-      SCM args, *ve = &ras;
+      SCM args;
+      SCM const*ve = &ras;
       unsigned long k, i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
@@ -1661,8 +1635,8 @@ rafe (SCM ra0,SCM proc,SCM ras)
       for (; i <= n; i++, i0 += inc0, i1 += inc1)
 	{
 	  args = SCM_EOL;
-	  for (k = SCM_INUM (scm_uniform_vector_length (ras)); k--;)
-	    args = scm_cons (scm_uniform_vector_ref (ve[k], SCM_MAKINUM (i)), args);
+	  for (k = scm_to_ulong (scm_uniform_vector_length (ras)); k--;)
+	    args = scm_cons (scm_uniform_vector_ref (ve[k], scm_from_long (i)), args);
 	  args = scm_cons2 (scm_cvref (ra0, i0, SCM_UNDEFINED), scm_cvref (ra1, i1, SCM_UNDEFINED), args);
 	  scm_apply_0 (proc, args);
 	}
@@ -1677,7 +1651,7 @@ SCM_DEFINE (scm_array_for_each, "array-for-each", 2, 0, 1,
 	    "in row-major order.  The value returned is unspecified.")
 #define FUNC_NAME s_scm_array_for_each
 {
-  SCM_VALIDATE_PROC (1,proc);
+  SCM_VALIDATE_PROC (1, proc);
   SCM_VALIDATE_REST_ARGUMENT (lra);
   scm_ramapc (rafe, proc, ra0, lra, FUNC_NAME);
   return SCM_UNSPECIFIED;
@@ -1706,8 +1680,8 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 #define FUNC_NAME s_scm_array_index_map_x
 {
   unsigned long i;
-  SCM_VALIDATE_NIM (1,ra);
-  SCM_VALIDATE_PROC (2,proc);
+  SCM_VALIDATE_NIM (1, ra);
+  SCM_VALIDATE_PROC (2, proc);
   switch (SCM_TYP7(ra))
     {
     default:
@@ -1715,9 +1689,8 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
     case scm_tc7_vector:
     case scm_tc7_wvect:
       {
-	SCM *ve = SCM_VELTS (ra);
 	for (i = 0; i < SCM_VECTOR_LENGTH (ra); i++)
-	  ve[i] = scm_call_1 (proc, SCM_MAKINUM (i));
+	  SCM_VECTOR_SET(ra, i, scm_call_1 (proc, scm_from_long (i)));
 	return SCM_UNSPECIFIED;
       }
     case scm_tc7_string:
@@ -1726,24 +1699,24 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
     case scm_tc7_uvect:
     case scm_tc7_ivect:
     case scm_tc7_svect:
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
     case scm_tc7_llvect:
 #endif
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
       {
-	unsigned long int length = SCM_INUM (scm_uniform_vector_length (ra));
+	unsigned long int length = scm_to_ulong (scm_uniform_vector_length (ra));
 	for (i = 0; i < length; i++)
-	  scm_array_set_x (ra, scm_call_1 (proc, SCM_MAKINUM (i)),
-			   SCM_MAKINUM (i));
+	  scm_array_set_x (ra, scm_call_1 (proc, scm_from_ulong (i)),
+			   scm_from_ulong (i));
 	return SCM_UNSPECIFIED;
       }
     case scm_tc7_smob:
       SCM_ASRTGO (SCM_ARRAYP (ra), badarg);
       {
 	SCM args = SCM_EOL;
-	SCM inds = scm_make_uve (SCM_ARRAY_NDIM (ra), SCM_MAKINUM (-1L));
+	SCM inds = scm_make_uve (SCM_ARRAY_NDIM (ra), scm_from_int (-1));
 	long *vinds = (long *) SCM_VELTS (inds);
 	int j, k, kmax = SCM_ARRAY_NDIM (ra) - 1;
 	if (kmax < 0)
@@ -1760,10 +1733,10 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 		for (; vinds[k] <= SCM_ARRAY_DIMS (ra)[k].ubnd; vinds[k]++)
 		  {
 		    for (j = kmax + 1, args = SCM_EOL; j--;)
-		      args = scm_cons (SCM_MAKINUM (vinds[j]), args);
+		      args = scm_cons (scm_from_long (vinds[j]), args);
 		    scm_array_set_x (SCM_ARRAY_V (ra),
 				     scm_apply_0 (proc, args),
-				     SCM_MAKINUM (i));
+				     scm_from_ulong (i));
 		    i += SCM_ARRAY_DIMS (ra)[k].inc;
 		  }
 		k--;
@@ -1787,7 +1760,7 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 
 
 static int
-raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
+raeql_1 (SCM ra0, SCM as_equal, SCM ra1)
 {
   SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
   unsigned long i0 = 0, i1 = 0;
@@ -1802,7 +1775,7 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
       ra0 = SCM_ARRAY_V (ra0);
     }
   else
-    n = SCM_INUM (scm_uniform_vector_length (ra0));
+    n = scm_to_ulong (scm_uniform_vector_length (ra0));
   if (SCM_ARRAYP (ra1))
     {
       i1 = SCM_ARRAY_BASE (ra1);
@@ -1816,19 +1789,19 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
     default:
       for (; n--; i0 += inc0, i1 += inc1)
 	{
-	  if (SCM_FALSEP (as_equal))
+	  if (scm_is_false (as_equal))
 	    {
-	      if (SCM_FALSEP (scm_array_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
+	      if (scm_is_false (scm_array_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
 		return 0;
 	    }
-	  else if (SCM_FALSEP (scm_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
+	  else if (scm_is_false (scm_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
 	    return 0;
 	}
       return 1;
     case scm_tc7_string:
       {
-	char *v0 = SCM_STRING_CHARS (ra0) + i0;
-	char *v1 = SCM_STRING_CHARS (ra1) + i1;
+	const char *v0 = scm_i_string_chars (ra0) + i0;
+	const char *v1 = scm_i_string_chars (ra1) + i1;
 	for (; n--; v0 += inc0, v1 += inc1)
 	  if (*v0 != *v1)
 	    return 0;
@@ -1867,7 +1840,7 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
 	    return 0;
 	return 1;
       }
-#ifdef HAVE_LONG_LONGS
+#if SCM_SIZEOF_LONG_LONG != 0
     case scm_tc7_llvect:
       {
 	long long *v0 = (long long *) SCM_VELTS (ra0) + i0;
@@ -1915,7 +1888,7 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
 
 
 static int
-raeql (SCM ra0,SCM as_equal,SCM ra1)
+raeql (SCM ra0, SCM as_equal, SCM ra1)
 {
   SCM v0 = ra0, v1 = ra1;
   scm_t_array_dim dim0, dim1;
@@ -1933,7 +1906,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
     {
       s0->inc = 1;
       s0->lbnd = 0;
-      s0->ubnd = SCM_INUM (scm_uniform_vector_length (v0)) - 1;
+      s0->ubnd = scm_to_long (scm_uniform_vector_length (v0)) - 1;
       unroll = 0;
     }
   if (SCM_ARRAYP (ra1))
@@ -1953,7 +1926,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
 	return 0;
       s1->inc = 1;
       s1->lbnd = 0;
-      s1->ubnd = SCM_INUM (scm_uniform_vector_length (v1)) - 1;
+      s1->ubnd = scm_to_long (scm_uniform_vector_length (v1)) - 1;
       unroll = 0;
     }
   if (SCM_TYP7 (v0) != SCM_TYP7 (v1))
@@ -1968,7 +1941,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
 	  vlen *= s0[k].ubnd - s1[k].lbnd + 1;
 	}
     }
-  if (unroll && bas0 == bas1 && SCM_EQ_P (v0, v1))
+  if (unroll && bas0 == bas1 && scm_is_eq (v0, v1))
     return 1;
   return scm_ramapc (raeql_1, as_equal, ra0, scm_cons (ra1, SCM_EOL), "");
 }
@@ -1977,7 +1950,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
 SCM
 scm_raequal (SCM ra0, SCM ra1)
 {
-  return SCM_BOOL(raeql (ra0, SCM_BOOL_T, ra1));
+  return scm_from_bool(raeql (ra0, SCM_BOOL_T, ra1));
 }
 
 #if 0
@@ -2013,7 +1986,6 @@ scm_array_equal_p (SCM ra0, SCM ra1)
     case scm_tc7_byvect:
     case scm_tc7_uvect:
     case scm_tc7_ivect:
-    case scm_tc7_svect:
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
@@ -2033,7 +2005,6 @@ scm_array_equal_p (SCM ra0, SCM ra1)
     case scm_tc7_byvect:
     case scm_tc7_uvect:
     case scm_tc7_ivect:
-    case scm_tc7_svect:
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
@@ -2044,7 +2015,7 @@ scm_array_equal_p (SCM ra0, SCM ra1)
       if (!SCM_ARRAYP (ra1))
 	goto callequal;
     }
-  return SCM_BOOL(raeql (ra0, SCM_BOOL_F, ra1));
+  return scm_from_bool(raeql (ra0, SCM_BOOL_F, ra1));
 }
 
 
@@ -2053,7 +2024,7 @@ init_raprocs (ra_iproc *subra)
 {
   for (; subra->name; subra++)
     {
-      SCM sym = scm_str2symbol (subra->name);
+      SCM sym = scm_from_locale_symbol (subra->name);
       SCM var =
 	scm_sym2var (sym, scm_current_module_lookup_closure (), SCM_BOOL_F);
       if (var != SCM_BOOL_F)
