@@ -1,43 +1,20 @@
-/* Copyright (C) 1995, 1996, 1997, 1999, 2000, 2001, 2006 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1999,2000,2001,2003, 2004 Free Software
+ * Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -55,13 +32,15 @@
 #include "libguile/ports.h"
 #include "libguile/root.h"
 #include "libguile/strings.h"
+#include "libguile/strports.h"
 #include "libguile/vectors.h"
-
 #include "libguile/validate.h"
+
 #include "libguile/read.h"
 
 
 
+SCM_GLOBAL_SYMBOL (scm_sym_dot, ".");
 SCM_SYMBOL (scm_keyword_prefix, "prefix");
 
 scm_t_option scm_read_opts[] = {
@@ -72,9 +51,57 @@ scm_t_option scm_read_opts[] = {
   { SCM_OPTION_BOOLEAN, "case-insensitive", 0,
     "Convert symbols to lower case."},
   { SCM_OPTION_SCM, "keywords", SCM_UNPACK (SCM_BOOL_F),
-    "Style of keyword recognition: #f or 'prefix"}
+    "Style of keyword recognition: #f or 'prefix."}
+#if SCM_ENABLE_ELISP
+  ,
+  { SCM_OPTION_BOOLEAN, "elisp-vectors", 0,
+    "Support Elisp vector syntax, namely `[...]'."},
+  { SCM_OPTION_BOOLEAN, "elisp-strings", 0,
+    "Support `\\(' and `\\)' in strings."}
+#endif
 };
 
+/*
+  Give meaningful error messages for errors
+
+  We use the format
+
+  FILE:LINE:COL: MESSAGE
+  This happened in ....
+
+  This is not standard GNU format, but the test-suite likes the real
+  message to be in front.
+
+ */
+
+
+static void
+scm_input_error (char const *function,
+		 SCM port, const char *message, SCM arg)
+{
+  SCM fn = (scm_is_string (SCM_FILENAME(port))
+	    ? SCM_FILENAME(port)
+	    : scm_from_locale_string ("#<unknown port>"));
+
+  SCM string_port = scm_open_output_string ();
+  SCM string = SCM_EOL;
+  scm_simple_format (string_port,
+		     scm_from_locale_string ("~A:~S:~S: ~A"),
+		     scm_list_4 (fn,
+				 scm_from_int (SCM_LINUM (port) + 1),
+				 scm_from_int (SCM_COL (port) + 1),
+				 scm_from_locale_string (message)));
+    
+  string = scm_get_output_string (string_port);
+  scm_close_output_port (string_port);
+  scm_error_scm (scm_from_locale_symbol ("read-error"),
+		 scm_from_locale_string (function),
+		 string,
+		 arg,
+		 SCM_BOOL_F);
+}
+
+
 SCM_DEFINE (scm_read_options, "read-options-interface", 0, 1, 0, 
             (SCM setting),
 	    "Option interface for the read options. Instead of using\n"
@@ -107,14 +134,14 @@ SCM_DEFINE (scm_read, "read", 0, 1, 0,
 
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
-  SCM_VALIDATE_OPINPORT (1,port);
+  SCM_VALIDATE_OPINPORT (1, port);
 
   c = scm_flush_ws (port, (char *) NULL);
   if (EOF == c)
     return SCM_EOF_VAL;
   scm_ungetc (c, port);
 
-  tok_buf = scm_allocate_string (30);
+  tok_buf = scm_c_make_string (30, SCM_UNDEFINED);
   return scm_lreadr (&tok_buf, port, &copy);
 }
 #undef FUNC_NAME
@@ -124,15 +151,17 @@ SCM_DEFINE (scm_read, "read", 0, 1, 0,
 char *
 scm_grow_tok_buf (SCM *tok_buf)
 {
-  size_t oldlen = SCM_STRING_LENGTH (*tok_buf);
-  SCM newstr = scm_allocate_string (2 * oldlen);
+  size_t oldlen = scm_i_string_length (*tok_buf);
+  const char *olddata = scm_i_string_chars (*tok_buf);
+  char *newdata;
+  SCM newstr = scm_i_make_string (2 * oldlen, &newdata);
   size_t i;
 
   for (i = 0; i != oldlen; ++i)
-    SCM_STRING_CHARS (newstr) [i] = SCM_STRING_CHARS (*tok_buf) [i];
+    newdata[i] = olddata[i];
 
   *tok_buf = newstr;
-  return SCM_STRING_CHARS (newstr);
+  return newdata;
 }
 
 
@@ -148,12 +177,10 @@ scm_flush_ws (SCM port, const char *eoferr)
       goteof:
 	if (eoferr)
 	  {
-	    if (!SCM_FALSEP (SCM_FILENAME (port)))
-	      scm_misc_error (eoferr,
-			      "end of file in ~A",
-			      scm_list_1 (SCM_FILENAME (port)));
-	    else
-	      scm_misc_error (eoferr, "end of file", SCM_EOL);
+	    scm_input_error (eoferr,
+			     port,
+			     "end of file",
+			     SCM_EOL);
 	  }
 	return c;
       case ';':
@@ -183,7 +210,7 @@ int
 scm_casei_streq (char *s1, char *s2)
 {
   while (*s1 && *s2)
-    if (scm_downcase((int)*s1) != scm_downcase((int)*s2))
+    if (scm_c_downcase((int)*s1) != scm_c_downcase((int)*s2))
       return 0;
     else
       {
@@ -193,13 +220,24 @@ scm_casei_streq (char *s1, char *s2)
   return !(*s1 || *s2);
 }
 
+static int
+scm_i_casei_streq (const char *s1, const char *s2, size_t len2)
+{
+  while (*s1 && len2 > 0)
+    if (scm_c_downcase((int)*s1) != scm_c_downcase((int)*s2))
+      return 0;
+    else
+      {
+	++s1;
+	++s2;
+	--len2;
+      }
+  return !(*s1 || len2 > 0);
+}
 
 /* recsexpr is used when recording expressions
  * constructed by read:sharp.
  */
-#ifndef DEBUG_EXTENSIONS
-#define recsexpr(obj, line, column, filename) (obj)
-#else
 static SCM
 recsexpr (SCM obj, long line, int column, SCM filename)
 {
@@ -210,7 +248,7 @@ recsexpr (SCM obj, long line, int column, SCM filename)
     /* If this sexpr is visible in the read:sharp source, we want to
        keep that information, so only record non-constant cons cells
        which haven't previously been read by the reader. */
-    if (SCM_FALSEP (scm_whash_lookup (scm_source_whash, obj)))
+    if (scm_is_false (scm_whash_lookup (scm_source_whash, obj)))
       {
 	if (SCM_COPY_SOURCE_P)
 	  {
@@ -245,17 +283,18 @@ recsexpr (SCM obj, long line, int column, SCM filename)
     return obj;
   }
 }
-#endif
 
 /* Consume an SCSH-style block comment.  Assume that we've already
    read the initial `#!', and eat characters until we get a
-   newline/exclamation-point/sharp-sign/newline sequence.  */
+   newline/exclamation-point/sharp-sign/newline sequence. 
+
+   A carriage return is also reocgnized as a newline. */
 
 static void
 skip_scsh_block_comment (SCM port)
 #define FUNC_NAME "skip_scsh_block_comment"
 {
-  int bang_seen = 0;
+  int state = 0;
 
   for (;;)
     {
@@ -264,23 +303,29 @@ skip_scsh_block_comment (SCM port)
       if (c == EOF)
 	SCM_MISC_ERROR ("unterminated `#! ... !#' comment", SCM_EOL);
 
-      if (c == '!')
-	bang_seen = 1;
-      else if (c == '#' && bang_seen)
+      if (state == 1 && c == '!')
+	state = 2;
+      else if (state == 2 && c == '#')
+	state = 3;
+      else if (state == 3 && (c == '\n' || c == '\r'))
 	return;
+      else if (c == '\n' || c == '\r')
+	state = 1;
       else
-	bang_seen = 0;
+	state = 0;
     }
 }
 #undef FUNC_NAME
 
 
 static SCM scm_get_hash_procedure(int c);
+static SCM scm_i_lreadparen (SCM *, SCM, char *, SCM *, char);
 
 static char s_list[]="list";
+static char s_vector[]="vector";
 
 SCM 
-scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
+scm_lreadr (SCM *tok_buf, SCM port, SCM *copy)
 #define FUNC_NAME "scm_lreadr"
 {
   int c;
@@ -298,11 +343,20 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
     case '(':
       return SCM_RECORD_POSITIONS_P
 	? scm_lreadrecparen (tok_buf, port, s_list, copy)
-	: scm_lreadparen (tok_buf, port, s_list, copy);
+	: scm_i_lreadparen (tok_buf, port, s_list, copy, ')');
     case ')':
-      SCM_MISC_ERROR ("unexpected \")\"", SCM_EOL);
+      scm_input_error (FUNC_NAME, port,"unexpected \")\"", SCM_EOL);
       goto tryagain;
     
+#if SCM_ENABLE_ELISP
+    case '[':
+      if (SCM_ELISP_VECTORS_P)
+	{
+	  p = scm_i_lreadparen (tok_buf, port, s_vector, copy, ']');
+	  return SCM_NULLP (p) ? scm_nullvect : scm_vector (p);
+	}
+      goto read_token;
+#endif
     case '\'':
       p = scm_sym_quote;
       goto recquote;
@@ -342,14 +396,14 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	/* Check for user-defined hash procedure first, to allow
 	   overriding of builtin hash read syntaxes.  */
 	SCM sharp = scm_get_hash_procedure (c);
-	if (!SCM_FALSEP (sharp))
+	if (scm_is_true (sharp))
 	  {
 	    int line = SCM_LINUM (port);
 	    int column = SCM_COL (port) - 2;
 	    SCM got;
 
 	    got = scm_call_2 (sharp, SCM_MAKE_CHAR (c), port);
-	    if (SCM_EQ_P (got, SCM_UNSPECIFIED))
+	    if (scm_is_eq (got, SCM_UNSPECIFIED))
 	      goto handle_sharp;
 	    if (SCM_RECORD_POSITIONS_P)
 	      return *copy = recsexpr (got, line, column,
@@ -362,7 +416,7 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
       switch (c)
 	{
 	case '(':
-	  p = scm_lreadparen (tok_buf, port, "vector", copy);
+	  p = scm_i_lreadparen (tok_buf, port, s_vector, copy, ')');
 	  return SCM_NULLP (p) ? scm_nullvect : scm_vector (p);
 
 	case 't':
@@ -396,11 +450,11 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	  c = scm_flush_ws (port, (char *)NULL);
 	  goto tryagain_no_flush_ws;
 
-#ifdef HAVE_ARRAYS
+#if SCM_HAVE_ARRAYS
 	case '*':
 	  j = scm_read_token (c, tok_buf, port, 0);
-	  p = scm_istr2bve (SCM_STRING_CHARS (*tok_buf) + 1, (long) (j - 1));
-	  if (!SCM_FALSEP (p))
+	  p = scm_istr2bve (scm_c_substring_shared (*tok_buf, 1, j-1));
+	  if (scm_is_true (p))
 	    return p;
 	  else
 	    goto unkshrp;
@@ -408,7 +462,7 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 
 	case '{':
 	  j = scm_read_token (c, tok_buf, port, 1);
-	  return scm_mem2symbol (SCM_STRING_CHARS (*tok_buf), j);
+	  return scm_string_to_symbol (scm_c_substring_copy (*tok_buf, 0, j));
 
 	case '\\':
 	  c = scm_getc (port);
@@ -417,20 +471,27 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	    return SCM_MAKE_CHAR (c);
 	  if (c >= '0' && c < '8')
 	    {
-	      p = scm_istr2int (SCM_STRING_CHARS (*tok_buf), (long) j, 8);
-	      if (!SCM_FALSEP (p))
-		return SCM_MAKE_CHAR (SCM_INUM (p));
+	      /* Dirk:FIXME::  This type of character syntax is not R5RS
+	       * compliant.  Further, it should be verified that the constant
+	       * does only consist of octal digits.  Finally, it should be
+	       * checked whether the resulting fixnum is in the range of
+	       * characters.  */
+	      p = scm_i_mem2number (scm_i_string_chars (*tok_buf), j, 8);
+	      if (SCM_I_INUMP (p))
+		return SCM_MAKE_CHAR (SCM_I_INUM (p));
 	    }
 	  for (c = 0; c < scm_n_charnames; c++)
 	    if (scm_charnames[c]
-		&& (scm_casei_streq (scm_charnames[c], SCM_STRING_CHARS (*tok_buf))))
+		&& (scm_i_casei_streq (scm_charnames[c],
+				       scm_i_string_chars (*tok_buf), j)))
 	      return SCM_MAKE_CHAR (scm_charnums[c]);
-	  SCM_MISC_ERROR ("unknown # object", SCM_EOL);
+	  scm_input_error (FUNC_NAME, port, "unknown character name ~a",
+			   scm_list_1 (scm_c_substring (*tok_buf, 0, j)));
 
 	  /* #:SYMBOL is a syntax for keywords supported in all contexts.  */
 	case ':':
 	  j = scm_read_token ('-', tok_buf, port, 0);
-	  p = scm_mem2symbol (SCM_STRING_CHARS (*tok_buf), j);
+	  p = scm_string_to_symbol (scm_c_substring_copy (*tok_buf, 0, j));
 	  return scm_make_keyword_from_dash_symbol (p);
 
 	default:
@@ -438,14 +499,14 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	  {
 	    SCM sharp = scm_get_hash_procedure (c);
 
-	    if (!SCM_FALSEP (sharp))
+	    if (scm_is_true (sharp))
 	      {
 		int line = SCM_LINUM (port);
 		int column = SCM_COL (port) - 2;
 		SCM got;
 
 		got = scm_call_2 (sharp, SCM_MAKE_CHAR (c), port);
-		if (SCM_EQ_P (got, SCM_UNSPECIFIED))
+		if (scm_is_eq (got, SCM_UNSPECIFIED))
 		  goto unkshrp;
 		if (SCM_RECORD_POSITIONS_P)
 		  return *copy = recsexpr (got, line, column,
@@ -455,8 +516,8 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	      }
 	  }
 	unkshrp:
-	scm_misc_error (s_scm_read, "Unknown # object: ~S",
-			scm_list_1 (SCM_MAKE_CHAR (c)));
+	scm_input_error (FUNC_NAME, port, "Unknown # object: ~S",
+		     scm_list_1 (SCM_MAKE_CHAR (c)));
 	}
 
     case '"':
@@ -464,14 +525,26 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
       while ('"' != (c = scm_getc (port)))
 	{
 	  if (c == EOF)
-	    SCM_MISC_ERROR ("end of file in string constant", SCM_EOL);
+	    str_eof: scm_input_error (FUNC_NAME, port, "end of file in string constant", SCM_EOL);
 
-	  while (j + 2 >= SCM_STRING_LENGTH (*tok_buf))
+	  while (j + 2 >= scm_i_string_length (*tok_buf))
 	    scm_grow_tok_buf (tok_buf);
 
 	  if (c == '\\')
 	    switch (c = scm_getc (port))
 	      {
+	      case EOF:
+		goto str_eof;
+	      case '"':
+	      case '\\':
+		break;
+#if SCM_ENABLE_ELISP
+	      case '(':
+	      case ')':
+		if (SCM_ESCAPED_PARENS_P)
+		  break;
+		goto bad_escaped;
+#endif
 	      case '\n':
 		continue;
 	      case '0':
@@ -495,51 +568,80 @@ scm_lreadr (SCM *tok_buf,SCM port,SCM *copy)
 	      case 'v':
 		c = '\v';
 		break;
+	      case 'x':
+		{
+		  int a, b;
+		  a = scm_getc (port);
+		  if (a == EOF) goto str_eof;
+		  b = scm_getc (port);
+		  if (b == EOF) goto str_eof;
+		  if      ('0' <= a && a <= '9') a -= '0';
+		  else if ('A' <= a && a <= 'F') a = a - 'A' + 10;
+		  else if ('a' <= a && a <= 'f') a = a - 'a' + 10;
+		  else goto bad_escaped;
+		  if      ('0' <= b && b <= '9') b -= '0';
+		  else if ('A' <= b && b <= 'F') b = b - 'A' + 10;
+		  else if ('a' <= b && b <= 'f') b = b - 'a' + 10;
+		  else goto bad_escaped;
+		  c = a * 16 + b;
+		  break;
+		}
+	      default:
+	      bad_escaped:
+		scm_input_error(FUNC_NAME, port,
+				"illegal character in escape sequence: ~S",
+				scm_list_1 (SCM_MAKE_CHAR (c)));
 	      }
-	  SCM_STRING_CHARS (*tok_buf)[j] = c;
+	  scm_c_string_set_x (*tok_buf, j, SCM_MAKE_CHAR (c));
 	  ++j;
 	}
       if (j == 0)
 	return scm_nullstr;
-      SCM_STRING_CHARS (*tok_buf)[j] = 0;
-      return scm_mem2string (SCM_STRING_CHARS (*tok_buf), j);
+      return scm_c_substring_copy (*tok_buf, 0, j);
 
-      case'0':case '1':case '2':case '3':case '4':
-    case '5':case '6':case '7':case '8':case '9':
+    case '0': case '1': case '2': case '3': case '4':
+    case '5': case '6': case '7': case '8': case '9':
     case '.':
     case '-':
     case '+':
     num:
-		j = scm_read_token (c, tok_buf, port, 0);
-		p = scm_istring2number (SCM_STRING_CHARS (*tok_buf), (long) j, 10L);
-		if (!SCM_FALSEP (p))
-		  return p;
-		if (c == '#')
-		  {
-		    if ((j == 2) && (scm_getc (port) == '('))
-		      {
-			scm_ungetc ('(', port);
-			c = SCM_STRING_CHARS (*tok_buf)[1];
-			goto callshrp;
-		      }
-		    SCM_MISC_ERROR ("unknown # object", SCM_EOL);
-		  }
-		goto tok;
+      j = scm_read_token (c, tok_buf, port, 0);
+      if (j == 1 && (c == '+' || c == '-'))
+	/* Shortcut:  Detected symbol '+ or '- */
+	goto tok;
+
+      p = scm_i_mem2number (scm_i_string_chars (*tok_buf), j, 10);
+      if (scm_is_true (p))
+	return p;
+      if (c == '#')
+	{
+	  if ((j == 2) && (scm_getc (port) == '('))
+	    {
+	      scm_ungetc ('(', port);
+	      c = scm_i_string_chars (*tok_buf)[1];
+	      goto callshrp;
+	    }
+	  scm_input_error (FUNC_NAME, port, "unknown # object", SCM_EOL);
+	}
+      goto tok;
 
     case ':':
-      if (SCM_EQ_P (SCM_PACK (SCM_KEYWORD_STYLE), scm_keyword_prefix))
+      if (scm_is_eq (SCM_PACK (SCM_KEYWORD_STYLE), scm_keyword_prefix))
 	{
 	  j = scm_read_token ('-', tok_buf, port, 0);
-	  p = scm_mem2symbol (SCM_STRING_CHARS (*tok_buf), j);
+	  p = scm_string_to_symbol (scm_c_substring (*tok_buf, 0, j));
 	  return scm_make_keyword_from_dash_symbol (p);
 	}
       /* fallthrough */
     default:
+#if SCM_ENABLE_ELISP
+    read_token:
+#endif
       j = scm_read_token (c, tok_buf, port, 0);
       /* fallthrough */
 
     tok:
-      return scm_mem2symbol (SCM_STRING_CHARS (*tok_buf), j);
+      return scm_string_to_symbol (scm_c_substring (*tok_buf, 0, j));
     }
 }
 #undef FUNC_NAME
@@ -552,44 +654,49 @@ _Pragma ("noopt");		/* # pragma _CRI noopt */
 size_t 
 scm_read_token (int ic, SCM *tok_buf, SCM port, int weird)
 {
-  register size_t j;
-  register int c;
-  register char *p;
-
-  c = (SCM_CASE_INSENSITIVE_P ? scm_downcase(ic) : ic);
-  p = SCM_STRING_CHARS (*tok_buf);
+  size_t j;
+  int c;
 
+  c = (SCM_CASE_INSENSITIVE_P ? scm_c_downcase(ic) : ic);
+					    
   if (weird)
     j = 0;
   else
     {
       j = 0;
-      while (j + 2 >= SCM_STRING_LENGTH (*tok_buf))
-	p = scm_grow_tok_buf (tok_buf);
-      p[j] = c;
+      while (j + 2 >= scm_i_string_length (*tok_buf))
+	scm_grow_tok_buf (tok_buf);
+      scm_c_string_set_x (*tok_buf, j, SCM_MAKE_CHAR (c));
       ++j;
     }
 
   while (1)
     {
-      while (j + 2 >= SCM_STRING_LENGTH (*tok_buf))
-	p = scm_grow_tok_buf (tok_buf);
+      while (j + 2 >= scm_i_string_length (*tok_buf))
+	scm_grow_tok_buf (tok_buf);
       c = scm_getc (port);
       switch (c)
 	{
 	case '(':
 	case ')':
+#if SCM_ENABLE_ELISP
+	case '[':
+	case ']':
+#endif
 	case '"':
 	case ';':
 	case SCM_WHITE_SPACES:
 	case SCM_LINE_INCREMENTORS:
-	  if (weird)
+	  if (weird
+#if SCM_ENABLE_ELISP
+	      || ((!SCM_ELISP_VECTORS_P) && ((c == '[') || (c == ']')))
+#endif
+	      )
 	    goto default_case;
 
 	  scm_ungetc (c, port);
 	case EOF:
 	eof_case:
-	  p[j] = 0;
 	  return j;
 	case '\\':
 	  if (!weird)
@@ -609,7 +716,6 @@ scm_read_token (int ic, SCM *tok_buf, SCM port, int weird)
 	  c = scm_getc (port);
 	  if (c == '#')
 	    {
-	      p[j] = 0;
 	      return j;
 	    }
 	  else
@@ -622,8 +728,8 @@ scm_read_token (int ic, SCM *tok_buf, SCM port, int weird)
 	default:
 	default_case:
 	  {
-	    c = (SCM_CASE_INSENSITIVE_P ? scm_downcase(c) : c);
-	    p[j] = c;
+	    c = (SCM_CASE_INSENSITIVE_P ? scm_c_downcase(c) : c);
+            scm_c_string_set_x (*tok_buf, j, SCM_MAKE_CHAR (c));
 	    ++j;
 	  }
 
@@ -635,9 +741,9 @@ scm_read_token (int ic, SCM *tok_buf, SCM port, int weird)
 _Pragma ("opt");		/* # pragma _CRI opt */
 #endif
 
-SCM 
-scm_lreadparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
-#define FUNC_NAME "scm_lreadparen"
+static SCM 
+scm_i_lreadparen (SCM *tok_buf, SCM port, char *name, SCM *copy, char term_char)
+#define FUNC_NAME "scm_i_lreadparen"
 {
   SCM tmp;
   SCM tl;
@@ -645,22 +751,22 @@ scm_lreadparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
   int c;
 
   c = scm_flush_ws (port, name);
-  if (')' == c)
+  if (term_char == c)
     return SCM_EOL;
   scm_ungetc (c, port);
-  if (SCM_EQ_P (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
+  if (scm_is_eq (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
     {
       ans = scm_lreadr (tok_buf, port, copy);
     closeit:
-      if (')' != (c = scm_flush_ws (port, name)))
-	SCM_MISC_ERROR ("missing close paren", SCM_EOL);
+      if (term_char != (c = scm_flush_ws (port, name)))
+	scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
       return ans;
     }
   ans = tl = scm_cons (tmp, SCM_EOL);
-  while (')' != (c = scm_flush_ws (port, name)))
+  while (term_char != (c = scm_flush_ws (port, name)))
     {
       scm_ungetc (c, port);
-      if (SCM_EQ_P (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
+      if (scm_is_eq (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
 	{
 	  SCM_SETCDR (tl, scm_lreadr (tok_buf, port, copy));
 	  goto closeit;
@@ -689,11 +795,11 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
   if (')' == c)
     return SCM_EOL;
   scm_ungetc (c, port);
-  if (SCM_EQ_P (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
+  if (scm_is_eq (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
     {
       ans = scm_lreadr (tok_buf, port, copy);
       if (')' != (c = scm_flush_ws (port, name)))
-	SCM_MISC_ERROR ("missing close paren", SCM_EOL);
+	scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
       return ans;
     }
   /* Build the head of the list structure. */
@@ -708,7 +814,7 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
       SCM new_tail;
 
       scm_ungetc (c, port);
-      if (SCM_EQ_P (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
+      if (scm_is_eq (scm_sym_dot, (tmp = scm_lreadr (tok_buf, port, copy))))
 	{
 	  SCM_SETCDR (tl, tmp = scm_lreadr (tok_buf, port, copy));
 	  if (SCM_COPY_SOURCE_P)
@@ -717,7 +823,7 @@ scm_lreadrecparen (SCM *tok_buf, SCM port, char *name, SCM *copy)
 				       : tmp,
 				       SCM_EOL));
 	  if (')' != (c = scm_flush_ws (port, name)))
-	    SCM_MISC_ERROR ("missing close paren", SCM_EOL);
+	    scm_input_error (FUNC_NAME, port, "missing close paren", SCM_EOL);
 	  goto exit;
 	}
 
@@ -764,8 +870,8 @@ SCM_DEFINE (scm_read_hash_extend, "read-hash-extend", 2, 0, 0,
   SCM prev;
 
   SCM_VALIDATE_CHAR (1, chr);
-  SCM_ASSERT (SCM_FALSEP (proc)
-	      || SCM_EQ_P (scm_procedure_p (proc), SCM_BOOL_T),
+  SCM_ASSERT (scm_is_false (proc)
+	      || scm_is_eq (scm_procedure_p (proc), SCM_BOOL_T),
 	      proc, SCM_ARG2, FUNC_NAME);
 
   /* Check if chr is already in the alist.  */
@@ -776,20 +882,20 @@ SCM_DEFINE (scm_read_hash_extend, "read-hash-extend", 2, 0, 0,
       if (SCM_NULLP (this))
 	{
 	  /* not found, so add it to the beginning.  */
-	  if (!SCM_FALSEP (proc))
+	  if (scm_is_true (proc))
 	    {
 	      *scm_read_hash_procedures = 
 		scm_cons (scm_cons (chr, proc), *scm_read_hash_procedures);
 	    }
 	  break;
 	}
-      if (SCM_EQ_P (chr, SCM_CAAR (this)))
+      if (scm_is_eq (chr, SCM_CAAR (this)))
 	{
 	  /* already in the alist.  */
-	  if (SCM_FALSEP (proc))
+	  if (scm_is_false (proc))
 	    {
 	      /* remove it.  */
-	      if (SCM_FALSEP (prev))
+	      if (scm_is_false (prev))
 		{
 		  *scm_read_hash_procedures =
 		    SCM_CDR (*scm_read_hash_procedures);
