@@ -1,43 +1,18 @@
-/*	Copyright (C) 1997, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/*	Copyright (C) 1997, 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
- *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 
@@ -51,6 +26,10 @@
    libraries which do not agree with the Spencer implementation may
    produce varying behavior.  Sigh. */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <sys/types.h>
 
 #include "libguile/_scm.h"
@@ -95,40 +74,29 @@ static size_t
 regex_free (SCM obj)
 {
   regfree (SCM_RGX (obj));
-  free (SCM_RGX (obj));
-  return sizeof(regex_t);
+  scm_gc_free (SCM_RGX (obj), sizeof(regex_t), "regex");
+  return 0;
 }
 
 
 
 SCM_SYMBOL (scm_regexp_error_key, "regular-expression-syntax");
 
-static char *
+static SCM
 scm_regexp_error_msg (int regerrno, regex_t *rx)
 {
-  SCM errmsg;
+  char *errmsg;
   int l;
 
-  /* FIXME: must we wrap any external calls in SCM_DEFER_INTS...SCM_ALLOW_INTS?
-     Or are these only necessary when a SCM object may be left in an
-     undetermined state (half-formed)?  If the latter then I believe we
-     may do without the critical section code. -twp */
-
-  /* We could simply make errmsg a char pointer, and allocate space with
-     malloc.  But since we are about to pass the pointer to scm_error, which
-     never returns, we would never have the opportunity to free it.  Creating
-     it as a SCM object means that the system will GC it at some point. */
-
-  errmsg = scm_make_string (SCM_MAKINUM (80), SCM_UNDEFINED);
-  SCM_DEFER_INTS;
-  l = regerror (regerrno, rx, SCM_STRING_CHARS (errmsg), 80);
+  errmsg = scm_malloc (80);
+  l = regerror (regerrno, rx, errmsg, 80);
   if (l > 80)
     {
-      errmsg = scm_make_string (SCM_MAKINUM (l), SCM_UNDEFINED);
-      regerror (regerrno, rx, SCM_STRING_CHARS (errmsg), l);
+      free (errmsg);
+      errmsg = scm_malloc (l);
+      regerror (regerrno, rx, errmsg, l);
     }
-  SCM_ALLOW_INTS;
-  return SCM_STRING_CHARS (errmsg);
+  return scm_take_locale_string (errmsg);
 }
 
 SCM_DEFINE (scm_regexp_p, "regexp?", 1, 0, 0,
@@ -137,7 +105,7 @@ SCM_DEFINE (scm_regexp_p, "regexp?", 1, 0, 0,
 	    "or @code{#f} otherwise.")
 #define FUNC_NAME s_scm_regexp_p
 {
-  return SCM_BOOL(SCM_RGXP (obj));
+  return scm_from_bool(SCM_RGXP (obj));
 }
 #undef FUNC_NAME
 
@@ -185,10 +153,10 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
   SCM flag;
   regex_t *rx;
   int status, cflags;
+  char *c_pat;
 
   SCM_VALIDATE_STRING (1, pat);
   SCM_VALIDATE_REST_ARGUMENT (flags);
-  SCM_STRING_COERCE_0TERMINATION_X (pat);
 
   /* Examine list of regexp flags.  If REG_BASIC is supplied, then
      turn off REG_EXTENDED flag (on by default). */
@@ -196,25 +164,29 @@ SCM_DEFINE (scm_make_regexp, "make-regexp", 1, 0, 1,
   flag = flags;
   while (!SCM_NULLP (flag))
     {
-      if (SCM_INUM (SCM_CAR (flag)) == REG_BASIC)
+      if (scm_to_int (SCM_CAR (flag)) == REG_BASIC)
 	cflags &= ~REG_EXTENDED;
       else
-	cflags |= SCM_INUM (SCM_CAR (flag));
+	cflags |= scm_to_int (SCM_CAR (flag));
       flag = SCM_CDR (flag);
     }
 
-  rx = SCM_MUST_MALLOC_TYPE (regex_t);
-  status = regcomp (rx, SCM_STRING_CHARS (pat),
+  rx = scm_gc_malloc (sizeof(regex_t), "regex");
+  c_pat = scm_to_locale_string (pat);
+  status = regcomp (rx, c_pat,
 		    /* Make sure they're not passing REG_NOSUB;
                        regexp-exec assumes we're getting match data.  */
 		    cflags & ~REG_NOSUB);
+  free (c_pat);
   if (status != 0)
     {
-      scm_error (scm_regexp_error_key,
-		 FUNC_NAME,
-		 scm_regexp_error_msg (status, rx),
-		 SCM_BOOL_F,
-		 SCM_BOOL_F);
+      SCM errmsg = scm_regexp_error_msg (status, rx);
+      scm_gc_free (rx, sizeof(regex_t), "regex");
+      scm_error_scm (scm_regexp_error_key,
+		     scm_from_locale_string (FUNC_NAME),
+		     errmsg,
+		     SCM_BOOL_F,
+		     SCM_BOOL_F);
       /* never returns */
     }
   SCM_RETURN_NEWSMOB (scm_tc16_regex, rx);
@@ -248,48 +220,51 @@ SCM_DEFINE (scm_regexp_exec, "regexp-exec", 2, 2, 0,
   regmatch_t *matches;
   SCM mvec = SCM_BOOL_F;
 
-  SCM_VALIDATE_RGXP (1,rx);
+  SCM_VALIDATE_RGXP (1, rx);
   SCM_VALIDATE_STRING (2, str);
-  SCM_VALIDATE_INUM_DEF_COPY (3,start,0,offset);
-  SCM_ASSERT_RANGE (3,start, offset >= 0 && offset <= SCM_STRING_LENGTH (str));
+
+  if (SCM_UNBNDP (start))
+    offset = 0;
+  else
+    offset = scm_to_signed_integer (start, 0, scm_i_string_length (str));
+
   if (SCM_UNBNDP (flags))
     flags = SCM_INUM0;
-  SCM_VALIDATE_INUM (4,flags);
-  SCM_STRING_COERCE_0TERMINATION_X (str);
 
   /* re_nsub doesn't account for the `subexpression' representing the
      whole regexp, so add 1 to nmatches. */
 
   nmatches = SCM_RGX(rx)->re_nsub + 1;
   SCM_DEFER_INTS;
-  matches = SCM_MUST_MALLOC_TYPE_NUM (regmatch_t,nmatches);
-  status = regexec (SCM_RGX (rx), SCM_STRING_CHARS (str) + offset,
+  matches = scm_malloc (sizeof (regmatch_t) * nmatches);
+  status = regexec (SCM_RGX (rx), scm_i_string_chars (str) + offset,
 		    nmatches, matches,
-		    SCM_INUM (flags));
+		    scm_to_int (flags));
   if (!status)
     {
       int i;
       /* The match vector must include a cell for the string that was matched,
 	 so add 1. */
       mvec = scm_c_make_vector (nmatches + 1, SCM_UNSPECIFIED);
-      SCM_VELTS(mvec)[0] = str;
+      SCM_VECTOR_SET(mvec,0, str);
       for (i = 0; i < nmatches; ++i)
 	if (matches[i].rm_so == -1)
-	  SCM_VELTS(mvec)[i+1] = scm_cons (SCM_MAKINUM (-1), SCM_MAKINUM (-1));
+	  SCM_VECTOR_SET(mvec, i+1,
+			 scm_cons (scm_from_int (-1), scm_from_int (-1)));
 	else
-	  SCM_VELTS(mvec)[i+1]
-	    = scm_cons (scm_long2num (matches[i].rm_so + offset),
-			scm_long2num (matches[i].rm_eo + offset));
+	  SCM_VECTOR_SET(mvec, i+1,
+			 scm_cons (scm_from_long (matches[i].rm_so + offset),
+				   scm_from_long (matches[i].rm_eo + offset)));
     }
-  scm_must_free ((char *) matches);
+  free (matches);
   SCM_ALLOW_INTS;
 
   if (status != 0 && status != REG_NOMATCH)
-    scm_error (scm_regexp_error_key,
-	       FUNC_NAME,
-	       scm_regexp_error_msg (status, SCM_RGX (rx)),
-	       SCM_BOOL_F,
-	       SCM_BOOL_F);
+    scm_error_scm (scm_regexp_error_key,
+		   scm_from_locale_string (FUNC_NAME),
+		   scm_regexp_error_msg (status, SCM_RGX (rx)),
+		   SCM_BOOL_F,
+		   SCM_BOOL_F);
   return mvec;
 }
 #undef FUNC_NAME
@@ -301,14 +276,14 @@ scm_init_regex_posix ()
   scm_set_smob_free (scm_tc16_regex, regex_free);
 
   /* Compilation flags.  */
-  scm_c_define ("regexp/basic", scm_long2num (REG_BASIC));
-  scm_c_define ("regexp/extended", scm_long2num (REG_EXTENDED));
-  scm_c_define ("regexp/icase", scm_long2num (REG_ICASE));
-  scm_c_define ("regexp/newline", scm_long2num (REG_NEWLINE));
+  scm_c_define ("regexp/basic", scm_from_long (REG_BASIC));
+  scm_c_define ("regexp/extended", scm_from_long (REG_EXTENDED));
+  scm_c_define ("regexp/icase", scm_from_long (REG_ICASE));
+  scm_c_define ("regexp/newline", scm_from_long (REG_NEWLINE));
 
   /* Execution flags.  */
-  scm_c_define ("regexp/notbol", scm_long2num (REG_NOTBOL));
-  scm_c_define ("regexp/noteol", scm_long2num (REG_NOTEOL));
+  scm_c_define ("regexp/notbol", scm_from_long (REG_NOTBOL));
+  scm_c_define ("regexp/noteol", scm_from_long (REG_NOTEOL));
 
 #include "libguile/regex-posix.x"
 
