@@ -1,46 +1,25 @@
-/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001, 2003 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <stdio.h>
 #include <errno.h>
@@ -68,6 +47,14 @@
 long scm_numsmob;
 scm_smob_descriptor scm_smobs[MAX_SMOB_COUNT];
 
+/* Lower 16 bit of data must be zero. 
+*/
+void
+scm_i_set_smob_flags (SCM x, scm_t_bits data)
+{
+  SCM_SET_CELL_WORD_0 (x, (SCM_CELL_WORD_0 (x) & 0xFFFF) | data);
+}
+
 /* {Mark}
  */
 
@@ -107,8 +94,11 @@ scm_free0 (SCM ptr SCM_UNUSED)
 size_t
 scm_smob_free (SCM obj)
 {
-  scm_must_free ((char *) SCM_CELL_WORD_1 (obj));
-  return scm_smobs[SCM_SMOBNUM (obj)].size;
+  long n = SCM_SMOBNUM (obj);
+  if (scm_smobs[n].size > 0)
+    scm_gc_free ((void *) SCM_CELL_WORD_1 (obj), 
+		 scm_smobs[n].size, SCM_SMOBNAME (n));
+  return 0;
 }
 
 /* {Print}
@@ -134,11 +124,11 @@ scm_smob_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
 
 #define SCM_SMOB_APPLY0(SMOB) \
   SCM_SMOB_DESCRIPTOR (SMOB).apply (SMOB)
-#define SCM_SMOB_APPLY1(SMOB,A1) \
+#define SCM_SMOB_APPLY1(SMOB, A1) \
   SCM_SMOB_DESCRIPTOR (SMOB).apply (SMOB, A1)
-#define SCM_SMOB_APPLY2(SMOB,A1,A2) \
+#define SCM_SMOB_APPLY2(SMOB, A1, A2) \
   SCM_SMOB_DESCRIPTOR (SMOB).apply (SMOB, A1, A2)
-#define SCM_SMOB_APPLY3(SMOB,A1,A2,A3) \
+#define SCM_SMOB_APPLY3(SMOB, A1, A2, A3) \
   SCM_SMOB_DESCRIPTOR (SMOB).apply (SMOB, A1, A2, A3)
 
 static SCM
@@ -287,7 +277,7 @@ scm_smob_apply_3_error (SCM smob,
 
 
 scm_t_bits 
-scm_make_smob_type (char *name, size_t size)
+scm_make_smob_type (char const *name, size_t size)
 #define FUNC_NAME "scm_make_smob_type"
 {
   long new_smob;
@@ -310,7 +300,7 @@ scm_make_smob_type (char *name, size_t size)
 
   /* Make a class object if Goops is present. */
   if (scm_smob_class)
-    scm_smob_class[new_smob] = scm_make_extended_class (name);
+    scm_smob_class[new_smob] = scm_make_extended_class (name, 0);
 
   return scm_tc7_smob + new_smob * 256;
 }
@@ -349,8 +339,7 @@ scm_set_smob_apply (scm_t_bits tc, SCM (*apply) (),
   SCM (*apply_1) (SCM, SCM);
   SCM (*apply_2) (SCM, SCM, SCM);
   SCM (*apply_3) (SCM, SCM, SCM, SCM);
-  const int type = SCM_GSUBR_MAKTYPE (req, opt, rst);
-  const long i = SCM_TC2SMOBNUM (tc);
+  int type = SCM_GSUBR_MAKTYPE (req, opt, rst);
 
   if (rst > 1 || req + opt + rst > 3)
     {
@@ -444,15 +433,15 @@ scm_set_smob_apply (scm_t_bits tc, SCM (*apply) (),
       apply_3 = scm_smob_apply_3_error; break;
     }
 
-  /* We compute i above and use it below because just using
-     SCM_TC2SMOBNUM (tc) directly as the array index below was causing
-     a build segfault on m68k machines */ 
-  scm_smobs[i].apply   = apply;
-  scm_smobs[i].apply_0 = apply_0;
-  scm_smobs[i].apply_1 = apply_1;
-  scm_smobs[i].apply_2 = apply_2;
-  scm_smobs[i].apply_3 = apply_3;
-  scm_smobs[i].gsubr_type = type;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].apply   = apply;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].apply_0 = apply_0;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].apply_1 = apply_1;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].apply_2 = apply_2;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].apply_3 = apply_3;
+  scm_smobs[SCM_TC2SMOBNUM (tc)].gsubr_type = type;
+  
+  if (scm_smob_class)
+    scm_i_inherit_applicable (scm_smob_class[SCM_TC2SMOBNUM (tc)]);
 }
 
 SCM
@@ -460,70 +449,29 @@ scm_make_smob (scm_t_bits tc)
 {
   long n = SCM_TC2SMOBNUM (tc);
   size_t size = scm_smobs[n].size;
-  SCM z;
-  SCM_NEWCELL (z);
-  if (size != 0)
-    {
-#if 0
-      if (scm_smobs[n].mark != 0)
-	{
-	  fprintf
-	    (stderr,
-	     "forbidden operation for smobs with GC data, use SCM_NEWSMOB\n");
-	  abort ();
-	}
-#endif
-      SCM_SET_SMOB_DATA (z, scm_must_malloc (size, SCM_SMOBNAME (n)));
-    }
-  SCM_SET_CELL_TYPE (z, tc);
-  return z;
+  scm_t_bits data = (size > 0
+		     ? (scm_t_bits) scm_gc_malloc (size, SCM_SMOBNAME (n))
+		     : 0);
+  return scm_cell (tc, data);
 }
 
 
-/* {Deprecated stuff}
- */
-
-#if (SCM_DEBUG_DEPRECATED == 0)
-
-long
-scm_make_smob_type_mfpe (char *name, size_t size,
-                        SCM (*mark) (SCM),
-                        size_t (*free) (SCM),
-                        int (*print) (SCM, SCM, scm_print_state *),
-                        SCM (*equalp) (SCM, SCM))
-{
-  long answer = scm_make_smob_type (name, size);
-  scm_set_smob_mfpe (answer, mark, free, print, equalp);
-  return answer;
-}
-
-void
-scm_set_smob_mfpe (long tc, 
-		   SCM (*mark) (SCM),
-		   size_t (*free) (SCM),
-		   int (*print) (SCM, SCM, scm_print_state *),
-		   SCM (*equalp) (SCM, SCM))
-{
-  if (mark) scm_set_smob_mark (tc, mark);
-  if (free) scm_set_smob_free (tc, free);
-  if (print) scm_set_smob_print (tc, print);
-  if (equalp) scm_set_smob_equalp (tc, equalp);
-}
-
-#endif  /* SCM_DEBUG_DEPRECATED == 0 */
-
-
-/* {Initialization for i/o types, float, bignum, the type of free cells}
+/* {Initialization for the type of free cells}
  */
 
 static int
 free_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
   char buf[100];
-
   sprintf (buf, "#<freed cell %p; GC missed a reference>",
 	   (void *) SCM_UNPACK (exp));
   scm_puts (buf, port);
+  
+#if (SCM_DEBUG_CELL_ACCESSES == 1)
+  if (scm_debug_cell_accesses_p)
+    abort();
+#endif
+  
 
   return 1;
 }
@@ -551,21 +499,9 @@ scm_smob_prehistory ()
       scm_smobs[i].gsubr_type = 0;
     }
 
-  /* WARNING: These scm_make_smob_type calls must be done in this order */
+  /* WARNING: This scm_make_smob_type call must be done first.  */
   tc = scm_make_smob_type ("free", 0);
   scm_set_smob_print (tc, free_print);
-
-  tc = scm_make_smob_type ("big", 0);  /* freed in gc */
-  scm_set_smob_print (tc, scm_bigprint);
-  scm_set_smob_equalp (tc, scm_bigequal);
-
-  tc = scm_make_smob_type ("real", 0);	/* freed in gc */
-  scm_set_smob_print (tc, scm_print_real);
-  scm_set_smob_equalp (tc, scm_real_equalp);
-
-  tc = scm_make_smob_type ("complex", 0);  /* freed in gc */
-  scm_set_smob_print (tc, scm_print_complex);
-  scm_set_smob_equalp (tc, scm_complex_equalp);
 }
 
 /*
