@@ -1,46 +1,19 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002 Free Software Foundation
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
- *
- * The author can be reached at djurfeldt@nada.kth.se
- * Mikael Djurfeldt, SANS/NADA KTH, 10044 STOCKHOLM, SWEDEN */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -98,8 +71,8 @@ srcprops_mark (SCM obj)
 static size_t
 srcprops_free (SCM obj)
 {
-  *((scm_t_srcprops **) SCM_CELL_WORD_1 (obj)) = srcprops_freelist;
-  srcprops_freelist = (scm_t_srcprops *) SCM_CELL_WORD_1 (obj);
+  *((scm_t_srcprops **) SCM_SMOB_DATA (obj)) = srcprops_freelist;
+  srcprops_freelist = (scm_t_srcprops *) SCM_SMOB_DATA (obj);
   return 0; /* srcprops_chunks are not freed until leaving guile */
 }
 
@@ -117,6 +90,14 @@ srcprops_print (SCM obj, SCM port, scm_print_state *pstate)
 }
 
 
+int
+scm_c_source_property_breakpoint_p (SCM form)
+{
+  SCM obj = scm_whash_lookup (scm_source_whash, form);
+  return SRCPROPSP (obj) && SRCPROPBRK (obj);
+}
+
+
 SCM
 scm_make_srcprops (long line, int col, SCM filename, SCM copy, SCM plist)
 {
@@ -130,10 +111,11 @@ scm_make_srcprops (long line, int col, SCM filename, SCM copy, SCM plist)
       scm_t_srcprops_chunk *mem;
       size_t n = sizeof (scm_t_srcprops_chunk)
 	            + sizeof (scm_t_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
-      SCM_SYSCALL (mem = (scm_t_srcprops_chunk *) malloc (n));
+      SCM_SYSCALL (mem = (scm_t_srcprops_chunk *) scm_malloc (n));
       if (mem == NULL)
 	scm_memory_error ("srcprops");
-      scm_mallocated += n;
+      scm_gc_register_collectable_memory (mem, n, "srcprops");
+      
       mem->next = srcprops_chunklist;
       srcprops_chunklist = mem;
       ptr = &mem->srcprops[0];
@@ -159,9 +141,9 @@ scm_srcprops_to_plist (SCM obj)
     plist = scm_acons (scm_sym_copy, SRCPROPCOPY (obj), plist);
   if (!SCM_UNBNDP (SRCPROPFNAME (obj)))
     plist = scm_acons (scm_sym_filename, SRCPROPFNAME (obj), plist);
-  plist = scm_acons (scm_sym_column, SCM_MAKINUM (SRCPROPCOL (obj)), plist);
-  plist = scm_acons (scm_sym_line, SCM_MAKINUM (SRCPROPLINE (obj)), plist);
-  plist = scm_acons (scm_sym_breakpoint, SRCPROPBRK (obj), plist);
+  plist = scm_acons (scm_sym_column, scm_from_int (SRCPROPCOL (obj)), plist);
+  plist = scm_acons (scm_sym_line, scm_from_int (SRCPROPLINE (obj)), plist);
+  plist = scm_acons (scm_sym_breakpoint, scm_from_bool (SRCPROPBRK (obj)), plist);
   return plist;
 }
 
@@ -171,13 +153,11 @@ SCM_DEFINE (scm_source_properties, "source-properties", 1, 0, 0,
 #define FUNC_NAME s_scm_source_properties
 {
   SCM p;
-  SCM_VALIDATE_NIM (1,obj);
+  SCM_VALIDATE_NIM (1, obj);
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
-#ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (!SCM_CONSP (obj))
     SCM_WRONG_TYPE_ARG (1, obj);
-#endif
   p = scm_hashq_ref (scm_source_whash, obj, SCM_EOL);
   if (SRCPROPSP (p))
     return scm_srcprops_to_plist (p);
@@ -196,13 +176,11 @@ SCM_DEFINE (scm_set_source_properties_x, "set-source-properties!", 2, 0, 0,
 #define FUNC_NAME s_scm_set_source_properties_x
 {
   SCM handle;
-  SCM_VALIDATE_NIM (1,obj);
+  SCM_VALIDATE_NIM (1, obj);
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
-#ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (!SCM_CONSP (obj))
     SCM_WRONG_TYPE_ARG(1, obj);
-#endif
   handle = scm_hashq_create_handle_x (scm_source_whash, obj, plist);
   SCM_SETCDR (handle, plist);
   return plist;
@@ -216,21 +194,19 @@ SCM_DEFINE (scm_source_property, "source-property", 2, 0, 0,
 #define FUNC_NAME s_scm_source_property
 {
   SCM p;
-  SCM_VALIDATE_NIM (1,obj);
+  SCM_VALIDATE_NIM (1, obj);
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
-#ifndef SCM_RECKLESS
-  else if (SCM_NECONSP (obj))
+  else if (!SCM_CONSP (obj))
     SCM_WRONG_TYPE_ARG (1, obj);
-#endif
   p = scm_hashq_ref (scm_source_whash, obj, SCM_EOL);
-  if (SCM_IMP (p) || !SRCPROPSP (p))
+  if (!SRCPROPSP (p))
     goto plist;
-  if      (SCM_EQ_P (scm_sym_breakpoint, key)) p = SRCPROPBRK (p);
-  else if (SCM_EQ_P (scm_sym_line,       key)) p = SCM_MAKINUM (SRCPROPLINE (p));
-  else if (SCM_EQ_P (scm_sym_column,     key)) p = SCM_MAKINUM (SRCPROPCOL (p));
-  else if (SCM_EQ_P (scm_sym_filename,   key)) p = SRCPROPFNAME (p);
-  else if (SCM_EQ_P (scm_sym_copy,       key)) p = SRCPROPCOPY (p);
+  if      (scm_is_eq (scm_sym_breakpoint, key)) p = scm_from_bool (SRCPROPBRK (p));
+  else if (scm_is_eq (scm_sym_line,       key)) p = scm_from_int (SRCPROPLINE (p));
+  else if (scm_is_eq (scm_sym_column,     key)) p = scm_from_int (SRCPROPCOL (p));
+  else if (scm_is_eq (scm_sym_filename,   key)) p = SRCPROPFNAME (p);
+  else if (scm_is_eq (scm_sym_copy,       key)) p = SRCPROPCOPY (p);
   else
     {
       p = SRCPROPPLIST (p);
@@ -250,13 +226,11 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
 {
   scm_whash_handle h;
   SCM p;
-  SCM_VALIDATE_NIM (1,obj);
+  SCM_VALIDATE_NIM (1, obj);
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
-#ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (!SCM_CONSP (obj))
     SCM_WRONG_TYPE_ARG (1, obj);
-#endif
   h = scm_whash_get_handle (scm_source_whash, obj);
   if (SCM_WHASHFOUNDP (h))
     p = SCM_WHASHREF (scm_source_whash, h);
@@ -265,11 +239,11 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
       h = scm_whash_create_handle (scm_source_whash, obj);
       p = SCM_EOL;
     }
-  if (SCM_EQ_P (scm_sym_breakpoint, key))
+  if (scm_is_eq (scm_sym_breakpoint, key))
     {
       if (SRCPROPSP (p))
 	{
-	  if (SCM_FALSEP (datum))
+	  if (scm_is_false (datum))
 	    CLEARSRCPROPBRK (p);
 	  else
 	    SETSRCPROPBRK (p);
@@ -278,40 +252,38 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
 	{
 	  SCM sp = scm_make_srcprops (0, 0, SCM_UNDEFINED, SCM_UNDEFINED, p);
 	  SCM_WHASHSET (scm_source_whash, h, sp);
-	  if (SCM_FALSEP (datum))
+	  if (scm_is_false (datum))
 	    CLEARSRCPROPBRK (sp);
 	  else
 	    SETSRCPROPBRK (sp);
 	}
     }
-  else if (SCM_EQ_P (scm_sym_line, key))
+  else if (scm_is_eq (scm_sym_line, key))
     {
-      SCM_VALIDATE_INUM (3,datum);
       if (SRCPROPSP (p))
-	SETSRCPROPLINE (p, SCM_INUM (datum));
+	SETSRCPROPLINE (p, scm_to_int (datum));
       else
 	SCM_WHASHSET (scm_source_whash, h,
-		      scm_make_srcprops (SCM_INUM (datum), 0,
+		      scm_make_srcprops (scm_to_int (datum), 0,
 					 SCM_UNDEFINED, SCM_UNDEFINED, p));
     }
-  else if (SCM_EQ_P (scm_sym_column, key))
+  else if (scm_is_eq (scm_sym_column, key))
     {
-      SCM_VALIDATE_INUM (3,datum);
       if (SRCPROPSP (p))
-	SETSRCPROPCOL (p, SCM_INUM (datum));
+	SETSRCPROPCOL (p, scm_to_int (datum));
       else
 	SCM_WHASHSET (scm_source_whash, h,
-		      scm_make_srcprops (0, SCM_INUM (datum),
+		      scm_make_srcprops (0, scm_to_int (datum),
 					 SCM_UNDEFINED, SCM_UNDEFINED, p));
     }
-  else if (SCM_EQ_P (scm_sym_filename, key))
+  else if (scm_is_eq (scm_sym_filename, key))
     {
       if (SRCPROPSP (p))
 	SRCPROPFNAME (p) = datum;
       else
 	SCM_WHASHSET (scm_source_whash, h, scm_make_srcprops (0, 0, datum, SCM_UNDEFINED, p));
     }
-  else if (SCM_EQ_P (scm_sym_copy, key))
+  else if (scm_is_eq (scm_sym_copy, key))
     {
       if (SRCPROPSP (p))
 	SRCPROPCOPY (p) = datum;
@@ -319,7 +291,12 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
 	SCM_WHASHSET (scm_source_whash, h, scm_make_srcprops (0, 0, SCM_UNDEFINED, datum, p));
     }
   else
-    SCM_WHASHSET (scm_source_whash, h, scm_acons (key, datum, p));
+    {
+      if (SRCPROPSP (p))
+	SRCPROPPLIST (p) = scm_acons (key, datum, SRCPROPPLIST (p));
+      else
+	SCM_WHASHSET (scm_source_whash, h, scm_acons (key, datum, p));
+    }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -333,7 +310,7 @@ scm_init_srcprop ()
   scm_set_smob_free (scm_tc16_srcprops, srcprops_free);
   scm_set_smob_print (scm_tc16_srcprops, srcprops_print);
 
-  scm_source_whash = scm_make_weak_key_hash_table (SCM_MAKINUM (2047));
+  scm_source_whash = scm_make_weak_key_hash_table (scm_from_int (2047));
   scm_c_define ("source-whash", scm_source_whash);
 
 #include "libguile/srcprop.x"
@@ -343,12 +320,13 @@ void
 scm_finish_srcprop ()
 {
   register scm_t_srcprops_chunk *ptr = srcprops_chunklist, *next;
+  size_t n= sizeof (scm_t_srcprops_chunk)
+    + sizeof (scm_t_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
   while (ptr)
     {
       next = ptr->next;
+      scm_gc_unregister_collectable_memory (ptr, n, "srcprops");
       free ((char *) ptr);
-      scm_mallocated -= sizeof (scm_t_srcprops_chunk)
-	                + sizeof (scm_t_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
       ptr = next;
     }
 }
