@@ -1,46 +1,24 @@
-/* Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
-
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 #include "libguile/chars.h"
@@ -81,16 +59,16 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
   SCM_VALIDATE_STRING (1, fields);
 
   { /* scope */
-    char * field_desc;
+    const char * field_desc;
     size_t len;
     int x;
 
-    len = SCM_STRING_LENGTH (fields);
+    len = scm_i_string_length (fields);
     if (len % 2 == 1)
       SCM_MISC_ERROR ("odd length field specification: ~S", 
 		      scm_list_1 (fields));
 
-    field_desc = SCM_STRING_CHARS (fields);
+    field_desc = scm_i_string_chars (fields);
 
     for (x = 0; x < len; x += 2)
       {
@@ -136,15 +114,16 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
 	  {
 	    if (field_desc[x + 2] != '-')
 	      SCM_MISC_ERROR ("missing dash field at position ~A",
-			      scm_list_1 (SCM_MAKINUM (x / 2)));
+			      scm_list_1 (scm_from_int (x / 2)));
 	    x += 2;
 	    goto recheck_ref;
 	  }
 #endif
       }
-    new_sym = scm_mem2symbol (field_desc, len);
+    new_sym = scm_string_to_symbol (fields);
   }
-  return scm_return_first (new_sym, fields);
+  scm_remember_upto_here_1 (fields);
+  return new_sym;
 }
 #undef FUNC_NAME
 
@@ -155,9 +134,10 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
 static void
 scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM inits)
 {
-  unsigned char * fields_desc = (unsigned char *) SCM_SYMBOL_CHARS (layout) - 2;
+  unsigned const char *fields_desc =
+    (unsigned const char *) scm_i_symbol_chars (layout) - 2;
   unsigned char prot = 0;
-  int n_fields = SCM_SYMBOL_LENGTH (layout) / 2;
+  int n_fields = scm_i_symbol_length (layout) / 2;
   int tailp = 0;
 
   while (n_fields)
@@ -185,7 +165,7 @@ scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM in
 	    *mem = 0;
 	  else
 	    {
-	      *mem = scm_num2long (SCM_CAR (inits), SCM_ARGn, "scm_struct_init");
+	      *mem = scm_to_long (SCM_CAR (inits));
 	      inits = SCM_CDR (inits);
 	    }
 	  break;
@@ -196,9 +176,7 @@ scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM in
 	    *mem = 0;
 	  else
 	    {
-	      *mem = scm_num2ulong (SCM_CAR (inits),
-				    SCM_ARGn,
-				    "scm_struct_init");
+	      *mem = scm_to_ulong (SCM_CAR (inits));
 	      inits = SCM_CDR (inits);
 	    }
 	  break;
@@ -244,7 +222,7 @@ SCM_DEFINE (scm_struct_p, "struct?", 1, 0, 0,
 	    "@code{#f}.")
 #define FUNC_NAME s_scm_struct_p
 {
-  return SCM_BOOL(SCM_STRUCTP (x));
+  return scm_from_bool(SCM_STRUCTP (x));
 }
 #undef FUNC_NAME
 
@@ -255,25 +233,27 @@ SCM_DEFINE (scm_struct_vtable_p, "struct-vtable?", 1, 0, 0,
 {
   SCM layout;
   scm_t_bits * mem;
+  int tmp;
 
   if (!SCM_STRUCTP (x))
     return SCM_BOOL_F;
 
   layout = SCM_STRUCT_LAYOUT (x);
 
-  if (SCM_SYMBOL_LENGTH (layout) < SCM_STRING_LENGTH (required_vtable_fields))
+  if (scm_i_symbol_length (layout)
+      < scm_i_string_length (required_vtable_fields))
     return SCM_BOOL_F;
 
-  if (strncmp (SCM_SYMBOL_CHARS (layout), SCM_STRING_CHARS (required_vtable_fields),
-	       SCM_STRING_LENGTH (required_vtable_fields)))
+  tmp = strncmp (scm_i_symbol_chars (layout),
+		 scm_i_string_chars (required_vtable_fields),
+		 scm_i_string_length (required_vtable_fields));
+  scm_remember_upto_here_1 (required_vtable_fields);
+  if (tmp)
     return SCM_BOOL_F;
 
   mem = SCM_STRUCT_DATA (x);
 
-  if (mem[1] != 0)
-    return SCM_BOOL_F;
-
-  return SCM_BOOL (SCM_SYMBOLP (SCM_PACK (mem[0])));
+  return scm_from_bool (scm_is_symbol (SCM_PACK (mem[scm_vtable_index_layout])));
 }
 #undef FUNC_NAME
 
@@ -309,10 +289,10 @@ SCM_DEFINE (scm_struct_vtable_p, "struct-vtable?", 1, 0, 0,
 
 
 scm_t_bits *
-scm_alloc_struct (int n_words, int n_extra, char *who)
+scm_alloc_struct (int n_words, int n_extra, const char *what)
 {
   int size = sizeof (scm_t_bits) * (n_words + n_extra) + 7;
-  void * block = scm_must_malloc (size, who);
+  void * block = scm_gc_malloc (size, what);
 
   /* Adjust the pointer to hide the extra words.  */
   scm_t_bits * p = (scm_t_bits *) block + n_extra;
@@ -329,36 +309,33 @@ scm_alloc_struct (int n_words, int n_extra, char *who)
   return p;
 }
 
-size_t
+void
 scm_struct_free_0 (scm_t_bits * vtable SCM_UNUSED,
 		   scm_t_bits * data SCM_UNUSED)
 {
-  return 0;
 }
 
-size_t
+void
 scm_struct_free_light (scm_t_bits * vtable, scm_t_bits * data)
 {
-  scm_must_free (data);
-  return vtable [scm_struct_i_size] & ~SCM_STRUCTF_MASK;
+  size_t n = vtable [scm_struct_i_size] & ~SCM_STRUCTF_MASK;
+  scm_gc_free (data, n, "struct");
 }
 
-size_t
+void
 scm_struct_free_standard (scm_t_bits * vtable SCM_UNUSED, scm_t_bits * data)
 {
   size_t n = (data[scm_struct_i_n_words] + scm_struct_n_extra_words)
 	     * sizeof (scm_t_bits) + 7;
-  scm_must_free ((void *) data[scm_struct_i_ptr]);
-  return n;
+  scm_gc_free ((void *) data[scm_struct_i_ptr], n, "heavy struct");
 }
 
-size_t
+void
 scm_struct_free_entity (scm_t_bits * vtable SCM_UNUSED, scm_t_bits * data)
 {
   size_t n = (data[scm_struct_i_n_words] + scm_struct_entity_n_extra_words)
 	     * sizeof (scm_t_bits) + 7;
-  scm_must_free ((void *) data[scm_struct_i_ptr]);
-  return n;
+  scm_gc_free ((void *) data[scm_struct_i_ptr], n, "entity struct");
 }
 
 static void *
@@ -366,7 +343,7 @@ scm_struct_gc_init (void *dummy1 SCM_UNUSED,
 		    void *dummy2 SCM_UNUSED,
 		    void *dummy3 SCM_UNUSED)
 {
-  scm_structs_to_free = SCM_EOL;
+  scm_i_structs_to_free = SCM_EOL;
   return 0;
 }
 
@@ -375,7 +352,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 		  void *dummy2 SCM_UNUSED,
 		  void *dummy3 SCM_UNUSED)
 {
-  SCM newchain = scm_structs_to_free;
+  SCM newchain = scm_i_structs_to_free;
   do
     {
       /* Mark vtables in GC chain.  GC mark set means delay freeing. */
@@ -384,7 +361,7 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 	{
 	  SCM vtable = SCM_STRUCT_VTABLE (chain);
 	  if (SCM_STRUCT_GC_CHAIN (vtable) != 0 && vtable != chain)
-	    SCM_SETGCMARK (vtable);
+	    SCM_SET_GC_MARK (vtable);
 	  chain = SCM_STRUCT_GC_CHAIN (chain);
 	}
       /* Free unmarked structs.  */
@@ -394,16 +371,16 @@ scm_free_structs (void *dummy1 SCM_UNUSED,
 	{
 	  SCM obj = chain;
 	  chain = SCM_STRUCT_GC_CHAIN (chain);
-	  if (SCM_GCMARKP (obj))
+	  if (SCM_GC_MARK_P (obj))
 	    {
-	      SCM_CLRGCMARK (obj);
+	      SCM_CLEAR_GC_MARK (obj);
 	      SCM_SET_STRUCT_GC_CHAIN (obj, newchain);
 	      newchain = obj;
 	    }
 	  else
 	    {
-	      scm_t_bits word0 = SCM_CELL_WORD_0 (obj) - scm_tc3_cons_gloc;
-	      /* access as struct */
+	      /* XXX - use less explicit code. */
+	      scm_t_bits word0 = SCM_CELL_WORD_0 (obj) - scm_tc3_struct;
 	      scm_t_bits * vtable_data = (scm_t_bits *) word0;
 	      scm_t_bits * data = SCM_STRUCT_DATA (obj);
 	      scm_t_struct_free free_struct_data
@@ -441,36 +418,34 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
 #define FUNC_NAME s_scm_make_struct
 {
   SCM layout;
-  int basic_size;
-  int tail_elts;
+  size_t basic_size;
+  size_t tail_elts;
   scm_t_bits * data;
   SCM handle;
 
-  SCM_VALIDATE_VTABLE (1,vtable);
-  SCM_VALIDATE_INUM (2,tail_array_size);
+  SCM_VALIDATE_VTABLE (1, vtable);
   SCM_VALIDATE_REST_ARGUMENT (init);
 
   layout = SCM_PACK (SCM_STRUCT_DATA (vtable) [scm_vtable_index_layout]);
-  basic_size = SCM_SYMBOL_LENGTH (layout) / 2;
-  tail_elts = SCM_INUM (tail_array_size);
-  SCM_NEWCELL2 (handle);
+  basic_size = scm_i_symbol_length (layout) / 2;
+  tail_elts = scm_to_size_t (tail_array_size);
   SCM_DEFER_INTS;
   if (SCM_STRUCT_DATA (vtable)[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
     {
       data = scm_alloc_struct (basic_size + tail_elts,
 			       scm_struct_entity_n_extra_words,
-			       "make-struct");
+			       "entity struct");
       data[scm_struct_i_procedure] = SCM_UNPACK (SCM_BOOL_F);
       data[scm_struct_i_setter] = SCM_UNPACK (SCM_BOOL_F);
     }
   else
     data = scm_alloc_struct (basic_size + tail_elts,
 			     scm_struct_n_extra_words,
-			     "make-struct");
-  SCM_SET_CELL_WORD_1 (handle, data);
-  SCM_SET_STRUCT_GC_CHAIN (handle, 0);
+			     "struct");
+  handle = scm_double_cell ((((scm_t_bits) SCM_STRUCT_DATA (vtable))
+			     + scm_tc3_struct),
+			    (scm_t_bits) data, 0, 0);
   scm_struct_init (handle, layout, data, tail_elts, init);
-  SCM_SET_CELL_WORD_0 (handle, (scm_t_bits) SCM_STRUCT_DATA (vtable) + scm_tc3_cons_gloc);
   SCM_ALLOW_INTS;
   return handle;
 }
@@ -528,30 +503,27 @@ SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
 {
   SCM fields;
   SCM layout;
-  int basic_size;
-  int tail_elts;
-  scm_t_bits * data;
+  size_t basic_size;
+  size_t tail_elts;
+  scm_t_bits *data;
   SCM handle;
 
   SCM_VALIDATE_STRING (1, user_fields);
-  SCM_VALIDATE_INUM (2, tail_array_size);
   SCM_VALIDATE_REST_ARGUMENT (init);
 
   fields = scm_string_append (scm_list_2 (required_vtable_fields,
 					  user_fields));
   layout = scm_make_struct_layout (fields);
-  basic_size = SCM_SYMBOL_LENGTH (layout) / 2;
-  tail_elts = SCM_INUM (tail_array_size);
-  SCM_NEWCELL2 (handle);
+  basic_size = scm_i_symbol_length (layout) / 2;
+  tail_elts = scm_to_size_t (tail_array_size);
   SCM_DEFER_INTS;
   data = scm_alloc_struct (basic_size + tail_elts,
 			   scm_struct_n_extra_words,
-			   "make-vtable-vtable");
-  SCM_SET_CELL_WORD_1 (handle, data);
-  SCM_SET_STRUCT_GC_CHAIN (handle, 0);
+			   "struct");
+  handle = scm_double_cell ((scm_t_bits) data + scm_tc3_struct,
+			    (scm_t_bits) data, 0, 0);
   data [scm_vtable_index_layout] = SCM_UNPACK (layout);
   scm_struct_init (handle, layout, data, tail_elts, scm_cons (layout, init));
-  SCM_SET_CELL_WORD_0 (handle, (scm_t_bits) data + scm_tc3_cons_gloc);
   SCM_ALLOW_INTS;
   return handle;
 }
@@ -572,25 +544,26 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
   SCM answer = SCM_UNDEFINED;
   scm_t_bits * data;
   SCM layout;
-  int p;
+  size_t layout_len;
+  size_t p;
   scm_t_bits n_fields;
-  char * fields_desc;
+  const char *fields_desc;
   char field_type = 0;
   
 
-  SCM_VALIDATE_STRUCT (1,handle);
-  SCM_VALIDATE_INUM (2,pos);
+  SCM_VALIDATE_STRUCT (1, handle);
 
   layout = SCM_STRUCT_LAYOUT (handle);
   data = SCM_STRUCT_DATA (handle);
-  p = SCM_INUM (pos);
+  p = scm_to_size_t (pos);
 
-  fields_desc = SCM_SYMBOL_CHARS (layout);
+  fields_desc = scm_i_symbol_chars (layout);
+  layout_len = scm_i_symbol_length (layout);
   n_fields = data[scm_struct_i_n_words];
   
-  SCM_ASSERT_RANGE(1,pos, p < n_fields);
+  SCM_ASSERT_RANGE(1, pos, p < n_fields);
 
-  if (p * 2 < SCM_SYMBOL_LENGTH (layout))
+  if (p * 2 < layout_len)
     {
       char ref;
       field_type = fields_desc[p * 2];
@@ -603,20 +576,20 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
 	    SCM_MISC_ERROR ("ref denied for field ~A", scm_list_1 (pos));
 	}
     }
-  else if (fields_desc[SCM_SYMBOL_LENGTH (layout) - 1] != 'O')    
-    field_type = fields_desc[SCM_SYMBOL_LENGTH (layout) - 2];
+  else if (fields_desc[layout_len - 1] != 'O')    
+    field_type = fields_desc[layout_len - 2];
   else
     SCM_MISC_ERROR ("ref denied for field ~A", scm_list_1 (pos));
   
   switch (field_type)
     {
     case 'u':
-      answer = scm_ulong2num (data[p]);
+      answer = scm_from_ulong (data[p]);
       break;
 
 #if 0
     case 'i':
-      answer = scm_long2num (data[p]);
+      answer = scm_from_long (data[p]);
       break;
 
     case 'd':
@@ -649,24 +622,25 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
 {
   scm_t_bits * data;
   SCM layout;
-  int p;
+  size_t layout_len;
+  size_t p;
   int n_fields;
-  char * fields_desc;
+  const char *fields_desc;
   char field_type = 0;
 
-  SCM_VALIDATE_STRUCT (1,handle);
-  SCM_VALIDATE_INUM (2,pos);
+  SCM_VALIDATE_STRUCT (1, handle);
 
   layout = SCM_STRUCT_LAYOUT (handle);
   data = SCM_STRUCT_DATA (handle);
-  p = SCM_INUM (pos);
+  p = scm_to_size_t (pos);
 
-  fields_desc = SCM_SYMBOL_CHARS (layout);
+  fields_desc = scm_i_symbol_chars (layout);
+  layout_len = scm_i_symbol_length (layout);
   n_fields = data[scm_struct_i_n_words];
 
-  SCM_ASSERT_RANGE (1,pos, p < n_fields);
+  SCM_ASSERT_RANGE (1, pos, p < n_fields);
 
-  if (p * 2 < SCM_SYMBOL_LENGTH (layout))
+  if (p * 2 < layout_len)
     {
       char set_x;
       field_type = fields_desc[p * 2];
@@ -674,8 +648,8 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
       if (set_x != 'w')
 	SCM_MISC_ERROR ("set! denied for field ~A", scm_list_1 (pos));
     }
-  else if (fields_desc[SCM_SYMBOL_LENGTH (layout) - 1] == 'W')    
-    field_type = fields_desc[SCM_SYMBOL_LENGTH (layout) - 2];
+  else if (fields_desc[layout_len - 1] == 'W')    
+    field_type = fields_desc[layout_len - 2];
   else
     SCM_MISC_ERROR ("set! denied for field ~A", scm_list_1 (pos));
   
@@ -717,7 +691,7 @@ SCM_DEFINE (scm_struct_vtable, "struct-vtable", 1, 0, 0,
 	    "Return the vtable structure that describes the type of @var{struct}.")
 #define FUNC_NAME s_scm_struct_vtable
 {
-  SCM_VALIDATE_STRUCT (1,handle);
+  SCM_VALIDATE_STRUCT (1, handle);
   return SCM_STRUCT_VTABLE (handle);
 }
 #undef FUNC_NAME
@@ -728,8 +702,8 @@ SCM_DEFINE (scm_struct_vtable_tag, "struct-vtable-tag", 1, 0, 0,
 	    "Return the vtable tag of the structure @var{handle}.")
 #define FUNC_NAME s_scm_struct_vtable_tag
 {
-  SCM_VALIDATE_VTABLE (1,handle);
-  return scm_long2num ((long) SCM_STRUCT_DATA (handle) >> 3);
+  SCM_VALIDATE_VTABLE (1, handle);
+  return scm_from_ulong (((unsigned long)SCM_STRUCT_DATA (handle)) >> 3);
 }
 #undef FUNC_NAME
 
@@ -757,7 +731,7 @@ scm_struct_create_handle (SCM obj)
 					    scm_struct_ihashq,
 					    scm_sloppy_assq,
 					    0);
-  if (SCM_FALSEP (SCM_CDR (handle)))
+  if (scm_is_false (SCM_CDR (handle)))
     SCM_SETCDR (handle, scm_cons (SCM_BOOL_F, SCM_BOOL_F));
   return handle;
 }
@@ -767,7 +741,7 @@ SCM_DEFINE (scm_struct_vtable_name, "struct-vtable-name", 1, 0, 0,
 	    "Return the name of the vtable @var{vtable}.")
 #define FUNC_NAME s_scm_struct_vtable_name
 {
-  SCM_VALIDATE_VTABLE (1,vtable);
+  SCM_VALIDATE_VTABLE (1, vtable);
   return SCM_STRUCT_TABLE_NAME (SCM_CDR (scm_struct_create_handle (vtable)));
 }
 #undef FUNC_NAME
@@ -777,8 +751,8 @@ SCM_DEFINE (scm_set_struct_vtable_name_x, "set-struct-vtable-name!", 2, 0, 0,
 	    "Set the name of the vtable @var{vtable} to @var{name}.")
 #define FUNC_NAME s_scm_set_struct_vtable_name_x
 {
-  SCM_VALIDATE_VTABLE (1,vtable);
-  SCM_VALIDATE_SYMBOL (2,name);
+  SCM_VALIDATE_VTABLE (1, vtable);
+  SCM_VALIDATE_SYMBOL (2, name);
   SCM_SET_STRUCT_TABLE_NAME (SCM_CDR (scm_struct_create_handle (vtable)),
 			     name);
   return SCM_UNSPECIFIED;
@@ -791,14 +765,14 @@ SCM_DEFINE (scm_set_struct_vtable_name_x, "set-struct-vtable-name!", 2, 0, 0,
 void
 scm_print_struct (SCM exp, SCM port, scm_print_state *pstate)
 {
-  if (SCM_NFALSEP (scm_procedure_p (SCM_STRUCT_PRINTER (exp))))
+  if (scm_is_true (scm_procedure_p (SCM_STRUCT_PRINTER (exp))))
     scm_printer_apply (SCM_STRUCT_PRINTER (exp), exp, port, pstate);
   else
     {
       SCM vtable = SCM_STRUCT_VTABLE (exp);
       SCM name = scm_struct_vtable_name (vtable);
       scm_puts ("#<", port);
-      if (SCM_NFALSEP (name))
+      if (scm_is_true (name))
 	scm_display (name, port);
       else
 	scm_puts ("struct", port);
@@ -813,22 +787,25 @@ scm_print_struct (SCM exp, SCM port, scm_print_state *pstate)
 void
 scm_struct_prehistory ()
 {
-  scm_c_hook_add (&scm_before_mark_c_hook, scm_struct_gc_init, 0, 0);
-  scm_c_hook_add (&scm_after_sweep_c_hook, scm_free_structs, 0, 0);
+  scm_i_structs_to_free = SCM_EOL;
+  scm_c_hook_add (&scm_before_sweep_c_hook, scm_struct_gc_init, 0, 0);
+  /* With the new lazy sweep GC, the point at which the entire heap is
+     swept is just before the mark phase. */
+  scm_c_hook_add (&scm_before_mark_c_hook, scm_free_structs, 0, 0);
 }
 
 void
 scm_init_struct ()
 {
   scm_struct_table
-    = scm_permanent_object (scm_make_weak_key_hash_table (SCM_MAKINUM (31)));
-  required_vtable_fields = scm_makfrom0str ("pruosrpw");
+    = scm_permanent_object (scm_make_weak_key_hash_table (scm_from_int (31)));
+  required_vtable_fields = scm_from_locale_string ("prsrpw");
   scm_permanent_object (required_vtable_fields);
-  scm_c_define ("vtable-index-layout", SCM_MAKINUM (scm_vtable_index_layout));
-  scm_c_define ("vtable-index-vtable", SCM_MAKINUM (scm_vtable_index_vtable));
+  scm_c_define ("vtable-index-layout", scm_from_int (scm_vtable_index_layout));
+  scm_c_define ("vtable-index-vtable", scm_from_int (scm_vtable_index_vtable));
   scm_c_define ("vtable-index-printer",
-		SCM_MAKINUM (scm_vtable_index_printer));
-  scm_c_define ("vtable-offset-user", SCM_MAKINUM (scm_vtable_offset_user));
+		scm_from_int (scm_vtable_index_printer));
+  scm_c_define ("vtable-offset-user", scm_from_int (scm_vtable_offset_user));
 #include "libguile/struct.x"
 }
 
