@@ -1,46 +1,25 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003, 2004 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include "libguile/_scm.h"
 #include "libguile/chars.h"
@@ -55,6 +34,8 @@
 #include "libguile/hashtab.h"
 #include "libguile/weaks.h"
 #include "libguile/modules.h"
+#include "libguile/read.h"
+#include "libguile/srfi-13.h"
 
 #include "libguile/validate.h"
 #include "libguile/symbols.h"
@@ -83,25 +64,49 @@ SCM_DEFINE (scm_sys_symbols, "%symbols", 0, 0, 0,
 /* {Symbols}
  */
 
+/* In order to optimize reading speed, this function breaks part of
+ * the hashtable abstraction.  The optimizations are:
+ *
+ * 1. The argument string can be compared directly to symbol objects
+ *    without first creating an SCM string object.  (This would have
+ *    been necessary if we had used the hashtable API in hashtab.h.)
+ *
+ * 2. We can use the raw hash value stored in scm_i_symbol_hash (sym)
+ *    to speed up lookup.
+ *
+ * Both optimizations might be possible without breaking the
+ * abstraction if the API in hashtab.c is improved.
+ */
 
-SCM
-scm_mem2symbol (const char *name, size_t len)
+unsigned long
+scm_i_hash_symbol (SCM obj, unsigned long n, void *closure)
 {
-  size_t raw_hash = scm_string_hash ((const unsigned char *) name, len);
-  size_t hash = raw_hash % SCM_VECTOR_LENGTH (symbols);
+  return scm_i_symbol_hash (obj) % n;
+}
+
+static SCM
+scm_i_mem2symbol (SCM str)
+{
+  const char *name = scm_i_string_chars (str);
+  size_t len = scm_i_string_length (str);
+
+  size_t raw_hash = scm_string_hash ((const unsigned char *) name, len) / 2;
+  size_t hash = raw_hash % SCM_HASHTABLE_N_BUCKETS (symbols);
 
   {
     /* Try to find the symbol in the symbols table */
 
     SCM l;
 
-    for (l = SCM_VELTS (symbols) [hash]; !SCM_NULLP (l); l = SCM_CDR (l))
+    for (l = SCM_HASHTABLE_BUCKETS (symbols) [hash];
+	 !SCM_NULLP (l);
+	 l = SCM_CDR (l))
       {
 	SCM sym = SCM_CAAR (l);
-	if (SCM_SYMBOL_HASH (sym) == raw_hash
-	    && SCM_SYMBOL_LENGTH (sym) == len)
+	if (scm_i_symbol_hash (sym) == raw_hash
+	    && scm_i_symbol_length (sym) == len)
 	  {
-	    char *chrs = SCM_SYMBOL_CHARS (sym);
+	    const char *chrs = scm_i_symbol_chars (sym);
 	    size_t i = len;
 
 	    while (i != 0)
@@ -120,30 +125,31 @@ scm_mem2symbol (const char *name, size_t len)
 
   {
     /* The symbol was not found - create it. */
+    SCM symbol = scm_i_make_symbol (str, raw_hash,
+				    scm_cons (SCM_BOOL_F, SCM_EOL));
 
-    SCM symbol;
-    SCM cell;
-    SCM slot;
-
-    SCM_NEWCELL2 (symbol);
-    SCM_SET_SYMBOL_CHARS (symbol, scm_must_strndup (name, len));
-    SCM_SET_SYMBOL_HASH (symbol, raw_hash);
-    SCM_SET_PROP_SLOTS (symbol, scm_cons (SCM_BOOL_F, SCM_EOL));
-    SCM_SET_SYMBOL_LENGTH (symbol, (long) len);
-
-    slot = SCM_VELTS (symbols) [hash];
-    cell = scm_cons (symbol, SCM_UNDEFINED);
-    SCM_VELTS (symbols) [hash] = scm_cons (cell, slot);
+    SCM slot = SCM_HASHTABLE_BUCKETS (symbols) [hash];
+    SCM cell = scm_cons (symbol, SCM_UNDEFINED);
+    SCM_SET_HASHTABLE_BUCKET (symbols, hash, scm_cons (cell, slot));
+    SCM_HASHTABLE_INCREMENT (symbols);
+    if (SCM_HASHTABLE_N_ITEMS (symbols) > SCM_HASHTABLE_UPPER (symbols))
+      scm_i_rehash (symbols, scm_i_hash_symbol, 0, "scm_mem2symbol");
 
     return symbol;
   }
 }
 
-
-SCM
-scm_str2symbol (const char *str)
+static SCM
+scm_i_mem2uninterned_symbol (SCM str)
 {
-  return scm_mem2symbol (str, strlen (str));
+  const char *name = scm_i_string_chars (str);
+  size_t len = scm_i_string_length (str);
+
+  size_t raw_hash = (scm_string_hash ((const unsigned char *) name, len)/2
+		     + SCM_T_BITS_MAX/2 + 1);
+
+  return scm_i_make_symbol (str, raw_hash,
+			    scm_cons (SCM_BOOL_F, SCM_EOL));
 }
 
 SCM_DEFINE (scm_symbol_p, "symbol?", 1, 0, 0, 
@@ -152,7 +158,30 @@ SCM_DEFINE (scm_symbol_p, "symbol?", 1, 0, 0,
 	    "@code{#f}.")
 #define FUNC_NAME s_scm_symbol_p
 {
-  return SCM_BOOL (SCM_SYMBOLP (obj));
+  return scm_from_bool (scm_is_symbol (obj));
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_symbol_interned_p, "symbol-interned?", 1, 0, 0, 
+	    (SCM symbol),
+	    "Return @code{#t} if @var{symbol} is interned, otherwise return\n"
+	    "@code{#f}.")
+#define FUNC_NAME s_scm_symbol_interned_p
+{
+  SCM_VALIDATE_SYMBOL (1, symbol);
+  return scm_from_bool (scm_i_symbol_is_interned (symbol));
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_make_symbol, "make-symbol", 1, 0, 0,
+	    (SCM name),
+	    "Return a new uninterned symbol with the name @var{name}.  " 
+	    "The returned symbol is guaranteed to be unique and future "
+	    "calls to @code{string->symbol} will not return it.")
+#define FUNC_NAME s_scm_make_symbol
+{
+  SCM_VALIDATE_STRING (1, name);
+  return scm_i_mem2uninterned_symbol (name);
 }
 #undef FUNC_NAME
 
@@ -183,11 +212,8 @@ SCM_DEFINE (scm_symbol_to_string, "symbol->string", 1, 0, 0,
 	    "@end lisp")
 #define FUNC_NAME s_scm_symbol_to_string
 {
-  SCM str;
   SCM_VALIDATE_SYMBOL (1, s);
-  str = scm_mem2string (SCM_SYMBOL_CHARS (s), SCM_SYMBOL_LENGTH (s));
-  scm_remember_upto_here_1 (s);
-  return str;
+  return scm_i_symbol_substring (s, 0, scm_i_symbol_length (s));
 }
 #undef FUNC_NAME
 
@@ -217,15 +243,25 @@ SCM_DEFINE (scm_string_to_symbol, "string->symbol", 1, 0, 0,
 #define FUNC_NAME s_scm_string_to_symbol
 {
   SCM_VALIDATE_STRING (1, string);
-  return scm_mem2symbol (SCM_STRING_CHARS (string),
-			 SCM_STRING_LENGTH (string));
+  return scm_i_mem2symbol (string);
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_string_ci_to_symbol, "string-ci->symbol", 1, 0, 0,
+	    (SCM str),
+	    "Return the symbol whose name is @var{str}.  @var{str} is\n"
+	    "converted to lowercase before the conversion is done, if Guile\n"
+	    "is currently reading symbols case-insensitively.")
+#define FUNC_NAME s_scm_string_ci_to_symbol
+{
+  return scm_string_to_symbol (SCM_CASE_INSENSITIVE_P
+			       ? scm_string_downcase(str)
+			       : str);
 }
 #undef FUNC_NAME
 
 #define MAX_PREFIX_LENGTH 30
 
-static int gensym_counter;
-
 SCM_DEFINE (scm_gensym, "gensym", 0, 1, 0,
             (SCM prefix),
 	    "Create a new symbol with a name constructed from a prefix and\n"
@@ -235,30 +271,24 @@ SCM_DEFINE (scm_gensym, "gensym", 0, 1, 0,
 	    "resetting the counter.")
 #define FUNC_NAME s_scm_gensym
 {
-  char buf[MAX_PREFIX_LENGTH + SCM_INTBUFLEN];
-  char *name = buf;
-  size_t len;
+  static int gensym_counter = 0;
+  
+  SCM suffix, name;
+  int n, n_digits;
+  char buf[SCM_INTBUFLEN];
+
   if (SCM_UNBNDP (prefix))
-    {
-      name[0] = ' ';
-      name[1] = 'g';
-      len = 2;
-    }
-  else
-    {
-      SCM_VALIDATE_STRING (1, prefix);
-      len = SCM_STRING_LENGTH (prefix);
-      if (len > MAX_PREFIX_LENGTH)
-	name = SCM_MUST_MALLOC (len + SCM_INTBUFLEN);
-      memcpy (name, SCM_STRING_CHARS (prefix), len);
-    }
-  {
-    int n_digits = scm_iint2str (gensym_counter++, 10, &name[len]);
-    SCM res = scm_mem2symbol (name, len + n_digits);
-    if (name != buf)
-      scm_must_free (name);
-    return res;
-  }
+    prefix = scm_from_locale_string (" g");
+  
+  /* mutex in case another thread looks and incs at the exact same moment */
+  scm_mutex_lock (&scm_i_misc_mutex);
+  n = gensym_counter++;
+  scm_mutex_unlock (&scm_i_misc_mutex);
+
+  n_digits = scm_iint2str (n, 10, buf);
+  suffix = scm_from_locale_stringn (buf, n_digits);
+  name = scm_string_append (scm_list_2 (prefix, suffix));
+  return scm_string_to_symbol (name);
 }
 #undef FUNC_NAME
 
@@ -268,7 +298,7 @@ SCM_DEFINE (scm_symbol_hash, "symbol-hash", 1, 0, 0,
 #define FUNC_NAME s_scm_symbol_hash
 {
   SCM_VALIDATE_SYMBOL (1, symbol);
-  return SCM_MAKINUM (SCM_SYMBOL_HASH (symbol));
+  return scm_from_ulong (scm_i_symbol_hash (symbol));
 }
 #undef FUNC_NAME
 
@@ -277,8 +307,8 @@ SCM_DEFINE (scm_symbol_fref, "symbol-fref", 1, 0, 0,
 	    "Return the contents of @var{symbol}'s @dfn{function slot}.")
 #define FUNC_NAME s_scm_symbol_fref
 {
-  SCM_VALIDATE_SYMBOL (1,s);
-  return SCM_SYMBOL_FUNC (s);
+  SCM_VALIDATE_SYMBOL (1, s);
+  return SCM_CAR (SCM_CELL_OBJECT_3 (s));
 }
 #undef FUNC_NAME
 
@@ -288,8 +318,8 @@ SCM_DEFINE (scm_symbol_pref, "symbol-pref", 1, 0, 0,
 	    "Return the @dfn{property list} currently associated with @var{symbol}.")
 #define FUNC_NAME s_scm_symbol_pref
 {
-  SCM_VALIDATE_SYMBOL (1,s);
-  return SCM_SYMBOL_PROPS (s);
+  SCM_VALIDATE_SYMBOL (1, s);
+  return SCM_CDR (SCM_CELL_OBJECT_3 (s));
 }
 #undef FUNC_NAME
 
@@ -299,8 +329,8 @@ SCM_DEFINE (scm_symbol_fset_x, "symbol-fset!", 2, 0, 0,
 	    "Change the binding of @var{symbol}'s function slot.")
 #define FUNC_NAME s_scm_symbol_fset_x
 {
-  SCM_VALIDATE_SYMBOL (1,s);
-  SCM_SET_SYMBOL_FUNC (s, val);
+  SCM_VALIDATE_SYMBOL (1, s);
+  SCM_SETCAR (SCM_CELL_OBJECT_3 (s), val);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -311,18 +341,28 @@ SCM_DEFINE (scm_symbol_pset_x, "symbol-pset!", 2, 0, 0,
 	    "Change the binding of @var{symbol}'s property slot.")
 #define FUNC_NAME s_scm_symbol_pset_x
 {
-  SCM_VALIDATE_SYMBOL (1,s);
-  SCM_DEFER_INTS;
-  SCM_SET_SYMBOL_PROPS (s, val);
-  SCM_ALLOW_INTS;
+  SCM_VALIDATE_SYMBOL (1, s);
+  SCM_SETCDR (SCM_CELL_OBJECT_3 (s), val);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
+SCM
+scm_from_locale_symbol (const char *sym)
+{
+  return scm_string_to_symbol (scm_from_locale_string (sym));
+}
+
+SCM
+scm_from_locale_symboln (const char *sym, size_t len)
+{
+  return scm_string_to_symbol (scm_from_locale_stringn (sym, len));
+}
+
 void
 scm_symbols_prehistory ()
 {
-  symbols = scm_make_weak_key_hash_table (SCM_MAKINUM (1009));
+  symbols = scm_make_weak_key_hash_table (scm_from_int (2139));
   scm_permanent_object (symbols);
 }
 
@@ -330,11 +370,7 @@ scm_symbols_prehistory ()
 void
 scm_init_symbols ()
 {
-  gensym_counter = 0;
 #include "libguile/symbols.x"
-#if SCM_ENABLE_VCELLS
-  scm_init_symbols_deprecated ();
-#endif
 }
 
 /*
