@@ -1,43 +1,19 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -50,14 +26,13 @@
 #include "libguile/eq.h"
 #include "libguile/dynwind.h"
 #include "libguile/backtrace.h"
-#ifdef DEBUG_EXTENSIONS
 #include "libguile/debug.h"
-#endif
 #include "libguile/continuations.h"
 #include "libguile/stackchk.h"
 #include "libguile/stacks.h"
 #include "libguile/fluids.h"
 #include "libguile/ports.h"
+#include "libguile/lang.h"
 
 #include "libguile/validate.h"
 #include "libguile/throw.h"
@@ -75,11 +50,9 @@ static scm_t_bits tc16_jmpbuffer;
   (SCM_SET_CELL_WORD_0 ((x), (SCM_CELL_WORD_0 (x) & ~(1L << 16L))))
 
 #define JBJMPBUF(OBJ)           ((jmp_buf *) SCM_CELL_WORD_1 (OBJ))
-#define SETJBJMPBUF(x,v)        (SCM_SET_CELL_WORD_1 ((x), (v)))
-#ifdef DEBUG_EXTENSIONS
+#define SETJBJMPBUF(x, v)        (SCM_SET_CELL_WORD_1 ((x), (scm_t_bits) (v)))
 #define SCM_JBDFRAME(x)         ((scm_t_debug_frame *) SCM_CELL_WORD_2 (x))
-#define SCM_SETJBDFRAME(x,v)    (SCM_SET_CELL_WORD_2 ((x), (v)))
-#endif
+#define SCM_SETJBDFRAME(x, v)    (SCM_SET_CELL_WORD_2 ((x), (scm_t_bits) (v)))
 
 static int
 jmpbuffer_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
@@ -97,11 +70,7 @@ make_jmpbuf (void)
   SCM answer;
   SCM_REDEFER_INTS;
   {
-#ifdef DEBUG_EXTENSIONS
     SCM_NEWSMOB2 (answer, tc16_jmpbuffer, 0, 0);
-#else
-    SCM_NEWSMOB (answer, tc16_jmpbuffer, 0);
-#endif
     SETJBJMPBUF(answer, (jmp_buf *)0);
     DEACTIVATEJB(answer);
   }
@@ -178,9 +147,7 @@ scm_internal_catch (SCM tag, scm_t_catch_body body, void *body_data, scm_t_catch
   answer = SCM_EOL;
   scm_dynwinds = scm_acons (tag, jmpbuf, scm_dynwinds);
   SETJBJMPBUF(jmpbuf, &jbr.buf);
-#ifdef DEBUG_EXTENSIONS
   SCM_SETJBDFRAME(jmpbuf, scm_last_debug_frame);
-#endif
   if (setjmp (jbr.buf))
     {
       SCM throw_tag;
@@ -401,13 +368,13 @@ scm_handle_by_proc_catching_all (void *handler_data, SCM tag, SCM throw_args)
 int
 scm_exit_status (SCM args)
 {
-  if (SCM_NNULLP (args))
+  if (!SCM_NULL_OR_NIL_P (args))
     {
       SCM cqa = SCM_CAR (args);
       
-      if (SCM_INUMP (cqa))
-	return (SCM_INUM (cqa));
-      else if (SCM_FALSEP (cqa))
+      if (scm_is_integer (cqa))
+	return (scm_to_int (cqa));
+      else if (scm_is_false (cqa))
 	return 1;
     }
   return 0;
@@ -428,7 +395,7 @@ handler_message (void *handler_data, SCM tag, SCM args)
       SCM parts   = SCM_CADDR (args);
       SCM rest    = SCM_CDDDR (args);
 
-      if (SCM_BACKTRACE_P && SCM_NFALSEP (stack))
+      if (SCM_BACKTRACE_P && scm_is_true (stack))
 	{
 	  scm_puts ("Backtrace:\n", p);
 	  scm_display_backtrace (stack, p, SCM_UNDEFINED, SCM_UNDEFINED);
@@ -477,7 +444,7 @@ handler_message (void *handler_data, SCM tag, SCM args)
 SCM
 scm_handle_by_message (void *handler_data, SCM tag, SCM args)
 {
-  if (SCM_NFALSEP (scm_eq_p (tag, scm_str2symbol ("quit"))))
+  if (scm_is_true (scm_eq_p (tag, scm_from_locale_symbol ("quit"))))
     {
       exit (scm_exit_status (args));
     }
@@ -535,7 +502,7 @@ SCM_DEFINE (scm_catch, "catch", 3, 0, 0,
 {
   struct scm_body_thunk_data c;
 
-  SCM_ASSERT (SCM_SYMBOLP (key) || SCM_EQ_P (key, SCM_BOOL_T),
+  SCM_ASSERT (scm_is_symbol (key) || scm_is_eq (key, SCM_BOOL_T),
 	      key, SCM_ARG1, FUNC_NAME);
 
   c.tag = key;
@@ -563,7 +530,7 @@ SCM_DEFINE (scm_lazy_catch, "lazy-catch", 3, 0, 0,
 {
   struct scm_body_thunk_data c;
 
-  SCM_ASSERT (SCM_SYMBOLP (key) || SCM_EQ_P (key, SCM_BOOL_T),
+  SCM_ASSERT (scm_is_symbol (key) || scm_is_eq (key, SCM_BOOL_T),
 	      key, SCM_ARG1, FUNC_NAME);
 
   c.tag = key;
@@ -594,7 +561,7 @@ SCM_DEFINE (scm_throw, "throw", 1, 0, 1,
 	    "If there is no handler at all, Guile prints an error and then exits.")
 #define FUNC_NAME s_scm_throw
 {
-  SCM_VALIDATE_SYMBOL (1,key);
+  SCM_VALIDATE_SYMBOL (1, key);
   return scm_ithrow (key, args, 1);
 }
 #undef FUNC_NAME
@@ -617,7 +584,7 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
 	{
 	  SCM this_key = SCM_CAR (dynpair);
 
-	  if (SCM_EQ_P (this_key, SCM_BOOL_T) || SCM_EQ_P (this_key, key))
+	  if (scm_is_eq (this_key, SCM_BOOL_T) || scm_is_eq (this_key, key))
 	    break;
 	}
     }
@@ -632,19 +599,13 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
     }
 
   /* If the wind list is malformed, bail.  */
-  /* remember calls added below to fix segfaults on ia64 and (I think)
-     powerpc */
   if (!SCM_CONSP (winds))
-    {
-      scm_remember_upto_here_1 (winds);
-      abort ();
-    }
-  scm_remember_upto_here_1 (winds);  
+    abort ();
       
   jmpbuf = SCM_CDR (dynpair);
   
   for (wind_goal = scm_dynwinds;
-       !SCM_EQ_P (SCM_CDAR (wind_goal), jmpbuf);
+       !scm_is_eq (SCM_CDAR (wind_goal), jmpbuf);
        wind_goal = SCM_CDR (wind_goal))
     ;
 
@@ -673,9 +634,7 @@ scm_ithrow (SCM key, SCM args, int noreturn SCM_UNUSED)
       jbr = (struct jmp_buf_and_retval *)JBJMPBUF (jmpbuf);
       jbr->throw_tag = key;
       jbr->retval = args;
-#ifdef DEBUG_EXTENSIONS
       scm_last_debug_frame = SCM_JBDFRAME (jmpbuf);
-#endif
       longjmp (*JBJMPBUF (jmpbuf), 1);
     }
 
