@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,2000,2001, 2003 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,12 +44,24 @@
 
 #include "libguile/_scm.h"
 #include "libguile/vectors.h"
+#include "libguile/lang.h"
+#include "libguile/hashtab.h"
 
 #include "libguile/validate.h"
 #include "libguile/weaks.h"
 
 
 
+/* 1. The current hash table implementation in hashtab.c uses weak alist
+ *    vectors (formerly called weak hash tables) internally.
+ *
+ * 2. All hash table operations still work on alist vectors.
+ *
+ * 3. The weak vector and alist vector Scheme API is accessed through
+ *    the module (ice-9 weak-vector).
+ */
+
+
 /* {Weak Vectors}
  */
 
@@ -59,54 +71,44 @@
  * elements which are initialized with the 'fill' object, or, if 'fill' is
  * undefined, with an unspecified object.
  */
-static SCM
-allocate_weak_vector (scm_t_bits type, SCM size, SCM fill, const char* caller)
+SCM
+scm_i_allocate_weak_vector (scm_t_bits type, SCM size, SCM fill, const char* caller)
 #define FUNC_NAME caller
 {
-  if (SCM_INUMP (size))
+  size_t c_size;
+  SCM v;
+
+  c_size = scm_to_unsigned_integer (size, 0, SCM_VECTOR_MAX_LENGTH);
+
+  if (c_size > 0)
     {
-      size_t c_size;
-      SCM v;
-
-      SCM_ASSERT_RANGE (1, size, SCM_INUM (size) >= 0);
-      c_size = SCM_INUM (size);
-
-      SCM_NEWCELL2 (v);
-      SCM_SET_WVECT_GC_CHAIN (v, SCM_EOL);
-      SCM_SET_WVECT_TYPE (v, type);
-
-      if (c_size > 0)
-	{
-	  scm_t_bits *base;
-	  size_t j;
-
-	  if (SCM_UNBNDP (fill))
-	    fill = SCM_UNSPECIFIED;
-
-	  SCM_ASSERT_RANGE (1, size, c_size <= SCM_VECTOR_MAX_LENGTH);
-	  base = scm_must_malloc (c_size * sizeof (scm_t_bits), FUNC_NAME);
-	  for (j = 0; j != c_size; ++j)
-	    base[j] = SCM_UNPACK (fill);
-	  SCM_SET_VECTOR_BASE (v, base);
-	  SCM_SET_VECTOR_LENGTH (v, c_size, scm_tc7_wvect);
-	  scm_remember_upto_here_1 (fill);
-	}
-      else
-	{
-	  SCM_SET_VECTOR_BASE (v, NULL);
-	  SCM_SET_VECTOR_LENGTH (v, 0, scm_tc7_wvect);
-	}
-
-      return v;
+      scm_t_bits *base;
+      size_t j;
+      
+      if (SCM_UNBNDP (fill))
+	fill = SCM_UNSPECIFIED;
+      
+      base = scm_gc_malloc (c_size * sizeof (scm_t_bits), "weak vector");
+      for (j = 0; j != c_size; ++j)
+	base[j] = SCM_UNPACK (fill);
+      v = scm_double_cell (SCM_MAKE_VECTOR_TAG (c_size, scm_tc7_wvect),
+			   (scm_t_bits) base,
+			   type,
+			   SCM_UNPACK (SCM_EOL));
+      scm_remember_upto_here_1 (fill);
     }
-  else if (SCM_BIGP (size))
-    SCM_OUT_OF_RANGE (1, size);
   else
-    SCM_WRONG_TYPE_ARG (1, size);
+    {
+      v = scm_double_cell (SCM_MAKE_VECTOR_TAG (0, scm_tc7_wvect),
+			   (scm_t_bits) NULL,
+			   type,
+			   SCM_UNPACK (SCM_EOL));
+    }
+
+  return v;
 }
 #undef FUNC_NAME
 
-
 SCM_DEFINE (scm_make_weak_vector, "make-weak-vector", 1, 1, 0,
 	    (SCM size, SCM fill),
 	    "Return a weak vector with @var{size} elements. If the optional\n"
@@ -115,7 +117,7 @@ SCM_DEFINE (scm_make_weak_vector, "make-weak-vector", 1, 1, 0,
 	    "empty list.")
 #define FUNC_NAME s_scm_make_weak_vector
 {
-  return allocate_weak_vector (0, size, fill, FUNC_NAME);
+  return scm_i_allocate_weak_vector (0, size, fill, FUNC_NAME);
 }
 #undef FUNC_NAME
 
@@ -139,10 +141,13 @@ SCM_DEFINE (scm_weak_vector, "weak-vector", 0, 0, 1,
      while the vector is being created. */
   i = scm_ilength (l);
   SCM_ASSERT (i >= 0, l, SCM_ARG1, FUNC_NAME);
-  res = scm_make_weak_vector (SCM_MAKINUM (i), SCM_UNSPECIFIED);
-  data = SCM_VELTS (res);
+  res = scm_make_weak_vector (scm_from_int (i), SCM_UNSPECIFIED);
 
-  while (!SCM_NULLP (l))
+  /*
+    no alloc, so  this loop is safe.
+  */     
+  data = SCM_WRITABLE_VELTS (res);
+  while (!SCM_NULL_OR_NIL_P (l))
     {
       *data++ = SCM_CAR (l);
       l = SCM_CDR (l);
@@ -159,81 +164,84 @@ SCM_DEFINE (scm_weak_vector_p, "weak-vector?", 1, 0, 0,
 	    "weak hashes are also weak vectors.")
 #define FUNC_NAME s_scm_weak_vector_p
 {
-  return SCM_BOOL (SCM_WVECTP (obj) && !SCM_IS_WHVEC (obj));
+  return scm_from_bool (SCM_WVECTP (obj) && !SCM_IS_WHVEC (obj));
 }
 #undef FUNC_NAME
 
 
 
-SCM_DEFINE (scm_make_weak_key_hash_table, "make-weak-key-hash-table", 1, 0, 0, 
+SCM_DEFINE (scm_make_weak_key_alist_vector, "make-weak-key-alist-vector", 0, 1, 0, 
 	    (SCM size),
-	    "@deffnx {Scheme Procedure} make-weak-value-hash-table size\n"
-	    "@deffnx {Scheme Procedure} make-doubly-weak-hash-table size\n"
+	    "@deffnx {Scheme Procedure} make-weak-value-alist-vector size\n"
+	    "@deffnx {Scheme Procedure} make-doubly-weak-alist-vector size\n"
 	    "Return a weak hash table with @var{size} buckets. As with any\n"
 	    "hash table, choosing a good size for the table requires some\n"
 	    "caution.\n"
 	    "\n"
 	    "You can modify weak hash tables in exactly the same way you\n"
 	    "would modify regular hash tables. (@pxref{Hash Tables})")
-#define FUNC_NAME s_scm_make_weak_key_hash_table
+#define FUNC_NAME s_scm_make_weak_key_alist_vector
 {
-  return allocate_weak_vector (1, size, SCM_EOL, FUNC_NAME);
+  return scm_i_allocate_weak_vector
+    (1, SCM_UNBNDP (size) ? scm_from_int (31) : size, SCM_EOL, FUNC_NAME);
 }
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_make_weak_value_hash_table, "make-weak-value-hash-table", 1, 0, 0, 
+SCM_DEFINE (scm_make_weak_value_alist_vector, "make-weak-value-alist-vector", 0, 1, 0, 
             (SCM size),
 	    "Return a hash table with weak values with @var{size} buckets.\n"
 	    "(@pxref{Hash Tables})")
-#define FUNC_NAME s_scm_make_weak_value_hash_table
+#define FUNC_NAME s_scm_make_weak_value_alist_vector
 {
-  return allocate_weak_vector (2, size, SCM_EOL, FUNC_NAME);
+  return scm_i_allocate_weak_vector
+    (2, SCM_UNBNDP (size) ? scm_from_int (31) : size, SCM_EOL, FUNC_NAME);
 }
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_make_doubly_weak_hash_table, "make-doubly-weak-hash-table", 1, 0, 0, 
+SCM_DEFINE (scm_make_doubly_weak_alist_vector, "make-doubly-weak-alist-vector", 1, 0, 0, 
             (SCM size),
 	    "Return a hash table with weak keys and values with @var{size}\n"
 	    "buckets.  (@pxref{Hash Tables})")
-#define FUNC_NAME s_scm_make_doubly_weak_hash_table
+#define FUNC_NAME s_scm_make_doubly_weak_alist_vector
 {
-  return allocate_weak_vector (3, size, SCM_EOL, FUNC_NAME);
+  return scm_i_allocate_weak_vector
+    (3, SCM_UNBNDP (size) ? scm_from_int (31) : size, SCM_EOL, FUNC_NAME);
 }
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_weak_key_hash_table_p, "weak-key-hash-table?", 1, 0, 0, 
+SCM_DEFINE (scm_weak_key_alist_vector_p, "weak-key-alist-vector?", 1, 0, 0, 
            (SCM obj),
-	    "@deffnx {Scheme Procedure} weak-value-hash-table? obj\n"
-	    "@deffnx {Scheme Procedure} doubly-weak-hash-table? obj\n"
+	    "@deffnx {Scheme Procedure} weak-value-alist-vector? obj\n"
+	    "@deffnx {Scheme Procedure} doubly-weak-alist-vector? obj\n"
 	    "Return @code{#t} if @var{obj} is the specified weak hash\n"
 	    "table. Note that a doubly weak hash table is neither a weak key\n"
 	    "nor a weak value hash table.")
-#define FUNC_NAME s_scm_weak_key_hash_table_p
+#define FUNC_NAME s_scm_weak_key_alist_vector_p
 {
-  return SCM_BOOL (SCM_WVECTP (obj) && SCM_IS_WHVEC (obj));
+  return scm_from_bool (SCM_WVECTP (obj) && SCM_IS_WHVEC (obj));
 }
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_weak_value_hash_table_p, "weak-value-hash-table?", 1, 0, 0, 
+SCM_DEFINE (scm_weak_value_alist_vector_p, "weak-value-alist-vector?", 1, 0, 0, 
             (SCM obj),
 	    "Return @code{#t} if @var{obj} is a weak value hash table.")
-#define FUNC_NAME s_scm_weak_value_hash_table_p
+#define FUNC_NAME s_scm_weak_value_alist_vector_p
 {
-  return SCM_BOOL (SCM_WVECTP (obj) && SCM_IS_WHVEC_V (obj));
+  return scm_from_bool (SCM_WVECTP (obj) && SCM_IS_WHVEC_V (obj));
 }
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_doubly_weak_hash_table_p, "doubly-weak-hash-table?", 1, 0, 0, 
+SCM_DEFINE (scm_doubly_weak_alist_vector_p, "doubly-weak-alist-vector?", 1, 0, 0, 
             (SCM obj),
 	    "Return @code{#t} if @var{obj} is a doubly weak hash table.")
-#define FUNC_NAME s_scm_doubly_weak_hash_table_p
+#define FUNC_NAME s_scm_doubly_weak_alist_vector_p
 {
-  return SCM_BOOL (SCM_WVECTP (obj) && SCM_IS_WHVEC_B (obj));
+  return scm_from_bool (SCM_WVECTP (obj) && SCM_IS_WHVEC_B (obj));
 }
 #undef FUNC_NAME
 
@@ -260,7 +268,7 @@ scm_mark_weak_vector_spines (void *dummy1 SCM_UNUSED,
     {
       if (SCM_IS_WHVEC_ANY (w))
 	{
-	  SCM *ptr;
+	  SCM const *ptr;
 	  SCM obj;
 	  long j;
 	  long n;
@@ -274,11 +282,11 @@ scm_mark_weak_vector_spines (void *dummy1 SCM_UNUSED,
 
 	      alist = ptr[j];
 	      while (   SCM_CONSP (alist)
-		     && !SCM_GCMARKP (alist)
+		     && !SCM_GC_MARK_P (alist)
 		     && SCM_CONSP  (SCM_CAR (alist)))
 		{
-		  SCM_SETGCMARK (alist);
-		  SCM_SETGCMARK (SCM_CAR (alist));
+		  SCM_SET_GC_MARK (alist);
+		  SCM_SET_GC_MARK (SCM_CAR (alist));
 		  alist = SCM_CDR (alist);
 		}
 	    }
@@ -288,6 +296,7 @@ scm_mark_weak_vector_spines (void *dummy1 SCM_UNUSED,
   return 0;
 }
 
+#define UNMARKED_CELL_P(x) (SCM_NIMP(x) && !SCM_GC_MARK_P (x))
 
 static void *
 scm_scan_weak_vectors (void *dummy1 SCM_UNUSED,
@@ -301,13 +310,15 @@ scm_scan_weak_vectors (void *dummy1 SCM_UNUSED,
 	{
 	  register long j, n;
 
-	  ptr = SCM_VELTS (w);
+	  ptr = SCM_GC_WRITABLE_VELTS (w);
 	  n = SCM_VECTOR_LENGTH (w);
 	  for (j = 0; j < n; ++j)
-	    if (SCM_FREE_CELL_P (ptr[j]))
+	    if (UNMARKED_CELL_P (ptr[j]))
 	      ptr[j] = SCM_BOOL_F;
 	}
-      else /* if (SCM_IS_WHVEC_ANY (scm_weak_vectors[i])) */
+      /* check if we should scan the alist vector here (hashtables
+	 have their own scan function in hashtab.c). */
+      else if (!SCM_WVECT_NOSCAN_P (w))
 	{
 	  SCM obj = w;
 	  register long n = SCM_VECTOR_LENGTH (w);
@@ -315,7 +326,7 @@ scm_scan_weak_vectors (void *dummy1 SCM_UNUSED,
           int weak_keys = SCM_IS_WHVEC (obj) || SCM_IS_WHVEC_B (obj);
           int weak_values = SCM_IS_WHVEC_V (obj) || SCM_IS_WHVEC_B (obj);
 
-	  ptr = SCM_VELTS (w);
+	  ptr = SCM_GC_WRITABLE_VELTS (w);
 
 	  for (j = 0; j < n; ++j)
 	    {
@@ -325,16 +336,16 @@ scm_scan_weak_vectors (void *dummy1 SCM_UNUSED,
 	      fixup = ptr + j;
 	      alist = *fixup;
 
-	      while (   SCM_CONSP (alist)
-			&& SCM_CONSP (SCM_CAR (alist)))
+	      while (SCM_CONSP (alist)
+		     && SCM_CONSP (SCM_CAR (alist)))
 		{
 		  SCM key;
 		  SCM value;
 
 		  key = SCM_CAAR (alist);
 		  value = SCM_CDAR (alist);
-		  if (   (weak_keys && SCM_FREE_CELL_P (key))
-			 || (weak_values && SCM_FREE_CELL_P (value)))
+		  if (   (weak_keys && UNMARKED_CELL_P (key))
+			 || (weak_values && UNMARKED_CELL_P (value)))
 		    {
 		      *fixup = SCM_CDR (alist);
 		    }
@@ -360,10 +371,18 @@ scm_weaks_prehistory ()
 }
 
 
+SCM
+scm_init_weaks_builtins ()
+{
+#include "libguile/weaks.x"
+  return SCM_UNSPECIFIED;
+}
+
 void
 scm_init_weaks ()
 {
-#include "libguile/weaks.x"
+  scm_c_define_gsubr ("%init-weaks-builtins", 0, 0, 0,
+		      scm_init_weaks_builtins);
 }
 
 
